/*************************************************************************

	formZ by autodessys, Inc
	Copyright c 1990,1991,1992 autodessys, Inc.  All rights reserved.

	File :	waveOUT.c
	By :	Vassiles Gougoulides.

	Created on :  		15 August 1994.
	Updated on :  		10 September 1994.
	Updated on :  		16 September 1994,	option to save groups as files.
	Last Updated on :  	13 March 1995,	newlines for non mac destinations.
	Description :		export WaveFront format.

**************************************************************************

	List of routines in this file :

*************************************************************************/


#include "wave_prvt.h"




/************************************************************************
	This function writes a normal vector.
*************************************************************************/
static fzrt_error_td wave_write_Normal3D(wave_out_td *wave, fz_xyz_td *p)
/************************************************************************/
{
	fzrt_error_td		err = FZRT_NOERR;
	fz_xyz_td			xyz = *p;
	char				tempstr[256];
	fzrt_int				accrx, accry, accrz;
	fzrt_int				num_dec = wave->num_dec < 6 ? 6 : wave->num_dec;
	double				res_min = pow(10.0, (float)(-num_dec));

	/* This seed is used to adjust normal indices when Use Absolute Indices custom option is selected. */
	++wave->normal_seed;

	/* Reverse the normals if sepcified by Normals Outward transform option. */
	if (wave->inv_norms)
	{
		xyz.x = -xyz.x;
		xyz.y = -xyz.y;
		xyz.z = -xyz.z;
	}

	/* Determine the number of characters to write for each value based on the numerical accuracy from the project working units. */
	sprintf(tempstr, "%d", (fzrt_int)xyz.x);
	accrx = strlen(tempstr);
	accrx += num_dec;
	sprintf(tempstr, "%d", (fzrt_int)xyz.y);
	accry = strlen(tempstr);
	accry += num_dec;
	sprintf(tempstr, "%d", (fzrt_int)xyz.z);
	accrz = strlen(tempstr);
	accrz += num_dec;
	
	if(fabs(xyz.x) < res_min)		xyz.x = 0.0;
	if(fabs(xyz.y) < res_min)		xyz.y = 0.0;
	if(fabs(xyz.z) < res_min)		xyz.z = 0.0;

	/* Write the normal vector. */
	if (fprintf(wave->fp, "vn %.*lf %.*lf %.*lf%s", accrx, xyz.x, accry, xyz.y, accrz, xyz.z, wave->nl_str) == EOF)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes a spline coordinate in 2D parameter space.
	The coordinate is actually a 3d coordinate with Z representing
	the weight.
*************************************************************************/
static fzrt_error_td wave_write_CntrlPoint2D(wave_out_td *wave, fz_xyz_td *p)
/************************************************************************/
{
	fzrt_error_td		err = FZRT_NOERR;
	char				tempstr[256];
	fzrt_int				accrx, accry, accrz;
	double				res_min = pow(10.0, (float)(-wave->num_dec));

	/* This seed is used to adjust vertex indices when Use Absolute Indices custom option is selected. */
	++wave->point_seed;

	/* Determine the number of characters to write for each value based on the numerical accuracy from the project working units. */
	sprintf(tempstr, "%d", (fzrt_int)p->x);
	accrx = strlen(tempstr);
	accrx += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->y);
	accry = strlen(tempstr);
	accry += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->z);
	accrz = strlen(tempstr);
	accrz += wave->num_dec;
	
	if(fabs(p->x) < res_min)		p->x = 0.0;
	if(fabs(p->y) < res_min)		p->y = 0.0;
	if(fabs(p->z) < res_min)		p->z = 0.0;

	/* Write the coordinate. */
	if (fprintf(wave->fp, "vp %.*lf %.*lf %.*lf%s", accrx, p->x, accry, p->y, accrz, p->z, wave->nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes a 3D point location
*************************************************************************/
static fzrt_error_td wave_write_Point3D(wave_out_td *wave, fz_xyz_td *p)
/************************************************************************/
{
	fzrt_error_td		err = FZRT_NOERR;
	char				tempstr[256];
	fzrt_int				accrx, accry, accrz;
	double				res_min = pow(10.0, (float)(-wave->num_dec));

	/* This seed is used to adjust vertex indices when Use Absolute Indices custom option is selected. */
	++wave->point_seed;

	/* Determine the number of characters to write for each value based on the numerical accuracy from the project working units. */
	sprintf(tempstr, "%d", (fzrt_int)p->x);
	accrx = strlen(tempstr);
	accrx += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->y);
	accry = strlen(tempstr);
	accry += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->z);
	accrz = strlen(tempstr);
	accrz += wave->num_dec;
	
	if(fabs(p->x) < res_min)		p->x = 0.0;
	if(fabs(p->y) < res_min)		p->y = 0.0;
	if(fabs(p->z) < res_min)		p->z = 0.0;

	/* Write the point location. */
	if (fprintf(wave->fp, "v %.*lf %.*lf %.*lf%s", accrx, p->x, accry, p->y, accrz, p->z, wave->nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes a 4D point location.  This is generally used to write
	3D spline vertices.  The fourth component of the coordinate is the weight.
*************************************************************************/
static fzrt_error_td wave_write_Point4D(wave_out_td *wave, fz_xyzw_td *p)
/************************************************************************/
{
	fzrt_error_td		err = FZRT_NOERR;
	char				tempstr[256];
	fzrt_int				accrx, accry, accrz, accrw;
	double				res_min = pow(10.0, (float)(-wave->num_dec));

	/* This seed is used to adjust vertex indices when Use Absolute Indices custom option is selected. */
	++wave->point_seed;

	/* Determine the number of characters to write for each value based on the numerical accuracy from the project working units. */
	sprintf(tempstr, "%d", (fzrt_int)p->x);
	accrx = strlen(tempstr);
	accrx += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->y);
	accry = strlen(tempstr);
	accry += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->z);
	accrz = strlen(tempstr);
	accrz += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)p->w);
	accrw = strlen(tempstr);
	accrw += wave->num_dec;
	
	if(fabs(p->x) < res_min)		p->x = 0.0;
	if(fabs(p->y) < res_min)		p->y = 0.0;
	if(fabs(p->z) < res_min)		p->z = 0.0;
	if(fabs(p->w) < res_min)		p->w = 0.0;

	/* Write the point location. */
	if (fprintf(wave->fp, "v %.*lf %.*lf %.*lf %.*lf%s", accrx, p->x, accry, p->y, accrz, p->z, accrw, p->w, wave->nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes a texture uv coordinate.
*************************************************************************/
static fzrt_error_td wave_write_uv(wave_out_td *wave, fz_xy_td *uv)
{
	fzrt_error_td		err = FZRT_NOERR;
	char				tempstr[256];
	fzrt_int				accru, accrv;
	double				res_min = pow(10.0, (float)(-wave->num_dec));

	/* This seed is used to adjust vertex indices when Use Absolute Indices custom option is selected. */
	++wave->uv_seed;

	/* Determine the number of characters to write for each value based on the numerical accuracy from the project working units. */
	sprintf(tempstr, "%d", (fzrt_int)uv->x);
	accru = strlen(tempstr);
	accru += wave->num_dec;
	sprintf(tempstr, "%d", (fzrt_int)uv->y);
	accrv = strlen(tempstr);
	accrv += wave->num_dec;
	
	if(fabs(uv->x) < res_min)		uv->x = 0.0;
	if(fabs(uv->y) < res_min)		uv->y = 0.0;

	/* Write the point location. */
	if (fprintf(wave->fp, "vt %.*lf %.*lf%s", accru, uv->x, accrv, uv->y, wave->nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes a header for a Wavefront file.  This applies
	to object (.obj) files and material (.mtl) files.
*************************************************************************/
static fzrt_error_td wave_write_header(wave_out_td *wave, FILE *fp, char *type)
{
	fzrt_error_td		err = FZRT_NOERR;
	time_t				now;
	struct tm			*dt;
	char				version[256], date[256], comp[256], name[256], user[256], cpyrt[768];
	char				*nl_str = wave->nl_str;

	/* Get the version string for the formZ application. */
	_syst_funcs.fz_syst_appl_get_info(FZ_APPL_VERSION, version, 256);

	/* Create a date/time string. */
	now = time(NULL);
	dt = localtime(&now);
	strftime(date, 256, "%c", dt);

	/* Get the uner information strings from the formZ application. */
	_syst_funcs.fz_syst_appl_get_info(FZ_APPL_USER_COMPANY, comp, 256);
	_syst_funcs.fz_syst_appl_get_info(FZ_APPL_USER_NAME, name, 256);
	_syst_funcs.fz_syst_appl_get_info(FZ_APPL_USER_SERIAL, user, 256);
	_syst_funcs.fz_syst_appl_get_info(FZ_APPL_COPYRIGHT, cpyrt, 768);

	/* Write the file header. */
	if (fprintf(fp, "# Creator form*Z %s, %s %s", version, cpyrt, nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	if (err == FZRT_NOERR && fprintf(fp, "# Date %s%s", date, nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	if (err == FZRT_NOERR && fprintf(fp, "# User %s:%s:%s%s", comp, name, user, nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	if (err == FZRT_NOERR && fprintf(fp, "# %s File%s%s", type, nl_str, nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function writes all surface styles to a Wavefront material (.mtl) file.
*************************************************************************/
static fzrt_error_td wave_write_surface_styles(wave_out_td *wave)
{
	fzrt_error_td				err = FZRT_NOERR;
	fzrt_int						sindx, num_styles;
	char						mtl_name[64];
	fz_xyz_td					rgb;
	FILE						*fp = NULL;
	fz_ffmt_tmap_style_ptr		style;
	fz_ffmt_tmap_file_ptr		file;
	char						*nl_str = wave->nl_str;
	fzrt_int						num;
	double						ambient, diffuse, specular;
	double						transp, spec_expo, refr;
	fz_rgb_float_td				diff_col, spec_col;
	float						bmp_amp;
	fzrt_int						color, trans, bump;
	char						name2[128];
	fzrt_floc_ptr				floc2;
	fz_ffmt_tmap_filetab_ptr	filetab;
	fz_ffmt_tmap_styletab_ptr	styletab;

	/* Open the material file */
	if((fp = fopen(wave->lib_name, "wb")) == NULL)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	else
	{
		if((err = wave_write_header(wave, fp, "Material")) == FZRT_NOERR)
		{
			/* Get the surface style table. */
			err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);

			/* Get the number of surface styles in the table. */
			if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_count(styletab, &num);

			/* Loop over the surface styles. */
			for(sindx = 1, num_styles = num; sindx <= num_styles && err == FZRT_NOERR; ++sindx)
			{
				/* Get an entry from the surface style table. */
				if(_ffmt_3d_funcs.fz_ffmt_data_model_styletab_entry(styletab, sindx, &style))
				{
					/* Get and write the surface style's name. */
					err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
					if(err == FZRT_NOERR)
					{
						/* Replace illegal characters wile '_'. */
						_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
						if (fprintf(fp, "newmtl %s%s", mtl_name, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Get and write the surface style's ambient reflection color. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_ambient(style, &ambient);
						rgb.x = rgb.y = rgb.z = ambient;
						if (err == FZRT_NOERR && fprintf(fp, "Ka %lG %lG %lG%s", rgb.x, rgb.y, rgb.z, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
					
					/* Get and write the diffuse reflection color. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_diffuse(style, &diffuse);
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_diffuse_color(style, &diff_col);
					if(err == FZRT_NOERR && wave->do_tmaps != 0)
					{
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_color_file_id(style, &color);
						if (err == FZRT_NOERR && color != 0)
						{
							rgb.x = rgb.y = rgb.z =  diffuse;
						}
						else
						{
							rgb.x = (diffuse * diff_col.red);
							rgb.y = (diffuse * diff_col.green);
							rgb.z = (diffuse * diff_col.blue);
						}
					}
					else
					{
						rgb.x = (diffuse * diff_col.red);
						rgb.y = (diffuse * diff_col.green);
						rgb.z = (diffuse * diff_col.blue);
					}
					if (err == FZRT_NOERR && fprintf(fp, "Kd %lG %lG %lG%s", rgb.x, rgb.y, rgb.z, nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
					
					/* Get and write the diffuse reflection color. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_specular(style, &specular);
					if(err == FZRT_NOERR)
					{
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_specular_color(style, &spec_col);
						rgb.x = (specular * spec_col.red);
						rgb.y = (specular * spec_col.green);
						rgb.z = (specular * spec_col.blue);
						if (err == FZRT_NOERR && fprintf(fp, "Ks %lG %lG %lG%s", rgb.x, rgb.y, rgb.z, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}

					/* Get the transparency. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_transparency(style, &transp);

					/* Get the specular exponent. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_spec_expo(style, &spec_expo);
					if(err == FZRT_NOERR)
					{
						/* Get the specular index of refraction. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_refraction(style, &refr);

						/* Write the transparency. */
						rgb.x = rgb.y = rgb.z = transp;
						if (err == FZRT_NOERR && fprintf(fp, "Tf %lG %lG %lG%s", rgb.x, rgb.y, rgb.z, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						if (err == FZRT_NOERR && fprintf(fp, "illum 4%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						if (err == FZRT_NOERR && fprintf(fp, "d 1.0%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}

						/* Write the specular exponent. */
						if (err == FZRT_NOERR && fprintf(fp, "Ns %lG%s",spec_expo, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						if (err == FZRT_NOERR && fprintf(fp, "sharpness 60%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}

						/* Write the specular index of refraction. */
						if (err == FZRT_NOERR && fprintf(fp, "Ni %lG%s", refr, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}

					/* If exporting textures. */
					if (err == FZRT_NOERR && wave->do_tmaps != 0)
					{
						_fzrt_file_funcs.fzrt_file_floc_init(&floc2);

						/* Get the texture file table. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab(wave->ffmt_id, &filetab);

						/* If this surface style has a color texture map, get the texture map's file table entry. */
						if (err == FZRT_NOERR && color > 0 && _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_entry(filetab, color, &file)) 
						{
							/* Get and write the texture map file name. */
							err = _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_dst_floc(file, floc2);
							if(err == FZRT_NOERR)	err = _fzrt_file_funcs.fzrt_file_floc_get_name(floc2, name2, 127);
							if (err == FZRT_NOERR && fprintf(fp, "map_Ka %s%s", name2, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							if (err == FZRT_NOERR && fprintf(fp, "map_Kd %s%s", name2, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							if (err == FZRT_NOERR && fprintf(fp, "map_Ks %s%s", name2, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
						
						if(err == FZRT_NOERR)
						{
							/* If this surface style has a transparency texture map, get the texture map's file table entry. */
							err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_transparency_file_id(style, &trans);
							if (err == FZRT_NOERR && trans > 0 && _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_entry(filetab, trans, &file)) 
							{	
								/* Get and write the texture map file name. */
								err = _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_dst_floc(file, floc2);
								if(err == FZRT_NOERR)	err = _fzrt_file_funcs.fzrt_file_floc_get_name(floc2, name2, 127);
								if (err == FZRT_NOERR && fprintf(fp, "decal %s%s", name2, nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}

						if(err == FZRT_NOERR)
						{
							/* If this surface style has a bump texture map, get the texture map's file table entry. */
							err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_bmap_file_id(style, &bump);
							if (err == FZRT_NOERR && bump > 0 && _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_entry(filetab, bump, &file)) 
							{
								/* Get and write the texture map file name and bump amplitude. */
								err = _ffmt_3d_funcs.fz_ffmt_data_model_tmap_filetab_dst_floc(file, floc2);
								if(err == FZRT_NOERR)	err = _fzrt_file_funcs.fzrt_file_floc_get_name(floc2, name2, 127);
								if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_bmap_amplitude(style, &bmp_amp);
								if (err == FZRT_NOERR && fprintf(fp, "bump -bm %f %s%s", bmp_amp, name2, nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}

						_fzrt_file_funcs.fzrt_file_floc_finit(&floc2);

						/* Write a new line string for the last line. */
						if (err == FZRT_NOERR && fprintf(fp, "%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
				}
				else 
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(FZRT_UNKNOWN_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0, 1021);
				}
			}
		}
	}

	/* Close the material file. */
	if (fp != NULL)
	{
		fclose(fp);
		fp = NULL;
		if (err == FZRT_NOERR)
		{
			/* Set the file's macintosh file type and creator as defined by the
				translator's get file type function (wave_get_ftype). */
			_ffmt_funcs.fz_file_set_attributes(wave->ffmt_id, 1, wave->floc);
		}
		else
		{
			/* If an error occurred, delete the file. */
			_ffmt_funcs.fz_file_remove(wave->floc);
		}
	}

	return err;
}

/************************************************************************
	This function creates a name for an object and guarantees it to be unique.
*************************************************************************/
static fzrt_error_td wave_make_object_name(wave_out_td *wave, fz_objt_ptr obj, char *name64, fzrt_int *obj_seed)
{
	fzrt_error_td	err = FZRT_NOERR;

	/* Get the name of the object. */
	err = _objt_attr_funcs.fz_objt_attr_get_objt_name(wave->windex, obj, name64);
	if ( err != FZRT_NOERR)
	{
		/* Append a unique number to it to guarantee a unique name. */
		sprintf(name64, "object %d", ++(*obj_seed));
	}

	/* Replace illegal characters wile '_'. */
	_ffmt_funcs.fz_ffmt_make_identifier(name64);

	return FZRT_NOERR;
}

/************************************************************************
	This function writes an object's vertices as points.
*************************************************************************/
fzrt_error_td wave_write_points(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fz_xyz_td *vertex_normals, fz_xy_td *vertex_texture_uvs)
{
	fzrt_error_td		err = FZRT_NOERR;
	fzrt_int				i, ncord = 0;
	char				name64[64];
	wave_out_td			*wave = (wave_out_td *)inst;
	char				*nl_str = wave->nl_str;
	FILE				*fp = wave->fp;
	fz_xyz_td			cord;
	fzrt_int				cnt;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the number of the vertices in the object. */
			err = _model_funcs.fz_objt_get_point_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &ncord);

			/* Loop over all the object's vertices. */
			for (i = 0; i < ncord && err == FZRT_NOERR; ++i)
			{
				/* Get the location of the vertex. */
				err = _model_funcs.fz_objt_point_get_xyz(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &cord);

				/* Write the location of the vertex. */
				if(err == FZRT_NOERR)	err = wave_write_Point3D(wave, &cord);
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			if(err == FZRT_NOERR)
			{
				/* Write the point tag. */
				if (fprintf(fp, "p ") < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}

				/* Loop over all the object's vertices. */
				for (i = 0, cnt = 0; i < ncord && err == FZRT_NOERR; ++i)
				{
					/* Write the adjusted (as specified by the Use Absolute Indices custom option) vertex indices. */
					if(fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)(-ncord + i))) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
					
					/* Write 8 points per line. */
					if(cnt % 8 == 0 && err == FZRT_NOERR)
					{
						if (fprintf(fp, "%sp ", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}
				}

				/* Write a new line string for the last line. */
				if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
			}
		}
	}
										/*----return error code----*/
	return err;
}

/************************************************************************
	This function writes an object's segments as a line segments.
*************************************************************************/
fzrt_error_td wave_write_lines(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fz_xyz_td *vertex_normals, fz_xy_td *vertex_texture_uvs)
{
	fzrt_error_td		err = FZRT_NOERR;
	fzrt_int				i, ncord = 0;
	char				name64[64];
	wave_out_td			*wave = (wave_out_td *)inst;
	char				*nl_str = wave->nl_str;
	FILE				*fp = wave->fp;
	fz_xyz_td			cord;
	fzrt_int				nsegt;
	char 				*segt_marker = NULL;
	fzrt_int				ri, pindx1, pindx2;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the number of the vertices in the object. */
			err = _model_funcs.fz_objt_get_point_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &ncord);

			/* Loop over all the object's vertices. */
			for (i = 0; i < ncord && err == FZRT_NOERR; ++i)
			{
				/* Get the location of the vertex. */
				err = _model_funcs.fz_objt_point_get_xyz(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &cord);

				/* Write the location of the vertex. */
				if(err == FZRT_NOERR)	err = wave_write_Point3D(wave, &cord);
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			if(err == FZRT_NOERR)
			{
				/* Get the number of the segments in the object. */
				_model_funcs.fz_objt_get_segt_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &nsegt);
				
				/* Create a segment marker array.  Segments on an object may be stored twice (once for each direction).  formZ maintains
					a link between these segments (one is called the reverse of the other).  Each segment should ony be
					written to the obj file once.  So for each segment, it's reverse segment is found and marked as written. */
				if(_fzrt_mem_funcs.fzrt_new_ptr_clear((fzrt_size)nsegt*sizeof(char), (fzrt_ptr*)&segt_marker) != FZRT_NOERR)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0, 1021);
				}
				
				/* Loop over all the object's segments. */
				for (i = 0; i < nsegt && err == FZRT_NOERR; ++i)
				{
					/* If the segment has not been marked as written, write it. */
					if(!segt_marker[i])
					{
						/* Get the segments reverse segment. */
						err = _model_funcs.fz_objt_segt_get_reverse(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &ri);
						
						/* Mark the reverse segment as written. */
						if(err == FZRT_NOERR && ri > -1) segt_marker[ri] = TRUE;
						
						/* Get the adjusted (as specified by the Use Absolute Indices custom option) endpoint vertex indices of the segment. */
						err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &pindx1);
						if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_segt_get_end_pindx(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &pindx2);
						
						/* Write the endpoint vertex indices of the segment. */
						if (err == FZRT_NOERR && fprintf(fp, "l %ld %ld%s",
											APINDEX(wave, (fzrt_int)(-ncord + pindx1)),
											APINDEX(wave, (fzrt_int)(-ncord + pindx2)),
											nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}
				}

				/* Write a new line string for the last line. */
				if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				
				/* Free the memory for the segment marker. */
				if(segt_marker != NULL) _fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)segt_marker);
			}
		}
	}

	return err;
}

/************************************************************************
	This function writes an object's curves as polylines.
*************************************************************************/
fzrt_error_td wave_write_pline(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fz_xyz_td *vertex_normals, fz_xy_td *vertex_texture_uvs)
{
	fzrt_error_td		err = FZRT_NOERR;
	fzrt_int				i, ncord = 0, cindx, scindx, segt, psegt, pindx, pindex;
	fzrt_int				tindex;
	char				name64[64];
	wave_out_td			*wave;
	char				*nl_str;
	FILE				*fp;
	fz_xyz_td			cord;

	wave = (wave_out_td *)inst;
	nl_str = wave->nl_str;
	fp = wave->fp;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the number of the vertices in the object. */
			err = _model_funcs.fz_objt_get_point_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &ncord);

			/* Loop over all the object's vertices. */
			for (i = 0; i < ncord && err == FZRT_NOERR; ++i)
			{
				/* Get the location of the vertex. */
				err = _model_funcs.fz_objt_point_get_xyz(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &cord);

				/* Write the location of the vertex. */
				if(err == FZRT_NOERR)	err = wave_write_Point3D(wave, &cord);
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			if(err == FZRT_NOERR)
			{
				/* Write the line tag. */
				if (fprintf(fp, "l ") < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				else
				{
					/* Get the first curve of the face. */
					err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, 0, FZ_OBJT_MODEL_TYPE_FACT, &cindx);
					scindx = cindx;
					do 
					{
						/* Get the first segment of the curve. */
						if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_curv_get_sindx(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &segt);

						/* Ensure that there aren't any segments ahead of psegt. */
						if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_segt_get_prev(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &psegt);
						while (psegt != -1 && psegt != segt && err == FZRT_NOERR)
						{
							err = _model_funcs.fz_objt_segt_get_prev(wave->windex, obj, psegt, FZ_OBJT_MODEL_TYPE_FACT, &psegt);
						}

						/* Loop over the curve's segments and write the topology of the polyline. */
						psegt = segt;
						if(err == FZRT_NOERR)
						{
							do {
								/* Get the index of the first vertex of the segment. */
								err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, psegt, FZ_OBJT_MODEL_TYPE_FACT, &pindx);

								if(err == FZRT_NOERR)
								{
									/* Adjust the vertex indexing as specified by the Use Absolute Indices custom option. */
									tindex = pindx - ncord;
									pindex = APINDEX(wave, tindex);
														
										/* Write the vertex index. */
									if (fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)(-ncord + pindx))) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}

									/* Get the next segment. */
									if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_segt_get_next(wave->windex, obj, psegt, FZ_OBJT_MODEL_TYPE_FACT, &psegt);
								}

								if(err == FZRT_NOERR)
								{
									/* Update the wait cursor. */
									_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

									/* Check for a cancel from the user. */
									if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
								}

								/* Loop until we reach the starting segment or a segment index of -1. */
							} while (psegt != -1 && psegt != segt && err == FZRT_NOERR);

							/* Write a new line string for the last line. */
							if (fprintf(fp, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}

						/* Get the next curve from the face. */
						err = _model_funcs.fz_objt_curv_get_next(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &cindx);

						/* Loop until we reach the starting curve or a curve index of -1. */
					} while (cindx != -1 && cindx != scindx && err == FZRT_NOERR);
				}
			}
		}
	}

	return err;
}

/************************************************************************
	This function writes an object as polygonal meshes.
	This function assumes all faces have no holes.
*************************************************************************/
fzrt_error_td wave_write_mesh(
			fzrt_int			windex,
			fz_ffmt_ref_td	ffmt_id,
			void			*inst,
			fz_objt_ptr		obj,
			fz_xyz_td		*face_normals,
			fz_xyz_td		*vertex_normals,
			fz_xy_td		*vertex_texture_uvs)
{
	fzrt_error_td				err = FZRT_NOERR;
	fzrt_int						i, j, nface = 0, ncord = 0, segt, sindx, *pindx_map = NULL;
	char						name64[64];
	fzrt_int						tindex, pindex, nindex, uindex;
	fzrt_int						cindx = -1, old_cindx = -1;
	char						mtl_name[64];
	fz_ffmt_tmap_style_ptr		style;
	wave_out_td					*wave = (wave_out_td *)inst;
	char						*nl_str = wave->nl_str;
	FILE						*fp = wave->fp;
	fz_xyz_td					cord;
	fzrt_int						obj_lface, pindx;
	fz_ffmt_tmap_styletab_ptr	styletab;

	wave = (wave_out_td *)inst;
	nl_str = wave->nl_str;
	fp = wave->fp;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the number of the vertices in the object. */
			err = _model_funcs.fz_objt_get_point_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &ncord);

			/* Loop over all the object's vertices. */
			for (i = 0; i < ncord && err == FZRT_NOERR; ++i)
			{
				/* Get the location of the vertex. */
				err = _model_funcs.fz_objt_point_get_xyz(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &cord);

				/* Write the location of the vertex. */
				if (err == FZRT_NOERR)	err = wave_write_Point3D(wave, &cord);
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			/* If applicable, export the vertex normals */
			if (vertex_normals != NULL && err == FZRT_NOERR)
			{
				/* The normals in the vertex_normals array correspond to the 
					vertices in the object's vertex array. So, they are indexed in the same way. */
				for (i = 0; i < ncord; ++i)
				{
					err = wave_write_Normal3D(wave, &vertex_normals[i]);
				}
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			/* if applicable export the texture uv coordinates. */
			if (vertex_texture_uvs != NULL && err == FZRT_NOERR)
			{
				/* The texture coordinates in the vertex_texture_uvs array correspond to the 
					vertices in the object's vertex array. So, they are indexed in the same way. */
				for (i = 0; i < ncord && err == FZRT_NOERR; ++i)
				{
					err = wave_write_uv(wave, &vertex_texture_uvs[i]);
				}
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}

			if(err == FZRT_NOERR)
			{
				/* Get the number of the faces in the object. */
				err = _model_funcs.fz_objt_get_face_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &obj_lface);

				/* Loop over all the faces. */
				for (i = 0; i < obj_lface && err == FZRT_NOERR; ++i)
				{
					/* Write the face's material. */
					/* Get the style table from formZ. */
					err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);

					/* Get the style pointer for this face. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_sid_from_face(wave->windex, wave->ffmt_id, styletab, obj, i, &style);

					/* Get the style's id. */
					if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_id(style, &cindx);

					/* The material only needs to be written if it changes. */
					if (err == FZRT_NOERR && cindx != old_cindx)
					{
						old_cindx = cindx;
										
						/* Get the surface style name. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
						if(err == FZRT_NOERR)
						{
							mtl_name[63] = '\0';
												
							/* Replace illegal characters wile '_'. */
							_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
							if (fprintf(fp, "usemtl %s%s", mtl_name, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
					}

					/* Write the face tag. */
					if (err == FZRT_NOERR && fprintf(fp, "f ") < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}

					/* This assumes the face has no holes.  Get the first (only) curve of the face. */
					if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, i, FZ_OBJT_MODEL_TYPE_FACT, &cindx);

					/* Loop over the curve's segments and write the topology of the face. */
					if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_curv_get_sindx(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &sindx);
					if(err == FZRT_NOERR)
					{
						segt = sindx;
						j = 0;
						do {
							/* Write 20 vertex indices per line. */
							if (((j % 20) == 0) && (j > 0))
							{
								if (fprintf(fp, "\\%s  ", nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}

							/* Get the index of the first vertex of the segment. */
							if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &pindx);

							if(err == FZRT_NOERR)
							{
								/* Adjust the vertex indexing as specified by the Use Absolute Indices custom option. */
								tindex = pindx - ncord;
								pindex = APINDEX(wave, tindex);
								nindex = ANINDEX(wave, tindex);
								uindex = AUVINDEX(wave, tindex);
													
								/* Write the appropriate combination of vertices, normals and texture coordinates indices. */
								if ((vertex_normals == NULL) && (vertex_texture_uvs == NULL))
								{
									if (fprintf(fp, "%ld ", pindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else if ((vertex_normals != NULL) && (vertex_texture_uvs == NULL))
								{
									if (fprintf(fp, "%ld//%ld ", pindex, nindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else if ((vertex_normals == NULL) && (vertex_texture_uvs != NULL))
								{
									if (fprintf(fp, "%ld/%ld ", pindex, uindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else /* This is the ((vertex_normals != NULL) && (vertex_texture_uvs != NULL)) case. */
								{
									if (fprintf(fp, "%ld/%ld/%ld ", pindex, uindex, nindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								++j;

								/* Get the next or previous segment depending on the Inverse Faces transform option. */
								if(wave->inv_faces)
								{
									err = _model_funcs.fz_objt_segt_get_prev(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
								}
								else
								{
									err = _model_funcs.fz_objt_segt_get_next(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
								}

								if(err == FZRT_NOERR)
								{
									/* Update the wait cursor. */
									_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

									/* Check for a cancel from the user. */
									if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
								}
							}

							/* Loop until we reach the starting segment or a segment index of -1. */
						} while (segt != sindx && segt != -1 && err == FZRT_NOERR);

						/* Write a new line string for the last line. */
						if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
				}
			}
		}
	}

	return err;
}


/************************************************************************
	This function writes an object as individual faces.
	This function assumes all faces have no holes.
*************************************************************************/
fzrt_error_td wave_write_faces(
			fzrt_int			windex,
			fz_ffmt_ref_td	ffmt_id,
			void			*inst,
			fz_objt_ptr		obj,
			fz_xyz_td		*face_normals,
			fz_xyz_td		*vertex_normals,
			fz_xy_td		*vertex_texture_uvs)
{
	fzrt_error_td				err = FZRT_NOERR;
	fzrt_int						f, i, nsegt, sindx, segt;
	fzrt_int						tindex, pindex, nindex, uindex;
	fzrt_int						cindx = -1, old_cindx = -1;
	fzrt_int						obj_lface;
	fzrt_int						pindx;
	char						mtl_name[64];
	char						name64[64];
	wave_out_td					*wave;
	char						*nl_str;
	FILE						*fp;
	fz_xyz_td					cord;
	fz_ffmt_tmap_style_ptr		style;
	fz_ffmt_tmap_styletab_ptr	styletab;
	
	wave = (wave_out_td *)inst;
	nl_str = wave->nl_str;
	fp = wave->fp;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
			if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Loop over all faces in the object. */
			_model_funcs.fz_objt_get_face_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_FACT, &obj_lface);
			for (f = 0; f < obj_lface && err == FZRT_NOERR; ++f)
			{
				/* Write the face's vertices. */
				/* This assumes the face has no holes.  Get the first (only) curve of the face. */
				err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, f, FZ_OBJT_MODEL_TYPE_FACT, &cindx);
				
				/* Loop over the curve's segments and get the coordinate for the first vertex of the segment. */
				if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_curv_get_sindx(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &sindx);
				if(err == FZRT_NOERR)
				{
					nsegt = 0; segt = sindx;
					do
					{
						/* Count the vertices that are written. */
						++nsegt;

						/* Get the index of the first vertex of the segment. */
						err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &pindx);
						
						/* Get the location of the vertex. */
						if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_point_get_xyz(wave->windex, obj, pindx, FZ_OBJT_MODEL_TYPE_FACT, &cord);
						
						/* Write the coordinate of the vertex. */
						if(err == FZRT_NOERR)	err = wave_write_Point3D(wave, &cord);
						
						if(err == FZRT_NOERR)
						{
							/* Get the next or previous segment depending on the Inverse Faces transform option. */
							if(wave->inv_faces)
							{
								_model_funcs.fz_objt_segt_get_prev(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
							}
							else
							{
								_model_funcs.fz_objt_segt_get_next(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
							}
						}
						
						/* Loop until we reach the starting segment or a segment index of -1. */
					} while (segt != sindx && segt != -1 && err == FZRT_NOERR);
				}

				if(err == FZRT_NOERR)
				{
					/* Update the wait cursor. */
					_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

					/* Check for a cancel from the user. */
					if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
				}

				if(err == FZRT_NOERR)
				{
					/* If applicable, write the vertex normals */
					if (vertex_normals != NULL)
					{
						/* Loop over the faces segments as before.  The normals in the vertex_normals array correspond to the 
							vertices in the object's vertex array. So, they are indexed in the same way. */
						/* This assumes the face has no holes.  Get the first (only) curve of the face. */
						err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, f, FZ_OBJT_MODEL_TYPE_FACT, &cindx);

						/* Loop over the curve's segments and get the coordinate for the first vertex of the segment. */
						if(err == FZRT_NOERR)	_model_funcs.fz_objt_curv_get_sindx(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &sindx);
						if(err == FZRT_NOERR)	
						{
								segt = sindx;
							do
							{
								/* Get the index of the first vertex of the segment. */
								err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &pindx);

								/* Use the vertex index to lookup and write the normal of the vertex. */
								if(err == FZRT_NOERR)	err = wave_write_Normal3D(wave, &vertex_normals[pindx]);
							
								if(err == FZRT_NOERR)	
								{
									/* Get the next or previous segment depending on the Inverse Faces transform option. */
									if(wave->inv_faces)
									{
										_model_funcs.fz_objt_segt_get_prev(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
									}
									else
									{
										_model_funcs.fz_objt_segt_get_next(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
									}
								}

								/* Loop until we reach the starting segment or a segment index of -1. */
							} while (segt != sindx && segt != -1 && err == FZRT_NOERR);
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}

					/* If applicable, write the texture uv coordinates */
					if (err == FZRT_NOERR && vertex_texture_uvs != NULL)
					{
						/* Loop over the faces segments as before.  The texture coordinates in the vertex_texture_uvs array
							correspond to the vertices in the object's vertex array. So, they are indexed in the same way. */
						/* This assumes the face has no holes.  Get the first (only) curve of the face. */
						err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, f, FZ_OBJT_MODEL_TYPE_FACT, &cindx);

						/* Loop over the curve's segments and get the coordinate for the first vertex of the segment. */
						if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_curv_get_sindx(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_FACT, &sindx);
						if(err == FZRT_NOERR)
						{
							segt = sindx;
							do {
								/* Get the index of the first vertex of the segment. */
								err = _model_funcs.fz_objt_segt_get_start_pindx(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &pindx);

								/* Use the vertex index to lookup and write the texture coordinate of the vertex. */
								if(err == FZRT_NOERR)	err = wave_write_uv(wave, &vertex_texture_uvs[pindx]);
							
								if(err == FZRT_NOERR)
								{
									/* Get the next or previous segment depending on the Inverse Faces transform option. */
									if(wave->inv_faces)
									{
										_model_funcs.fz_objt_segt_get_prev(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
									}
									else
									{
										_model_funcs.fz_objt_segt_get_next(wave->windex, obj, segt, FZ_OBJT_MODEL_TYPE_FACT, &segt);
									}
								}
							
								/* Loop until we reach the starting segment or a segment index of -1. */
							} while (segt != sindx && segt != -1 && err == FZRT_NOERR);
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}

					if(err == FZRT_NOERR)
					{
						/* Write the face's material. */
						/* Get the style table from formZ. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);
						
						/* Get the style pointer for this face. */
						if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_sid_from_face(wave->windex, wave->ffmt_id, styletab, obj, f, &style);
						
						/* Get the style's id. */
						if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_id(style, &cindx);

						/* The material only needs to be written if it changes. */
						if (err == FZRT_NOERR && cindx != old_cindx)
						{
							old_cindx = cindx;
							
							/* Get the surface style name. */
							err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
							if(err == FZRT_NOERR)
							{
								mtl_name[63] = '\0';
								
								/* Replace illegal characters wile '_'. */
								_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
								
								/* Write the name of the surface style.  This is the same name written to the mtl file. */
								if (fprintf(fp, "usemtl %s%s", mtl_name, nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}

					if(err == FZRT_NOERR)
					{
						/* Write the face tag. */
						if (fprintf(fp, "f ") < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						
						/* Loop for each vertex that was written for this face. */
						for (i = 0; i < nsegt && err == FZRT_NOERR; ++i)
						{
							/* Write 20 vertex indices per line. */
							if (((i % 20) == 0) && (i > 0))
							{
								if (fprintf(fp, "\\%s  ", nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
							
							if(err == FZRT_NOERR)
							{
								/* Adjust the vertex indexing as specified by the Use Absolute Indices custom option. */
								tindex = (fzrt_int)(-nsegt + i);
								pindex = APINDEX(wave, tindex);
								nindex = ANINDEX(wave, tindex);
								uindex = AUVINDEX(wave, tindex);
								
								/* Write the appropriate combination of vertices, normals and texture coordinates indices. */
								if ((vertex_normals == NULL) && (vertex_texture_uvs == NULL))
								{
									if (fprintf(fp, "%ld ", pindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else if ((vertex_normals != NULL) && (vertex_texture_uvs == NULL))
								{
									if (fprintf(fp, "%ld//%ld ", pindex, nindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else if ((vertex_normals == NULL) && (vertex_texture_uvs != NULL))
								{
									if (fprintf(fp, "%ld/%ld ", pindex, uindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
								else /* This is the ((vertex_normals != NULL) && (vertex_texture_uvs != NULL)) case */
								{
									if (fprintf(fp, "%ld/%ld/%ld ", pindex, uindex, nindex) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
							}
						}
					}

					/* Write a new line string for the last line. */
					if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				}
			}
		}
	}

	return err;
}

/************************************************************************
	This function writes trimmed nurbs surfaces.
*************************************************************************/
fzrt_error_td wave_write_trimmed(
				fzrt_int			windex,
				fz_ffmt_ref_td	ffmt_id,
				void			*inst,
				fz_objt_ptr		obj,
				fzrt_boolean	start,
				fzrt_boolean	end,
				fzrt_boolean	do_tmap)
{
	fzrt_error_td				err = FZRT_NOERR;
	fzrt_int						f, i, j,
								*ncord = NULL,
								ncurv /*, cindx, curv*/ ;
	fzrt_int						cindx = -1, old_cindx = -1, scindx;
	char						mtl_name[64];
	fz_ffmt_tmap_style_ptr		style;
	char						name64[64];
	wave_out_td					*wave;
	char						*nl_str;
	FILE						*fp;
	fzrt_int						obj_lface;
	fz_ffmt_tmap_styletab_ptr	styletab;
	fz_nurbs_cur_ptr			ncrv;
	fz_nurbs_srf_ptr			nsrf;
	fzrt_int						num_crv;
	double						*weights = NULL;
	double						*Uknot = NULL, *Vknot = NULL;
	fz_xyz_td					*pnts = NULL;
	fz_xyz_td					*cord2d = NULL;
	fz_xyzw_td					*cord = NULL;
	fz_xy_td					*Uminmax;
	fz_xy_td					uv;
	double						Umin, Umax, Vmin, Vmax;
	double						step_u, step_v;
	fzrt_int						nU, degU;
	fzrt_int						nV, degV;
	fzrt_int						num;
	fzrt_int						tindex;
	fz_tag_td					col_tag;
	fzrt_boolean				has_trim = FALSE;
	
	wave = (wave_out_td *)inst;
	nl_str = wave->nl_str;
	fp = wave->fp;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the number of faces in the object. Each face is a trimmed surface. */
			err = _model_funcs.fz_objt_get_face_count(wave->windex, obj, FZ_OBJT_MODEL_TYPE_SMOD, &obj_lface);

			/* Loop over all the faces. */
			for (f = 0; f < obj_lface && err == FZRT_NOERR; ++f)
			{
				/* Write face's surface style */
				/* Get the surface style table. */
				err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);
						
				/* Get the style pointer for this face. */
				if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_sid_from_face(wave->windex, wave->ffmt_id, styletab, obj, f, &style);
						
				/* Get the style's id. */
				if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_id(style, &cindx);

				/* The material only needs to be written if it changes. */
				if (err == FZRT_NOERR && cindx != old_cindx)
				{
					old_cindx = cindx;
					/* Get and write the surface style's name. */
					err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
					if(err == FZRT_NOERR)
					{
						mtl_name[63] = '\0';
								
						/* Replace illegal characters wile '_'. */
						_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
						if (fprintf(fp, "usemtl %s%s", mtl_name, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
				}

				/* Allocate memory for an array of trimming curve TODO TODO TODO. */
				if(err == FZRT_NOERR)	err = _model_funcs.fz_objt_face_get_cindx(wave->windex, obj, f, FZ_OBJT_MODEL_TYPE_SMOD, &cindx);
				scindx = cindx;
				if(err == FZRT_NOERR)
				{
					err = _model_funcs.fz_objt_face_get_curv_count(wave->windex, obj, f, FZ_OBJT_MODEL_TYPE_SMOD, &ncurv);
					_fzrt_mem_funcs.fzrt_new_ptr(ncurv*sizeof(fz_xy_td), (fzrt_ptr*)&Uminmax);
					if(Uminmax == NULL)
					{
						err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
					}
				}
		
				if(err == FZRT_NOERR)
				{
					/* Loop over the face's curves.  Each curve represents a trimming curve.
						Write each curve. */
					i = 0;
					do {
						err = _model_funcs.fz_objt_alys_get_curv_pspace_trim(windex, obj, cindx, &num_crv, &ncrv);

						if(ncrv != NULL && num_crv > 0)
						{
							has_trim = TRUE;
						/* Get the nurbs parameters from the nurbs curve description. */
							if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_npts(ncrv, &nU);
							if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_degree(ncrv, &degU);
					
							if(err == FZRT_NOERR)
							{
								/* Allocate memory for controls points, weights, and knots. */
								_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(fz_xyz_td), (fzrt_ptr*)&pnts);
								if(pnts == NULL)
								{
									err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
								}
								if(err == FZRT_NOERR)
								{
									_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(double), (fzrt_ptr*)&weights);
									if(weights == NULL)
									{
										err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
									}
								}
								if(err == FZRT_NOERR)
								{
									_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(fz_xyz_td), (fzrt_ptr*)&cord2d);
									if(cord2d == NULL)
									{
										err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
									}
								}
								if(err == FZRT_NOERR)
								{
									_fzrt_mem_funcs.fzrt_new_ptr((nU+degU+1)*sizeof(double), (fzrt_ptr*)&Uknot);
									if(Uknot == NULL)
									{
										err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
									}
								}
							}

							/* Get the control points, weights, and knots from the nurbs surface description. */
							if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_control_points(ncrv, pnts);
							if(err == FZRT_NOERR)	err = 	_ncrv_funcs.fz_ncrv_get_weights(ncrv, weights);
							if(err == FZRT_NOERR)	err = 	_ncrv_funcs.fz_ncrv_get_knots(ncrv, Uknot);

							/* Copy the control points and weights to a combined array. */
							for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								cord2d[j].x = pnts[j].x;
								cord2d[j].y = pnts[j].y;
								cord2d[j].z = weights[j];
							}

							/* Write the control points and weights. */
							for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								err = wave_write_CntrlPoint2D(wave, &cord2d[j]);
							}

							/* Write the curve type */
							if (err == FZRT_NOERR && fprintf(fp, "cstype rat bspline%s", nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
					
							/* Write the degree */
							if (err == FZRT_NOERR && fprintf(fp, "deg %ld%s", (fzrt_int)(degU), nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
				
							/* Write the umin/max and point indices */
							if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_domain(ncrv, &Umin, &Umax);
							if (err == FZRT_NOERR && fprintf(fp, "curv2 ") < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								if (fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)(-nU + j))) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
							if (err == FZRT_NOERR && fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)(-nU))) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							
							/* Write the knots */
							if (err == FZRT_NOERR && fprintf(fp, "parm u ") < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							for (j = 0; j < nU + degU + 1 && err == FZRT_NOERR; ++j)
							{
								if (fprintf(fp, "%lG ", Uknot[j]) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
							if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							Uminmax[i].x = 0.0;
							if(Uknot != NULL)	Uminmax[i].y = Uknot[nU + degU];
							else	Uminmax[i].y = 1.0;
						
							/* End the curve. */
							if (err == FZRT_NOERR && fprintf(fp, "end%s", nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							
							if(Uknot != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Uknot);
							if(cord2d != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)cord2d);
							if(weights != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)weights);
							if(pnts != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)pnts);
						}
						/* Get the next curve from the face. */
						if (err == FZRT_NOERR)	err = _model_funcs.fz_objt_curv_get_next(wave->windex, obj, cindx, FZ_OBJT_MODEL_TYPE_SMOD, &cindx);
						i++;
					}
					while(cindx != scindx && cindx != -1 && err == FZRT_NOERR);

					if(err == FZRT_NOERR)
					{
						/* Write the nurbs surface. */

						/* Get the object's nurbs surface description. */
						err = _model_funcs.fz_objt_get_nsrf(wave->windex, obj, &nsrf);

						/* Get the nurbs parameters from the nurbs surface description. */
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_u_npts(nsrf, &nU);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_npts(nsrf, &nV);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_u_degree(nsrf, &degU);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_degree(nsrf, &degV);

						if(err == FZRT_NOERR)
						{
							/* Allocate memory for controls points, weights, and knots. */
							num = nU*nV;
							_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(fz_xyz_td), (fzrt_ptr*)&pnts);
							if(pnts == NULL)
							{
								err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
							}
							if(err == FZRT_NOERR)
							{
								_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(double), (fzrt_ptr*)&weights);
								if(weights == NULL)
								{
									err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
								}
							}
							if(err == FZRT_NOERR)
							{
								_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(fz_xyzw_td), (fzrt_ptr*)&cord);
								if(cord == NULL)
								{
									err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
								}
							}
							if(err == FZRT_NOERR)
							{
								_fzrt_mem_funcs.fzrt_new_ptr((nU+degU+1)*sizeof(double), (fzrt_ptr*)&Uknot);
								if(Uknot == NULL)
								{
									err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
								}
							}
							if(err == FZRT_NOERR)
							{
								_fzrt_mem_funcs.fzrt_new_ptr((nV+degV+1)*sizeof(double), (fzrt_ptr*)&Vknot);
								if(Vknot == NULL)
								{
									err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
								}
							}
						}
						
						/* Get the control points, weights, and knots from the nurbs surface description. */
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_control_points(nsrf, pnts);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_weights(nsrf, weights);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_u_knots(nsrf, Uknot);
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_knots(nsrf, Vknot);

						/* Copy the control points and weights to a combined array. */
						for (j = 0; j < num && err == FZRT_NOERR; ++j)
						{
							cord[j].x = pnts[j].x / weights[j];
							cord[j].y = pnts[j].y / weights[j];
							cord[j].z = pnts[j].z / weights[j];
							cord[j].w = weights[j];
						}

						/* Write the control points and weights. */
						for (i = 0; i < nV && err == FZRT_NOERR; ++i)
						{
							for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								err = wave_write_Point4D(wave, &cord[j * nV + i]);
							}
							
							if(err == FZRT_NOERR)
							{
								/* Update the wait cursor. */
								_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

								/* Check for a cancel from the user. */
								if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
							}
						}

						/* If applicable, Write the texture map controls */
						if (err == FZRT_NOERR && do_tmap)
						{
							step_u = 1.0 / (double)nU;
							step_v = 1.0 / (double)nV;
							for (i = 0; i < nV && err == FZRT_NOERR; ++i)
							{
								for (j = 0; j < nU && err == FZRT_NOERR; ++j)
								{
									uv.x = (double)j * step_u; uv.y = (double)i * step_v;
									err = wave_write_uv(wave, &uv);
								}
							
								if(err == FZRT_NOERR)
								{
									/* Update the wait cursor. */
									_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

									/* Check for a cancel from the user. */
									if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
								}
							}
						}
						
						/* Write the surface type */
						if (err == FZRT_NOERR && fprintf(fp, "cstype rat bspline%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						
						/* Write the degree in u/v */
						if (err == FZRT_NOERR && fprintf(fp, "deg %ld %ld%s", (fzrt_int)(degU), (fzrt_int)(degV), nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}

						/* Write the surface style */
						/* Get the surface style table. */
						if(err == FZRT_NOERR)	err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);

						/* Get the style tag for this object. */
						if(err == FZRT_NOERR)	err = _objt_attr_funcs.fz_objt_attr_get_objt_rmtl(wave->windex, obj, TRUE, &col_tag);
								
						/* Get the style pointer for this face. */
						if (err == FZRT_NOERR && _ffmt_3d_funcs.fz_ffmt_data_model_styletab_entry_by_surf_tag(styletab, &col_tag, &style))
						{
							/* Get and write the surface style's name. */
							err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
							if(err == FZRT_NOERR)
							{
								mtl_name[63] = '\0';
												
								/* Replace illegal characters wile '_'. */
								_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
								if (fprintf(fp, "usemtl %s%s", mtl_name, nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}
						
						/* Write the umin/max, vmin/max and point indices */
						if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_domain(nsrf, &Umin, &Umax, &Vmin, &Vmax);
						if (fprintf(fp, "surf %lG %lG %lG %lG \\%s", Umin, Umax, Vmin, Vmax, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						for (i = 0; i < nV && err == FZRT_NOERR; ++i)
						{
							if (err == FZRT_NOERR && fprintf(fp, "\t") < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							if (do_tmap)
							{
								for (j = 0; j < nU && err == FZRT_NOERR; ++j)
								{
									tindex = (fzrt_int)((i - nV) * nU + j);
									if (fprintf(fp, "%ld/%ld ", APINDEX(wave, tindex), AUVINDEX(wave, tindex)) < 0)
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
							}
							else
							{	for (j = 0; j < nU && err == FZRT_NOERR; ++j)
								{
									if (fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)((i - nV) * nU + j))) < 0)			/* VAS 02/05/96 */
									{
										err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
									}
								}
							}
							if (err == FZRT_NOERR && fprintf(fp, ((i == (nV - 1))?"%s":"\\%s"), nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
						
						/* Write the U knots */
						if (err == FZRT_NOERR && fprintf(fp, "parm u ") < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						for (i = 0; i < nU + degU + 1 && err == FZRT_NOERR; ++i)
						{
							if (fprintf(fp, "%lG ", Uknot[i]) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
						if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}

						if(err == FZRT_NOERR)
						{
							/* Update the wait cursor. */
							_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

							/* Check for a cancel from the user. */
							if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
						}
						
						/* Write the V knots */
						if (err == FZRT_NOERR && fprintf(fp, "parm v ") < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						for (i = 0; i < nV + degV + 1 && err == FZRT_NOERR; ++i)
						{
							if (fprintf(fp, "%lG ", Vknot[i]) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
						if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}

						if(err == FZRT_NOERR)
						{
							/* Update the wait cursor. */
							_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

							/* Check for a cancel from the user. */
							if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
						}
						
						if(has_trim)
						{
							if (err == FZRT_NOERR && fprintf(fp, "trim %lG %lG %d%s", (double)Uminmax[0].x, (double)Uminmax[0].y, wave->curv2_seed + 1/*AC2INDEX(wave, -ncurv)*/, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
							for (i = 1; i < ncurv && err == FZRT_NOERR; ++i)
							{
								if (fprintf(fp, "hole %lG %lG %d%s", (double)Uminmax[i].x, (double)Uminmax[i].y, wave->curv2_seed + 1 + i/*AC2INDEX(wave, -ncurv + i)*/, nl_str) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}
						wave->curv2_seed += ncurv;
						
						/* End the surface. */
						if (err == FZRT_NOERR && fprintf(fp, "end%s", nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						
						if(Vknot != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Vknot);
						if(Uknot != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Uknot);
						if(cord != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)cord);
						if(weights != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)weights);
						if(pnts != NULL)	_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)pnts);
					}
				}
			}
		}
	}

	return err;
}

/************************************************************************
	This function determines if an object can be exported as a smooth object.
*************************************************************************/
fzrt_error_td wave_can_do_smooth(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fzrt_boolean do_tmap, fz_ffmt_data_model_write_smod_meth_enum *smod_method)
{
	wave_out_td								*wave = (wave_out_td *)inst;
	fz_ffmt_data_model_write_smod_meth_enum	rv = FZ_FFMT_DATA_MODEL_WRITE_SMOD_AS_FACT;
	fzrt_error_td							err = FZRT_NOERR;

	if(*smod_method == FZ_FFMT_DATA_MODEL_WRITE_SMOD_AS_TRIMMED)
	{
		rv = FZ_FFMT_DATA_MODEL_WRITE_SMOD_AS_TRIMMED;
	}
	*smod_method = rv;
	
	return(err);
}

/************************************************************************
	This function exports a nurbs curve.
	This function assumes one curve per object.
*************************************************************************/
static fzrt_error_td wave_write_curve(fz_objt_ptr obj, void *inst)
{
	fzrt_error_td		err = FZRT_NOERR;
	fzrt_int				j, nU, degU;
	char				name64[64];
	wave_out_td			*wave;
	char				*nl_str;
	FILE				*fp;
	double				Umin, Umax;
	fz_xyz_td			*pnts = NULL;
	double				*weights = NULL;
	double				*Uknot = NULL;
	fz_xyzw_td			*cord = NULL;
	fz_nurbs_cur_ptr	ncrv;
	
	wave = (wave_out_td *)inst;
	fp = wave->fp;
	nl_str = wave->nl_str;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		/* Write the object's name. */
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the object's nurbs curve description. */
			err = _model_funcs.fz_objt_get_ncrv(wave->windex, obj, &ncrv);

			/* Get the nurbs parameters from the nurbs curve description. */
			if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_npts(ncrv, &nU);
			if(err == FZRT_NOERR)	err = _ncrv_funcs.fz_ncrv_get_degree(ncrv, &degU);

			if(err == FZRT_NOERR)	
			{
				/* Allocate memory for controls points, weights, and knots. */
				_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(fz_xyz_td), (fzrt_ptr*)&pnts);
				if(pnts == NULL)
				{
					err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
				}
				if(err == FZRT_NOERR)
				{
					_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(double), (fzrt_ptr*)&weights);
					if(weights == NULL)
					{
						err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
					}
				}
				if(err == FZRT_NOERR)
				{
					_fzrt_mem_funcs.fzrt_new_ptr(nU*sizeof(fz_xyzw_td), (fzrt_ptr*)&cord);
					if(cord == NULL)
					{
						err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
					}
				}
				if(err == FZRT_NOERR)
				{
					_fzrt_mem_funcs.fzrt_new_ptr((nU+degU+1)*sizeof(double), (fzrt_ptr*)&Uknot);
					if(Uknot == NULL)
					{
						err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
					}
				}
			}

			if(err == FZRT_NOERR)	
			{
				/* Get the control points, weights, and knots from the nurbs surface description. */
				err = _ncrv_funcs.fz_ncrv_get_control_points(ncrv, pnts);
				if(err == FZRT_NOERR)	err = 	_ncrv_funcs.fz_ncrv_get_weights(ncrv, weights);
				if(err == FZRT_NOERR)	err = 	_ncrv_funcs.fz_ncrv_get_knots(ncrv, Uknot);

				if(err == FZRT_NOERR)	
				{
					/* Copy the control points and weights to a combined array. */
					for (j = 0; j < nU; ++j)
					{
						cord[j].x = pnts[j].x;
						cord[j].y = pnts[j].y;
						cord[j].z = pnts[j].z;
						cord[j].w = weights[j];
					}
				}
			}

			if(err == FZRT_NOERR)	
			{
				/* Write the control points and weights. */
				for (j = 0; j < nU && err == FZRT_NOERR; ++j)
				{
					err = wave_write_Point4D(wave, &cord[j]);
				}
			}

			if(err == FZRT_NOERR)
			{
				/* Update the wait cursor. */
				_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

				/* Check for a cancel from the user. */
				if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
			}
			
			if(err == FZRT_NOERR)
			{
				/* Write the curve type */
				if (fprintf(fp, "cstype rat bspline%s", nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
			
				/* Write the degree */
				if (err == FZRT_NOERR && fprintf(fp, "deg %ld%s", (fzrt_int)(degU), nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
			}

			if(err == FZRT_NOERR)
			{
				/* Write the umin/max and point indices */
				err = _ncrv_funcs.fz_ncrv_get_domain(ncrv, &Umin, &Umax);
				if (err == FZRT_NOERR && fprintf(fp, "curv %lG %lG ", Umin, Umax) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				for (j = 0; j < nU && err == FZRT_NOERR; ++j)
				{
					if (fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)(-nU + j))) < 0)				/* VAS 02/05/96 */
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				}
				if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				
				/* Write the knots */
				if (err == FZRT_NOERR && fprintf(fp, "parm u ") < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				for (j = 0; j < (nU + degU + 1) && err == FZRT_NOERR; ++j)
				{
					if (fprintf(fp, "%lG ", Uknot[j]) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				}
				if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				
				if(err == FZRT_NOERR)
				{
					/* Write the curve type */
					if (fprintf(fp, "cstype rat bspline%s", nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				
					/* Write the degree */
					if (err == FZRT_NOERR && fprintf(fp, "deg %ld%s", (fzrt_int)(degU), nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				}
				
				/* End the curve. */
				if (err == FZRT_NOERR && fprintf(fp, "end%s", nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
			}
		}
	}

	/* Free memory for the control points, weights, and knots. */
	if (Uknot != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Uknot);
		Uknot = NULL;
	}
	if (cord != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)cord);
		cord = NULL;
	}
	if (pnts != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)pnts);
		pnts = NULL;
	}
	if (weights != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)weights);
		weights = NULL;
	}

	return err;
}

/************************************************************************
	This function exports a nurbs surface.
*************************************************************************/
static fzrt_error_td wave_write_nurbs(fz_objt_ptr obj, fzrt_boolean do_tmaps, void *inst)
{
	fzrt_error_td				err = FZRT_NOERR;
	fzrt_int						i, j;
	fzrt_int						nU, degU;
	fzrt_int						nV, degV;
	fzrt_int						tindex;
	fzrt_int						num;
	char						mtl_name[64];
	fz_ffmt_tmap_style_ptr		style;
	fz_xy_td					uv;
	double						Umin, Umax, Vmin, Vmax;
	double						step_u, step_v;
	char						name64[64];
	wave_out_td					*wave;
	char						*nl_str;
	FILE						*fp;
	fz_ffmt_tmap_styletab_ptr	styletab;
	fz_nurbs_srf_ptr			nsrf;
	fz_xyz_td					*pnts = NULL;
	double						*weights = NULL;
	double						*Uknot = NULL, *Vknot = NULL;
	fz_xyzw_td					*cord = NULL;
	fz_tag_td					col_tag;
	
	wave = (wave_out_td *)inst;
	fp = wave->fp;
	nl_str = wave->nl_str;

	/* Create a unique object name.  In formZ, several objects can have the same name, obj does not allow this. */
	if ((err = wave_make_object_name(wave, obj, name64, &wave->obj_seed)) == FZRT_NOERR)
	{
		/* Write the object's name. */
		if (fprintf(fp, "o %s%s", name64, nl_str) < 0)
		{
			err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
		}
		else
		{
			/* Get the object's nurbs surface description. */
			err = _model_funcs.fz_objt_get_nsrf(wave->windex, obj, &nsrf);

			if(err == FZRT_NOERR)
			{
				/* Get the nurbs parameters from the nurbs surface description. */
				err = _nsrf_funcs.fz_nsrf_get_u_npts(nsrf, &nU);
				if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_npts(nsrf, &nV);
				if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_u_degree(nsrf, &degU);
				if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_degree(nsrf, &degV);

				if(err == FZRT_NOERR)
				{
					/* Allocate memory for controls points, weights, and knots. */
					num = nU*nV;
					_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(fz_xyz_td), (fzrt_ptr*)&pnts);
					if(pnts == NULL)
					{
						err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
					}
					if(err == FZRT_NOERR)
					{
						_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(double), (fzrt_ptr*)&weights);
						if(weights == NULL)
						{
							err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
						}
					}
					if(err == FZRT_NOERR)
					{
						_fzrt_mem_funcs.fzrt_new_ptr(num*sizeof(fz_xyzw_td), (fzrt_ptr*)&cord);
						if(cord == NULL)
						{
							err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
						}
					}
					if(err == FZRT_NOERR)
					{
						_fzrt_mem_funcs.fzrt_new_ptr((nU+degU+1)*sizeof(double), (fzrt_ptr*)&Uknot);
						if(Uknot == NULL)
						{
							err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
						}
					}
					if(err == FZRT_NOERR)
					{
						_fzrt_mem_funcs.fzrt_new_ptr((nV+degV+1)*sizeof(double), (fzrt_ptr*)&Vknot);
						if(Vknot == NULL)
						{
							err = _fzrt_base_funcs.fzrt_error_set(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0);
						}
					}
				}
				
				/* Get the control points, weights, and knots from the nurbs surface description. */
				if(err == FZRT_NOERR)
				{
					err = _nsrf_funcs.fz_nsrf_get_control_points(nsrf, pnts);
					if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_weights(nsrf, weights);
					if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_u_knots(nsrf, Uknot);
					if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_v_knots(nsrf, Vknot);

					/* Copy the control points and weights to a combined array. */
					for (j = 0; j < num && err == FZRT_NOERR; ++j)
					{
						cord[j].x = pnts[j].x / weights[j];
						cord[j].y = pnts[j].y / weights[j];
						cord[j].z = pnts[j].z / weights[j];
						cord[j].w = weights[j];
					}
				}

				/* Write the control points and weights. */
				for (i = 0; i < nV && err == FZRT_NOERR; ++i)
				{
					for (j = 0; j < nU && err == FZRT_NOERR; ++j)
					{
						err = wave_write_Point4D(wave, &cord[j * nV + i]);
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}
				}

				/* If exporting texture maps, write the texture map controls */
				if (err == FZRT_NOERR && do_tmaps)
				{
					step_u = 1.0 / (double)nU;
					step_v = 1.0 / (double)nV;
					for (i = 0; i < nV && err == FZRT_NOERR; ++i)
					{
						for (j = 0; j < nU && err == FZRT_NOERR; ++j)
						{
							uv.x = (double)j * step_u; uv.y = (double)i * step_v;
							err = wave_write_uv(wave, &uv);
						}

						if(err == FZRT_NOERR)
						{
							/* Update the wait cursor. */
							_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

							/* Check for a cancel from the user. */
							if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
						}
					}
				}
				
				/* Write the surface type */
				if (err == FZRT_NOERR && fprintf(fp, "cstype rat bspline%s", nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}
				
				/* Write the degree in u/v */
				if (err == FZRT_NOERR && fprintf(fp, "deg %ld %ld%s", (fzrt_int)(degU), (fzrt_int)(degV), nl_str) < 0)
				{
					err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
				}

				if(err == FZRT_NOERR)
				{
					/* Write the surface style */
					/* Get the surface style table. */
					err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab(wave->ffmt_id, &styletab);

					/* Get the style tag for this face. */
					if(err == FZRT_NOERR)	err = _objt_attr_funcs.fz_objt_attr_get_objt_rmtl(wave->windex, obj, TRUE, &col_tag);
						
					/* Get the style pointer for this face. */
					if (err == FZRT_NOERR && _ffmt_3d_funcs.fz_ffmt_data_model_styletab_entry_by_surf_tag(styletab, &col_tag, &style))
					{
						/* Get and write the surface style's name. */
						err = _ffmt_3d_funcs.fz_ffmt_data_model_styletab_surf_name(style, mtl_name, 63);
						if(err == FZRT_NOERR)
						{
							mtl_name[63] = '\0';
								
							/* Replace illegal characters wile '_'. */
							_ffmt_funcs.fz_ffmt_make_identifier(mtl_name);
							if (fprintf(fp, "usemtl %s%s", mtl_name, nl_str) < 0)
							{
								err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
							}
						}
					}
				
					/* Write the umin/max, vmin/max and point indices */
					if(err == FZRT_NOERR)	err = _nsrf_funcs.fz_nsrf_get_domain(nsrf, &Umin, &Umax, &Vmin, &Vmax);
					if (err == FZRT_NOERR && fprintf(fp, "surf %lG %lG %lG %lG \\%s", Umin, Umax, Vmin, Vmax, nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
					for (i = 0; i < nV && err == FZRT_NOERR; ++i)
					{
						if (err == FZRT_NOERR && fprintf(fp, "\t") < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
						if (err == FZRT_NOERR && do_tmaps)
						{
							for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								tindex = (fzrt_int)((i - nV) * nU + j);
								if (err == FZRT_NOERR && fprintf(fp, "%ld/%ld ", APINDEX(wave, tindex), AUVINDEX(wave, tindex)) < 0)
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}
						else
						{	for (j = 0; j < nU && err == FZRT_NOERR; ++j)
							{
								if (err == FZRT_NOERR && fprintf(fp, "%ld ", APINDEX(wave, (fzrt_int)((i - nV) * nU + j))) < 0)			/* VAS 02/05/96 */
								{
									err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
								}
							}
						}
						if (err == FZRT_NOERR && fprintf(fp, ((i == (nV - 1))?"%s":"\\%s"), nl_str) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
					
					/* Write the U knots */
					if (err == FZRT_NOERR && fprintf(fp, "parm u ") < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
					for (i = 0; i < nU + degU + 1 && err == FZRT_NOERR; ++i)
					{
						if (fprintf(fp, "%lG ", Uknot[i]) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
					if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}
					
					/* Write the V knots */
					if (err == FZRT_NOERR && fprintf(fp, "parm v ") < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
					for (i = 0; i < nV + degV + 1 && err == FZRT_NOERR; ++i)
					{
						if (err == FZRT_NOERR && fprintf(fp, "%lG ", Vknot[i]) < 0)
						{
							err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
						}
					}
					if (err == FZRT_NOERR && fprintf(fp, nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}

					if(err == FZRT_NOERR)
					{
						/* Update the wait cursor. */
						_fuim_funcs.fz_fuim_curs_wait(FZ_FUIM_CURS_WAIT_TURN);

						/* Check for a cancel from the user. */
						if (_fuim_funcs.fz_fuim_key_cancel())	err = FZ_USER_CANCEL;
					}
					
					/* End the surface. */
					if (err == FZRT_NOERR && fprintf(fp, "end%s", nl_str) < 0)
					{
						err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
					}
				}
			}
		}
	}

	/* Free memory for the control points, weights, and knots. */
	if (Uknot != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Uknot);
		Uknot = NULL;
	}
	if (Vknot != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)Vknot);
		Vknot = NULL;
	}
	if (cord != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)cord);
		cord = NULL;
	}
	if (weights != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)weights);
		cord = NULL;
	}
	if (pnts != NULL)
	{
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)pnts);
		cord = NULL;
	}

	return err;
}


/************************************************************************
	This function checks to see if a controlled object can be exported to a file.
	The obj format only supports nurbs curvs and surfaces.
	Since we set the FZ_FFMT_FORMAT_ATTR_SUPPORTS_NURBS_BIT in the
	get info function (wave_get_info), this function my be called
	twice for each object.  If the controlled object is not a nurbs
	surface, it will be called once for the origonal object; this function
	will return FALSE.  Then formZ will convert the object to a nurbs surface
	and call this function again; this function will then return TRUE.
*************************************************************************/
fzrt_boolean wave_can_do_ctrl(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fzrt_boolean do_tmap, fz_ffmt_data_model_write_cntl_meth_enum cntl_method)
{
	fzrt_boolean	rv = FALSE;
	wave_out_td		*wave;
	fzrt_UUID_td	utype;
	
	wave = (wave_out_td *)inst;

	/* Get the object type (UUID) and compair it to the UUID's for
		nurbs curves and nurbs surfaces. */
	_model_funcs.fz_objt_cntl_get_uuid(wave->windex, obj, utype);
	if(_fzrt_base_funcs.fzrt_UUID_is_equal(utype, FZ_OBJT_TYPE_NCRV))
	{
		rv = TRUE;
	}
	else if(_fzrt_base_funcs.fzrt_UUID_is_equal(utype, FZ_OBJT_TYPE_NRBZ))
	{
		if(do_tmap)
		{
			/* If the object is textured, the texture mapping must be parametric. 
				Otherwise, the object is exported as a facetted object with
				uv texture coordinates. */
			if(_ffmt_3d_funcs.fz_ffmt_data_model_tmap_isparam(windex, obj))
			{
				rv = TRUE;
			}
		}
		else
		{
			rv = TRUE;
		}
	}
	
	return(rv);
}

/************************************************************************
	This function exports a controlled object.
*************************************************************************/
fzrt_error_td wave_write_ctrl(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_objt_ptr obj, fzrt_boolean do_tmap)
{
	fzrt_error_td	err = FZRT_NOERR;
	wave_out_td		*wave;
	fzrt_UUID_td	utype;
	
	wave = (wave_out_td *)inst;

	/* Get the object type (UUID) and compair it to the UUID's for
		nurbs curves and nurbs surfaces. */
	_model_funcs.fz_objt_cntl_get_uuid(wave->windex, obj, utype);
	if(_fzrt_base_funcs.fzrt_UUID_is_equal(utype, FZ_OBJT_TYPE_NCRV))
	{
		/* Export a nurbs curve. */
		err = (fzrt_error_td)wave_write_curve(obj, inst);
	}
	else if(_fzrt_base_funcs.fzrt_UUID_is_equal(utype, FZ_OBJT_TYPE_NRBZ))
	{
		/* Export a nurbs surface. */
		err = (fzrt_error_td)wave_write_nurbs(obj, do_tmap, inst);
	}
	
	return err;
}


/************************************************************************
	This function opens and writes a header to a file.  If the Separate Files options is set, this
	function will get called multiple times (once for each exported group).
*************************************************************************/
fzrt_error_td wave_write_file_begin(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fzrt_floc_ptr floc)
{
	fzrt_error_td		err = FZRT_NOERR;
	wave_out_td			*wave = (wave_out_td *)inst;
	char				*nl_str = wave->nl_str;

	/* Open the file. */
	if ((wave->fp = fopen(wave->workname, "wb")) == NULL)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	/* Set the file's buffer size. */
	else if (setvbuf(wave->fp, NULL, _IOFBF, 0x00008000L))
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	else
	{
		/* Write the file header. */
		if ((err = wave_write_header(wave, wave->fp, "Object")) == FZRT_NOERR)
		{
			/* Write the material library file. */
			if (fprintf(wave->fp, "mtllib %s%s", wave->lib_name, nl_str) == EOF)
			{
				err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
			}
			else
			{
				wave->normal_seed = 0;
				wave->point_seed = 0;
				wave->point2_seed = 0;
				wave->curv2_seed = 0;
				wave->uv_seed = 0;
			}
		}
	}

	return err;
}

/************************************************************************
	This function ends and closes a file.  If the Separate Files options is set, this
	function will get called multiple times (once for each exported group).
*************************************************************************/
fzrt_error_td wave_write_file_end(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fzrt_floc_ptr floc)
{
	fzrt_error_td		err = FZRT_NOERR;
	wave_out_td 		*wave;
	FILE				*fp;
	
	wave = (wave_out_td *)inst;
	fp = wave->fp;

	if (fp != NULL)
	{
		/* Close the file. */
		fclose(fp);
		wave->fp = NULL;
		
		/* Set the file's macintosh file type and creator as defined by the
			translator's get file type function (wave_get_ftype). */
		_ffmt_funcs.fz_file_set_attributes(wave->ffmt_id, 1, wave->floc);
	}

	return err;
}

/************************************************************************
	This function handles the start of a group.  This is not groups as shown in the
	formZ objects palette, but groups created as defined by the export
	grouping method.
*************************************************************************/
fzrt_error_td wave_write_group_begin(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_tag_td *fz_tag, char *name)
{
	fzrt_error_td			err = FZRT_NOERR;
	char					group[64];
	wave_out_td				*wave;
	char					*nl_str;
	FILE					*fp;
	
	wave = (wave_out_td *)inst;
	nl_str = wave->nl_str;

	strncpy(group, name, 63);

	fp = wave->fp;

	/* start a wavefront group */							
	/* Replace illegal characters wile '_'. */
	_ffmt_funcs.fz_ffmt_make_identifier(group);
	if (fprintf(fp, "g %s%s", group, nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	else if (fprintf(fp, "s off%s", nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}
	else if (fprintf(fp, "mg off%s", nl_str) < 0)
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(errno, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_ANSI, 0, 2008);
	}

	return err;
}

/************************************************************************
	This function handles the end of a group.
*************************************************************************/
fzrt_error_td wave_write_group_end(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void *inst, fz_tag_td *fz_tag, char *name)
{
	fzrt_error_td						err = FZRT_NOERR;

	return err;
}

/************************************************************************
	This function starts the export process and initializes working data.
*************************************************************************/
fzrt_error_td wave_write_begin(
	fzrt_int				windex,
	fz_ffmt_ref_td		ffmt_id,
	void				**wave_ptr,
	fzrt_floc_ptr		floc
	)
{
	fzrt_error_td						err = FZRT_NOERR;
	char								temp[256];
	wave_out_td							*wave = NULL;
	fz_ffmt_write_nl_meth_enum			nl_meth;
	fzrt_int								out_opts_flags, tform_opts_flags;
	fz_type_td							fz_type;
	fzrt_int								lval;
	char								tmp_str[256];

	_ffmt_3d_funcs.fz_ffmt_data_model_write_opts_parm_get(ffmt_id, FZ_FFMT_DATA_MODEL_WRITE_OPTS_PARM_NL_METHOD, &fz_type);
	_type_funcs.fz_type_get_int(&fz_type, &lval);
	nl_meth = (fz_ffmt_write_nl_meth_enum)lval;
	_ffmt_3d_funcs.fz_ffmt_data_model_write_opts_parm_get(ffmt_id, FZ_FFMT_DATA_MODEL_WRITE_OPTS_PARM_FLAGS, &fz_type);
	_type_funcs.fz_type_get_int(&fz_type, &out_opts_flags);

		/* Allocate working data. */
	if (_fzrt_mem_funcs.fzrt_new_ptr_clear(sizeof(wave_out_td), (fzrt_ptr*)wave_ptr) == FZRT_NOERR)
	{
		wave = *((wave_out_td **)wave_ptr);

		/* Initialize working data. */
		wave->windex = windex;
		wave->workname[0] = 0;
		_fzrt_file_funcs.fzrt_file_floc_init(&wave->floc);
		_fzrt_file_funcs.fzrt_file_floc_copy(floc, wave->floc);
		wave->fp = NULL;
		
		/* Get the line termination characters specified by the export options. */
		_ffmt_funcs.fz_ffmt_lterm_term_get_str(nl_meth, wave->nl_str);
		wave->ffmt_id = ffmt_id;
		wave->pos = wave->max_pos = 0L;

		wave->obj_seed = 0;
		wave->normal_seed = 0L;
		wave->point_seed = 0L;
		wave->point2_seed = 0L;
		wave->curv2_seed = 0L;
		wave->uv_seed = 0L;
		wave->do_tmaps = 0;
		wave->lib_name[0] = '\0';

		/* Get the numerical accuracy from the project working units.  This will determine the number
			of decimal places to use then writing coordinates to the wavefront files. */
		_proj_funcs.fz_proj_units_get_parm_data(windex, FZ_UNITS_PARM_DEC_DECIMAL_PLACES, &fz_type);
		_type_funcs.fz_type_get_short(&fz_type, &wave->num_dec);

		/* Get transform flags which determine directions of face and vertex normals. */
		_ffmt_3d_funcs.fz_ffmt_data_model_tform_opts_parm_get(ffmt_id, FZ_FFMT_DATA_MODEL_TFORM_OPTS_PARM_FLAGS, &fz_type);
		_type_funcs.fz_type_get_int(&fz_type, &tform_opts_flags);
		wave->inv_norms = (short)((FZ_CHKBIT(tform_opts_flags, FZ_FFMT_DATA_MODEL_TFORM_OPTS_NORMALSOUTWARD_BIT)) ? FALSE : TRUE);
		wave->inv_faces = (short)((FZ_CHKBIT(tform_opts_flags, FZ_FFMT_DATA_MODEL_TFORM_OPTS_INVERSEFACES_BIT)) ? TRUE : FALSE);

		/* Get the name of the file we're writing to. */
		_fzrt_file_funcs.fzrt_file_floc_get_name(floc, tmp_str, 256);
		/* Need to convert this file name to a character set which works with ansi file functions. */
		_fzrt_file_funcs.fzrt_file_cnvt_name(tmp_str, wave->workname, 256);

		wave->do_tmaps = (FZ_CHKBIT(out_opts_flags, FZ_FFMT_DATA_MODEL_WRITE_OPTS_TEXTUREMAPS_BIT) ? TRUE : FALSE);

		/* Since materials are saved in a separate file, they only need to be written once. */
		if (err == FZRT_NOERR)
		{
			/* Create the name of the mtl file. */
			strcpy(temp, wave->workname);
			_ffmt_funcs.fz_file_remove_ext(temp);
								
			/* Replace illegal characters wile '_'. */
			_ffmt_funcs.fz_ffmt_make_identifier(temp);
			
			/* fz_file_make_unique_name checks to see if a file of the name we created already exists,
				if a file of the same name exists, it adjusts the name to make it unique in the output folder. */
			if ((err = _ffmt_funcs.fz_file_make_unique_name(wave->floc, temp, "mtl", wave->lib_name)) == FZRT_NOERR)
			{
				/* Write all surface styles to the mtl file. */
				err = wave_write_surface_styles(wave);
			}
		}
		
		/* Open the file and begin the export. */
		if (err == FZRT_NOERR)	err = wave_write_file_begin(windex, ffmt_id, wave, floc);
	}
	else
	{
		err = _fzrt_base_funcs.fzrt_error_set_with_detail(FZRT_MALLOC_ERROR, FZRT_ERROR_SEVERITY_ERROR, FZRT_ERROR_CONTEXT_FZRT, 0, 1021);
	}

	return err;
}

/************************************************************************
	This function completes the export process and cleans up working data.
*************************************************************************/
fzrt_error_td wave_write_end(fzrt_int windex, fz_ffmt_ref_td ffmt_id, void **wave_ptr, fzrt_floc_ptr floc, fzrt_error_td err)
{
	wave_out_td		*wave = NULL;

	if (wave_ptr != NULL && *wave_ptr != NULL)
	{
		wave = *((wave_out_td **)wave_ptr);

		/* Finish writing to the file. */
		err = wave_write_file_end(windex, ffmt_id, wave, floc);

		/* Close the file. */
		if (wave->fp != NULL)
		{
			fclose(wave->fp);
			wave->fp = NULL;
		}
		
		/* If an error occurred during export, delete the potentially corrupt file. */
		if (err != FZRT_NOERR) _ffmt_funcs.fz_file_remove(wave->floc);

		_fzrt_file_funcs.fzrt_file_floc_finit(&wave->floc);

		/* Free the working data. */
		_fzrt_mem_funcs.fzrt_dispose_ptr((fzrt_ptr)wave);
		*wave_ptr = wave = NULL;
	}

	return(err);
}

/************************************************************************
	This function gets a general export failed string to display with an error message to the user.
*************************************************************************/
fzrt_error_td wave_write_err_lable(fzrt_int windex, fz_ffmt_ref_td ffmt_id, char *str, fzrt_int max_len, short *err_id)
{
	char str1[256];

	/* Get the string from the plugin's resource file. */
	_fzrt_base_funcs.fzrt_fzr_get_string(_fz_rsrc_ref, WAVE_STRINGS, WAVE_WRITE_ERR_STR, str1);
	strncpy(str, str1, max_len);
	
	/* Assign a numeric error number which will also be displayed on the error dialog. */
	*err_id = WAVE_WRITE_ERR_ID;
	
	return(FZRT_NOERR);
}
