/*********************************************************************************************
 *                                                                                           *
 *                                                                                           *
 *   Copyright 1989-2018 by AutoDesSys, Inc.                                                 *
 *                                                                                           *
 *   This software and its associated documentation contains proprietary,                    *
 *   confidential, and trade secret information of AutoDesSys Inc. and may not be            *
 *   disclosed, copied in any form, or used for any purpose except as specifically           *
 *   permitted in writing by AutoDesSys, Inc.                                                *
 *                                                                                           *
 *   AutoDesSys, Inc. grants permission to use the software to those who purchase            *
 *   a license.  Such a usage will be made according to the terms and conditions             *
 *   specified in the license agreement to which a licensee agrees before installing         *
 *   and commencing usage of the software.  The license agreement can be found               *
 *   on the envelop that contains the software CD.                                           *
 *                                                                                           *
 *   AutoDesSys, Inc. also grants permission to licensees to develop extensions to           *
 *   the software, which can be in the form of scripts or plugins.  The permission           *
 *   includes calling the API and call back functions provided by AutoDesSys, Inc.           *
 *   and accessing information relevant to these functions and the general workings          *
 *   of form.Z, but only up to the point to which AutoDesSys, Inc. has chosen to             *
 *   expose, through its form.Z SDK Manual.  All extensions developed by third               *
 *   parties will include a copy of this copyright note in an easily accessible place or     *
 *   a clear instruction to read this original note by AutoDesSys, Inc.                      *
 *                                                                                           *
 *   All the software distributed under a license, either for the purpose of using it or     *
 *   for the purpose of developing extensions, is distributed AS IS, WITHOUT                 *
 *   WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND                                    *
 *   AutoDesSys, Inc. HEREBY DISCLAIMS ALL SUCH WARRANTIES,                                  *
 *   INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF                                         *
 *   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.                                    *
 *                                                                                           *
 *   formZ, RenderZone, and RadioZity are registered trademarks of AutoDesSys, Inc.          *
 *                                                                                           *
 *   form.Z SDK, Version 8.6.3, Dec 03, 2018                                                 *
 *                                                                                           *
 *                                                                                           *
 *********************************************************************************************/



#ifndef __FORMZ_FUNCS_H
#define __FORMZ_FUNCS_H

#include "fzpl_plugin_extern.h"
#include "fzpl_plugin.h"






#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************

	formZ SDK


	Description
		formZ API Reference v 6.5.0.0.  This document contains all the API functions, types, constants, 
		etc., that are available to call or use within a plugin or script.

		Some functions, types, and constants are only available in plugins and some are only available 
		in scripts, but many are available in both. The icon  denotes 
		it is usable in a plugin, while the icon  denotes it is usable 
		in a script. 

		To use a function, type, or constant within a script, simply verify it is available to scripts. 
		To use a function, type, or constant within a plugin, the appropriate API header file (listed with each description) 
		needs to be included by the plugin, and also, for functions, the function set 
		in which it belongs needs to be acquired</i> by the plugin.  (See the Plugin Manager (FZPL) section). 

		All the functionality provided by the formZ API is divided into two camps:  the Callback Reference, and the 
		API Reference.  The Callback Reference contains descriptions of those functions that formZ calls, 
		and that a plugin or script "fills in" with the algorithm to perform the task at hand. 
		That is, formZ asks each extension to do certain tasks, and the callback functions provide the 
		functionality behind those tasks.  For example, formZ will ask a renderer plugin to "render" the 
		scene by calling the plugin renderer's callback function fz_rndr_cbak_image_disp.  The plugin renderer 
		is in charge of supplying the functionality of what it means to render the scene.  Or formZ 
		may ask the renderer to do something as simple as providing its name, by calling the 
		renderer's fz_rndr_cbak_name callback function. 

		The API Reference section contains descriptions of functions that a plugin or script calls to perform certain 
		tasks, query information about an object, perform editing operations, create objects, lights, views, 
		manipulate objects, lights, views, surface styles, texture maps, etc., or perform thousands of other 
		operations that formZ can do. 

		These sections provide a very comprehensive and powerful way to customize and automate formZ. 
		You can extend formZ to include new renderers, attributes, object types, shaders, 
		tools, commands, palettes, etc.  Also virtually, every conceivable thing that can be done to 
		something within formZ in a manual way can be customized and/or automated by doing it through 
		API function calls.
 ****************************************************************/

/****************************************************************

	4.0 Call Back Reference


	Description
		The Callback Reference contains descriptions of those functions that formZ calls, 
		and that a plugin or script "fills in" with the algorithm to perform the task at hand. 
		That is, formZ asks each extension to do certain tasks, and the callback functions provide the 
		functionality behind those tasks.  For example, formZ will ask a renderer plugin to "render" the 
		scene by calling the plugin renderer's callback function fz_rndr_cbak_image_disp.  The plugin renderer 
		is in charge of supplying the functionality of what it means to render the scene.  Or formZ 
		may ask the renderer to do something as simple as providing its name, by calling the 
		renderer's fz_rndr_cbak_name callback function.
 ****************************************************************/

/****************************************************************

	5.0 API Reference


	Description
		The API Reference section contains functions that a plugin or script calls to perform certain 
		tasks, query information about an object, perform editing operations, create objects, lights, views, 
		manipulate objects, lights, views, surface styles, texture maps, etc., or perform thousands of other 
		operations that formZ can do.
 ****************************************************************/

/****************************************************************

	System


	Description
		System-level API functions usually operate on general things within formZ. 
		This includes things such as file io, text and fonts, and math functions, etc.
 ****************************************************************/

/****************************************************************

	Project


	Description
		Project-level API functions usually operate on project-specific things within formZ. 
		This includes things such as modeling objects, managing lights, rendering, symbols, etc.
 ****************************************************************/



/****************************************************************

	FZ_FILE_DATA_EXTS_TYPE

	Description
		Data file translator extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_FILE_DATA_EXTS_TYPE		"\x41\x67\x5b\x0c\x93\xa8\x40\x10\xb2\x68\x3d\x75\x6a\xc1\xbb\x77"

/****************************************************************

	FZ_FILE_DATA_EXTS_VERSION

	Description
		Data file translator extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_FILE_DATA_EXTS_VERSION	FZPL_VERS_MAKE(5,0,0,0)




/****************************************************************

	FZ_FILE_IMAGE_EXTS_TYPE

	Description
		Image file translator extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_FILE_IMAGE_EXTS_TYPE		"\x56\x62\x63\x15\x2c\xb4\x47\x33\xae\x2d\x1d\x54\xd9\x48\xf0\xcd"

/****************************************************************

	FZ_FILE_IMAGE_EXTS_VERSION

	Description
		Image file translator extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_FILE_IMAGE_EXTS_VERSION	FZPL_VERS_MAKE(5,0,0,0)





/****************************************************************

	FZ_ATTR_EXTS_TYPE

	Description
		Attribute extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_ATTR_EXTS_TYPE			"\x2e\xab\xaf\x7f\x52\x2f\x4f\x49\x82\x37\x81\x20\x88\x6f\x4d\x80"

/****************************************************************

	FZ_ATTR_EXTS_VERSION

	Description
		Attribute extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_ATTR_EXTS_VERSION		FZPL_VERS_MAKE(5,0,0,0)




/****************************************************************

	FZ_OTYP_EXTS_TYPE

	Description
		Controlled object extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_OTYP_EXTS_TYPE			"\xdf\x8a\xb3\xc9\x4a\x03\x46\xf3\x90\xd8\xe6\x3b\x44\x2e\xa5\x1c"

/****************************************************************

	FZ_OTYP_EXTS_VERSION

	Description
		Controlled object extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_OTYP_EXTS_VERSION		FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_SHDR_EXTS_TYPE

	Description
		Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_EXTS_TYPE			"\x9b\xee\x34\x0f\x08\xc1\x47\xa2\x91\x17\xc5\x61\xe7\xcf\xd0\x98"

/****************************************************************

	FZ_SHDR_EXTS_VERSION

	Description
		Shader extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_EXTS_VERSION		FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_CMND_EXTS_TYPE

	Description
		Command extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_CMND_EXTS_TYPE						"\x44\x32\xb1\x81\xfc\x2c\x4c\x54\xa1\x7e\x63\xad\x26\x79\x60\xcb"

/****************************************************************

	FZ_CMND_EXTS_VERSION

	Description
		Command extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_CMND_EXTS_VERSION					FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_PALT_EXTS_TYPE

	Description
		Palette extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_PALT_EXTS_TYPE						"\xd9\x54\x34\x93\xae\x0a\x48\xc5\xad\x3f\xc3\xfd\xa6\xdc\x53\x76"

/****************************************************************

	FZ_PALT_EXTS_VERSION

	Description
		Palette extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_PALT_EXTS_VERSION					FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_RNDR_EXTS_TYPE

	Description
		Renderer extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_RNDR_EXTS_TYPE				"\x65\xce\xf2\x9c\x31\xe0\x46\x3e\xbd\x0d\x2e\x14\xbf\x30\x8a\x3b"

/****************************************************************

	FZ_RNDR_EXTS_VERSION

	Description
		Renderer extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_RNDR_EXTS_VERSION			FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_UTILITY_TYPE

	Description
		Utility extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTILITY_TYPE					"\xf2\xc5\x3c\x05\x92\xfd\x4d\xc1\x8f\x7a\x63\x9c\x1d\xaf\x52\xb0"

/****************************************************************

	FZ_UTILITY_VERSION

	Description
		Utility extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTILITY_VERSION				FZPL_VERS_MAKE(5,0,0,0)






/****************************************************************

	FZ_TOOL_EXTS_TYPE

	Description
		Tool extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_TOOL_EXTS_TYPE						"\x9a\xbf\x87\x7f\x04\xaa\x46\xfb\xb4\x4c\x6b\x85\x73\xd7\xc5\x6c"

/****************************************************************

	FZ_TOOL_EXTS_VERSION

	Description
		Tool extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_TOOL_EXTS_VERSION					FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_UTIL_SYST_EXTS_TYPE

	Description
		System utility extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTIL_SYST_EXTS_TYPE			"\xed\x1a\x2e\xa4\xd7\x6d\x48\xac\xbf\x1c\x62\x2e\x4e\xc4\x26\xf5"

/****************************************************************

	FZ_UTIL_SYST_EXTS_VERSION

	Description
		System utility extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTIL_SYST_EXTS_VERSION		FZPL_VERS_MAKE(5,0,0,0)



/****************************************************************

	FZ_UTIL_PROJ_EXTS_TYPE

	Description
		Project utility extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTIL_PROJ_EXTS_TYPE			"\x55\x88\x03\x95\x69\x26\x4b\xcc\xa2\x71\xa5\x6c\xeb\x6c\x15\xcb"

/****************************************************************

	FZ_UTIL_PROJ_EXTS_VERSION

	Description
		Project utility extension type version.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_UTIL_PROJ_EXTS_VERSION		FZPL_VERS_MAKE(5,0,0,0)


/****************************************************************

	FZ_SHDR_COLR_EXTS_TYPE

	Description
		Color Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_COLR_EXTS_TYPE			"\x78\x97\xce\x89\xaf\x53\x4f\x78\x88\xa6\x1a\xe9\x94\x0f\xda\x5a"
/****************************************************************

	FZ_SHDR_REFL_EXTS_TYPE

	Description
		Reflection Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_REFL_EXTS_TYPE			"\x89\x40\x7b\x91\x32\x4e\x4d\x5d\xbc\x75\x16\xce\x40\xa1\x8c\x65"
/****************************************************************

	FZ_SHDR_TRNS_EXTS_TYPE

	Description
		Transparency Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_TRNS_EXTS_TYPE			"\xfc\x0c\xe4\xdf\x08\x10\x46\x79\x92\x9d\xe2\xec\x29\xff\xa5\x9a"
/****************************************************************

	FZ_SHDR_BUMP_EXTS_TYPE

	Description
		Bump Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_BUMP_EXTS_TYPE			"\x0b\x27\xdf\x42\x39\xe1\x4c\x15\xb5\x8c\x52\x3a\x94\xf6\x13\x3a"
/****************************************************************

	FZ_SHDR_BGND_EXTS_TYPE

	Description
		Background Shader extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_BGND_EXTS_TYPE			"\xb3\xeb\x0e\x8b\x9b\x24\x4f\x41\x85\x72\x69\x62\x61\x5d\x55\xd7"
/****************************************************************

	FZ_SHDR_FGND_EXTS_TYPE

	Description
		Foreground Shader (or depth effect) extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_SHDR_FGND_EXTS_TYPE			"\x7e\xcb\x6b\x4f\x36\xbf\x4b\xb1\x9d\xd7\xc4\xce\x57\xd6\xdb\x1e"
/****************************************************************

	FZ_CMND_SYST_EXTS_TYPE

	Description
		System Command extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_CMND_SYST_EXTS_TYPE			"\x61\x9f\x51\x7e\x30\x72\x48\x68\x96\x51\xf6\x8c\x50\x34\x03\xd8"
/****************************************************************

	FZ_CMND_PROJ_EXTS_TYPE

	Description
		Project Command extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_CMND_PROJ_EXTS_TYPE			"\x03\xc4\x2e\xcf\x03\x64\x4b\xd8\xb8\x4b\x77\x4c\x47\x87\x30\xb5"
/****************************************************************

	FZ_PALT_SYST_EXTS_TYPE

	Description
		System Palette extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_PALT_SYST_EXTS_TYPE			"\x53\x79\xd6\x13\xfb\x7d\x4c\x4c\x9c\x99\xd4\x3d\x5d\x07\xab\x4e"
/****************************************************************

	FZ_PALT_PROJ_EXTS_TYPE

	Description
		Project Palette extension type id.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_PALT_PROJ_EXTS_TYPE			"\x5d\x6c\x9a\x23\x1c\xdb\x46\xed\xb9\x76\xd0\xeb\xa5\x5f\xf4\xa6"





/****************************************************************

	fz_unit_type_enum


	Description
		Working unit types. These represent the units of the Working units dialog.

	Members
		FZ_UNIT_TYPE_ENGLISH
			Project Units are English.  Position values stored within formZ are in inches.
		FZ_UNIT_TYPE_METRIC
			Project Units are Metric.  Position values stored within formZ are in centimeters.

	Availability	5.0.0.0


	See Also
		fz_ffmt_cbak_data_model_write_units_conv, fz_fuim_unit_convert, fz_notf_cbak_pref_default,  
		fz_notf_cbak_proj_units, fz_notf_cbak_syst_units, fz_notf_cbak_wind_units,  
		fz_proj_unit_enum

 ****************************************************************/
typedef enum fz_unit_type_enum
{
	FZ_UNIT_TYPE_ENGLISH = 0,
	FZ_UNIT_TYPE_METRIC,
	UNIT_MAX
} fz_unit_type_enum;

/****************************************************************

	fz_unit_scale_enum


	Description
		Working unit default scales. These represent the Data Scale menu of the Working units dialog.

	Members
		FZ_UNIT_SCAL_UNDEF
			Undefined scale
		FZ_UNIT_SCAL_SMALL
			Units are in the small scale (lamp)
		FZ_UNIT_SCAL_MEDIUM
			Units are in the medium scale (building)
		FZ_UNIT_SCAL_LARGE
			Units are in the large scale (city)
		FZ_UNIT_SCAL_MINI
			Units are in the miniature scale (watch)

	Availability	5.0.0.0


	See Also
		fz_fuim_unit_convert, fz_notf_cbak_pref_default, fz_notf_cbak_proj_units,  
		fz_notf_cbak_syst_units, fz_notf_cbak_wind_units, fz_proj_unit_enum

 ****************************************************************/
typedef enum fz_unit_scale_enum
{
	FZ_UNIT_SCAL_UNDEF	 = -1,
	FZ_UNIT_SCAL_SMALL	 =  0,
	FZ_UNIT_SCAL_MEDIUM  =  1,
	FZ_UNIT_SCAL_LARGE	 =  2,
	FZ_UNIT_SCAL_MINI	 =  3,
	
	FZ_UNIT_SCAL_MAX
} fz_unit_scale_enum;

/****************************************************************

	fz_wind_type_enum


	Description
		Project window types.

	Members
		FZ_WIND_TYPE_NONE
			No type
		FZ_WIND_TYPE_MODEL
			Modeling Window
		FZ_WIND_TYPE_DRAFT
			Drafting Window

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_wind_type_enum
{
	FZ_WIND_TYPE_NONE=0,
	FZ_WIND_TYPE_MODEL=5,
	FZ_WIND_TYPE_DRAFT=18,

	FZ_WIND_TYPE_MAX
} fz_wind_type_enum;


/****************************************************************

	FZ_FFMT_NONE


	Description
		A value which represents an unknown format reference id (fz_ffmt_ref_td).  This can 
		also be used in query functions to represent any file format.

	Availability	5.0.0.0

 ****************************************************************/
#define FZ_FFMT_NONE	(-1)

/****************************************************************

	fz_ffmt_ref_td


	Description
		A format reference identifier.

	Availability	5.0.0.0


	See Also
		fz_ffmt_keyword_to_ref_id, fz_ffmt_keyword_to_ref_id_list, fz_ffmt_cbak_data_model_read_dlog_cust, fz_ffmt_cbak_data_model_read_opts_changed, fz_ffmt_cbak_data_model_read_opts_default,  
		fz_ffmt_cbak_data_model_read_opts_flags, fz_ffmt_cbak_data_model_read_predloginit, fz_ffmt_cbak_data_model_tform_opts_changed,  
		fz_ffmt_cbak_data_model_tmap_list, fz_ffmt_cbak_data_model_write_begin, fz_ffmt_cbak_data_model_write_can_do_ctrl,  
		fz_ffmt_cbak_data_model_write_can_do_smooth, fz_ffmt_cbak_data_model_write_cmpt_def_end, fz_ffmt_cbak_data_model_write_cmpt_def_start,  
		fz_ffmt_cbak_data_model_write_cmpt_obsolete, fz_ffmt_cbak_data_model_write_ctrl, fz_ffmt_cbak_data_model_write_dlog_cust,  
		fz_ffmt_cbak_data_model_write_end, fz_ffmt_cbak_data_model_write_err_label, fz_ffmt_cbak_data_model_write_faces,  
		fz_ffmt_cbak_data_model_write_file_begin_func, fz_ffmt_cbak_data_model_write_file_end_func, fz_ffmt_cbak_data_model_write_grup_begin,  
		fz_ffmt_cbak_data_model_write_grup_end, fz_ffmt_cbak_data_model_write_lines, fz_ffmt_cbak_data_model_write_objt,  
		fz_ffmt_cbak_data_model_write_opts_changed, fz_ffmt_cbak_data_model_write_opts_default, fz_ffmt_cbak_data_model_write_opts_flags,  
		fz_ffmt_cbak_data_model_write_points, fz_ffmt_cbak_data_model_write_polylines, fz_ffmt_cbak_data_model_write_predloginit,  
		fz_ffmt_cbak_data_model_write_progress_str, fz_ffmt_cbak_data_model_write_smod_solid, fz_ffmt_cbak_data_model_write_smod_trimmed_surf,  
		fz_ffmt_cbak_data_model_write_symb_def_end, fz_ffmt_cbak_data_model_write_symb_def_start, fz_ffmt_cbak_data_model_write_units_conv,  
		fz_ffmt_cbak_image_bmap_read, fz_ffmt_cbak_image_bmap_read_info, fz_ffmt_cbak_image_bmap_write_err_label,  
		fz_ffmt_cbak_image_bmap_write_file_begin, fz_ffmt_cbak_image_bmap_write_file_end, fz_ffmt_cbak_image_bmap_write_image_begin,  
		fz_ffmt_cbak_image_bmap_write_image_end, fz_ffmt_cbak_image_bmap_write_image_scanline_byte, fz_ffmt_cbak_image_bmap_write_image_scanline_float,  
		fz_ffmt_cbak_image_bmap_write_progress_str, fz_ffmt_cbak_image_read_dlog_cust, fz_ffmt_cbak_image_read_opts_changed,  
		fz_ffmt_cbak_image_read_opts_default, fz_ffmt_cbak_image_read_opts_flags, fz_ffmt_cbak_image_read_predloginit,  
		fz_ffmt_cbak_image_vect_read_frame, fz_ffmt_cbak_image_vect_read_printer_data, fz_ffmt_cbak_image_vect_read_raw_data,  
		fz_ffmt_cbak_image_vect_write_arc, fz_ffmt_cbak_image_vect_write_begin_compound, fz_ffmt_cbak_image_vect_write_can_do_arc,  
		fz_ffmt_cbak_image_vect_write_end_compound, fz_ffmt_cbak_image_vect_write_err_label, fz_ffmt_cbak_image_vect_write_line,  
		fz_ffmt_cbak_image_vect_write_line_ref, fz_ffmt_cbak_image_vect_write_lineset, fz_ffmt_cbak_image_vect_write_lineset_ref,  
		fz_ffmt_cbak_image_vect_write_point, fz_ffmt_cbak_image_vect_write_point_ref, fz_ffmt_cbak_image_vect_write_progress_str,  
		fz_ffmt_cbak_image_vect_write_restore_gstate, fz_ffmt_cbak_image_vect_write_save_gstate, fz_ffmt_cbak_image_vect_write_set_fill_color,  
		fz_ffmt_cbak_image_vect_write_set_fill_pattern, fz_ffmt_cbak_image_vect_write_set_line_color, fz_ffmt_cbak_image_vect_write_set_line_style,  
		fz_ffmt_cbak_image_vect_write_set_line_weight, fz_ffmt_cbak_image_vect_write_set_text_angle, fz_ffmt_cbak_image_vect_write_set_text_font,  
		fz_ffmt_cbak_image_vect_write_simple_text, fz_ffmt_cbak_image_vect_write_string_font, fz_ffmt_cbak_image_vect_write_string_header,  
		fz_ffmt_cbak_image_vect_write_string_newline, fz_ffmt_cbak_image_vect_write_string_trailer, fz_ffmt_cbak_image_vect_write_string_write_angle,  
		fz_ffmt_cbak_image_vect_write_write_text_char, fz_ffmt_cbak_image_write_dlog_cust, fz_ffmt_cbak_image_write_opts_changed,  
		fz_ffmt_cbak_image_write_opts_default, fz_ffmt_cbak_image_write_opts_flags, fz_ffmt_cbak_image_write_predloginit,  
		fz_ffmt_data_get_import_symb_lib, fz_ffmt_data_model_import_cmpt, fz_ffmt_data_model_import_symb_def,  
		fz_ffmt_data_model_read_finish, fz_ffmt_data_model_read_get_dlog_flags, fz_ffmt_data_model_read_opts_parm_get,  
		fz_ffmt_data_model_read_opts_parm_set, fz_ffmt_data_model_read_set_dlog_flags, fz_ffmt_data_model_sid_from_face,  
		fz_ffmt_data_model_styletab, fz_ffmt_data_model_tform_opts_parm_get, fz_ffmt_data_model_tform_opts_parm_set,  
		fz_ffmt_data_model_tmap_filetab, fz_ffmt_data_model_tmap_opts_get_dst_floc, fz_ffmt_data_model_tmap_opts_parm_get,  
		fz_ffmt_data_model_tmap_opts_parm_set, fz_ffmt_data_model_tmap_read_filetab_add, fz_ffmt_data_model_write_all,  
		fz_ffmt_data_model_write_cmpts, fz_ffmt_data_model_write_get_dlog_flags, fz_ffmt_data_model_write_opts_get_dcomp_opts,  
		fz_ffmt_data_model_write_opts_parm_get, fz_ffmt_data_model_write_opts_parm_set, fz_ffmt_data_model_write_opts_set_dcomp_opts,  
		fz_ffmt_data_model_write_opts_units_menu_get_enable_flags, fz_ffmt_data_model_write_opts_units_menu_set_enable_flags, fz_ffmt_data_model_write_sdefs,  
		fz_ffmt_data_model_write_set_dlog_flags, fz_ffmt_data_symb_opts_parm_get, fz_ffmt_data_symb_opts_parm_set,  
		fz_ffmt_export_file, fz_ffmt_get_format_name, fz_ffmt_get_uuid,  
		fz_ffmt_image_bmap_get_read_funcs, fz_ffmt_image_bmap_get_write_funcs, fz_ffmt_image_bmap_load_pbuf,  
		fz_ffmt_image_bmap_write_pbuf, fz_ffmt_image_read_get_dlog_flags, fz_ffmt_image_read_opts_parm_get,  
		fz_ffmt_image_read_opts_parm_set, fz_ffmt_image_read_set_dlog_flags, fz_ffmt_image_vect_get_read_funcs,  
		fz_ffmt_image_vect_get_write_funcs, fz_ffmt_image_vect_write_func, fz_ffmt_image_write_get_dlog_flags,  
		fz_ffmt_image_write_opts_parm_get, fz_ffmt_image_write_opts_parm_set, fz_ffmt_image_write_set_dlog_flags,  
		fz_ffmt_import_file, fz_ffmt_keyword_set_str, fz_ffmt_supports_read_bitmap,  
		fz_ffmt_supports_write_bitmap, fz_ffmt_tform_get_martix, fz_ffmt_translator_get_class,  
		fz_ffmt_translator_is_class, fz_file_get_extension, fz_file_remove_extension,  
		fz_file_set_attributes, fz_rmtl_get_bmp_norm_shdr_rndr_txr, fz_rmtl_get_bmp_shdr_rndr_txr,  
		fz_rmtl_get_col_shdr_rndr_txr, fz_rmtl_get_obj_bmp_rndr_txr, fz_rmtl_get_obj_col_rndr_txr,  
		fz_rmtl_get_obj_reflmap_rndr_txr, fz_rmtl_get_obj_shaded_rndr_txr, fz_rmtl_get_obj_trn_rndr_txr,  
		fz_rmtl_get_reflmap_shdr_rndr_txr, fz_rmtl_get_trn_shdr_rndr_txr, fz_srep_cbak_bake_combined_textures,  
		fz_srep_cbak_bake_separate_textures, fz_srep_cbak_col_rndr_texture, fz_syst_anim_export,  
		fz_tmap_embd_to_tmap_file

 ****************************************************************/
typedef fzrt_int fz_ffmt_ref_td;


/****************************************************************

	fz_lite_ptr


	Description
		Data structure for a light. 
		The individual fields of this type can be accessed and set via 
		fz_lite_get_parm_common and fz_lite_set_parm_common and by get/set 
		functions specific to each light type.

	Availability	5.0.0.0


	See Also
		fz_light_rep_cbak_cedt, fz_lite_copy_to_windex, fz_lite_delete,  
		fz_lite_dist_get_sun, fz_lite_dist_set_dir_from_sun, fz_lite_duplicate,  
		fz_lite_get_grup_tag, fz_lite_get_intens_scale, fz_lite_get_lrep_data,  
		fz_lite_get_next_light, fz_lite_get_parm_area_obsolete, fz_lite_get_parm_common_obsolete,  
		fz_lite_get_parm_cone_obsolete, fz_lite_get_parm_custom_obsolete, fz_lite_get_parm_dist_obsolete,  
		fz_lite_get_parm_environment_obsolete, fz_lite_get_parm_line_obsolete, fz_lite_get_parm_point_obsolete,  
		fz_lite_get_parm_projector_obsolete, fz_lite_get_pick_status, fz_lite_get_prev_light,  
		fz_lite_get_symb, fz_lite_is_symb_light, fz_lite_ptr_to_tag,  
		fz_lite_set_parm_area_obsolete, fz_lite_set_parm_common_obsolete, fz_lite_set_parm_cone_obsolete,  
		fz_lite_set_parm_custom_obsolete, fz_lite_set_parm_dist_obsolete, fz_lite_set_parm_environment_obsolete,  
		fz_lite_set_parm_line_obsolete, fz_lite_set_parm_point_obsolete, fz_lite_set_parm_projector_obsolete,  
		fz_lite_set_pick_status, fz_lite_set_sun, fz_lite_transform,  
		FZRT_SPEC	*fz_lite_fuim_create_env_maps_items_func, FZRT_SPEC	*fz_lite_fuim_create_env_spin_items_func, FZRT_SPEC	*fz_lite_fuim_create_ignore_objt_color_items_func,  
		FZRT_SPEC	*fz_lite_fuim_create_inner_angle_items_func, FZRT_SPEC	*fz_lite_fuim_create_obj_from_pick_items_func, FZRT_SPEC	*fz_lite_fuim_create_outer_angle_items_func,  
		FZRT_SPEC	*fz_lite_fuim_create_quality_items_func, FZRT_SPEC	*fz_lite_fuim_create_radius_items_func, FZRT_SPEC	*fz_lite_fuim_create_render_objt_items_func,  
		FZRT_SPEC	*fz_lite_fuim_create_spin_items_func, FZRT_SPEC	*fz_lite_fuim_create_tmap_items_func, FZRT_SPEC	*fz_lite_get_parm_func,  
		FZRT_SPEC	*fz_lite_get_plugin_parm_func, FZRT_SPEC	*fz_lite_set_parm_func, FZRT_SPEC	*fz_lite_set_plugin_parm_func,  
		fz_notf_cbak_lite, fz_light_rep_cbak_data_are_equal, fz_light_rep_cbak_data_copy,  
		fz_light_rep_cbak_data_finit, fz_light_rep_cbak_data_init, fz_light_rep_cbak_data_io

 ****************************************************************/
typedef struct light_td 		*fz_lite_ptr;




#ifdef __cplusplus
}
#endif

#endif

