/*************************************************************************

	formZ by autodessys, Inc.
	Copyright c 1994 autodessys, Inc.  All rights reserved.
	
	
	File :	fz_cmnd_api.h
	
	By :	
	
	Created on :  		17 June 2002
	Last Updated on :  	17 June 2002
	
	Description :	
					
***************************************************************************/

#ifndef _FORMZ_CMND_API_H
#define _FORMZ_CMND_API_H 1

#include "fzpl_plugin.h"
#include "fz_iost_api.h"	
#include "fz_model_core_api.h"	
#include "fz_fuim_api.h"
#include "fz_api.h"
#include "fz_proj_api.h"

/* @doc EXTERNAL cmds_cbak */

#ifdef __cplusplus
extern "C" {
#endif

/********************************************************************************************************

	@module

	@desc	Command plugins are designed to execute a task or change a state. Command plugins are 
			executed by the user through a menu item, icon in an icon menu, a key shortcut, a click in a 
			project window or from a palette. There are five variants of command plugins: system command, project command, 
			tool, system palette, and project palette. <nl>

			All commands appear in the Key Shortcuts Manager dialog so that they may have key shortcuts 
			assigned for them. <nl>

			Command plugins may provide the fz_notf_cbak_fset function set to be notified when 
			changes occur within formZ.

	
	@toc	cmds_cbak

*********************************************************************************************************/



/********************************************************************************************************

	@module

	@desc	System commands are defined using the FZ_CMND_EXTS_SYST_PLUGIN_TYPE and the 
			fz_cmnd_cbak_syst_fset function set as described in the following sections. The user 
			invokes a system command plugin from a menu item in the Extensions menu, an icon from the 
			system icon palette, or from a key shortcut. System commands are not dependent on a project 
			window. 

	@toc	cmds_cbak_syst

*********************************************************************************************************/

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command initialization (Required). This function is called once by formZ as soon as the command 
				is registered. It estabilishes the level and kind of the command. The level disquishes the 
				context of the command (system, modeling, drafting, window etc.). The type tells formZ if the command is a tool, 
				action or state setting.
				
				Returning and error other than FZRT_NOERR will cause the command NOT to be registered

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_finit

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_init_func)(
	void
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command finalization (Optional). This function is called once by formZ when the command is unregistered at termination.
				This function is usful for finalizing ay data that was initialized in fz_cmnd_cbak_syst_init.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_init

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_finit_func)(
	void 
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command name (Recommended). This function is called by formZ to get the name of the command. It is recomended that 
				the command name is stored in .fzr file so that it is localizable. If this function is not provided, 
				the name of the plugin, name of the plugin file, name of the script, or name of the script file 
				will be used.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_uuid,fz_cmnd_cbak_syst_help

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_name_func)(	
	char 		*name, 					/* @parm	<out> | string to be filled with the command name 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of name parameter	*/
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command id (Recommended). This function is called by formZ to get the id (uuid) of the command. 
				This unique id is used by formZ to maintain all persistant information about the command. If this 
				function is not provided, formZ will assign a default id. Note that there are a limited number of 
				default ids and if the number of commands changes from session to session, the id will not be maintained. 
				In this situation, any persistant information (preferences for example) will be lost. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_name,fz_cmnd_cbak_syst_help

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_uuid_func)(
	fzrt_UUID_td 	uuid 				/* @parm	<out> | uuid to be filled with the command id 	*/	
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command help (Recommended). This function is called by formZ to get the help text of the command. 
				It is recomended that the help text is stored in .fzr file so that it is localizable. 
				If this function is not provided, no help string is shown. Help text is used in the 
				Help dilaogs and the Keyshortcuts manager.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_name,fz_cmnd_cbak_syst_uuid

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_help_func)(	
	char 		*help, 					/* @parm	<out> | string to be filled with the help text 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of help text parameter	*/
	);


/********************************************************************************************************

	@fsfunc_t	 

	@desc		Command active. This function is called by formZ at various times only for CMND_KIND_STATE commands to determine 
				if the command state is active. If this function is not provided, the command is assumed to be inactive.

	@return		TRUE for Active, FALSE for inactive

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_avail,fz_cmnd_cbak_syst_select

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_active_func)(
	fzrt_int 			*rv					/* @parm	<out> | TRUE for available , FALSE for not available (dimmed) 	*/		
	);
	
/********************************************************************************************************

	@fsfunc_t	

	@desc		Command available (Recommended). This function is called by formZ at various times to determine 
				if the command is available. If your command is dependant on certain conditions to be usable, then this function can be used 
				to prevent its use when conditions warrant. If this function is not provided, the command is assumed to be available.

	@return		TRUE for available, FALSE for unavailable

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_active,fz_cmnd_cbak_syst_select

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_avail_func)(
	fzrt_int 			*rv					/* @parm	<out> | TRUE for available , FALSE for not available (dimmed) 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command select (Recommended). This function is called by formZ when the command is selected by the user. 
				The user can select the command by clicking on an icon, selecting an item in a menu, 
				selecting an item in a palette, or executing a keyshortcut.
				For CMND_KIND_ACTION command, this is where action is performed
				For CMND_KIND_STATE command, this is where the state is changed
				For CMND_KIND_TOOL command, this indicates that the tool is now the active tool

	@return		TRUE for handled, FALSE for if not

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_avail,fz_cmnd_cbak_syst_active

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_select_func)(
	void
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from file (Optional). This function is called by formZ to load an icon for the command from an image file. 
				The image file can be in any supported image file format (or of an image format wich is support be a image file 
				translator plugin). It is recommened to use the TIFF image format as it is a cross platform supported format and 
				is always present. The source graphics are identified in the file by indicating the top left corner of the 
				graphic in the file. In addition to the graphic, a mask can be provided for use when the icon is highlighted. A 
				single file can be used for multiple icons across a variety of plugins by createing a grid of icons in the file 
				and specifing the location for each icon.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_icon_rsrc

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_icon_file_func)(
	fz_fuim_icon_enum 			which, 				/* @parm	<in> | which style of icon to load. */		
	fzrt_floc_ptr 			floc,				/* @parm	<out> | file location of file which contains the icon. */
	fzrt_int 					*hpos, 				/* @parm	<in> | left horizontal location of icon data in file. */
	fzrt_int 					*vpos, 				/* @parm	<in> | top vertical location of icon data in file. */
	fzrt_floc_ptr 			floc_mask, 			/* @parm	<out_opt> | file location of file which contains the icon mask data. */
	fzrt_int 					*hpos_mask, 		/* @parm	<in> | left horizontal location of icon mask data in file. */
	fzrt_int 					*vpos_mask			/* @parm	<in> | top vertical location of icon mask data in file. */
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from resource file (Optional). This function is called by formZ to load an 
				icon for the command from a platform specific resource file. This method of loading an icon 
				is less flexible than fz_cmnd_cbak_syst_icon_file and requires specific generation of the icon artwork 
				for each platform. This method is only provided as a convienence for developers who already have  
				a library of platform specific icons developed. <nl>
				
				The resource file can be the plugin file (.fzp) or an independant resource file. All icons 
				stored in 32 x 32 pixels resources, however, depening on the type of the icon, only part of 
				the resource will be used. Only the top left 30 x 30 of the 32 x 32 is used for the black and white 
				full icon size. The bottom and right two pixels are NOT used (and will be cropped). The entire 32 x 32 
				is used for the color full icon size. 
				For the alternate size icon, formZ uses the bottom left 20 x 16 pixels. The top 16 and right 12 pixels 
				are NOT used (and will be cropped). <nl>
								
				Macintosh: The back and white source graphic is stored in a 'ICON' resource. The color source is read 
				from a 32 x 32 'cicn' resource.  <nl>
				
				Windows: The back and white and color source graphics are stored in a 'Icon' resource.  <nl>

	@return		Error codes

	@avail		5.0.0.0

	@see		fzrt_rlib_get_resource,fz_cmnd_cbak_syst_icon_file

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_icon_rsrc_func)(
	fz_fuim_icon_enum 			which, 				/* @parm	<in> | which style of icon to load. */		
	fzrt_icon_ptr 			*icon				/* @parm	<out> | icon handle. */
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Preference file I/O (Optional). This function is called by formZ to read and write the commands preference information 
				when a preference file or the defaults file is read or written. The preference files use an IFF type of file format. 
				A block is read or written for each command which has a fz_cmnd_cbak_syst_icon_rsrc registered. This is usefull for storing 
				the any user settings used by a command such as the options for a tool command. <nl>
				
				Version control is handleed through the version parameter. When writing a block you should set the version variable to
				the variant of data that you are writing. The default is zero. It is recommedend that you increment this number each time
				when you change the format of the data that you write to the stream. When reading the block, the version parameter
				contains the version which was used to write the block. You can then take the approperate action to read the variant 
				of data in the block. 

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_pref_io_func)(
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 			/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);




/********************************************************************************************************

	@fsfunc_t	 

	@desc		Adds command icon to a toolbar (Optional). This function is called by formZ to add the command icon to the icon toolbar. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_icon_menu_adjacent,fz_fuim_exts_icon_group	

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_icon_menu_func)(
	const fzrt_UUID_td 			icon_bar_uuid,			/* @parm	<in> | id of the icon tool bar */ 
	fzrt_UUID_td 				group_uuid,				/* @parm	<out> | id of the icon group (row) to insert the icon
																			if NULL then a default Extensions group at the bottom of the toorbar is used,  */ 
	fz_fuim_icon_group_enum 		*group_pos, 			/* @parm	<out> | position to insert the command in the group (row) */ 
	fzrt_int 						*group_row, 			/* @parm	<out> | specifies exact row when group_pos is CMND_ICON_GROUP_CUSTOM */ 
	fzrt_int 						*group_col				/* @parm	<out> | specifies exact column when group_pos is CMND_ICON_GROUP_CUSTOM */ 
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Adds command icon to a toolbar, adjacent to another command. This function is called by formZ to add the command 
				icon to the icon toolbar. The position of the icon is adjacent to an existing icon.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_syst_icon_menu

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_icon_menu_adjacent_func)(
	const fzrt_UUID_td 		icon_bar_uuid, 		/* @parm	<in> | id of the icon tool bar */ 
	fzrt_UUID_td 			adjacent_uuid, 		/* @parm	<out> | id of the adjecant command */ 
	fz_fuim_icon_adjacent_enum *where				/* @parm	<out> | relationship to the adjacent command */ 
	);


/********************************************************************************************************

	@fsfunc_t	 

	@desc		Adds command to the extensions menu. This function is called by formZ to add the command icon to the extensions menu toolbar. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_fuim_exts_menu

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_menu_func)(
	fz_fuim_menu_ptr 				menu_ptr,				/* @parm	<in> | pointer to formz menu manager */ 
	const fzrt_UUID_td 			extensions_uuid,		/* @parm	<in> | menu id of the extensions menu */ 
	fzrt_UUID_td 				group_uuid,				/* @parm	<out_opt> | menu id of the group to insert the command in 
																			if NULL, then the item will appear in the Extensions menu */ 
	fzrt_int 						*position 				/* @parm	<in> | posttion in group_uuid to insert the command in 
																			if 0, item is inserted in alpha-order*/ 
	);

/************************************************************************

	@fsfunc_t

	@desc		Instructs a formZ extension to register its help data.
							
	@avail		6.0.0.0

	@toc		cmds_cbak_syst

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_syst_pdf_help_func)(void);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from file (Optional). This function is called by formZ to load an icon for the command from an image file. 
				The image file can be in any supported image file format (or of an image format wich is support be a image file 
				translator plugin). It is recommended to use the TIFF image format as it is a cross platform supported format and 
				is always present. The source graphics are identified in the file by indicating the top left corner of the 
				graphic in the file. A single file can be used for multiple icons across a variety of plugins by creating a grid 
				of icons in the file and specifing the location for each icon.

	@return		Error codes

	@avail		7.0.0.0

	@see		fz_cmnd_cbak_syst_icon_rsrc

	@exfile		

	@toc		cmds_cbak_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_syst_icon_file2_func)(
	fzrt_floc_ptr 			floc				/* @parm	<out> | file location of file which contains the icon. */
	);


/************************************************************************

	@plfset	fz_cmnd_fset
	
	@desc	Function set for commands. The functions in this set are supplied by the plugin or script to enable commands.  
			Commands are tools actions or states that are selected or invoked by a menu item, icon or key shortcut.

	@avail	5.0.0.0

	@toc	cmds_cbak_syst

*************************************************************************/

/********************************************************************************************************

	@fsdef	formZ Commands function set id.

*********************************************************************************************************/
/* UUID : {5a4244e6-f96e-49c0-b646-d5f1a8d57eff} */
#define FZ_CMND_CBAK_SYST_FSET_TYPE		"\x5a\x42\x44\xe6\xf9\x6e\x49\xc0\xb6\x46\xd5\xf1\xa8\xd5\x7e\xff"	/* Reference 1953 */
/********************************************************************************************************

	@fsdef	formZ Commands function set name.

*********************************************************************************************************/
#define FZ_CMND_CBAK_SYST_FSET_NAME		"formZ Command functions"
/********************************************************************************************************

	@fsdef	formZ Commands function set version.

*********************************************************************************************************/
#define FZ_CMND_CBAK_SYST_FSET_VERSION	FZPL_VERS_MAKE(6,0,0,0)

typedef struct fz_cmnd_cbak_syst_fset
{	
	fzpl_fset_prvt_data_td				prvt_data;

	fz_cmnd_cbak_syst_init_func				fz_cmnd_cbak_syst_init;				/* @fsfunc */
	fz_cmnd_cbak_syst_finit_func				fz_cmnd_cbak_syst_finit;				/* @fsfunc */
	fz_cmnd_cbak_syst_name_func				fz_cmnd_cbak_syst_name;				/* @fsfunc */
	fz_cmnd_cbak_syst_uuid_func				fz_cmnd_cbak_syst_uuid;				/* @fsfunc */
	fz_cmnd_cbak_syst_help_func				fz_cmnd_cbak_syst_help;				/* @fsfunc */
	
	fz_cmnd_cbak_syst_avail_func				fz_cmnd_cbak_syst_avail;				/* @fsfunc */
	fz_cmnd_cbak_syst_select_func				fz_cmnd_cbak_syst_select;				/* @fsfunc */
	fz_cmnd_cbak_syst_active_func				fz_cmnd_cbak_syst_active;				/* @fsfunc */

	fz_cmnd_cbak_syst_menu_func				fz_cmnd_cbak_syst_menu;				/* @fsfunc */
	fz_cmnd_cbak_syst_icon_menu_func			fz_cmnd_cbak_syst_icon_menu;			/* @fsfunc */
	fz_cmnd_cbak_syst_icon_menu_adjacent_func	fz_cmnd_cbak_syst_icon_menu_adjacent;	/* @fsfunc */
	fz_cmnd_cbak_syst_icon_rsrc_func			fz_cmnd_cbak_syst_icon_rsrc;			/* @fsfunc */
	fz_cmnd_cbak_syst_icon_file_func			fz_cmnd_cbak_syst_icon_file_obsolete;			/* @fsfunc */

	fz_cmnd_cbak_syst_pref_io_func				fz_cmnd_cbak_syst_pref_io;				/* @fsfunc */	
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_cmnd_cbak_syst_pdf_help_func				fz_cmnd_cbak_syst_pdf_help;				/* @fsfunc */
	fz_cmnd_cbak_syst_icon_file2_func			fz_cmnd_cbak_syst_icon_file2;			/* @fsfunc */

	
} fz_cmnd_cbak_syst_fset;



/********************************************************************************************************

	@module

	@desc	Project command plugins are implemented by the call back function set 
			fz_cmnd_cbak_proj_fset. There are three functions in this function set. Only the main 
			execution function is required unless the plugin has multiple fz_cmnd_cbak_proj_fset 
			function sets. The following shows the fill in of a fz_cmnd_cbak_proj_fset function set. 
			This function is provided to the fzpl_plugin_add_fset function call shown above. Note that 
			some of these functions are optional and some are mutually exclusive hence a plugin would 
			never implement all of these functions. 

	@toc	cmds_cbak_proj

*********************************************************************************************************/

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command initialization (Required). This function is called once by formZ as soon as the command 
				is registered. It estabilishes the level and kind of the command. The level disquishes the 
				context of the command (system, modeling, drafting, window etc.). The type tells formZ if the command is a tool, 
				action or state setting.
				
				Returning an error other than FZRT_NOERR will cause the command NOT to be registered

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_finit

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_init_func)(
	void
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command finalization (Optional). This function is called once by formZ when the command is unregistered at termination.
				This function is useful for finalizing or releasing any data that was initialized in fz_cmnd_cbak_proj_init.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_init

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_finit_func)(
	void 
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command initialization (Required). This function is called once by formZ as soon as the command 
				is registered. It estabilishes the level and kind of the command. The level disquishes the 
				context of the command (system, modeling, drafting, window etc.). The type tells formZ if the command is a tool, 
				action or state setting.
				
				Returning and error other than FZRT_NOERR will cause the command NOT to be registered

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_finit

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_info_func)(
	fz_proj_level_enum 		*level 				/* @parm	<out> | to be filled with the command level 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command name (Recommended). This function is called by formZ to get the name of the command. It is recomended that 
				the command name is stored in .fzr file so that it is localizable. If this function is not provided, 
				the name of the plugin, name of the plugin file, name of the script, or name of the script file 
				will be used.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_uuid,fz_cmnd_cbak_proj_help

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_name_func)(	
	char 		*name, 					/* @parm	<out> | string to be filled with the command name 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of name parameter	*/
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command id (Recommended). This function is called by formZ to get the id (uuid) of the command. 
				This unique id is used by formZ to maintain all persistant information about the command. If this 
				function is not provided, formZ will assign a default id. Note that there are a limited number of 
				default ids and if the number of commands changes from session to session, the id will not be maintained. 
				In this situation, any persistant information (preferences for example) will be lost. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_name,fz_cmnd_cbak_proj_help

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_uuid_func)(
	fzrt_UUID_td 	uuid 				/* @parm	<out> | uuid to be filled with the command id 	*/	
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command help (Recommended). This function is called by formZ to get the help text of the command. 
				It is recomended that the help text is stored in .fzr file so that it is localizable. 
				If this function is not provided, no help string is shown. Help text is used in the 
				Help dialogs and the Keyshortcuts manager.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_name,fz_cmnd_cbak_proj_uuid

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_help_func)(	
	char 		*help, 					/* @parm	<out> | string to be filled with the help text 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of help text parameter	*/
	);


/********************************************************************************************************

	@fsfunc_t	 

	@desc		This function is called by formZ at various times only for CMND_KIND_STATE commands to determine 
				if the command state is active. If this function is not provided, the command is assumed to be inactive.
				If the command is active, a checkmark will appear next to te command in the menu.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_avail,fz_cmnd_cbak_proj_select

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_active_func)(
	fzrt_int			windex,				/* @parm	<in> | 	project window index */
	fzrt_int 			*rv					/* @parm	<out> | TRUE for available , FALSE for not available (dimmed) 	*/		
	);
	
/********************************************************************************************************

	@fsfunc_t	

	@desc		Command available (Recommended). This function is called by formZ at various times to determine 
				if the command is available. If your command is dependant on certain conditions to be usable, then this function can be used 
				to prevent its use when conditions warrant. If this function is not provided, the command is assumed to be available.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_active,fz_cmnd_cbak_proj_select

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_avail_func)(
	fzrt_int			windex,					/* @parm	<in> | 	project window index */
	fzrt_int 			*rv					/* @parm	<out> | TRUE for available , FALSE for not available (dimmed) 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command select (Recommended). This function is called by formZ when the command is selected by the user. 
				The user can select the command by clicking on an icon, selecting an item in a menu, 
				selecting an item in a palette, or executing a keyshortcut.
				For CMND_KIND_ACTION command, this is where action is performed
				For CMND_KIND_STATE command, this is where the state is changed
				For CMND_KIND_TOOL command, this indicates that the tool is now the active tool

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_avail,fz_cmnd_cbak_proj_active

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_select_func)(
	fzrt_int			windex					/* @parm	<in> | 	project window index */
	);



/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from file (Optional). This function is called by formZ to load an icon for the command from an image file. 
				The image file can be in any supported image file format (or of an image format wich is support be a image file 
				translator plugin). It is recommened to use the TIFF image format as it is a cross platform supported format and 
				is always present. The source graphics are identified in the file by indicating the top left corner of the 
				graphic in the file. In addition to the graphic, a mask can be provided for use when the icon is highlighted. A 
				single file can be used for multiple icons across a variety of plugins by createing a grid of icons in the file 
				and specifing the location for each icon.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_icon_rsrc

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_icon_file_func)(
	fz_fuim_icon_enum 			which, 				/* @parm	<in> | which style of icon to load. */		
	fzrt_floc_ptr 			floc,				/* @parm	<out> | file location of file which contains the icon. */
	fzrt_int 					*hpos, 				/* @parm	<in> | left horizontal location of icon data in file. */
	fzrt_int 					*vpos, 				/* @parm	<in> | top vertical location of icon data in file. */
	fzrt_floc_ptr 			floc_mask, 			/* @parm	<out_opt> | file location of file which contains the icon mask data. */
	fzrt_int 					*hpos_mask, 		/* @parm	<in> | left horizontal location of icon mask data in file. */
	fzrt_int 					*vpos_mask			/* @parm	<in> | top vertical location of icon mask data in file. */
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from resource file (Optional). This function is called by formZ to load an 
				icon for the command from a platform specific resource file. This method of loading an icon 
				is less flexible than fz_cmnd_cbak_proj_icon_file and requires specific generation of the icon artwork 
				for each platform. This method is only provided as a convienence for developers who already have  
				a library of platform specific icons developed. <nl>
				
				The resource file can be the plugin file (.fzp) or an independant resource file. All icons 
				stored in 32 x 32 pixels resources, however, depening on the type of the icon, only part of 
				the resource will be used. Only the top left 30 x 30 of the 32 x 32 is used for the black and white 
				full icon size. The bottom and right two pixels are NOT used (and will be cropped). The entire 32 x 32 
				is used for the color full icon size. 
				For the alternate size icon, formZ uses the bottom left 20 x 16 pixels. The top 16 and right 12 pixels 
				are NOT used (and will be cropped). <nl>
								
				Macintosh: The back and white source graphic is stored in a 'ICON' resource. The color source is read 
				from a 32 x 32 'cicn' resource.  <nl>
				
				Windows: The back and white and color source graphics are stored in a 'Icon' resource.  <nl>

	@return		Error codes

	@avail		5.0.0.0

	@see		fzrt_rlib_get_resource,fz_cmnd_cbak_proj_icon_file

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_icon_rsrc_func)(
	fz_fuim_icon_enum 			which, 				/* @parm	<in> | which style of icon to load. */		
	fzrt_icon_ptr 			*icon				/* @parm	<out> | icon handle. */
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Preference file I/O (Optional). This function is called by formZ to read and write the commands preference information 
				when a preference file or the defaults file is read or written. The preference files use an IFF type of file format. 
				A block is read or written for each command which has a fz_cmnd_cbak_proj_icon_rsrc registered. This is useful for storing 
				any user settings used by a command such as the options for a tool command. <nl>
				
				Version control is handled through the version parameter. When writing a block you should set the version variable to
				the variant of data that you are writing. The default is zero. It is recommended that you increment this number each time
				when you change the format of the data that you write to the stream. When reading the block, the version parameter
				contains the version which was used to write the block. You can then take the appropriate action to read the variant 
				of data in the block. 

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_pref_io_func)(
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 			/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);





/********************************************************************************************************

	@fsfunc_t	 

	@desc		Adds command to the extensions menu. This function is called by formZ to add the command icon to the extensions menu toolbar. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_fuim_exts_menu

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_menu_func)(
	fz_fuim_menu_ptr			menu_ptr,				/* @parm	<in> | pointer to formz menu manager */ 
	const fzrt_UUID_td 			menu_uuid,				/* @parm	<in> | menu id of the extensions menu */ 
	fzrt_UUID_td 				group_uuid,				/* @parm	<out_opt> | menu id of the group to insert the command in 
																			if NULL, then the item will appear in the Extensions menu */ 
	fzrt_int 						*position 				/* @parm	<out_opt> | posttion in group_uuid to insert the command in 
																			if 0, item is inserted in alpha-order*/ 
	);


/********************************************************************************************************

	@fsfunc_t	 

	@desc		Adds command icon to a toolbar (Optional). This function is called by formZ to add the command icon to the icon toolbar. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_icon_menu_adjacent,fz_fuim_exts_icon_group	

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_icon_menu_func)(
	const fzrt_UUID_td 			icon_bar_uuid,			/* @parm	<in> | id of the icon tool bar */ 
	fzrt_UUID_td 				group_uuid,				/* @parm	<out> | id of the icon group (row) to insert the icon
																			if NULL, then a default Extensions group at the bottom of the toorbar is used */ 
	fz_fuim_icon_group_enum 	*group_pos, 			/* @parm	<out> | position to insert the command in the group (row) */ 
	fzrt_int 						*group_row, 			/* @parm	<out> | specifies exact row when group_pos is CMND_ICON_GROUP_CUSTOM */ 
	fzrt_int 						*group_col				/* @parm	<out> | specifies exact column when group_pos is CMND_ICON_GROUP_CUSTOM */ 
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Adds command icon to a toolbar, adjacent to another command. This function is called by formZ to add the command 
				icon to the icon toolbar. The position of the icon is adjacent to an existing icon.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_icon_menu

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_icon_menu_adjacent_func)(
	const fzrt_UUID_td 		icon_bar_uuid, 		/* @parm	<in> | id of the icon tool bar */ 
	fzrt_UUID_td 			adjacent_uuid, 		/* @parm	<out> | id of the adjecant command */ 
	fz_fuim_icon_adjacent_enum *where				/* @parm	<out> | relationship to the adjacent command */ 
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		formZ calls this function to read and write any command specific project data to a formZ project
				file. This function is called once when reading and writing formZ project files. The file IO is
				performed using the IO streams (iost) interface. This interface provides functions for reading and
				writing data from a file (stream) and handles all cross platform endian issues. The iost
				parameter is the pointer to the formZ project file and should be used in all IO Stream function
				calls. The IO Stream functions are fully documented in the formZ API reference.
				The dir parameter indicates if the file is being written with a value of FZ_IOST_WRITE or read
				with a value of FZ_IOST_READ. The version parameter should return the version of the data
				that was is written when writing a file. When reading a file, the version parameter contains the
				version of the data that was written to in the file (and hence being read). The size parameter is
				only valid when dir == FZ_IOST_READ (read). This is the size of the data that was written in
				the file.
				It is the responsibility of the plugin to maintain version changes of the plugin data.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_data_io_func)(
	fzrt_int					windex,				/* @parm	<in> | project window index */
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 				/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);



/********************************************************************************************************

	@fsfunc_t	

	@desc		formZ calls this function to read and write any command specific project window data to a
				formZ project file. This function is called once for each window in the project when reading and
				writing formZ project files. The file IO is performed using the IO streams (iost) interface. This
				interface provides functions for reading and writing data from a file (stream) and handles all cross
				platform endian issues. The iost parameter is the pointer to the formZ Project file and should
				be used in all IO Stream function calls. The IO Stream functions are fully documented in the
				formZ API reference.
				The dir parameter indicates if the file is being written with a value of FZ_IOST_WRITE or read
				with a value of FZ_IOST_READ. The version parameter should return the version of the data
				that was is written when writing a file. When reading a file, the version parameter contains the
				version of the data that was written to in the file (and hence being read). The size parameter is
				only valid when dir == FZ_IOST_READ (read). This is the size of the data that was written in
				the file.
				It is the responsibility of the plugin to maintain version changes of the plugin data.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_wind_data_io_func)(
	fzrt_int					windex,				/* @parm	<in> | project window index */
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 				/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);

/************************************************************************

	@fsfunc_t

	@desc		Instructs a formZ extension to register its help data.
							
	@avail		6.0.0.0

	@toc		cmds_cbak_proj

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_proj_pdf_help_func)(void);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Command icon from file (Optional). This function is called by formZ to load an icon for the command from an image file. 
				The image file can be in any supported image file format (or of an image format wich is support be a image file 
				translator plugin). It is recommened to use the TIFF image format as it is a cross platform supported format and 
				is always present. The source graphics are identified in the file by indicating the top left corner of the 
				graphic in the file. In addition to the graphic, a mask can be provided for use when the icon is highlighted. A 
				single file can be used for multiple icons across a variety of plugins by createing a grid of icons in the file 
				and specifing the location for each icon.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_cmnd_cbak_proj_icon_rsrc

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_cmnd_cbak_proj_icon_file2_func)(
	fzrt_floc_ptr 			floc				/* @parm	<out> | file location of file which contains the icon. */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Project data init function. 
				This function needs to be implemented if the cmnd needs to store descrete data for a project.
				This functionis called when each project is created.
				The plugin can return a single data reference (ie pointer to memory allocted by the plugin)
				The data pointer can be accessed using fz_proj_cmnd_data_get using the same UUID returned by fz_cmnd_cbak_proj_uuid

	@return		Error codes

	@avail		8.6.0.0

	@see		fz_cmnd_cbak_proj_data_finit,fz_proj_cmnd_data_get

	@exfile		

	@toc		rndr_cbak	

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_proj_data_init_func)(
	fzrt_int			windex,			/* @parm	<in> | project window index */
	fzrt_ptr			*proj_data		/* @parm	<out> | pointer to data */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Project data finit function.
				The project data finit function is complementary to the project data init function. 
				If implemented, it is called when a project is closed. It gives the cmnd the opportunity
				to finit any dynamic data, that was created since the project was opened. 

 
	@return		Error codes

	@avail		8.6.0.0

	@see		fz_cmnd_cbak_proj_data_init

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_proj_data_finit_func)(
	fzrt_int			windex,			/* @parm	<in> | project window index */
	fzrt_ptr			proj_data		/* @parm	<in> | pointer to data allocated in fz_cmnd_cbak_proj_data_init */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Project data init function. 
				This function needs to be implemented if the cmnd needs to store descrete data for a each window of a project.
				This functionis called when each window of the project is created.
				The plugin can return a single data reference (ie pointer to memory allocted by the plugin)
				The data pointer can be accessed using fz_proj_cmnd_wind_data_get using the same UUID returned by fz_cmnd_cbak_proj_uuid

	@return		Error codes

	@avail		8.6.0.0

	@see		fz_cmnd_cbak_proj_wind_data_finit

	@exfile		

	@toc		rndr_cbak	

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_proj_wind_data_init_func)(
	fzrt_int			windex,			/* @parm	<in> | project window index */
	fzrt_ptr			*wind_data		/* @parm	<out> | pointer to data */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Project data finit function.
				The project data finit function is complementary to the project data init function. 
				If implemented, it is called when a project windwo is closed. It gives the cmnd the opportunity
				to finit any dynamic data, that was created since the project window was opened. 

 
	@return		Error codes

	@avail		8.6.0.0

	@see		fz_cmnd_cbak_proj_wind_data_init

	@exfile		

	@toc		cmds_cbak_proj

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_cmnd_cbak_proj_wind_data_finit_func)(
	fzrt_int			windex,			/* @parm	<in> | project window index */
	fzrt_ptr			wind_data		/* @parm	<in> | pointer to data allocated in fz_cmnd_cbak_proj_wind_data_init */
	);
	
/************************************************************************

	@plfset	fz_cmnd_cbak_proj_fset

	@desc	Function set for commands. The functions in this set are supplied by the plugin or script to enable commands.  
			Commands are tools actions or states that are selected or invoked by a menu item, icon or key shortcut.

	@avail	5.0.0.0

	@toc	cmds_cbak_proj

*************************************************************************/

/********************************************************************************************************

	@fsdef	formZ Commands function set id.

*********************************************************************************************************/
/* UUID : {259896ba-3058-4888-81f0-20dea4eedb6a} */
#define FZ_CMND_CBAK_PROJ_FSET_TYPE		"\x25\x98\x96\xba\x30\x58\x48\x88\x81\xf0\x20\xde\xa4\xee\xdb\x6a"	/* Reference 800 */
/********************************************************************************************************

	@fsdef	formZ Commands function set name.

*********************************************************************************************************/
#define FZ_CMND_CBAK_PROJ_FSET_NAME		"formZ commands functions"
/********************************************************************************************************

	@fsdef	formZ Commands function set version.

*********************************************************************************************************/
#define FZ_CMND_CBAK_PROJ_FSET_VERSION	FZPL_VERS_MAKE(8,6,0,0)


typedef struct fz_cmnd_cbak_proj_fset
{	
	fzpl_fset_prvt_data_td						prvt_data;

	fz_cmnd_cbak_proj_init_func					fz_cmnd_cbak_proj_init;				/* @fsfunc */
	fz_cmnd_cbak_proj_finit_func				fz_cmnd_cbak_proj_finit;				/* @fsfunc */
	fz_cmnd_cbak_proj_info_func					fz_cmnd_cbak_proj_info;				/* @fsfunc */
	fz_cmnd_cbak_proj_name_func					fz_cmnd_cbak_proj_name;				/* @fsfunc */
	fz_cmnd_cbak_proj_uuid_func					fz_cmnd_cbak_proj_uuid;				/* @fsfunc */
	fz_cmnd_cbak_proj_help_func					fz_cmnd_cbak_proj_help;				/* @fsfunc */
	
	fz_cmnd_cbak_proj_avail_func				fz_cmnd_cbak_proj_avail;				/* @fsfunc */
	fz_cmnd_cbak_proj_select_func				fz_cmnd_cbak_proj_select;				/* @fsfunc */
	fz_cmnd_cbak_proj_active_func				fz_cmnd_cbak_proj_active;				/* @fsfunc */

	fz_cmnd_cbak_proj_menu_func					fz_cmnd_cbak_proj_menu;				/* @fsfunc */
	fz_cmnd_cbak_proj_icon_menu_func			fz_cmnd_cbak_proj_icon_menu;			/* @fsfunc */
	fz_cmnd_cbak_proj_icon_menu_adjacent_func	fz_cmnd_cbak_proj_icon_menu_adjacent;	/* @fsfunc */
	fz_cmnd_cbak_proj_icon_rsrc_func			fz_cmnd_cbak_proj_icon_rsrc;			/* @fsfunc */
	fz_cmnd_cbak_proj_icon_file_func			fz_cmnd_cbak_proj_icon_file_obsolete;			/* @fsfunc */

	fz_cmnd_cbak_proj_pref_io_func				fz_cmnd_cbak_proj_pref_io;				/* @fsfunc */	
	fz_cmnd_cbak_proj_data_io_func				fz_cmnd_cbak_proj_data_io;				/* @fsfunc */	
	fz_cmnd_cbak_proj_wind_data_io_func			fz_cmnd_cbak_proj_wind_data_io;		/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_cmnd_cbak_proj_pdf_help_func				fz_cmnd_cbak_proj_pdf_help;				/* @fsfunc */
	
	/************************************************************************************
	 *		@comm  New 7.0.0.0 functions.												*
	 ************************************************************************************/

	fz_cmnd_cbak_proj_icon_file2_func			fz_cmnd_cbak_proj_icon_file;			/* @fsfunc */

	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 7.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	fz_cmnd_cbak_proj_data_init_func			fz_cmnd_cbak_proj_data_init;
	fz_cmnd_cbak_proj_data_finit_func			fz_cmnd_cbak_proj_data_finit;
	fz_cmnd_cbak_proj_wind_data_init_func		fz_cmnd_cbak_proj_wind_data_init;
	fz_cmnd_cbak_proj_wind_data_finit_func		fz_cmnd_cbak_proj_wind_data_finit;

	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8.6.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

} fz_cmnd_cbak_proj_fset;



#ifdef __cplusplus
}
#endif


#endif /* _FORMZ_CMND_API_H  */
