/*********************************************************************************************
 *                                                                                           *
 *                                                                                           *
 *   Copyright 1989-2018 by AutoDesSys, Inc.                                                 *
 *                                                                                           *
 *   This software and its associated documentation contains proprietary,                    *
 *   confidential, and trade secret information of AutoDesSys Inc. and may not be            *
 *   disclosed, copied in any form, or used for any purpose except as specifically           *
 *   permitted in writing by AutoDesSys, Inc.                                                *
 *                                                                                           *
 *   AutoDesSys, Inc. grants permission to use the software to those who purchase            *
 *   a license.  Such a usage will be made according to the terms and conditions             *
 *   specified in the license agreement to which a licensee agrees before installing         *
 *   and commencing usage of the software.  The license agreement can be found               *
 *   on the envelop that contains the software CD.                                           *
 *                                                                                           *
 *   AutoDesSys, Inc. also grants permission to licensees to develop extensions to           *
 *   the software, which can be in the form of scripts or plugins.  The permission           *
 *   includes calling the API and call back functions provided by AutoDesSys, Inc.           *
 *   and accessing information relevant to these functions and the general workings          *
 *   of form.Z, but only up to the point to which AutoDesSys, Inc. has chosen to             *
 *   expose, through its form.Z SDK Manual.  All extensions developed by third               *
 *   parties will include a copy of this copyright note in an easily accessible place or     *
 *   a clear instruction to read this original note by AutoDesSys, Inc.                      *
 *                                                                                           *
 *   All the software distributed under a license, either for the purpose of using it or     *
 *   for the purpose of developing extensions, is distributed AS IS, WITHOUT                 *
 *   WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND                                    *
 *   AutoDesSys, Inc. HEREBY DISCLAIMS ALL SUCH WARRANTIES,                                  *
 *   INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF                                         *
 *   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.                                    *
 *                                                                                           *
 *   formZ, RenderZone, and RadioZity are registered trademarks of AutoDesSys, Inc.          *
 *                                                                                           *
 *   form.Z SDK, Version 8.6.3, Dec 03, 2018                                                 *
 *                                                                                           *
 *                                                                                           *
 *********************************************************************************************/



#ifndef __FORMZ_LITE_H
#define __FORMZ_LITE_H 1

#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************

	Lights Manager


	Description
		formZ lights management functions.  This module contains functions to create, manipulate, edit 
		their parameters, and delete lights.
 ****************************************************************/

#include "fzpl_plugin.h"

#include "fz_api.h"
#include "fz_type_api.h"
#include "fz_objt_api.h"
#include "fz_objt_symb_api.h"
#include "fz_light_rep_api.h"


/****************************************************************

	FZ_LIGHT_DATA_UUID

	Description
		Unique identifier for the formz Light Data

	Availability	7.0.0.0

 ****************************************************************/
#define FZ_LIGHT_DATA_UUID "\xfd\x8b\x54\xc7\x71\x2b\x42\x51\xa5\xe3\xfb\xa7\xdf\xf2\x7b\x19"

/****************************************************************

	fz_lite_sun_ptr


	Description
		Data structure for the sun options of a light. 
		The individual fields of this type can be accessed and set via 
		fz_lite_get_parm_sun and fz_lite_set_parm_sun.

	Availability	5.0.0.0

 ****************************************************************/
typedef struct sun_td			*fz_lite_sun_ptr;

/****************************************************************

	fz_lite_custom_data_ptr


	Description
		Data structure to hold options for editing a custom light. 
		The individual fields of this type can be accessed and set via 
		fz_lite_get_parm_custom_data and fz_lite_set_parm_custom_data.

	Availability	5.0.0.0


	See Also
		fz_lite_get_parm_custom_data, fz_lite_parm_cust_enum, fz_lite_set_parm_custom_data

 ****************************************************************/
typedef struct	light_custom_data_td		*fz_lite_custom_data_ptr;

/****************************************************************

	fz_lite_parm_common_enum


	Description
		Common parameter identifiers for a light. Used for the 
		"which" argument of fz_lite_get_parm_common and fz_lite_set_parm_common

	Members
		FZ_LITE_PARM_NAME
			Light name (32 characters max)
			Type: char*
		FZ_LITE_PARM_TYPE
			Light type
			Type: fz_lite_type_enum
			Default Value: FZ_LITE_TYPE_DISTANT
		FZ_LITE_PARM_COLOR
			Light color
			Type: fz_rgb_float_td
			Default Value: white {1.0,1.0,1.0}
		FZ_LITE_PARM_ACCURATE_INTENS_ON
			Use Light's accurate intensity
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_ACCURATE_INTENS_RADIOS_ON
			If on, accurate radiometric intensity (Watts) is used. Otherwise 
			the photometric intensity (lumens, candelas) is used.
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_PARM_ACCURATE_INTENS_WATTS
			Radiometric light intensity, expressed in watts.
			Type: double
			Default Value: 100.0
			Range: 0.0
		FZ_LITE_PARM_ACCURATE_INTENS_EFFICIENCY
			Efficiency of a light's radiometric intensity.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_ACCURATE_INTENS_COLTEMP
			The photometric intensity's color temperature in degrees Kelvin. 
			Meaningful values are in the range of 1000 to 10000 
			degrees Kelvin, but larger values are allowed.
			Type: double
			Default Value: 2000
			Range: 1000
		FZ_LITE_PARM_ACCURATE_INTENS_LUMENS
			Photometric light intensity, expressed in lumens.
			Type: double
			Default Value: 1000
			Range: 0.0
		FZ_LITE_PARM_SIMPLE_INTENS
			Simple light intensity. A value of 1.0 means 100%. 
			Negative values are allowed, meaning that the light source 
			will suck light away from the scene, instead of adding to it.
			Type: double
			Default Value: 1.0
		FZ_LITE_PARM_SIMPLE_FALLOFF
			Light falloff.
			Type: fz_lite_falloff_enum
			Default Value: FZ_LITE_FALLOFF_CONSTANT
		FZ_LITE_PARM_ORIGIN
			The coordinates of the light's origin.
			Type: fz_xyz_td
		FZ_LITE_PARM_COI
			The coordinates of the light's center of interest.
			Type: fz_xyz_td
		FZ_LITE_PARM_VISIBILITY
			The light's visibility status, when drawn in an 
			interactive rendering mode.
			Type: fz_lite_visible_enum
			Default Value: FZ_LITE_VISIBILITY_INVISIBLE
		FZ_LITE_PARM_LOCKED_ON
			The light's lock status.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SHINING_ON
			The light's shining status.
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_PARM_LENSFLARE_ON
			Light lens flares on/off
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_PARM_GLOW_ON
			Light glow on/off
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SHADOWS_ON
			Light shadows on/off
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SHADOW_TYPE
			Light shadow type
			Type: fz_lite_smap_type_enum
			Default Value: FZ_LITE_SHAD_TYPE_MAP
		FZ_LITE_PARM_TRANS_SHADOWS_ON
			Transparent shadows on/off
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SHADOW_MAP_QUALITY
			Shadow map quality
			Type: fz_lite_smap_quality_enum
			Default Value: FZ_LITE_SHAD_MAP_QUALITY_MEDIUM
		FZ_LITE_PARM_SHADOW_MAP_SOFTNESS
			Shadow map softness. 1.0 mean 100%. Meaningful values 
			are in the range of 0.0 to 1.0.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_PARM_SHADOW_MAP_TOLERANCE
			Shadow map tolerance. 1.0 mean 100%. Meaningful values 
			are in the range of 0.0 to 1.0.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_PARM_SHADOW_MAP_FIXED_RES_ON
			Shadow map fixed resolution on/off. If FALSE, the shadow 
			map resolution is determined as a factor of the image size. 
			if TRUE, the resolution is defined by a fixed number.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SHADOW_MAP_RES_IMAGESIZE
			Shadow map resolution as a factor of the image size.
			Type: double
			Default Value: 1.0
			Range: 0.0
		FZ_LITE_PARM_SHADOW_MAP_RES_FIXED
			Shadow map resolution as a fixed value.
			Type: fzrt_int
			Default Value: 512
			Range: 64
		FZ_LITE_PARM_SHADOW_MAP_LIMIT
			The type of shadow map limit for a distant light.
			Type: fz_lite_smap_limit_enum
			Default Value: FZ_LITE_SHAD_MAP_LIMIT_ALL
		FZ_LITE_PARM_GLOW_ACCURATE_ON
			Accurate light glow on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_GLOW_SIMPLE_INTENS
			Simple glow intensity. A value of 1.0 means 100%.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_GLOW_SIMPLE_FALLOFF
			Simple glow falloff
			Type: fz_lite_glow_falloff_enum
			Default Value: FZ_LITE_GLOW_SIMPLE_FALLOF_SQUARE
		FZ_LITE_PARM_GLOW_SIMPLE_LIGHT_DIST_ON
			Simple glow by user specified distance on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_GLOW_SIMPLE_DIST
			User specified simple glow distance.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_INTENS
			Accurate glow intensity. A value of 1.0 means 100%.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_ATTENUATION
			Accurate glow attenuation. A value of 1.0 means 100%.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_MAX_DIST
			Accurate glow maximum distance.
			Type: double
			Default Value: English 1200.0 in, Metric 5000.0 cm

			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_QUALITY
			Accurate glow quality.
			Type: fzrt_int
			Default Value: 5
			Range: 1
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_ON
			Accurate glow noise on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_SIZE
			Accurate glow noise size interval.
			Type: double
			Default Value: English 48.0 in, Metric 100.0 cm

			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_TYPE
			Accurate glow noise type.
			Type: fz_lite_glow_noise_enum
			Default Value: FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BETTER
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_NUM_IMPULSE
			Accurate glow noise, # of impulses.
			Type: fzrt_int
			Default Value: 3
			Range: 1
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_CONTRAST
			Accurate glow noise contrast.
			Type: double
			Default Value: 1.0
			Range: 0.0
		FZ_LITE_PARM_GLOW_ACCURATE_NOISE_DETAIL
			Accurate glow noise detail.
			Type: fzrt_int
			Default Value: 3
			Range: 1
		FZ_LITE_PARM_ACCURATE_INTENS_COLTEMP_ON
			Whether or not the Color Temperature is used 
			for the Photometric Accurate light intensity
			Type: fzrt_boolean
			Default Value: FALSE
			Availability: 6.1.2.0


	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_common_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_common_enum
{
	FZ_LITE_PARM_NAME = 0,
											
	FZ_LITE_PARM_TYPE,
													
	FZ_LITE_PARM_COLOR,
											
	FZ_LITE_PARM_ACCURATE_INTENS_ON,
											
	FZ_LITE_PARM_ACCURATE_INTENS_RADIOS_ON,
											
	FZ_LITE_PARM_ACCURATE_INTENS_WATTS,
											
	FZ_LITE_PARM_ACCURATE_INTENS_EFFICIENCY,
											
											
	FZ_LITE_PARM_ACCURATE_INTENS_COLTEMP,
											
	FZ_LITE_PARM_ACCURATE_INTENS_LUMENS,

	FZ_LITE_PARM_SIMPLE_INTENS,
											
	FZ_LITE_PARM_SIMPLE_FALLOFF,

	FZ_LITE_PARM_ORIGIN,

	FZ_LITE_PARM_COI,
											
	FZ_LITE_PARM_VISIBILITY,
											
	FZ_LITE_PARM_LOCKED_ON,
											
	FZ_LITE_PARM_SHINING_ON,
											
	FZ_LITE_PARM_LENSFLARE_ON,
											
	FZ_LITE_PARM_GLOW_ON,
											
	FZ_LITE_PARM_SHADOWS_ON,
											
	FZ_LITE_PARM_SHADOW_TYPE,
											
	FZ_LITE_PARM_TRANS_SHADOWS_ON,
											
	FZ_LITE_PARM_SHADOW_MAP_QUALITY,
											
	FZ_LITE_PARM_SHADOW_MAP_SOFTNESS,
											
	FZ_LITE_PARM_SHADOW_MAP_TOLERANCE,
											
	FZ_LITE_PARM_SHADOW_MAP_FIXED_RES_ON,
											
											
	FZ_LITE_PARM_SHADOW_MAP_RES_IMAGESIZE,

	FZ_LITE_PARM_SHADOW_MAP_RES_FIXED,
											
	FZ_LITE_PARM_SHADOW_MAP_LIMIT,
											
	FZ_LITE_PARM_GLOW_ACCURATE_ON,
											
	FZ_LITE_PARM_GLOW_SIMPLE_INTENS,
											
	FZ_LITE_PARM_GLOW_SIMPLE_FALLOFF,
											
	FZ_LITE_PARM_GLOW_SIMPLE_LIGHT_DIST_ON,
											
	FZ_LITE_PARM_GLOW_SIMPLE_DIST,
											
	FZ_LITE_PARM_GLOW_ACCURATE_INTENS,
											
	FZ_LITE_PARM_GLOW_ACCURATE_ATTENUATION,
											
	FZ_LITE_PARM_GLOW_ACCURATE_MAX_DIST,
											
	FZ_LITE_PARM_GLOW_ACCURATE_QUALITY,
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_ON,
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_SIZE,
											
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_TYPE,
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_NUM_IMPULSE,
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_CONTRAST,
											
	FZ_LITE_PARM_GLOW_ACCURATE_NOISE_DETAIL,
	FZ_LITE_PARM_ACCURATE_INTENS_COLTEMP_ON,

	FZ_LITE_PARAM_MAX
} fz_lite_parm_common_enum;

/****************************************************************

	fz_lite_type_enum


	Description
		Indicates what type of light the light is. 
		This enum is used in the parameter FZ_LITE_PARM_TYPE in the 
		functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_TYPE_DISTANT
			Distant light
		FZ_LITE_TYPE_POINT
			Point light
		FZ_LITE_TYPE_CONE
			Cone light
		FZ_LITE_TYPE_PROJECTOR
			Projector light
		FZ_LITE_TYPE_AREA
			Area light
		FZ_LITE_TYPE_CUSTOM
			Custom light
		FZ_LITE_TYPE_LINE
			Line light
		FZ_LITE_TYPE_ENVIRONMENT
			Environment light
		FZ_LITE_TYPE_AMBIENT
			Ambient light (v 7.0.0.0)
		FZ_LITE_TYPE_PLUGIN
			Light type installed by a plugin (v 7.0.0.0)

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_type_enum
{	FZ_LITE_TYPE_DISTANT = 0,
	FZ_LITE_TYPE_POINT,
	FZ_LITE_TYPE_CONE,
	FZ_LITE_TYPE_PROJECTOR,
FZ_LITE_TYPE_AREA,
	FZ_LITE_TYPE_CUSTOM,
	FZ_LITE_TYPE_LINE,
	FZ_LITE_TYPE_ENVIRONMENT,
	FZ_LITE_TYPE_AMBIENT,
	FZ_LITE_TYPE_PLUGIN,
	
	FZ_LITE_TYPE_MAX
} fz_lite_type_enum;

/****************************************************************

	fz_lite_falloff_enum


	Description
		Indicates what type of light falloff the light has. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_FALLOFF 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_SIMPLE_FALLOFF 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_FALLOFF_CONSTANT
			Constant falloff
		FZ_LITE_FALLOFF_LINEAR_CLAMPED
			Linear falloff (clamped)
		FZ_LITE_FALLOFF_SQUARE_CLAMPED
			Square falloff (clamped)
		FZ_LITE_FALLOFF_LINEAR_UNCLAMPED
			Linear falloff (unclamped)
		FZ_LITE_FALLOFF_SQUARE_UNCLAMPED
			Square falloff (unclamped)

	Availability	5.0.0.0


	See Also
		fz_lite_parm_common_enum, fz_lite_parm_grup_enum

 ****************************************************************/
typedef enum fz_lite_falloff_enum
{
	FZ_LITE_FALLOFF_CONSTANT = 0,
	FZ_LITE_FALLOFF_LINEAR_CLAMPED,
	FZ_LITE_FALLOFF_SQUARE_CLAMPED,
	FZ_LITE_FALLOFF_LINEAR_UNCLAMPED,
	FZ_LITE_FALLOFF_SQUARE_UNCLAMPED,
	
	FZ_LITE_FALLOFF_MAX
} fz_lite_falloff_enum;

/****************************************************************

	fz_lite_visible_enum


	Description
		Indicates what type of light visibility the light has. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_VISIBILITY 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_VISIBILITY 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_VISIBILITY_INVISIBLE
			Light is not visible (not drawn on screen)
		FZ_LITE_VISIBILITY_VISIBLE
			Light is visible
		FZ_LITE_VISIBILITY_GHOSTED
			Light is drawn in the project's ghost color

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_visible_enum
{
	FZ_LITE_VISIBILITY_INVISIBLE = 0,
	FZ_LITE_VISIBILITY_VISIBLE,
	FZ_LITE_VISIBILITY_GHOSTED,
	
	FZ_LITE_VISIBILITY_MAX
} fz_lite_visible_enum;

/****************************************************************

	fz_lite_smap_type_enum


	Description
		Indicates what type of shadow map the light has. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_TYPE 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_SHADOW_TYPE 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_SHAD_TYPE_MAP
			Shadows are created from a shadow map.
		FZ_LITE_SHAD_TYPE_RAYTRACE
			Shadows are raytraced.
		FZ_LITE_SHAD_TYPE_RAYTRACE_ACCL
			Shadows are raytraced with shadow map acceleration.

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_smap_type_enum
{
	FZ_LITE_SHAD_TYPE_MAP = 0,
	FZ_LITE_SHAD_TYPE_RAYTRACE,
	FZ_LITE_SHAD_TYPE_RAYTRACE_ACCL,
	
	FZ_LITE_SHAD_TYPE_MAX
} fz_lite_smap_type_enum;

/****************************************************************

	fz_lite_smap_quality_enum


	Description
		Indicates what type of shadow map quality the light has. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_QUALITY 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_SHADOW_MAP_QUALITY 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_SHAD_MAP_QUALITY_LOW
			Low shadow map quality.
		FZ_LITE_SHAD_MAP_QUALITY_MEDIUM
			Medium shadow map quality.
		FZ_LITE_SHAD_MAP_QUALITY_HIGH
			High shadow map quality.

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_smap_quality_enum
{
	FZ_LITE_SHAD_MAP_QUALITY_LOW = 0,
	FZ_LITE_SHAD_MAP_QUALITY_MEDIUM,
	FZ_LITE_SHAD_MAP_QUALITY_HIGH,
	
	FZ_LITE_SHAD_MAP_QUALITY_MAX
} fz_lite_smap_quality_enum;

/****************************************************************

	fz_lite_smap_limit_enum


	Description
		Indicates what type of shadow limit the light has. This applies only to 
		distant lights. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_LIMIT 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_SHADOW_MAP_LIMIT 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_SHAD_MAP_LIMIT_ALL
			Include all objects in the generation of a shadow map
		FZ_LITE_SHAD_MAP_LIMIT_VIS_FACES
			Include only completely visible faces in the generation of a shadow map
		FZ_LITE_SHAD_MAP_LIMIT_VIS_OBJS
			Include only completely visible objects in the generation of a shadow map

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_smap_limit_enum
{
	FZ_LITE_SHAD_MAP_LIMIT_ALL = 0,
	FZ_LITE_SHAD_MAP_LIMIT_VIS_FACES,
	FZ_LITE_SHAD_MAP_LIMIT_VIS_OBJS,
	
	FZ_LITE_SHAD_MAP_LIMIT_MAX
} fz_lite_smap_limit_enum;

/****************************************************************

	fz_lite_glow_falloff_enum


	Description
		Indicates what type of simple glow falloff the light has. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_FALLOFF 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_GLOW_SIMPLE_FALLOFF 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_GLOW_SIMPLE_FALLOF_LINEAR
			Constant falloff
		FZ_LITE_GLOW_SIMPLE_FALLOF_SQUARE
			Linear falloff
		FZ_LITE_GLOW_SIMPLE_FALLOF_CUBIC
			Square falloff

	Availability	5.0.0.0


	See Also
		fz_lite_parm_common_enum, fz_lite_parm_grup_enum

 ****************************************************************/
typedef enum fz_lite_glow_falloff_enum
{	FZ_LITE_GLOW_SIMPLE_FALLOF_LINEAR = 0,
	FZ_LITE_GLOW_SIMPLE_FALLOF_SQUARE,
	FZ_LITE_GLOW_SIMPLE_FALLOF_CUBIC,
	
	FZ_LITE_GLOW_SIMPLE_FALLOF_MAX
} fz_lite_glow_falloff_enum;

/****************************************************************

	fz_lite_glow_noise_enum


	Description
		Glow accurate noise type. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_TYPE 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_GLOW_ACCURATE_NOISE_TYPE 
		in the functions fz_lite_get_parm_common and fz_lite_set_parm_common.

	Members
		FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_SIMPLE
			Simple noise patterns
		FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BETTER
			Better nosie patterns
		FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BEST
			Best noise patterns

	Availability	5.0.0.0


	See Also
		fz_lite_parm_common_enum, fz_lite_parm_grup_enum

 ****************************************************************/
typedef enum fz_lite_glow_noise_enum
{	FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_SIMPLE = 0,
	FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BETTER,
	FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BEST,
	
	FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_MAX
} fz_lite_glow_noise_enum;


/****************************************************************

	fz_lite_parm_dist_enum


	Description
		Parameter identifiers for a distant light. Used for the 
		"which" argument of fz_lite_get_parm_dist and fz_lite_set_parm_dist

	Members
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ON_OBSOLETE
			Use the atmospheric option for accurate distant light intensity. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_ON.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_ON_OBSOLETE
			Use the simple atmospheric parameters . 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_SIMPLE_ON.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_QUALITY_OBSOLETE
			Accurate sky parameter : Quality of the atmospheric light. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_ACC_QUALITY.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_LUMINANCE_OBSOLETE
			Accurate sky parameter : Luminance value of the atmospheric light in candelas / sqft or candelas / sqMeter. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_ACC_LUMINANCE.
			Type: double
			Default Value: 50
			Range: 0.0
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_COVER_OBSOLETE
			Accurate sky parameter : Cloud coverage type of the atmospheric light. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_ACC_COVER.
			Type: fz_lite_parm_sky_intens_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_OVERCAST
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_COLOR_OBSOLETE
			Accurate sky parameter : Color type of the atmospheric light. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_ACC_COLOR_PRESET.
			Type: fz_lite_parm_sky_color_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_LIGHTBLUE
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_INTENS_OBSOLETE
			Simple  sky parameter : sky intensity. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_SIMPLE_INTENS.
			Type: double
			Range: 0.0
			Default Value: 1.0
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_NOISE_OBSOLETE
			Simple  sky parameter : sky noise. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_NOISE.
			Type: double
			Range: 0.0
			Default Value: 0.0
		FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_SAMPLES_OBSOLETE
			Simple  sky parameter : # of samples. 
			Obsolete: 6.5.0.0. Replaced by FZ_LITE_PARM_DIST_SKY_SIMPLE_SAMPLES.
			Type: fzrt_int
			Range: 3
			Default Value: 10
		FZ_LITE_PARM_DIST_SUN
			The sun data of a distant light.
			Type: fz_lite_sun_ptr
		FZ_LITE_PARM_DIST_SKY_ON
			Apply the atmospheric light also for this distant light.
			Type: fzrt_boolean
			Default Value: FALSE
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_SIMPLE_ON
			Use the simple atmospheric parameters.  If FALSE, then 
			the accurate atmospheric parameters are used.
			Type: fzrt_boolean
			Default Value: TRUE
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_QUALITY
			Accurate sky parameter : Quality of the atmospheric light.
			Type: double
			Default Value: 0.5
			Range: 0.0
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_LUMINANCE
			Accurate sky parameter : Luminance value of the atmospheric light (in candelas / sqft or candelas / sqMeter).
			Type: double
			Default Value: 50
			Range: 0.0
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_COVER
			Accurate sky parameter : Cloud coverage type of the atmospheric light.
			Type: fz_lite_parm_sky_intens_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_OVERCAST
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_COLOR_BY_TEMP
			Accurate sky parameter : If TRUE, the color of sky is determined by color temperature (in Kelvin) of FZ_LITE_PARM_DIST_SKY_ACC_COLOR_TEMP. 
			If FALSE, the color is determined by the specified color of FZ_LITE_PARM_DIST_SKY_COLOR.
			Type: fzrt_boolean
			Default Value: FALSE
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_COLOR_TEMP
			Accurate sky parameter : Color of sky is determined by this color temperature (in kelvin) if FZ_LITE_PARM_DIST_SKY_ACC_COLOR_BY_TEMP is TRUE.
			Type: float
			Default Value: 6500
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_ACC_COLOR_PRESET
			Accurate sky parameter : Color temperature preset of the atmospheric light. 
			This assigns the color temperature to a preset value. 
			Useful only if the FZ_LITE_PARM_DIST_SKY_ACC_COLOR_BY_TEMP parameter is set to TRUE.
			Type: fz_lite_parm_sky_color_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_LIGHTBLUE
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_SIMPLE_INTENS
			Simple sky parameter : sky intensity.
			Type: double
			Range: 0.0
			Default Value: 1.0
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_SIMPLE_SAMPLES
			Simple sky parameter : # of samples.
			Type: fzrt_int
			Range: 3
			Default Value: 10
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_COLOR
			Sky parameter : sky color.
			Type: fz_rgb_float_td
			Default Value: white (1,1,1)
			Availability: 6.5.0.0

		FZ_LITE_PARM_DIST_SKY_NOISE
			Sky parameter : sky noise.
			Type: double
			Range: 0.0
			Default Value: 0.0
			Availability: 6.5.0.0


	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_dist_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_dist_enum
{
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ON_OBSOLETE,
														
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_ON_OBSOLETE,

	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_QUALITY_OBSOLETE,
											
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_LUMINANCE_OBSOLETE,
											
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_COVER_OBSOLETE,
											
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_ACC_COLOR_OBSOLETE,
											
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_INTENS_OBSOLETE,
														
	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_NOISE_OBSOLETE,

	FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_SIMPLE_SAMPLES_OBSOLETE,
	FZ_LITE_PARM_DIST_SUN,
	FZ_LITE_PARM_DIST_SKY_ON,
	FZ_LITE_PARM_DIST_SKY_SIMPLE_ON,
	FZ_LITE_PARM_DIST_SKY_ACC_QUALITY,
											
	FZ_LITE_PARM_DIST_SKY_ACC_LUMINANCE,
	FZ_LITE_PARM_DIST_SKY_ACC_COVER,
	FZ_LITE_PARM_DIST_SKY_ACC_COLOR_BY_TEMP,
	FZ_LITE_PARM_DIST_SKY_ACC_COLOR_TEMP,
	FZ_LITE_PARM_DIST_SKY_ACC_COLOR_PRESET,
	FZ_LITE_PARM_DIST_SKY_SIMPLE_INTENS,
	FZ_LITE_PARM_DIST_SKY_SIMPLE_SAMPLES,
	FZ_LITE_PARM_DIST_SKY_COLOR,
	FZ_LITE_PARM_DIST_SKY_NOISE,
	FZ_LITE_PARAM_DIST_MAX
	
} fz_lite_parm_dist_enum;

/****************************************************************

	fz_lite_parm_sun_enum


	Description
		Parameter identifiers for a sun light. Used for the 
		"which" argument of fz_lite_get_parm_sun and fz_lite_set_parm_sun.

	Members
		FZ_LITE_PARM_SUN_ANGLE_NORTH
			Sun's north angle, measured relative to the world's positive y axis.
			Type: double
			Default Value: PI/2.0 (90 degrees)
		FZ_LITE_PARM_SUN_BY_SITE_ON
			Define sun direction by site on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SUN_SITE_FROM_CITY_ON
			Define sun direction from a city on/off
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SUN_CITY_ID
			city id
			Type: fzrt_int
			Range: 0
		FZ_LITE_PARM_SUN_LATITUDE
			Sun latitude angle (in radians), when specifying sun direction latitude/longitude/time zone (by city is off).
			Type: double
			Range: -PI/2
		FZ_LITE_PARM_SUN_LONGITUDE
			Sun longitude angle (in radians), when specifying sun direction latitude/longitude/time zone (by city is off)
			Type: double
			Range: -PI
		FZ_LITE_PARM_SUN_TIMEZONE
			Sun time zone (hours), when specifying sun direction latitude/longitude/time zone (by city is off)
			Type: double
			Range: -12.0
		FZ_LITE_PARM_SUN_DLST
			Time specification: Daylight savings time on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_SUN_YEAR
			Time specification: year
			Type: fzrt_int
			Range: 1900
		FZ_LITE_PARM_SUN_MONTH
			Time specification: month
			Type: fzrt_int
			Range: 1
		FZ_LITE_PARM_SUN_DAY
			Time specification: day
			Type: fzrt_int
			Range: 1
		FZ_LITE_PARM_SUN_HOUR
			Time specification: hour
			Type: fzrt_int
			Range: 0
		FZ_LITE_PARM_SUN_MINUTE
			Time specification: minute
			Type: fzrt_int
			Range: 0
		FZ_LITE_PARM_SUN_SECOND
			Time specification: second
			Type: fzrt_int
			Range: 0
		FZ_LITE_PARM_SUN_FROM_SOUTH_ON
			Sun azimuth angle measured from south on/off. Used when specifying 
			the sun direction by azimuth/altitude.
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_PARM_SUN_ALTITUDE
			Sun altitude angle. Used when specifying 
			the sun direction by azimuth/altitude.
			Type: double
			Range: -PI
		FZ_LITE_PARM_SUN_AZIMUTH
			Sun azimuth angle. Used when specifying 
			the sun direction by azimuth/altitude.
			Type: double
			Range: -PI

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_sun

 ****************************************************************/
typedef enum fz_lite_parm_sun_enum
{
	FZ_LITE_PARM_SUN_ANGLE_NORTH = 0,
									
	FZ_LITE_PARM_SUN_BY_SITE_ON,
									
	FZ_LITE_PARM_SUN_SITE_FROM_CITY_ON,
									
	FZ_LITE_PARM_SUN_CITY_ID,
									
	FZ_LITE_PARM_SUN_LATITUDE,
												
	FZ_LITE_PARM_SUN_LONGITUDE,
												
	FZ_LITE_PARM_SUN_TIMEZONE,
									
	FZ_LITE_PARM_SUN_DLST,
									
	FZ_LITE_PARM_SUN_YEAR,
									
	FZ_LITE_PARM_SUN_MONTH,
									
	FZ_LITE_PARM_SUN_DAY,
									
									
	FZ_LITE_PARM_SUN_HOUR,
									
	FZ_LITE_PARM_SUN_MINUTE,
									
	FZ_LITE_PARM_SUN_SECOND,
									
	FZ_LITE_PARM_SUN_FROM_SOUTH_ON,
									
	FZ_LITE_PARM_SUN_ALTITUDE,
									
	FZ_LITE_PARM_SUN_AZIMUTH,
	
	FZ_LITE_PARAM_SUN_MAX
} fz_lite_parm_sun_enum;

/****************************************************************

	fz_lite_parm_sky_intens_enum


	Description
		Distant light's atmospheric light cloud cover type. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COVER 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_COVER 
		in the functions fz_lite_get_parm_dist and fz_lite_set_parm_dist.

	Members
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_OVERCAST
			Sky of atmospheric light is overcast
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_MIXED
			Sky of atmospheric light has some clouds
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_CLEAR
			Sky of atmospheric light is has no clouds

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_parm_sky_intens_enum
{	FZ_LITE_DIST_ACCURATE_INTENS_SKY_OVERCAST = 0,
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_MIXED,
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_CLEAR,
	
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_COVER_MAX
} fz_lite_parm_sky_intens_enum;

/****************************************************************

	fz_lite_parm_sky_color_enum


	Description
		Distant light's atmospheric light color. 
		This enum is used in the parameter FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COLOR 
		in the functions fz_lite_grup_get_parm and fz_lite_grup_set_parm 
		and in the parameter FZ_LITE_PARM_DIST_ACCURATE_INTENS_SKY_COLOR 
		in the functions fz_lite_get_parm_dist and fz_lite_set_parm_dist.

	Members
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_LIGHTBLUE
			Atmospheric light's sky color is light blue
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_MEDIUMBLUE
			Atmospheric light's sky color is medium blue
		FZ_LITE_DIST_ACCURATE_INTENS_SKY_DARKBLUE
			Atmospheric light's sky color is dark blue

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_parm_sky_color_enum
{	FZ_LITE_DIST_ACCURATE_INTENS_SKY_LIGHTBLUE = 0,
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_MEDIUMBLUE,
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_DARKBLUE,
	
	FZ_LITE_DIST_ACCURATE_INTENS_SKY_COLOR_MAX
} fz_lite_parm_sky_color_enum;


/****************************************************************

	fz_lite_parm_pnt_enum


	Description
		Parameter identifiers for a point light. 
		Used for the "which" argument of fz_lite_get_parm_point and fz_lite_set_parm_point.

	Members
		FZ_LITE_PARM_POINT_RADIUS
			Point light radius.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_point_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_pnt_enum
{
	FZ_LITE_PARM_POINT_RADIUS = 0,
	
	FZ_LITE_PARAM_POINT_MAX
} fz_lite_parm_pnt_enum;

/****************************************************************

	fz_lite_parm_cone_enum


	Description
		Parameter identifiers for a cone light. 
		Used for the "which" argument of fz_lite_get_parm_cone and fz_lite_set_parm_cone.

	Members
		FZ_LITE_PARM_CONE_OUTER_ANGLE
			Cone light's outer angle. Must be larger than 
			the inner angle, but not larger than twice the inner angle.
			Type: double
			Default Value: PI/3.0 (60 degrees)
			Range: 0.0
		FZ_LITE_PARM_CONE_INNER_ANGLE
			Cone light's inner angle. Must be less than 
			the outer angle, but not less than half the outer angle.
			Type: double
			Default Value: PI/4.0 (45 degrees)
			Range: 0.0

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_cone_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_cone_enum
{
	FZ_LITE_PARM_CONE_OUTER_ANGLE = 0,

	FZ_LITE_PARM_CONE_INNER_ANGLE,
	
	FZ_LITE_PARAM_CONE_MAX
} fz_lite_parm_cone_enum;

/****************************************************************

	fz_lite_parm_pjtr_enum


	Description
		Parameter identifiers for a projector light. 
		Used for the "which" argument of fz_lite_get_parm_projector and fz_lite_set_parm_projector.

	Members
		FZ_LITE_PARM_PROJECTOR_ANGLE
			Projector light angle.
			Type: double
			Default Value: PI/3.0 (60 degrees)
			Range: 0.0
		FZ_LITE_PARM_PROJECTOR_SPIN
			Projector light spin (rotation around the light's direction).
			Type: double
			Default Value: 0.0
		FZ_LITE_PARM_PROJECTOR_TMAP
			Projector light's texture map.
			Type: fz_tmap_ptr

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_projector_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_pjtr_enum
{
	FZ_LITE_PARM_PROJECTOR_ANGLE = 0,

	FZ_LITE_PARM_PROJECTOR_SPIN,
												
	FZ_LITE_PARM_PROJECTOR_TMAP,
	
	FZ_LITE_PARAM_PROJECTOR_MAX
} fz_lite_parm_pjtr_enum;

/****************************************************************

	fz_lite_parm_area_enum


	Description
		Parameter identifiers for an area light. 
		Used for the "which" argument of fz_lite_get_parm_area and fz_lite_set_parm_area.

	Members
		FZ_LITE_PARM_AREA_RADIUS
			Area light's radius.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_PARM_AREA_RNDR_OBJ
			Render the area lights object on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_AREA_IGNORE_OBJ_COLOR
			Ignore the area light's object color on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_AREA_QUALITY
			Area light illumination and shadow quality.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_AREA_OBJECT
			Area light object.
			Type: fz_objt_ptr

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_area_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_area_enum
{
	FZ_LITE_PARM_AREA_RADIUS = 0,
												
	FZ_LITE_PARM_AREA_RNDR_OBJ,

	FZ_LITE_PARM_AREA_IGNORE_OBJ_COLOR,
												
	FZ_LITE_PARM_AREA_QUALITY,
	
	FZ_LITE_PARM_AREA_OBJECT,
	
	FZ_LITE_PARAM_AREA_MAX
} fz_lite_parm_area_enum;

/****************************************************************

	fz_lite_parm_line_enum


	Description
		Parameter identifiers for a line light. 
		Used for the "which" argument of fz_lite_get_parm_line and fz_lite_set_parm_line.

	Members
		FZ_LITE_PARM_LINE_RADIUS
			Line light's radius.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_PARM_LINE_QUALITY
			Line light illumination and shadow quality.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_PARM_LINE_OBJECT
			Line light object.
			Type: fz_objt_ptr

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_line_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_line_enum
{
	FZ_LITE_PARM_LINE_RADIUS = 0,

	FZ_LITE_PARM_LINE_QUALITY,
												
	FZ_LITE_PARM_LINE_OBJECT,
	
	FZ_LITE_PARAM_LINE_MAX
} fz_lite_parm_line_enum;

/****************************************************************

	fz_lite_parm_cust_enum


	Description
		Parameter identifiers for a custom light. 
		Used for the "which" argument of fz_lite_get_parm_custom and fz_lite_set_parm_custom.

	Members
		FZ_LITE_PARM_CUSTOM_RADIUS
			Custom light's radius.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_PARM_CUSTOM_SPIN
			Custom light spin.
			Type: double
			Default Value: 0.0
		FZ_LITE_PARM_CUSTOM_DIST_DATA
			Custom light intensity distribution data.
			Type: fz_lite_custom_data_ptr

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_custom_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_cust_enum
{
	FZ_LITE_PARM_CUSTOM_RADIUS = 0,

	FZ_LITE_PARM_CUSTOM_SPIN,
												
	FZ_LITE_PARM_CUSTOM_DIST_DATA,
	
	FZ_LITE_PARAM_CUSTOM_MAX
} fz_lite_parm_cust_enum;

/****************************************************************

	fz_lite_parm_env_enum


	Description
		Parameter identifiers for an environment light. 
		Used for the "which" argument of fz_lite_get_parm_environment and fz_lite_get_parm_environment.

	Members
		FZ_LITE_PARM_ENV_MAP_TYPE
			How to environment map.
			Type: fz_rndr_rzne_envr_type_enum
			Default Value: FZ_RNDR_RZNE_ENVR_TYPE_CUBIC
		FZ_LITE_PARM_ENV_FINITE
			is the environment considered finite (TRUE) or infinite (FALSE).
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_ENV_FINIT_CENTER
			The center of the environment, if it is set to be finite.
			Type: fz_xyz_td
			Default Value: {0,0,0}
		FZ_LITE_PARM_ENV_FINIT_SIZE
			The size of the environment, if it is set to be finite.
			Type: double
			Default Value: 10,000 feet
		FZ_LITE_PARM_ENV_CUBIC_SINGLE_MAP
			TRUE if using a single (cross) cubic map. FALSE if using six individual maps.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_PARM_ENV_CUBIC_MAP_CROSS
			Environment light's cubic cross map.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_TOP
			Environment light's cubic map at the top.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_BOTTOM
			Environment light's cubic map at the bottom.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_LEFT
			Environment light's cubic map at the left.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_RIGHT
			Environment light's cubic map at the right.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_FRONT
			Environment light's cubic map at the front.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_CUBIC_MAP_BACK
			Environment light's cubic map at the back.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_SPHERIC_MAP
			Environment light's spherical map.
			Type: fz_tmap_ptr
		FZ_LITE_PARM_ENV_SPHERIC_LAYOUT_TYPE
			The layout type of the spherical environment map.
			Type: fz_rndr_rzne_envr_sphr_layout_enum
			Default Value: FZ_RNDR_RZNE_ENVR_SPHR_LAYOUT_SPHERIC
		FZ_LITE_PARM_ENV_NUM_SAMPLES
			The number of samples to be used for the environment light.
			Type: fzrt_int
			Range: 3
			Default Value: 10
		FZ_LITE_PARM_ENV_SOFTNESS
			The softness parameter to be used for the environment light.
			Type: double
			Range: 0.0
			Default Value: 0.5
		FZ_LITE_PARM_ENV_COLOR_FACTOR
			The color factor parameter to be used for the environment light.
			Type: double
			Range: 0.0
			Default Value: 0.5

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_environment_obsolete

 ****************************************************************/
typedef enum fz_lite_parm_env_enum
{
	FZ_LITE_PARM_ENV_MAP_TYPE = 0,

	FZ_LITE_PARM_ENV_FINITE,
												
	FZ_LITE_PARM_ENV_FINIT_CENTER,
												
	FZ_LITE_PARM_ENV_FINIT_SIZE,
												
	FZ_LITE_PARM_ENV_CUBIC_SINGLE_MAP,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_CROSS,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_TOP,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_BOTTOM,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_LEFT,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_RIGHT,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_FRONT,
												
	FZ_LITE_PARM_ENV_CUBIC_MAP_BACK,
												
	FZ_LITE_PARM_ENV_SPHERIC_MAP,
	
	FZ_LITE_PARM_ENV_SPHERIC_LAYOUT_TYPE,
	
	FZ_LITE_PARM_ENV_NUM_SAMPLES,
	
	FZ_LITE_PARM_ENV_SOFTNESS,
	
	FZ_LITE_PARM_ENV_COLOR_FACTOR,

	FZ_LITE_PARM_ENV_MAX

} fz_lite_parm_env_enum;


/****************************************************************

	fz_light_env_ptr


	Description
		Data structure which represents the common environment paramters 
		for a light. The individualt fields of this data structure can 
		be accessed vua fz_lite_get_env_parm and fz_lite_set_env_parm. 
		The fields are defined by the enum fz_lite_env_parm_enum.

	Availability	7.0.0.0


	See Also
		FZRT_SPEC	*fz_lite_get_env_parm_func, FZRT_SPEC	*fz_lite_set_env_parm_func

 ****************************************************************/
typedef struct 	expr_env_td 	*fz_light_env_ptr;

/****************************************************************

	fz_light_env_type_enum


	Description
		Identifier for the map type of an environment light.

	Members
		FZ_LIGHT_ENV_TYPE_CUBIC
			Cubic environment mapping
		FZ_LIGHT_ENV_TYPE_SPHERICAL
			Spherical environment mapping

	Availability	7.0.0.0

 ****************************************************************/
typedef enum fz_light_env_type_enum
{
	FZ_LIGHT_ENV_TYPE_CUBIC = 0,
	FZ_LIGHT_ENV_TYPE_SPHERICAL,
	FZ_LIGHT_ENV_TYPE_MAX
}fz_light_env_type_enum;

/****************************************************************

	fz_light_env_sphr_layout_enum


	Description
		Parameter identifiers for a Spherical Environment map layout type.

	Members
		FZ_LIGHT_ENV_SPHR_LAYOUT_PANORAMIC
			Panoramic layout
		FZ_LIGHT_ENV_SPHR_LAYOUT_SPHERIC
			Spheric layout

	Availability	7.0.0.0


	See Also
		fz_lite_env_parm_enum

 ****************************************************************/
typedef enum fz_light_env_sphr_layout_enum
{
	FZ_LIGHT_ENV_SPHR_LAYOUT_PANORAMIC = 0,
	FZ_LIGHT_ENV_SPHR_LAYOUT_SPHERIC,
	
	FZ_LIGHT_ENV_SPHR_LAYOUT_MAX

} fz_light_env_sphr_layout_enum;

/****************************************************************

	fz_lite_env_parm_enum


	Description
		Identifiers for the common environment paramters of a light. 
		Used for the "which" argument of fz_lite_get_env_parm and fz_lite_set_env_parm.

	Members
		FZ_LITE_ENV_PARM_MAP_TYPE
			How to environment map.
			Type: fz_light_env_type_enum
			Default Value: FZ_LIGHT_ENV_TYPE_CUBIC
		FZ_LITE_ENV_PARM_CUBIC_SINGLE_MAP
			TRUE if using a single (cross) cubic map. FALSE if using six individual maps.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_ENV_PARM_CUBIC_MAP_CROSS
			Environment light's cubic cross map.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_TOP
			Environment light's cubic map at the top.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_BOTTOM
			Environment light's cubic map at the bottom.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_LEFT
			Environment light's cubic map at the left.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_RIGHT
			Environment light's cubic map at the right.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_FRONT
			Environment light's cubic map at the front.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_CUBIC_MAP_BACK
			Environment light's cubic map at the back.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_SPHERIC_MAP
			Environment light's spherical map.
			Type: fz_tmap_ptr
		FZ_LITE_ENV_PARM_SPHERIC_LAYOUT_TYPE
			The layout type of the spherical environment map.
			Type: fz_light_env_sphr_layout_enum
			Default Value: FZ_LIGHT_ENV_SPHR_LAYOUT_PANORAMIC

	Availability	7.0.0.0


	See Also
		FZRT_SPEC	*fz_lite_get_env_parm_func, FZRT_SPEC	*fz_lite_set_env_parm_func

 ****************************************************************/
typedef enum fz_lite_env_parm_enum
{
	FZ_LITE_ENV_PARM_MAP_TYPE = 0,

	FZ_LITE_ENV_PARM_CUBIC_SINGLE_MAP,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_CROSS,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_TOP,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_BOTTOM,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_LEFT,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_RIGHT,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_FRONT,
												
	FZ_LITE_ENV_PARM_CUBIC_MAP_BACK,
												
	FZ_LITE_ENV_PARM_SPHERIC_MAP,
	
	FZ_LITE_ENV_PARM_SPHERIC_LAYOUT_TYPE,
	
	FZ_LITE_ENV_PARM_MAX

} fz_lite_env_parm_enum;


/****************************************************************

	fz_lite_parm_cust_data_enum


	Description
		Parameter identifiers for a custom light data. 
		Used for the "which" argument of fz_lite_get_parm_custom_data and fz_lite_set_parm_custom_data.

	Members
		FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY
			Custom light data, distribution symmetry.
			Type: fz_lite_parm_cust_data_symmetry_enum
			Default Value: FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_AXIAL
		FZ_LITE_PARM_CUSTOM_DATA_HEMISPHERE
			Custom light data, distribution hemisphere.
			Type: fz_lite_parm_cust_data_hemi_enum
			Default Value: FZ_LITE_PARM_CUSTOM_DATA_HEMI_BOTH
		FZ_LITE_PARM_CUSTOM_DATA_NUMVERT
			Custom light data, number of vertical angles.
			Type: fzrt_int
			Default Value: 2
			Range: 2
		FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ
			Custom light data, number of horizontal distribution curves.
			Type: fzrt_int
			Default Value: 1
			Range: 1
		FZ_LITE_PARM_CUSTOM_DATA_VERTANG
			Custom light data, array of verical angles. The angles must be 
			in ascending order an be in the range of 0.0 <= angle <= PI.
			Type: double *
			Default Value: { 0.0, PI }
			Range: 0.0
		FZ_LITE_PARM_CUSTOM_DATA_HORIZANG
			Custom light data, array of horizontal angles. The angles must be 
			in ascending order an be in the range of 0.0 <= angle <= 2PI.
			Type: double *
			Default Value: { 0.0 }
			Range: 0.0
		FZ_LITE_PARM_CUSTOM_DATA_VALUES
			Custom light data, array of intensity values. There are 
			FZ_LITE_PARM_CUSTOM_DATA_NUMVERT * FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ 
			values in the array and the values are ordered in vertical major 
			order: 
			values[0] = value at vertical angle 0 , horizontal angle 0 
			values[1] = value at vertical angle 1 , horizontal angle 0 
			values[2] = value at vertical angle 2 , horizontal angle 0 
			. 
			. 
			. 
			values[numvert-1] = value at vertical angle numvert-1 , horizontal angle 0 
			 
			values[numvert * 1 + 0] = value at vertical angle 0 , horizontal angle 1 
			values[numvert * 1 + 1] = value at vertical angle 1 , horizontal angle 1 
			values[numvert * 1 + 2] = value at vertical angle 2 , horizontal angle 1 
			. 
			. 
			. 
			values[numvert * 1 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 1 
			 
			values[numvert * 2 + 0] = value at vertical angle 0 , horizontal angle 2 
			values[numvert * 2 + 1] = value at vertical angle 1 , horizontal angle 2 
			values[numvert * 2 + 2] = value at vertical angle 2 , horizontal angle 2 
			. 
			. 
			. 
			values[numvert * 2 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 2 
			 
			values[numvert * (numhorz-1) + 0] = value at vertical angle 0 , horizontal angle numhorz-1 
			values[numvert * (numhorz-1) + 1] = value at vertical angle 1 , horizontal angle numhorz-1 
			values[numvert * (numhorz-1) + 2] = value at vertical angle 2 , horizontal angle numhorz-1 
			. 
			. 
			. 
			values[numvert * (numhorz-1) + numvert-1] = value at vertical angle numvert-1 , horizontal angle numhorz-1
			Type: double *
			Default Value: { 0.0, 1.0 }
			Range: 0.0

	Availability	5.0.0.0


	See Also
		fz_lite_set_parm_custom_data

 ****************************************************************/
typedef enum fz_lite_parm_cust_data_enum
{	FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY = 0,
												
	FZ_LITE_PARM_CUSTOM_DATA_HEMISPHERE,
												
	FZ_LITE_PARM_CUSTOM_DATA_NUMVERT,
												
	FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ,
												
												
	FZ_LITE_PARM_CUSTOM_DATA_VERTANG,
												
	FZ_LITE_PARM_CUSTOM_DATA_HORIZANG,
												
	FZ_LITE_PARM_CUSTOM_DATA_VALUES,
	
	FZ_LITE_PARAM_CUSTOM_DATA_MAX
} fz_lite_parm_cust_data_enum;

/****************************************************************

	fz_lite_parm_cust_data_symmetry_enum


	Description
		Custom light symmetry type. 
		This enum is used in the parameter FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY 
		in the functions fz_lite_get_parm_custom_data and fz_lite_set_parm_custom_data.

	Members
		FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_NONE
			Custom light data, no symmetry
		FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_AXIAL
			Custom light data, axial symmetry
		FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_QUAD
			Custom light data, quadrant symmetry
		FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_PLANE
			Custom light data, plane symmetry

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_parm_cust_data_symmetry_enum
{	FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_NONE = 0,
	FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_AXIAL,
	FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_QUAD,
	FZ_LITE_PARM_CUSTOM_DATA_SYMMETRY_PLANE,
	
	FZ_LITE_PARAM_CUSTOM_DATA_SYMMETRY_MAX
} fz_lite_parm_cust_data_symmetry_enum;

/****************************************************************

	fz_lite_parm_cust_data_hemi_enum


	Description
		Custom light hemisphere type. 
		This enum is used in the parameter FZ_LITE_PARM_CUSTOM_DATA_HEMISPHERE 
		in the functions fz_lite_get_parm_custom_data and fz_lite_set_parm_custom_data.

	Members
		FZ_LITE_PARM_CUSTOM_DATA_HEMI_BOTH
			Custom light data, top and bottom hemispheres
		FZ_LITE_PARM_CUSTOM_DATA_HEMI_BOTTOM
			Custom light data, bottom only hemisphere
		FZ_LITE_PARM_CUSTOM_DATA_HEMI_TOP
			Custom light data, top only hemisphere

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_parm_cust_data_hemi_enum
{	FZ_LITE_PARM_CUSTOM_DATA_HEMI_BOTH = 0,
	FZ_LITE_PARM_CUSTOM_DATA_HEMI_BOTTOM,
	FZ_LITE_PARM_CUSTOM_DATA_HEMI_TOP,
	
	FZ_LITE_PARAM_CUSTOM_DATA_HEMI_MAX
} fz_lite_parm_cust_data_hemi_enum;

/****************************************************************

	fz_lite_parm_grup_enum


	Description
		Parameter identifiers for a light group. 
		Used for the "which" argument of fz_lite_grup_get_parm and fz_lite_grup_set_parm. When 
		getting the FZ_LITE_GROUP_PARM_VISIBILITY, FZ_LITE_GROUP_PARM_LOCKED_ON, FZ_LITE_GROUP_PARM_SHINING_ON 
		or FZ_LITE_GROUP_PARM_SHADOWS_ON parameters through fz_lite_grup_get_parm, the value may be -1 
		if the status of the respective attribute is different between the individual lights 
		contained in the group.

	Members
		FZ_LITE_GROUP_PARM_NAME
			Light group name (32 characters max).
			Type: char*
		FZ_LITE_GROUP_PARM_VISIBILITY
			Light group visibility status.
			Type: fz_lite_visible_enum or -1 when getting this parameter
			Default Value: FZ_LITE_VISIBILITY_INVISIBLE
		FZ_LITE_GROUP_PARM_LOCKED_ON
			Light group locked on status.
			Type: fzrt_int, TRUE/FALSE or -1 when getting this parameter
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_SHINING_ON
			Light group shining on status.
			Type: fzrt_int, TRUE/FALSE or -1 when getting this parameter
			Default Value: TRUE
		FZ_LITE_GROUP_PARM_SHADOWS_ON
			Light group shadows on status.
			Type: fzrt_int, TRUE/FALSE or -1 when getting this parameter
			Default Value: TRUE
		FZ_LITE_GROUP_PARM_OVERRIDE_COLOR_ON
			Light group override color (on/off)
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_COLOR
			Light group override color
			Type: fz_rgb_float_td
			Default Value: white {1.0,1.0,1.0}
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_ON
			Light group override intensity (on/off)
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ON
			Light group override: replace intensity (on/off)
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_SIMPLE_ON_OBSOLETE
			Light group override: replace intensity by specifying a 
			new simple intensity value (on/off). 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_SIMPLE_FACTOR
			Light group override: replace intensity by specifying a 
			new simple intensity value.
			Type: double
			Default Value: 1.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_FALLOFF
			Light group override: replace falloff by specifying a 
			new falloff value
			Type: fz_lite_falloff_enum
			Default Value: FZ_LITE_FALLOFF_CONSTANT
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_RADIOS_ON_OBSOLETE
			Light group override: replace intensity by using a new 
			radiometric intensity value (on) or photometric intensity 
			value (off) 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_WATTS_OBSOLETE
			Light group override: replace intensity by using a new 
			radiometric intensity value in watts 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 100.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_EFFICIENCY_OBSOLETE
			Light group override: replace efficiency by using a new 
			efficiency value 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_COLTEMP_OBSOLETE
			Light group override: replace color temperature by using a new 
			color temperature value. Meaningful values are in the range of 1000 to 10000 
			degrees Kelvin, but larger values are allowed. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 2000
			Range: 1000
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_LUMENS_OBSOLETE
			Light group override: replace intensity by using a new 
			photometric intensity value in lumens 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 1000
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_ON_OBSOLETE
			Light group override: when replacing the intensity of distant lights 
			turn the atmospheric light on/off. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_QUALITY_OBSOLETE
			Light group override when replacing the intensity of distant lights 
			and the atmospheric light is used, this is the override quality value of 
			the atmospheric light. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_LUMINANCE_OBSOLETE
			Light group override: when replacing the intensity of distant lights 
			and the atmospheric light is used, this is the override luminance of 
			the atmospheric light. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 50
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COVER_OBSOLETE
			Light group override: when replacing the intensity of distant lights 
			and the atmospheric light is used, this is the override cloud coverage of 
			the atmospheric light. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_parm_sky_intens_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_OVERCAST
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COLOR_OBSOLETE
			Light group override: when replacing the intensity of distant lights 
			and the atmospheric light is used, this is the override sky color of 
			the atmospheric light. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_parm_sky_color_enum
			Default Value: FZ_LITE_DIST_ACCURATE_INTENS_SKY_LIGHTBLUE
		FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_SCALE_FACTOR
			Light group override: replace intensity by specifying a scale factor by which 
			all intensities of the lights in a group are multiplied.
			Type: double
			Default Value: 1.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_ON_OBSOLETE
			Light group override: turn shadow type override on/off 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_TYPE_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new shadow type. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_smap_type_enum
			Default Value: FZ_LITE_SHAD_TYPE_MAP
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_TRANSP_ON_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for transparent shadows (on/off). 
			Obsolete: 7.0.0.0. Now longer available. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_QUALITY_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map quality. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_smap_quality_enum
			Default Value: FZ_LITE_SHAD_MAP_QUALITY_MEDIUM
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_SOFTNESS_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map softness. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_TOLERANCE_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map tolerance. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_FIXED_RES_ON_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the fixed resolution on/off option. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_RES_IMAGESIZE_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map resolution, as a factor of 
			the image size. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 1.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_RES_FIXED_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map fixed resolution. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_int
			Default Value: 512
			Range: 64
		FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_LIMIT_OBSOLETE
			Light group override: when overriding the shadow type, this 
			is the new value for the shadow map limit option. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_smap_limit_enum
			Default Value: FZ_LITE_SHAD_MAP_LIMIT_ALL
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ON_OBSOLETE
			Light group override: turn glow override on/off 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_GLOW_ON_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the glow option (on/off). 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_ON_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow option (on/off). 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_INTENS_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the simple glow intensity. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_FALLOFF_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the simple glow falloff. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_glow_falloff_enum
			Default Value: FZ_LITE_GLOW_SIMPLE_FALLOF_SQUARE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_LIGHT_DIST_ON_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the simple glow by distance option (on/off). 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_DIST_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the simple glow distance. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_INTENS_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow intensity. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.5
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_ATTENUATION_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow attenuation. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 0.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_MAX_DIST_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow maximum distance. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: English 1200.0 in, Metric 5000.0 cm

			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_QUALITY_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow quality. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_int
			Default Value: 5
			Range: 1
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_ON_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise option (on/off). 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_SIZE_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise size. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: English 48.0 in, Metric 100.0 cm

			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_TYPE_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise type. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fz_lite_glow_noise_enum
			Default Value: FZ_LITE_GLOW_ACCURATE_NOISE_TYPE_BETTER
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_NIMPULSE_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise number of impulses. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_int
			Default Value: 3
			Range: 1
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_CONTRAST_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise contrast. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: double
			Default Value: 1.0
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_DETAIL_OBSOLETE
			Light group override: when overriding the glow option, this 
			is the new value for the accurate glow noise detail. 
			Obsolete: 7.0.0.0. Now longer available.
			Type: fzrt_int
			Default Value: 3
			Range: 1
		FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_ON
			Light group override: turn radius / angle override on/off.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_RADIUS
			when overriding the radius / angle parameters, this 
			is the new value for the radius.
			Type: double
			Default Value: English 120.0 in, Metric 500.0 cm

			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_OUTER_ANGLE
			when overriding the radius / angle parameters, this 
			is the new value for the outer angle of cone lights and 
			the angle of projector lights.
			Type: double
			Default Value: PI/3.0 (60 degrees)
			Range: 0.0
		FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_INNER_ANGLE
			when overriding the radius / angle parameters, this 
			is the new value for the inner angle of cone lights.
			Type: double
			Default Value: PI/4.0 (45 degrees)
			Range: 0.0

	Availability	5.0.0.0

 ****************************************************************/
typedef enum fz_lite_parm_grup_enum
{	FZ_LITE_GROUP_PARM_NAME = 0,
																	
	FZ_LITE_GROUP_PARM_VISIBILITY,
	
	FZ_LITE_GROUP_PARM_LOCKED_ON,


	FZ_LITE_GROUP_PARM_SHINING_ON,

	FZ_LITE_GROUP_PARM_SHADOWS_ON,

	FZ_LITE_GROUP_PARM_OVERRIDE_COLOR_ON,

	FZ_LITE_GROUP_PARM_OVERRIDE_COLOR,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_ON,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ON,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_SIMPLE_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_SIMPLE_FACTOR,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_FALLOFF,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_RADIOS_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_WATTS_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_EFFICIENCY_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_COLTEMP_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_LUMENS_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_QUALITY_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_LUMINANCE_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COVER_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_REPL_ACC_SKY_COLOR_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_INTENS_SCALE_FACTOR,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_TYPE_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_TRANSP_ON_OBSOLETE,
																	
	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_QUALITY_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_SOFTNESS_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_TOLERANCE_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_FIXED_RES_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_RES_IMAGESIZE_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_RES_FIXED_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_SHADTYPE_MAP_LIMIT_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_GLOW_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_INTENS_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_FALLOFF_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_LIGHT_DIST_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_SIMPLE_DIST_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_INTENS_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_ATTENUATION_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_MAX_DIST_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_QUALITY_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_ON_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_SIZE_OBSOLETE,
																	
	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_TYPE_OBSOLETE,
																	
	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_NIMPULSE_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_CONTRAST_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_GLOW_ACCURATE_NOISE_DETAIL_OBSOLETE,

	FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_ON,
																	
	FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_RADIUS,

	FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_OUTER_ANGLE,

	FZ_LITE_GROUP_PARM_OVERRIDE_RADANG_INNER_ANGLE,
	
	FZ_LITE_GROUP_PARAM_MAX
} fz_lite_parm_grup_enum;

/****************************************************************

	fz_lite_get_num_lights



	Description
		Gets the number of lights in a windex.

	Parameters
		windex  [Input]
			project window index
		num_lite  [Result]
			number of lights

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_num_lights_func)(
	fzrt_int				windex,
	fzrt_int				*num_lite
	);

/****************************************************************

	fz_lite_tag_to_ptr



	Description
		Gets the light pointer from the light tag.

	Parameters
		windex  [Input]
			project window index
		tag  [Input]
			light tag
		lite  [Result]
			light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_ptr_to_tag

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_tag_to_ptr_func)(
	fzrt_int				windex,
	fz_tag_td			*tag,
	fz_lite_ptr			*lite
	);

/****************************************************************

	fz_lite_ptr_to_tag



	Description
		Gets the light tag from the light pointer.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		tag  [Result]
			light tag

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_tag_to_ptr

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_ptr_to_tag_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_tag_td			*tag
	);

/****************************************************************

	fz_lite_get_next_light



	Description
		Given a light pointer, get the next light pointer in the windex. 
		If lite_in is passed as NULL, the first light in the windex is 
		returned.

	Parameters
		windex  [Input]
			project window index
		lite_in  [Input]
			light
		lite_next  [Result]
			next light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_prev_light

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_next_light_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite_in,
	fz_lite_ptr			*lite_next
	);

/****************************************************************

	fz_lite_get_prev_light



	Description
		Given a light pointer, get the previous light pointer in the windex. 
		If lite_in is passed as NULL, the last light in the windex is returned.

	Parameters
		windex  [Input]
			project window index
		lite_in  [Input]
			light
		lite_prev  [Result]
			previous light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_next_light

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_prev_light_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite_in,
	fz_lite_ptr			*lite_prev
	);

/****************************************************************

	fz_lite_create_new



	Description
		Creates a new light in the given windex.

	Parameters
		windex  [Input]
			project window index
		lite  [Result]
			light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_create_new_func)(
	fzrt_int				windex,
	fz_lite_ptr			*lite
	);

/****************************************************************

	fz_lite_duplicate



	Description
		Creates a duplicate light from a given light.

	Parameters
		windex  [Input]
			project window index
		lite_in  [Input]
			source light
		lite_dupl  [Result]
			new light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_copy_to_windex

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_duplicate_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite_in,
	fz_lite_ptr			*lite_dupl
	);

/****************************************************************

	fz_lite_copy_to_windex



	Description
		Creates a copy of a given light in a given windex. This api function 
		makes it possible to copy a light from one project to another, if 
		source and destiantion windex represent different projects. 
		If source and destiantion windex are from the same project, 
		fz_lite_copy_to_windex performs the same task as fz_lite_duplicate

	Parameters
		src_windex  [Input]
			project window index of source light
		src_lite  [Input]
			source light
		dst_windex  [Input]
			project window index of new light
		dst_lite  [Result]
			new light

	Returns
		Error codes


	Availability	6.5.5.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_duplicate

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_copy_to_windex_func)(
	fzrt_int				src_windex,
	fz_lite_ptr			src_lite,
	fzrt_int				dst_windex,
	fz_lite_ptr			*dst_lite
	);

/****************************************************************

	fz_lite_delete



	Description
		Deletes a light. 
		Note: the light cannot be deleted if it is the only light 
		in the windex or if the light is the sun light. 
		If the light is a symbol light, it also cannot be deleted. 
		The whole light group which is associated with the symbol 
		instance must be deleted via the call light_delete_lite_group.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light to delete

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_delete_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite
	);

/****************************************************************

	fz_lite_get_sun



	Description
		Gets the sun light.

	Parameters
		windex  [Input]
			project window index
		sun_lite  [Result]
			sun light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_sun

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_sun_func)(
	fzrt_int				windex,
	fz_lite_ptr			*sun_lite
	);

/****************************************************************

	fz_lite_set_sun



	Description
		Makes a given light the sun light. 
		Note: The light must be a direct light.

	Parameters
		windex  [Input]
			project window index
		sun_lite  [Input]
			light to make the sun light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_sun

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_sun_func)(
	fzrt_int				windex,
	fz_lite_ptr			sun_lite
	);


/****************************************************************

	fz_lite_get_parm_common_obsolete



	Description
		Gets one of the common parameters of a light. 
		Note: not all common parameters are used by all lights. 
		In this case the parameter is still returned, but has no 
		effect on the behavior of the light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override 
		parameter in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_common

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_common_func_obsolete)(
	fzrt_int					windex,
	fz_lite_ptr				lite,
	fz_lite_parm_common_enum	which,
	fzrt_boolean			display,
	fz_type_td				*data
	);

/****************************************************************

	fz_lite_set_parm_common_obsolete



	Description
		Sets one of the common parameters of a light. 
		Note: not all common parameters are used by all lights. 
		In this case the parameter is still set, 
		but has no effect on the behavior of the light. 
		When setting the light type to an area or line light, a default 
		area or line object is created. 
		The default area object is a 1x1x1 unit cube centered at the light origin. 
		The default line light is a 1 unit length line, centered at the 
		light origin and extending in the x direction.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_common

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_common_func_obsolete)(
	fzrt_int					windex,
	fz_lite_ptr				lite,
	fz_lite_parm_common_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_lite_get_parm_dist_obsolete



	Description
		Gets one of the unique parameters of a distant light.

		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's override 
		option is on for that parameter, the override parameter is returned.

		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			direct light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_dist

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_dist_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_dist_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_dist_obsolete



	Description
		Sets one of the unique parameters of a distant light. 
		Note: the sun pointer (FZ_LITE_PARM_DIST_SUN) can only be retrieved, 
		not set. 
		Parameters of the sun position of a distant light are set through 
		fz_lite_set_parm_sun.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			direct light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_dist,fz_lite_set_parm_sun

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_dist_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_dist_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_point_obsolete



	Description
		Gets one of the unique parameters of a point light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			point light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_point

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_point_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_pnt_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_point_obsolete



	Description
		Sets one of the unique parameters of a point light.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			point light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_point

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_point_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_pnt_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_cone_obsolete



	Description
		Gets one of the unique parameters of a cone light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			cone light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_cone

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_cone_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_cone_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_cone_obsolete



	Description
		Sets one of the unique parameters of a cone light.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			cone light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_cone

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_cone_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_cone_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_projector_obsolete



	Description
		Gets one of the unique parameters of a projector light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			projector light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_projector

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_projector_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_pjtr_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_projector_obsolete



	Description
		Sets one of the unique parameters of a projector light.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			projector light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_projector

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_projector_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_pjtr_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_environment_obsolete



	Description
		Gets one of the unique parameters of an environment light.

		If the display parameter is TRUE, the parameter value returned reflects 
		the light parameters as used when a scene is displayed. That is, if 
		the light is in a light group and the group's override option is on for 
		that parameter, the override parameter is returned.

		If the display parameter is FALSE, the true parameter of the light 
		is returned. Not all parameters of a light have an equivalent override 
		parameter in the light group. In that case, the true parameter of the 
		light is always returned, regardless of the display parameter.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			environment light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_environment

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_environment_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_env_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_environment_obsolete



	Description
		Sets one of the unique parameters of an environment light. Note that 
		the map parameters can not be set. The fz_lite_get_parm_environment 
		returns pointers to the various environment maps. There are texture 
		map API functions supplied, which set the parameters of a map, such 
		as a new file name.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			environment light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_environment

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_environment_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_env_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_area_obsolete



	Description
		Gets one of the unique parameters of an area light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter. 
		Note: the object pointer retrieved from the area light needs to be 
		handled with care. 
		The content of this object cannot be edited, the object cannot be deleted.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			area light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_area

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_area_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_area_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_area_obsolete



	Description
		Sets one of the unique parameters of an area light. 
		When setting the object of an area light, the object passed 
		as the parameter will be copied into the area light's 
		object storage. 
		The object passed in remains unchanged.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			area light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_area

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_area_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_area_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_line_obsolete



	Description
		Gets one of the unique parameters of a line light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter. 
		Note: the object pointer retrieved from the line light needs to be 
		handled with care. 
		The content of this object cannot be edited, the object cannot be deleted.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			line light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_line

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_line_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_line_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_line_obsolete



	Description
		Sets one of the unique parameters of a line light. 
		When setting the object of an line light, the object passed 
		as the parameter will be copied into the area light's object storage. 
		The object passed in remains unchanged.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			line light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_line

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_line_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_line_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_custom_obsolete



	Description
		Gets one of the unique parameters of a custom light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override parameter 
		in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter. 
		Note: the content of the custom light distribution data retrieved/set with 
		fz_lite_get_parm_custom_data/fz_lite_set_parm_custom_data.

		This function is now obsolete. Use fz_lite_get_parm and fz_lite_get_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			custom light
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_custom

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_custom_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_cust_enum	which,
	fzrt_boolean		display,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_custom_obsolete



	Description
		Sets one of the unique parameters of a custom light. 
		Note: the content of the custom light distribution 
		data retrieved/set with fz_lite_get_parm_custom_data / fz_lite_set_parm_custom_data. 
		The LITE_PARAM_CUSTOM_DATA parameter is therefore read only 
		and cannot be set with this function.

		This function is now obsolete. Use fz_lite_set_parm and fz_lite_set_plugin_parm instead.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			custom light
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_custom

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_custom_func_obsolete)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_parm_cust_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_get_parm_custom_data



	Description
		Gets a parameter of a custom light's distribution data. 
		When retrieving the FZ_LITE_PARM_CUSTOM_DATA_VERTANG parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which is large enough to hold FZ_LITE_PARM_CUSTOM_DATA_NUMVERT values. 
		It is the responsibility of the calling function to allocate this array. 
		 
		When retrieving the FZ_LITE_PARM_CUSTOM_DATA_HORIZANG parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which is large enough to hold FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ values. 
		It is the responsibility of the calling function to allocate this array. 
		 
		When retrieving the FZ_LITE_PARM_CUSTOM_DATA_VALUES parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which is large enough to hold 
		FZ_LITE_PARM_CUSTOM_DATA_NUMVERT * FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ values. 
		It is the responsibility of the calling function to allocate this array. 
		The values will be ordered in vertical major order: 
		 
		values[0] = value at vertical angle 0 , horizontal angle 0 
		values[1] = value at vertical angle 1 , horizontal angle 0 
		values[2] = value at vertical angle 2 , horizontal angle 0 
		. 
		. 
		. 
		values[numvert-1] = value at vertical angle numvert-1 , horizontal angle 0 
		 
		values[numvert * 1 + 0] = value at vertical angle 0 , horizontal angle 1 
		values[numvert * 1 + 1] = value at vertical angle 1 , horizontal angle 1 
		values[numvert * 1 + 2] = value at vertical angle 2 , horizontal angle 1 
		. 
		. 
		. 
		values[numvert * 1 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 1 
		 
		values[numvert * 2 + 0] = value at vertical angle 0 , horizontal angle 2 
		values[numvert * 2 + 1] = value at vertical angle 1 , horizontal angle 2 
		values[numvert * 2 + 2] = value at vertical angle 2 , horizontal angle 2 
		. 
		. 
		. 
		values[numvert * 2 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 2 
		 
		values[numvert * (numhorz-1) + 0] = value at vertical angle 0 , horizontal angle numhorz-1 
		values[numvert * (numhorz-1) + 1] = value at vertical angle 1 , horizontal angle numhorz-1 
		values[numvert * (numhorz-1) + 2] = value at vertical angle 2 , horizontal angle numhorz-1 
		. 
		. 
		. 
		values[numvert * (numhorz-1) + numvert-1] = value at vertical angle numvert-1 , horizontal angle numhorz-1

	Parameters
		windex  [Input]
			project window index
		cust_data  [Input]
			edit options
		which  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_custom_data

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_custom_data_func)(
	fzrt_int						windex,
	fz_lite_custom_data_ptr		cust_data,
	fz_lite_parm_cust_data_enum	which,
	fz_type_td					*data
	);

/****************************************************************

	fz_lite_set_parm_custom_data



	Description
		Sets a parameter of a custom light's distribution data. 
		When setting the FZ_LITE_PARM_CUSTOM_DATA_VERTANG parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which has FZ_LITE_PARM_CUSTOM_DATA_NUMVERT values. 
		 
		When setting the FZ_LITE_PARM_CUSTOM_DATA_HORIZANG parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which has FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ values. 
		 
		When setting the FZ_LITE_PARM_CUSTOM_DATA_VALUES parameter, 
		the data argument must contain a pointer to an array of doubles, 
		which has FZ_LITE_PARM_CUSTOM_DATA_NUMVERT * FZ_LITE_PARM_CUSTOM_DATA_NUMHORIZ 
		values. 
		The values must be ordered in vertical major order: 
		 
		values[0] = value at vertical angle 0 , horizontal angle 0 
		values[1] = value at vertical angle 1 , horizontal angle 0 
		values[2] = value at vertical angle 2 , horizontal angle 0 
		. 
		. 
		. 
		values[numvert-1] = value at vertical angle numvert-1 , horizontal angle 0 
		 
		values[numvert * 1 + 0] = value at vertical angle 0 , horizontal angle 1 
		values[numvert * 1 + 1] = value at vertical angle 1 , horizontal angle 1 
		values[numvert * 1 + 2] = value at vertical angle 2 , horizontal angle 1 
		. 
		. 
		. 
		values[numvert * 1 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 1 
		 
		values[numvert * 2 + 0] = value at vertical angle 0 , horizontal angle 2 
		values[numvert * 2 + 1] = value at vertical angle 1 , horizontal angle 2 
		values[numvert * 2 + 2] = value at vertical angle 2 , horizontal angle 2 
		. 
		. 
		. 
		values[numvert * 2 + numvert-1] = value at vertical angle numvert-1 , horizontal angle 2 
		 
		values[numvert * (numhorz-1) + 0] = value at vertical angle 0 , horizontal angle numhorz-1 
		values[numvert * (numhorz-1) + 1] = value at vertical angle 1 , horizontal angle numhorz-1 
		values[numvert * (numhorz-1) + 2] = value at vertical angle 2 , horizontal angle numhorz-1 
		. 
		. 
		. 
		values[numvert * (numhorz-1) + numvert-1] = value at vertical angle numvert-1 , horizontal angle numhorz-1

	Parameters
		windex  [Input]
			project window index
		cust_data  [Input]
			edit options
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_custom_data

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_custom_data_func)(
	fzrt_int						windex,
	fz_lite_custom_data_ptr		cust_data,
	fz_lite_parm_cust_data_enum	which,
	fz_type_td					*data
	);

/****************************************************************

	fz_lite_is_symb_light



	Description
		Check whether a light is part of a symbol light group. 
		Remember that those lights are read only (i.e. their parameters 
		cannot be changed and they cannot be deleted individually).

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		is_symb_light  [Result]
			is light part of a symbol light group

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_is_symb_light_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fzrt_boolean		*is_symb_light
	);

/****************************************************************

	fz_lite_get_ambient_intensity



	Description
		Gets the intensity of the ambient light in a windex.

	Parameters
		windex  [Input]
			project window index
		intens  [Result]
			ambient light intensity

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_ambient_intensity

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_ambient_intensity_func)(
	fzrt_int				windex,
	double				*intens
	);

/****************************************************************

	fz_lite_set_ambient_intensity



	Description
		Sets the intensity of the ambient light in a windex.

	Parameters
		windex  [Input]
			project window index
		intens  [Input]
			ambient light intensity
			Range: 0.0

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_ambient_intensity

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_ambient_intensity_func)(
	fzrt_int				windex,
	double				intens
	);


/****************************************************************

	fz_lite_get_ambient_color



	Description
		Gets the color of the ambient light in a windex.

	Parameters
		windex  [Input]
			project window index
		color  [Result]
			ambient light color

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_ambient_color

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_ambient_color_func) (
	fzrt_int				windex,
	fz_rgb_float_td		*color
	);

/****************************************************************

	fz_lite_set_ambient_color



	Description
		Sets the color of the ambient light in a windex.

	Parameters
		windex  [Input]
			project window index
		color  [Input]
			ambient light color

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_ambient_color

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_ambient_color_func) (
	fzrt_int				windex,
	fz_rgb_float_td		*color
	);


/****************************************************************

	fz_lite_transform



	Description
		Transforms a light by the given matrix. 
		Lights which belong to a symbol light group cannot be 
		transformed with this function. 
		Transform the owning symbol instance to achieve 
		this (fz_objt_edit_transform_objt).

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		mat  [Input]
			matrix to transform light by

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_transform_func) (
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_mat4x4_td		*mat
	);

/****************************************************************

	fz_lite_get_grup_table



	Description
		Gets the light group table of a windex.

	Parameters
		windex  [Input]
			project window index
		gtable  [Result]
			light group table

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_grup_table_func) (
	fzrt_int				windex,
	fz_grup_table_ptr		*gtable
	);

/****************************************************************

	fz_lite_get_grup_tag



	Description
		Gets a light's group tag. 
		If the tag returned is -1, then the light is not contained 
		in a group, but is located at the top (root) level.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		group_tag  [Result]
			light's group tag

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_grup_tag_func) (
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_tag_td			*group_tag
	);



/****************************************************************

	fz_lite_grup_get_parm



	Description
		Gets a light group's parameter. 
		For the parameters 
		FZ_LITE_GROUP_PARM_VISIBILITY 
		FZ_LITE_GROUP_PARM_LOCKED_ON 
		FZ_LITE_GROUP_PARM_SHINING_ON 
		FZ_LITE_GROUP_PARM_SHADOWS_ON 
		if the return value is -1, the status of the lights in the group is not the same. 
		For example, if there are 3 lights in a group with two lights which have 
		visibility on and one light with visibility off, the FZ_LITE_GROUP_PARM_VISIBILITY 
		parameter will be returned as -1.

	Parameters
		windex  [Input]
			project window index
		group_tag  [Input]
			light group tag
		which  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_grup_set_parm

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_grup_get_parm_func) (
	fzrt_int					windex,
	fz_tag_td				*group_tag,
	fz_lite_parm_grup_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_lite_grup_set_parm



	Description
		Sets a light group's parameter.

	Parameters
		windex  [Input]
			project window index
		group_tag  [Input]
			light group tag
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_grup_get_parm

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_grup_set_parm_func) (
	fzrt_int					windex,
	fz_tag_td				*group_tag,
	fz_lite_parm_grup_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_lite_dist_set_dir_from_sun



	Description
		Sets a distant light's direction from its sun parameters. 
		Note: the sun parameters are set with fz_lite_set_parm_sun.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_dist_set_dir_from_sun_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite
	);

/****************************************************************

	fz_lite_get_parm_sun



	Description
		Gets one of the sun parameters.

	Parameters
		windex  [Input]
			project window index
		sun  [Input]
			sun
		which  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_parm_sun

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_parm_sun_func) (
	fzrt_int				windex,
	fz_lite_sun_ptr				sun,
	fz_lite_parm_sun_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_set_parm_sun



	Description
		Sets one of the sun parameters.

	Parameters
		windex  [Input]
			project window index
		sun  [Input]
			sun
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_parm_sun

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_set_parm_sun_func) (
	fzrt_int				windex,
	fz_lite_sun_ptr				sun,
	fz_lite_parm_sun_enum	which,
	fz_type_td			*data
	);

/****************************************************************

	fz_lite_sun_city_id_from_name



	Description
		Get the city id from the city name and country name.

	Parameters
		windex  [Input]
			project window index
		city_name  [Input]
			city name
		country_name  [Input]
			country name
		city_id  [Result]
			city id

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_name_from_id

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_id_from_name_func) (
	fzrt_int				windex,
	char				*city_name,
	char				*country_name,
	fzrt_int				*city_id
	);

/****************************************************************

	fz_lite_sun_city_id_from_indx



	Description
		Gets the id of the nth city in the geography database. The number 
		of cities can be retrieved via fz_lite_sun_get_num_cities.

	Parameters
		windex  [Input]
			project window index
		indx  [Input]
			the index of the nth city
		city_id  [Result]
			city id

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_get_num_cities

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_id_from_indx_func) (
	fzrt_int				windex,
	fzrt_int				indx,
	fzrt_int				*city_id
	);

/****************************************************************

	fz_lite_sun_get_num_cities



	Description
		Gets the number of cities in the geography database.

	Parameters
		windex  [Input]
			project window index
		n_cities  [Result]
			the number of cities

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_id_from_indx

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_get_num_cities_func) (
	fzrt_int				windex,
	fzrt_int				*n_cities
	);


/****************************************************************

	fz_lite_sun_city_name_from_id



	Description
		Get the city name and country name from the city id.

	Parameters
		windex  [Input]
			project window index
		city_id  [Input]
			city id
		city_name  [Result, Optional]
			city name
		country_name  [Result, Optional]
			country name

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_id_from_name

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_name_from_id_func) (
	fzrt_int				windex,
	fzrt_int				city_id,
	char				*city_name,
	char				*country_name
	);

/****************************************************************

	fz_lite_sun_city_add



	Description
		Add a new city to the geography database.

	Parameters
		windex  [Input]
			project window index
		city_name  [Input]
			city name
		country_name  [Input]
			country name
		latitude  [Input]
			latitude
			Range: -PI/2
		longitude  [Input]
			longitude
			Range: -PI
		time_zone  [Input]
			time zone
			Range: -12.0
		city_id  [Result, Optional]
			city id

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_delete,fz_lite_sun_country_add

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_add_func) (
	fzrt_int				windex,
	char				*city_name,
	char				*country_name,
	double				latitude,
	double				longitude,
	double				time_zone,
	fzrt_int				*city_id
	);

/****************************************************************

	fz_lite_sun_country_add



	Description
		Add a new country to the geography database.

	Parameters
		windex  [Input]
			project window index
		country_name  [Input]
			country name

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_add

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_country_add_func) (
	fzrt_int				windex,
	char				*country_name
	);

/****************************************************************

	fz_lite_sun_city_delete



	Description
		Deletes a city from the geography database.

	Parameters
		windex  [Input]
			project window index
		city_id  [Input]
			city id

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_add

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_delete_func) (
	fzrt_int				windex,
	fzrt_int				city_id
	);

/****************************************************************

	fz_lite_sun_city_set_parm



	Description
		Sets the parameters of a city.

	Parameters
		windex  [Input]
			project window index
		city_id  [Input]
			city id
		city_name  [Input, Optional]
			city name
		country_name  [Input, Optional]
			country name
		latitude  [Input]
			latitude
			Range: -PI/2
		longitude  [Input]
			longitude
			Range: -PI
		time_zone  [Input]
			time zone
			Range: -12.0

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_get_parm

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_set_parm_func) (
	fzrt_int				windex,
	fzrt_int				city_id,
	char				*city_name,
	char				*country_name,
	double				latitude,
	double				longitude,
	double				time_zone
	);

/****************************************************************

	fz_lite_sun_city_get_parm



	Description
		Gets the parameters of a city.

	Parameters
		windex  [Input]
			project window index
		city_id  [Input]
			city id
		city_name  [Result, Optional]
			city name
		country_name  [Result, Optional]
			country name
		latitude  [Result, Optional]
			latitude
		longitude  [Result, Optional]
			longitude
		time_zone  [Result, Optional]
			time zone

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_sun_city_set_parm

 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_sun_city_get_parm_func) (
	fzrt_int				windex,
	fzrt_int				city_id,
	char				*city_name,
	char				*country_name,
	double				*latitude,
	double				*longitude,
	double				*time_zone
	);

/****************************************************************

	fz_lite_get_symb



	Description
		Gets the symbol a symbol light belongs to.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		sym_ptr  [Input]
			symbol pointer

	Returns
		Error codes 
		MODEL_ERR_INVALID_LIGHT_TYPE - light is not a symbol light


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_symb_func)(
	fzrt_int				windex,
	fz_lite_ptr		lite,
	fz_symb_ins_ptr		*sym_ptr
	);

/****************************************************************

	fz_lite_opts_parm_enum


	Description
		Light setup parameters. 
		Used for the "which" argument of fz_lite_get_opts_parm_data and fz_lite_set_opts_parm_data.

	Members
		FZ_LITE_OPTS_SHOW_COLOR
			Indicates if the names of the lights will be drawn in the color that 
			was assigned to them in the lights palette. If not set, the names 
			are all drawn the same.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_OPTS_HIGHLIGHT_PICKED
			When set, the light name in the lights palette will be highlighted whenever 
			a light is picked.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_LITE_OPTS_LIGHT_NAME
			The default light name (32 characters max).
			Type: char*
		FZ_LITE_OPTS_GROUP_NAME
			The default light group name (32 characters max).
			Type: char*

	Availability	5.0.0.0


	See Also
		fz_lite_set_opts_parm_data

 ****************************************************************/
typedef enum fz_lite_opts_parm_enum
{	FZ_LITE_OPTS_SHOW_COLOR,

	FZ_LITE_OPTS_HIGHLIGHT_PICKED,

	FZ_LITE_OPTS_LIGHT_NAME,

	FZ_LITE_OPTS_GROUP_NAME,

	FZ_LITE_OPTS_MAX
} fz_lite_opts_parm_enum;

/****************************************************************

	fz_lite_get_opts_parm_data



	Description
		Gets light setup parameters.

	Parameters
		windex  [Input]
			project window index
		which  [Input]
			parameter identifier
		value  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_set_opts_parm_data

 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_get_opts_parm_data_func)(
	fzrt_int						windex,
	fz_lite_opts_parm_enum		which,
	fz_type_td					*value
	);

/****************************************************************

	fz_lite_set_opts_parm_data



	Description
		Sets light setup parameters.

	Parameters
		windex  [Input]
			project window index
		which  [Input]
			parameter identifier
		value  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl

	See Also
		fz_lite_get_opts_parm_data

 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_set_opts_parm_data_func)(
	fzrt_int						windex,
	fz_lite_opts_parm_enum		which,
	fz_type_td					*value
	);

/****************************************************************

	fz_lite_sort



	Description
		Sorts the entries in the lights palette list in alphabetical order. 
		Upper case characters precede the lower case.

	Parameters
		windex  [Input]
			project window index

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_sort_func)(
	fzrt_int						windex
	);

/****************************************************************

	fz_lite_clear



	Description
		Clears all lights from the list except the light being used as the sun. 
		The sun can't be removed.

	Parameters
		windex  [Input]
			project window index

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_clear_func)(
	fzrt_int						windex
	);


/****************************************************************

	fz_lite_get_pick_status



	Description
		Checks whether a light is picked. The picked parameter also 
		returns whether the coi or the eye or both are picked. For eye, bit 
		1 (0x00000001) is set, for coi bit 2 (0x00000002) is set. For light 
		types that don't have a specific light vector, such as area lights, 
		this parameter always has both bits set (0x00000001 | 0x00000002) if 
		the light is picked.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			the light for which to get the pick status
		picked  [Result]
			the bit encoded pick status of the light. 
			bit 1 = eye is picked, bit 2 = coi is picked. 
			No bits set = light is not picked

	Returns
		Error codes


	Availability	6.0.0.5

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_get_pick_status_func)(
	fzrt_int						windex,
	fz_lite_ptr					lite,
	fzrt_int						*picked
	);

/****************************************************************

	fz_lite_set_pick_status



	Description
		Sets the pick status of a light. If picked is passed as 0, the light 
		is unpicked. For lights with a direction vector, such as a cone light, 
		the eye and coi can be picked separately. If the 1 bit is set (0x00000001) 
		the eye is picked. If bit 2 is set (0x00000002) the coi is picked. 
		For ligths without a direction vector, such as area lights, no such 
		distinction is necessary and picked must be passed as 1 for all cases. 
		Note, that only bits 1 and 2 are checked. Any other bits are ignored.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			the light for which to set the pick status
		picked  [Input]
			the bit encoded pick status of the light. 
			bit 1 = set pick of eye, bit 2 = set pick of coi. 
			not bits set = set light to not picked

	Returns
		Error codes


	Availability	6.0.0.5

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_set_pick_status_func)(
	fzrt_int						windex,
	fz_lite_ptr					lite,
	fzrt_int						picked
	);

/****************************************************************

	fz_lite_pick



	Description
		Performs a pick action among the lights of a project, based on the screen point 
		passed in. This screen point is usually supplied by formZ in a click callback function 
		such as the click function of a tool plugin. 
		Note, that if a light was clicked on with the screen point, 
		the light may be picked or unpicked, based on the previous pick status of the light. 
		If no light was picked, lite will be passed back as NULL. If a valid 
		light was picked or unpicked it is passed back. The api function 
		fz_lite_get_pick_status can then be called to determine the pick status of 
		the light.

	Parameters
		windex  [Input]
			project window index
		screen_pnt  [Input]
			the screen point where the mouse was clicked
		lite  [Result]
			the light which was picked or unpicked by the mouse click

	Returns
		Error codes


	Availability	6.0.0.5

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td 	 (*fz_lite_pick_func)(
	fzrt_int						windex,
	fzrt_point					*screen_pnt,
	fz_lite_ptr					*lite
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_radius_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's radius parameter. They are a static title text and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		title_item_id  [Result, Optional]
			the id of the title item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_radius_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*title_item_id,
	fzrt_int				*edit_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_spin_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's spin parameter. They are a slider and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		slider_item_id  [Result, Optional]
			the id of the slider item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_spin_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*slider_item_id,
	fzrt_int				*edit_item_id
	);
	
/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_quality_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's quality parameter. They are a text title, a slider and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		title_item_id  [Result, Optional]
			the id of the title item
		slider_item_id  [Result, Optional]
			the id of the slider item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_quality_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*title_item_id,
	fzrt_int				*slider_item_id,
	fzrt_int				*edit_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_outer_angle_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's outer angle parameter. They are a slider and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		slider_item_id  [Result, Optional]
			the id of the slider item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_outer_angle_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*slider_item_id,
	fzrt_int				*edit_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_inner_angle_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's inner angle parameter. They are a slider and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		slider_item_id  [Result, Optional]
			the id of the slider item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_inner_angle_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*slider_item_id,
	fzrt_int				*edit_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_tmap_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim item for 
		the light's texture map parameter. This is a custom item with the texture preview.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim item
		item_id  [Result, Optional]
			the id of the custom item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_tmap_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*item_id
	);
	
/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_render_objt_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim item for 
		the light's render object parameter. This is a check box.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim item
		tem_id  [Result, Optional]
			the id of the check box item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_render_objt_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*tem_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_ignore_objt_color_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim item for 
		the light's ignore object color parameter. This is a check box.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim item
		tem_id  [Result, Optional]
			the id of the check box item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_ignore_objt_color_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*tem_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_env_maps_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's environment maps parameter. They are a menu with the mapping type and an options 
		button to access the respective texture map dialogs.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		menu_item_id  [Result, Optional]
			the id of the menu item
		button_item_id  [Result, Optional]
			the id of the button item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_env_maps_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*menu_item_id,
	fzrt_int				*button_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_env_spin_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's environment spin parameter. They are a slider and an editable text field.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		slider_item_id  [Result, Optional]
			the id of the slider item
		edit_item_id  [Result, Optional]
			the id of the text edit item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_env_spin_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*slider_item_id,
	fzrt_int				*edit_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_fuim_create_obj_from_pick_items_func



	Description
		This function is intended to be used by true plugin lights to put up the standard fuim items for 
		the light's Update Object From Pick button. It is meant for lights types which use a light 
		object, that can be derived from a picked object.

	Parameters
		fuim_tmpl  [Input]
			fuim manager
		parent  [Input]
			parent group
		title  [Input, Optional]
			title for the item. If NULL is passed, the 
			default title will be used.
		windex  [Input]
			project window index
		light  [Input]
			the light for which to create the fuim items
		button_item_id  [Result, Optional]
			the id of the button item

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_fuim_create_obj_from_pick_items_func) (
	fz_fuim_tmpl_ptr	fuim_tmpl,
	fzrt_int				parent,
	fz_string_td		title,
	fzrt_int				windex,
	fz_lite_ptr			light,
	fzrt_int				*button_item_id
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_get_parm_func



	Description
		Gets one of the common parameters of a light. 
		If the display parameter is TRUE, the parameter value returned 
		reflects the light parameters as used when a scene is displayed. 
		That is, if the light is in a light group and the group's 
		override option is on for that parameter, the override 
		parameter is returned. 
		If the display parameter is FALSE, the true parameter of the light 
		is returned. 
		Not all parameters of a light have an equivalent override 
		parameter in the light group. 
		In that case, the true parameter of the light is always returned, 
		regardless of the display parameter.

		Not all lights use all common parameters. For example, a point light 
		does not use the outer angle parameters by default. If the outer angle 
		paramter is retrieved for such a light, an error is returned.

		fz_lite_get_parm and fz_lite_set_parm replace older api functions that 
		get and set the parameters for each individual light type, such as 
		fz_lite_get_parm_dist and fz_lite_set_parm_dist. These should not be 
		used anymore.

	Parameters
		windex  [Input]
			project window index
		light  [Input]
			the light for which to get the parameter
		which  [Input]
			parameter identifier
		display  [Input]
			if TRUE, the display parameter is returned, 
			otherwise the true parameter is returned
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_get_parm_func) (
	fzrt_int					windex,
	fz_lite_ptr				light,
	fz_light_rep_parm_enum	which,
	fzrt_boolean			display,
	fz_type_td				*data
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_set_parm_func



	Description
		Sets one of the common parameters of a light. 
		Note: not all common parameters are used by all lights. 
		In this case the parameter is not set.  
		When setting the light type to a type which requires an object, such as an area light, a default 
		object is created. This a 1x1x1 unit cube centered at the light origin.

		It is possible to further change the object used by a light by passing an 
		object pointer with the FZ_LIGHT_REP_PARM_USE_OBJ parameter identifier. In this 
		case, the content of the object passed is copied into the object stored with the 
		light. The object passed in stays unchanged.

		The FZ_LIGHT_REP_PARM_ENV_MAPS parameter is a read only parameter. Individual 
		parameters of a lights environment are get/set with the fz_lite_get_env_parm and 
		fz_lite_set_env_parm api functions.

		The FZ_LIGHT_REP_PARM_TMAP parameter is also a read only parameter. A texture 
		map can be modified with the fz_tmap_fset function set (fz_tmap_xxx functions for scripts).

		fz_lite_get_parm and fz_lite_set_parm replace older api functions that 
		get and set the parameters for each individual light type, such as 
		fz_lite_get_parm_dist and fz_lite_set_parm_dist. These should not be 
		used anymore.

	Parameters
		windex  [Input]
			project window index
		light  [Input]
			the light for which to set the parameter
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_set_parm_func) (
	fzrt_int					windex,
	fz_lite_ptr				light,
	fz_light_rep_parm_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_get_env_parm_func



	Description
		Gets one of the common parameters of an environment, used by a light. 
		The env argument is a pointer, that can be retrieved from a light, which 
		uses the standard environment parameter, with the fz_lite_get_parm api 
		function using the FZ_LIGHT_REP_PARM_ENV_MAPS parameter identifier.

	Parameters
		windex  [Input]
			project window index
		env  [Input]
			the environment for which to get the parameter
		which  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_get_env_parm_func) (
	fzrt_int					windex,
	fz_light_env_ptr		env,
	fz_lite_env_parm_enum	which,
	fz_type_td				*data
	);
	
/****************************************************************

	FZRT_SPEC	*fz_lite_set_env_parm_func



	Description
		Sets one of the common parameters of an environment, used by a light. 
		The env argument is a pointer, that can be retrieved from a light, which 
		uses the standard environment parameter, with the fz_lite_get_parm api 
		function using the FZ_LIGHT_REP_PARM_ENV_MAPS parameter identifier.

	Parameters
		windex  [Input]
			project window index
		env  [Input]
			the environment for which to set the parameter
		which  [Input]
			parameter identifier
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_set_env_parm_func) (
	fzrt_int					windex,
	fz_light_env_ptr		env,
	fz_lite_env_parm_enum	which,
	fz_type_td				*data
	);


/****************************************************************

	FZRT_SPEC	*fz_lite_get_plugin_parm_func



	Description
		Gets a parameters of a light, which is defined by a light representation plugin. 
		The light type may be a common light type, or it may be a light type that 
		is entirely defined by a plugin. If it is a common light type, the plugin 
		has defined additional parameters. If it is a plugin light type, the 
		plugin has defined all parameters. 
		The lrep_uuid parameter identifies the plugin which defines the light type. The 
		parm_uuid parameter defines the parameter itself. These UUID based parameters 
		are defined in the respective header file of the plugin. For example, the 
		lrep_uuid for the RenderZone light representation plugin are defined in 
		fz_rzne_lite_api.h, together with all the renderzone parameter UUID's.

	Parameters
		windex  [Input]
			project window index
		light  [Input]
			the light for which to get the parameter
		lrep_uuid  [Input]
			light representation plugin identifier
		parm_uuid  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_get_plugin_parm_func) (
	fzrt_int					windex,
	fz_lite_ptr				light,
	fzrt_UUID_td			lrep_uuid,
	fzrt_UUID_td			parm_uuid,
	fz_type_td				*data
	);

/****************************************************************

	FZRT_SPEC	*fz_lite_set_plugin_parm_func



	Description
		Sets a parameters of a light, which is defined by a light representation plugin. 
		The light type may be a common light type, or it may be a light type that 
		is entirely defined by a plugin. If it is a common light type, the plugin 
		has defined additional parameters. If it is a plugin light type, the 
		plugin has defined all parameters. 
		The lrep_uuid parameter identifies the plugin which defines the light type. The 
		parm_uuid parameter defines the parameter itself. These UUID based parameters 
		are defined in the respective header file of the plugin. For example, the 
		lrep_uuid for the RenderZone light representation plugin are defined in 
		fz_rzne_lite_api.h, together with all the renderzone parameter UUID's.

	Parameters
		windex  [Input]
			project window index
		light  [Input]
			the light for which to set the parameter
		lrep_uuid  [Input]
			light representation plugin identifier
		parm_uuid  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	7.0.0.0

 ****************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC	*fz_lite_set_plugin_parm_func) (
	fzrt_int					windex,
	fz_lite_ptr				light,
	fzrt_UUID_td			lrep_uuid,
	fzrt_UUID_td			parm_uuid,
	fz_type_td				*data
	);

/****************************************************************

	fz_lite_get_lrep_data



	Description
		This function allows a plugin to retrieve the data associated with 
		a light representation. It is most useful for a plugin renderer 
		that has implemented its own light representation plugin 
		and needs to get to its light data when executing the 
		rendering. The lrep_uuid argument identifies the light 
		representation. If it is passed as NULL, the lrep_indx argument 
		is used. For speed purpose, the lrep_indx is faster, as it is a run time 
		index, where as the lrep_uuid requires searching. The lrep_indx can be 
		retrieved from the uuid of a light representation by the api 
		call fz_lite_get_lrep_index.

	Parameters
		windex  [Input]
			project window index
		lite_ptr  [Input]
			the light from which to get the data
		lrep_uuid  [Input, Optional]
			the uuid which identifies the light representation. 
			If passed as NULL, lrep_indx is used.
		lrep_indx  [Input, Optional]
			the run time index which identifies the light representation. 
			If lrep_uuid is passed as NULL, this argument is used and must be a valid index.
		lrep_data  [Result]
			a pointer to the light data

	Returns
		Error codes


	Availability	7.0.0.0


	See Also
		fz_lite_get_lrep_index

 ****************************************************************/
typedef fzrt_error_td  (*fz_lite_get_lrep_data_func) (
	fzrt_int					windex,
	fz_lite_ptr				lite_ptr,
	fzrt_UUID_td			lrep_uuid,
	fzrt_int					lrep_indx,
	void					**lrep_data
	);

/****************************************************************

	fz_lite_get_lrep_index



	Description
		Given the uuid of a light representation, get its run time index. 
		The run time index allows for faster access of the data of the 
		light when using the api fz_lite_get_lrep_data. Note, that 
		this index is guaranteed to be valid only for a single run time session 
		of formZ. That is, this index should not be saved to a file and used 
		in a subsequent session.

	Parameters
		lrep_uuid  [Input]
			the uuid of the light representation.
		lrep_indx  [Result]
			the run time index of the light representation.

	Returns
		Error codes


	Availability	7.0.0.0


	See Also
		fz_lite_get_lrep_data

 ****************************************************************/
typedef fzrt_error_td  (*fz_lite_get_lrep_index_func) (
	fzrt_UUID_td			lrep_uuid,
	fzrt_int					*lrep_indx
	);

/****************************************************************

	fz_lite_get_intens_scale



	Description
		Gets the intensity scale factor of a light. If the light is not 
		in a light group, this is always 1.0. If the light is in a light 
		group and the group has the intensity override wit hthe scaler option on 
		the scale factor is returned. 
		This function is intended to be used by a plugin renderer to honor 
		the intensity override option of a light group. That is, when setting 
		up a rendering looking at the lights in a project, the plugin renderer 
		should always inquire this parameter and apply it to its own 
		light intenisty settings. Note, that the simple intensity light 
		parameter(fz_lite_get_parm(windex,light,FZ_LIGHT_REP_PARM_INTENS_BRIGHT,display,&data), 
		that is maintained by formZ already has this scale factor applied if the 
		display parameter is set to TRUE.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			the light from which to get intensity scale
		scale  [Result]
			the intensity scale

	Returns
		Error codes


	Availability	7.0.0.0


	See Also
		fz_lite_get_parm

 ****************************************************************/
typedef fzrt_error_td  (*fz_lite_get_intens_scale_func) (
	fzrt_int					windex,
	fz_lite_ptr				lite,
	double					*scale
	);

/****************************************************************

	fz_lite_dist_get_sun



	Description
		Returns the sun data for a given light. Note, that in the interface 
		only distant lights give access to the sun. However, this function 
		can be called for every light type, as sun data is actually stored 
		with each light, even if its not a distant light.

	Parameters
		windex  [Input]
			project window index
		lite  [Input]
			light
		sun_ptr  [Result]
			sun

	Returns
		Error codes


	Availability	7.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_dist_get_sun_func)(
	fzrt_int				windex,
	fz_lite_ptr			lite,
	fz_lite_sun_ptr		*sun_ptr
	);


/****************************************************************

	fz_lite_get_sun_legacy



	Description
		Gets the sun light for a project that originated in a version 
		prior to version 7 and bonzai3d. If the project is newer, NULL 
		will be returned.

	Parameters
		windex  [Input]
			project window index
		sun_lite  [Result]
			former sun light

	Returns
		Error codes


	Availability	7.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_lights.fsl
 ****************************************************************/
typedef fzrt_error_td	 (*fz_lite_get_sun_legacy_func)(
	fzrt_int				windex,
	fz_lite_ptr			*sun_lite
	);

/****************************************************************

	fz_lite_fset


	FZ_LITE_FSET_TYPE
		formZ lights function set UUID.

	FZ_LITE_FSET_NAME
		formZ lights function set name.

	FZ_LITE_FSET_VERSION
		formZ lights function set version.

	Description
		formZ Lights Function Set

	Availability	5.0.0.0

 ****************************************************************/


#define FZ_LITE_FSET_TYPE		"\x04\x19\xf1\xe2\x0d\xc7\x49\xda\xa4\xdf\x88\xf7\xcf\xbd\x72\x97"

#define FZ_LITE_FSET_NAME		"formZ Lights Functions"

#define FZ_LITE_FSET_VERSION	FZPL_VERS_MAKE(7,0,1,0)

typedef struct fz_lite_fset
{
	fzpl_fset_prvt_data_td				prvt_data;

	
	fz_lite_get_num_lights_func			fz_lite_get_num_lights;
	fz_lite_tag_to_ptr_func				fz_lite_tag_to_ptr;
	fz_lite_ptr_to_tag_func				fz_lite_ptr_to_tag;
	fz_lite_get_next_light_func			fz_lite_get_next_light;
	fz_lite_get_prev_light_func			fz_lite_get_prev_light;
	fz_lite_create_new_func				fz_lite_create_new;
	fz_lite_duplicate_func				fz_lite_duplicate;
	fz_lite_delete_func					fz_lite_delete;
	fz_lite_get_sun_func				fz_lite_get_sun;
	fz_lite_set_sun_func				fz_lite_set_sun;
	fz_lite_get_parm_common_func_obsolete			fz_lite_get_parm_common_obsolete;
	fz_lite_set_parm_common_func_obsolete			fz_lite_set_parm_common_obsolete;
	fz_lite_get_parm_dist_func_obsolete				fz_lite_get_parm_dist_obsolete;
	fz_lite_set_parm_dist_func_obsolete				fz_lite_set_parm_dist_obsolete;
	fz_lite_get_parm_point_func_obsolete			fz_lite_get_parm_point_obsolete;
	fz_lite_set_parm_point_func_obsolete			fz_lite_set_parm_point_obsolete;
	fz_lite_get_parm_cone_func_obsolete				fz_lite_get_parm_cone_obsolete;
	fz_lite_set_parm_cone_func_obsolete				fz_lite_set_parm_cone_obsolete;
	fz_lite_get_parm_projector_func_obsolete		fz_lite_get_parm_projector_obsolete;
	fz_lite_set_parm_projector_func_obsolete		fz_lite_set_parm_projector_obsolete;
	fz_lite_get_parm_area_func_obsolete				fz_lite_get_parm_area_obsolete;
	fz_lite_set_parm_area_func_obsolete				fz_lite_set_parm_area_obsolete;
	fz_lite_get_parm_line_func_obsolete				fz_lite_get_parm_line_obsolete;
	fz_lite_set_parm_line_func_obsolete				fz_lite_set_parm_line_obsolete;
	fz_lite_get_parm_custom_func_obsolete			fz_lite_get_parm_custom_obsolete;
	fz_lite_set_parm_custom_func_obsolete			fz_lite_set_parm_custom_obsolete;
	fz_lite_get_parm_custom_data_func	fz_lite_get_parm_custom_data;
	fz_lite_set_parm_custom_data_func	fz_lite_set_parm_custom_data;
	fz_lite_is_symb_light_func			fz_lite_is_symb_light;
	fz_lite_get_ambient_intensity_func	fz_lite_get_ambient_intensity;
	fz_lite_set_ambient_intensity_func	fz_lite_set_ambient_intensity;
	fz_lite_get_ambient_color_func		fz_lite_get_ambient_color;
	fz_lite_set_ambient_color_func		fz_lite_set_ambient_color;
	fz_lite_transform_func				fz_lite_transform;
	fz_lite_get_grup_table_func		fz_lite_get_grup_table;
	fz_lite_get_grup_tag_func			fz_lite_get_grup_tag;
	fz_lite_grup_get_parm_func			fz_lite_grup_get_parm;
	fz_lite_grup_set_parm_func			fz_lite_grup_set_parm;
	fz_lite_dist_set_dir_from_sun_func	fz_lite_dist_set_dir_from_sun;
	fz_lite_get_parm_sun_func			fz_lite_get_parm_sun;
	fz_lite_set_parm_sun_func			fz_lite_set_parm_sun;
	fz_lite_sun_city_id_from_name_func	fz_lite_sun_city_id_from_name;
	fz_lite_sun_city_name_from_id_func	fz_lite_sun_city_name_from_id;
	fz_lite_sun_city_add_func			fz_lite_sun_city_add;
	fz_lite_sun_country_add_func		fz_lite_sun_country_add;
	fz_lite_sun_city_delete_func		fz_lite_sun_city_delete;
	fz_lite_sun_city_get_parm_func		fz_lite_sun_city_get_parm;
	fz_lite_sun_city_set_parm_func		fz_lite_sun_city_set_parm;
	
	fz_lite_get_symb_func				fz_lite_get_symb;
	fz_lite_get_opts_parm_data_func		fz_lite_get_opts_parm_data;
	fz_lite_set_opts_parm_data_func		fz_lite_set_opts_parm_data;
	fz_lite_sort_func					fz_lite_sort;
	fz_lite_clear_func					fz_lite_clear;

	fz_lite_get_parm_environment_func_obsolete	fz_lite_get_parm_environment_obsolete;
	fz_lite_set_parm_environment_func_obsolete	fz_lite_set_parm_environment_obsolete;
	
	fz_lite_sun_city_id_from_indx_func	fz_lite_sun_city_id_from_indx;
	fz_lite_sun_get_num_cities_func		fz_lite_sun_get_num_cities;
	
	
	
	fz_lite_get_pick_status_func		fz_lite_get_pick_status;
	fz_lite_set_pick_status_func		fz_lite_set_pick_status;
	fz_lite_pick_func					fz_lite_pick;

	fz_lite_copy_to_windex_func			fz_lite_copy_to_windex;

	
	fz_lite_fuim_create_radius_items_func				fz_lite_fuim_create_radius_items;
	fz_lite_fuim_create_spin_items_func					fz_lite_fuim_create_spin_items;
	fz_lite_fuim_create_quality_items_func				fz_lite_fuim_create_quality_items;
	fz_lite_fuim_create_outer_angle_items_func			fz_lite_fuim_create_outer_angle_items;
	fz_lite_fuim_create_inner_angle_items_func			fz_lite_fuim_create_inner_angle_items;
	fz_lite_fuim_create_tmap_items_func					fz_lite_fuim_create_tmap_items;
	fz_lite_fuim_create_render_objt_items_func			fz_lite_fuim_create_render_objt_items;
	fz_lite_fuim_create_ignore_objt_color_items_func	fz_lite_fuim_create_ignore_objt_color_items;
	fz_lite_fuim_create_env_maps_items_func				fz_lite_fuim_create_env_maps_items;
	fz_lite_fuim_create_env_spin_items_func				fz_lite_fuim_create_env_spin_items;
	fz_lite_fuim_create_obj_from_pick_items_func		fz_lite_fuim_create_obj_from_pick_items;
	
	
	fz_lite_get_parm_func								fz_lite_get_parm;
	fz_lite_set_parm_func								fz_lite_set_parm;
	
	fz_lite_get_env_parm_func							fz_lite_get_env_parm;
	fz_lite_set_env_parm_func							fz_lite_set_env_parm;
	
	fz_lite_get_plugin_parm_func						fz_lite_get_plugin_parm;
	fz_lite_set_plugin_parm_func						fz_lite_set_plugin_parm;
	
	fz_lite_get_lrep_data_func							fz_lite_get_lrep_data;
	fz_lite_get_lrep_index_func							fz_lite_get_lrep_index;
	fz_lite_get_intens_scale_func						fz_lite_get_intens_scale;
	
	fz_lite_dist_get_sun_func							fz_lite_dist_get_sun;
	fz_lite_get_sun_legacy_func							fz_lite_get_sun_legacy;
}  fz_lite_fset;


#ifdef __cplusplus
}
#endif

#endif

