/*********************************************************************************************
 *                                                                                           *
 *                                                                                           *
 *   Copyright 1989-2018 by AutoDesSys, Inc.                                                 *
 *                                                                                           *
 *   This software and its associated documentation contains proprietary,                    *
 *   confidential, and trade secret information of AutoDesSys Inc. and may not be            *
 *   disclosed, copied in any form, or used for any purpose except as specifically           *
 *   permitted in writing by AutoDesSys, Inc.                                                *
 *                                                                                           *
 *   AutoDesSys, Inc. grants permission to use the software to those who purchase            *
 *   a license.  Such a usage will be made according to the terms and conditions             *
 *   specified in the license agreement to which a licensee agrees before installing         *
 *   and commencing usage of the software.  The license agreement can be found               *
 *   on the envelop that contains the software CD.                                           *
 *                                                                                           *
 *   AutoDesSys, Inc. also grants permission to licensees to develop extensions to           *
 *   the software, which can be in the form of scripts or plugins.  The permission           *
 *   includes calling the API and call back functions provided by AutoDesSys, Inc.           *
 *   and accessing information relevant to these functions and the general workings          *
 *   of form.Z, but only up to the point to which AutoDesSys, Inc. has chosen to             *
 *   expose, through its form.Z SDK Manual.  All extensions developed by third               *
 *   parties will include a copy of this copyright note in an easily accessible place or     *
 *   a clear instruction to read this original note by AutoDesSys, Inc.                      *
 *                                                                                           *
 *   All the software distributed under a license, either for the purpose of using it or     *
 *   for the purpose of developing extensions, is distributed AS IS, WITHOUT                 *
 *   WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND                                    *
 *   AutoDesSys, Inc. HEREBY DISCLAIMS ALL SUCH WARRANTIES,                                  *
 *   INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF                                         *
 *   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.                                    *
 *                                                                                           *
 *   formZ, RenderZone, and RadioZity are registered trademarks of AutoDesSys, Inc.          *
 *                                                                                           *
 *   form.Z SDK, Version 8.6.3, Dec 03, 2018                                                 *
 *                                                                                           *
 *                                                                                           *
 *********************************************************************************************/


#ifndef _FORMZ_MODEL_SWEP_API_H
#define _FORMZ_MODEL_SWEP_API_H

#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************

	Sweep


	Description
		formZ Model Sweep objects.
 ****************************************************************/

#include "fz_model_core_api.h"



/****************************************************************

	FZ_OBJT_TYPE_SWEEP


	Description
		Unique identifier for the sweep object type.
 ****************************************************************/
#define FZ_OBJT_TYPE_SWEEP		"\xc6\xba\x21\xd5\xdc\x38\x4e\xa2\x99\x90\xb5\xcb\xc7\x00\xa1\xbe"

/****************************************************************

	fz_objt_sweep_cnstr_opts_ptr


	Description
		data structure to hold options for the constructions of a sweep object. 
		This type is used to pass additional construction options to the 
		function fz_objt_cnstr_sweep_boundary,fz_objt_cnstr_sweep_2path 
		fz_objt_cnstr_sweep_2source and fz_objt_cnstr_sweep_axial . The individual 
		fields of this type can be accessed and set via fz_objt_cnstr_sweep_opts_get 
		and fz_objt_cnstr_sweep_opts_set.

	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_boundary,fz_objt_cnstr_sweep_2path, 
		fz_objt_cnstr_sweep_2source,fz_objt_cnstr_sweep_axial, 
		fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set, fz_objt_cnstr_sweep_2path

 ****************************************************************/
typedef struct	swep_cnstr_opts_td	*fz_objt_sweep_cnstr_opts_ptr;

/****************************************************************

	fz_objt_sweep_parm_enum


	Description
		Parameter identifiers for a parametric sweep object. Used for the 
		"which" argument of fz_objt_edit_sweep_parm_get, fz_objt_edit_sweep_parm_set, 
		fz_objt_cnstr_sweep_opts_get and fz_objt_cnstr_sweep_opts_set. 
		After creating a sweep object, the source and path objects can be 
		replaced by passing in a new object using the fz_objt_edit_sweep_parm_set 
		API function. Note, that the same restrictions apply for the new objects 
		as for the ones used to construct the sweep initially. When getting 
		the source/path objects from an existing sweep object via fz_objt_edit_sweep_parm_get, 
		a copy of the source or path is made and passed back. It is the responsibility 
		of the calling function to delete this new object afterwards, using fz_objt_edit_delete_objt.

	Members
		FZ_OBJT_SWEEP_PARM_SRC_ALIGN
			Construction - Source alignment.
			Type: fz_objt_sweep_srcalign_enum
			Default Value: FZ_OBJT_SWEEP_SRCALIGN_CENTROID
		FZ_OBJT_SWEEP_PARM_FACT_TRIANG
			Construction and Editing - Triangulate on (TRUE) or off (FALSE) for facetted objects.
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_OBJT_SWEEP_PARM_CAP_START_ON
			Construction and Editing - Start cap on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_OBJT_SWEEP_PARM_CAP_END_ON
			Construction and Editing - End cap on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: TRUE
		FZ_OBJT_SWEEP_PARM_AXL_PERP_TO_PLANE
			Construction and Editing - Axial sweep, perpendicular to plane on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_OBJT_SWEEP_PARM_AXL_PLANE_NORM
			Construction and Editing - Axial sweep, normal direction for perpendicular to plane option . 
			This parameter is a normalized vector which indicates 
			the perpendicular normal) direction of the plane to 
			which the sweep cross section is held perpendicular, if 
			the Perpendicular To Plane option is used.
			Type: fz_xyz_td
		FZ_OBJT_SWEEP_PARM_AXL_SRC_SCL_ON
			Construction and Editing - Axial sweep, source scale on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_OBJT_SWEEP_PARM_AXL_SRC_SCL
			Construction and Editing - Axial sweep, source scale factor.
			Type: fz_xy_td
			Default Value: {1.0,1.0}
			Range: 0.0
		FZ_OBJT_SWEEP_PARM_AXL_SRC_ROT_ON
			Construction and Editing - Axial sweep, source rotation on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_OBJT_SWEEP_PARM_AXL_SRC_ROT
			Construction and Editing - Axial sweep, source rotation angle.
			Type: double
			Default Value: 0.0
		FZ_OBJT_SWEEP_PARM_2PTH_PSRV_HGT
			Construction and Editing - 2 path sweep, preserve height on (TRUE) or off (FALSE).
			Type: fzrt_boolean
			Default Value: FALSE
		FZ_OBJT_SWEEP_PARM_NRBZ_DEG_U
			Construction and Editing - Length degree of nurbz object.
			Type: fzrt_int
			Default Value: 2
			Range: 1
		FZ_OBJT_SWEEP_PARM_NRBZ_DEG_V
			Construction and Editing - Depth degree of nurbz object.
			Type: fzrt_int
			Default Value: 2
			Range: 1
		FZ_OBJT_SWEEP_PARM_TYPE
			Editing - Sweep type. 
			This is a read only parameter and indicates whether the 
			sweep is an axial, 2 source, 2 path or boundary sweep
			Type: fz_objt_sweep_type_enum
		FZ_OBJT_SWEEP_PARM_MODEL_TYPE
			Editing - Model type.
			Type: fz_objt_model_type_enum
		FZ_OBJT_SWEEP_PARM_SMOOTH_TYPE
			Editing - Smooth type.
			Type: fz_objt_smooth_type_enum
		FZ_OBJT_SWEEP_PARM_SRC_OBJ1
			Editing - Sweep source object 1. 
			This is the source object for all sweep types.
			Type: fz_objt_ptr
		FZ_OBJT_SWEEP_PARM_SRC_OBJ2
			Editing - Sweep source object 2. 
			This is the 2nd source object for the 2 source sweep.
			Type: fz_objt_ptr
		FZ_OBJT_SWEEP_PARM_PTH_OBJ1
			Editing - Sweep path object 1. 
			This is the path object for all sweep types.
			Type: fz_objt_ptr
		FZ_OBJT_SWEEP_PARM_PTH_OBJ2
			Editing - Sweep path object 2. 
			This is the 2nd path object for the 2 path sweep.
			Type: fz_objt_ptr

	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_get,fz_objt_edit_sweep_parm_set, 
		fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set, fz_objt_cnstr_sweep_opts_get

 ****************************************************************/
typedef enum fz_objt_sweep_parm_enum
{
	
	FZ_OBJT_SWEEP_PARM_SRC_ALIGN = 0,
	
	
	FZ_OBJT_SWEEP_PARM_FACT_TRIANG,
											
	FZ_OBJT_SWEEP_PARM_CAP_START_ON,
											
	FZ_OBJT_SWEEP_PARM_CAP_END_ON,
											
	FZ_OBJT_SWEEP_PARM_AXL_PERP_TO_PLANE,
											
	FZ_OBJT_SWEEP_PARM_AXL_PLANE_NORM,
													
	FZ_OBJT_SWEEP_PARM_AXL_SRC_SCL_ON,
											
	FZ_OBJT_SWEEP_PARM_AXL_SRC_SCL,
											
	FZ_OBJT_SWEEP_PARM_AXL_SRC_ROT_ON,
											
	FZ_OBJT_SWEEP_PARM_AXL_SRC_ROT,
											
	FZ_OBJT_SWEEP_PARM_2PTH_PSRV_HGT,
											
	FZ_OBJT_SWEEP_PARM_NRBZ_DEG_U,
											
	FZ_OBJT_SWEEP_PARM_NRBZ_DEG_V,

	
	FZ_OBJT_SWEEP_PARM_TYPE,
											
	FZ_OBJT_SWEEP_PARM_MODEL_TYPE,
											
	FZ_OBJT_SWEEP_PARM_SMOOTH_TYPE,
											
	FZ_OBJT_SWEEP_PARM_SRC_OBJ1,
											
	FZ_OBJT_SWEEP_PARM_SRC_OBJ2,
											
	FZ_OBJT_SWEEP_PARM_PTH_OBJ1,
											
	FZ_OBJT_SWEEP_PARM_PTH_OBJ2,
											
	FZ_OBJT_SWEEP_PARM_MAX
	
} fz_objt_sweep_parm_enum;


/****************************************************************

	fz_objt_sweep_type_enum


	Description
		Indicates what type of sweep a parametric sweep object is. This enum 
		is used in the read only parameter FZ_OBJT_SWEEP_PARM_TYPE in the 
		function fz_objt_edit_sweep_parm_get.

	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set, 

 ****************************************************************/
typedef enum fz_objt_sweep_type_enum
{
	FZ_OBJT_SWEEP_TYPE_AXIAL = 0,
	FZ_OBJT_SWEEP_TYPE_2SOURCE,
	FZ_OBJT_SWEEP_TYPE_2PATH,
	FZ_OBJT_SWEEP_TYPE_BOUNDARY
	
} fz_objt_sweep_type_enum;

/****************************************************************

	fz_objt_sweep_srcalign_enum


	Description
		Values for sweep source alignment. Use by the FZ_OBJT_SWEEP_PARM_SRC_ALIGN 
		parameter in fz_objt_cnstr_sweep_opts_get and fz_objt_cnstr_sweep_opts_set.

	Members
		FZ_OBJT_SWEEP_SRCALIGN_ORIGIN
			Align source relative to world origin
		FZ_OBJT_SWEEP_SRCALIGN_CENTROID
			Align source using its centroid
		FZ_OBJT_SWEEP_SRCALIGN_1STPNT
			Align source using its 1st point

	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set, 

 ****************************************************************/
typedef enum fz_objt_sweep_srcalign_enum
{
	FZ_OBJT_SWEEP_SRCALIGN_ORIGIN = 0,
	FZ_OBJT_SWEEP_SRCALIGN_CENTROID,
	FZ_OBJT_SWEEP_SRCALIGN_1STPNT

} fz_objt_sweep_srcalign_enum;

/****************************************************************

	fz_objt_cnstr_sweep_axial



	Description
		Creates an axial sweep object. The 
		source object must have a single face. It can be an open or 
		closed wire or a single face surface object. The path must 
		also be an open or closed wire or a single face surface object 
		with a single curve.

	Parameters
		windex  [Input]
			project window index
		src_obj  [Input]
			source object
		pth_obj  [Input]
			path object
		model_type  [Input]
			model type
		smooth_type  [Input]
			smooth type
		swep_opts  [Input, Optional]
			construction options. For defaults fz_objt_sweep_parm_enum
		refn_opts  [Input, Optional]
			display resolution options. For defaults fz_attr_refn_parm_enum
		obj  [Result]
			new object

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_set,fz_objt_edit_sweep_parm_get

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_axial_func) (
	fzrt_int 					windex,
	fz_objt_ptr			src_obj,
	fz_objt_ptr			pth_obj,
	fz_objt_model_type_enum			model_type,
	fz_objt_smooth_type_enum	smooth_type,
	fz_objt_sweep_cnstr_opts_ptr		swep_opts,
	fz_attr_refn_ptr		refn_opts,
	fz_objt_ptr			*obj
	);

/****************************************************************

	fz_objt_cnstr_sweep_2source



	Description
		Creates a 2 source sweep object. The 
		source objects must have a single face. They can be an open or 
		closed wires or single face surface objects. The path must 
		also be an open or closed wire or a single face surface object 
		with a single curve. The source objects must both be closed 
		or both be open.

	Parameters
		windex  [Input]
			project window index
		src_obj1  [Input]
			source object 1
		src_obj2  [Input]
			source object 2
		pth_obj  [Input]
			path object
		model_type  [Input]
			model type
		smooth_type  [Input]
			smooth type
		swep_opts  [Input, Optional]
			construction options. For defaults see fz_objt_sweep_parm_enum
		refn_opts  [Input, Optional]
			display resolution options. For defaults see fz_attr_refn_parm_enum
		obj  [Result]
			new object

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_set,fz_objt_edit_sweep_parm_get

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_2source_func) (
	fzrt_int 					windex,
	fz_objt_ptr			src_obj1,
	fz_objt_ptr			src_obj2,
	fz_objt_ptr			pth_obj,
	fz_objt_model_type_enum			model_type,
	fz_objt_smooth_type_enum	smooth_type,
	fz_objt_sweep_cnstr_opts_ptr		swep_opts,
	fz_attr_refn_ptr		refn_opts,
	fz_objt_ptr			*obj
	);

/****************************************************************

	fz_objt_cnstr_sweep_2path



	Description
		Creates a 2 path sweep object. The 
		source object must have a single face. It can be an open or 
		closed wire or a single face surface object. The paths must 
		also be open or closed wires or single face surface objects 
		with a single curve. The path objects must both be closed 
		or both be open.

	Parameters
		windex  [Input]
			project window index
		src_obj  [Input]
			source object
		pth_obj1  [Input]
			path object 1
		pth_obj2  [Input]
			path object 2
		model_type  [Input]
			model type
		smooth_type  [Input]
			smooth type
		swep_opts  [Input, Optional]
			construction options. For defaults fz_objt_sweep_parm_enum
		refn_opts  [Input, Optional]
			display resolution options. For defaults fz_attr_refn_parm_enum
		obj  [Result]
			new object

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_set,fz_objt_edit_sweep_parm_get

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_2path_func) (
	fzrt_int 					windex,
	fz_objt_ptr			src_obj,
	fz_objt_ptr			pth_obj1,
	fz_objt_ptr			pth_obj2,
	fz_objt_model_type_enum			model_type,
	fz_objt_smooth_type_enum	smooth_type,
	fz_objt_sweep_cnstr_opts_ptr		swep_opts,
	fz_attr_refn_ptr		refn_opts,
	fz_objt_ptr			*obj
	);

/****************************************************************

	fz_objt_cnstr_sweep_boundary



	Description
		Creates a boundary sweep object. The source object 
		must have a single face and be an open wire. 
		The path must be an open or closed wire or single face 
		surface object with a single curve. Note, that the 
		boundary sweep cannot be executed as a smooth nurbz 
		operation.

	Parameters
		windex  [Input]
			project window index
		src_obj  [Input]
			source object
		pth_obj  [Input]
			path object
		model_type  [Input]
			model type
		swep_opts  [Input, Optional]
			construction options. For defaults fz_objt_sweep_parm_enum
		refn_opts  [Input, Optional]
			display resolution options. For defaults fz_attr_refn_parm_enum
		obj  [Result]
			new object

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_set,fz_objt_edit_sweep_parm_get

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_boundary_func) (
	fzrt_int 					windex,
	fz_objt_ptr			src_obj,
	fz_objt_ptr			pth_obj,
	fz_objt_model_type_enum			model_type,
	fz_objt_sweep_cnstr_opts_ptr		swep_opts,
	fz_attr_refn_ptr		refn_opts,
	fz_objt_ptr			*obj
	);

/****************************************************************

	fz_objt_cnstr_sweep_opts_init



	Description
		Allocate sweep construction options and fill with defaults

	Parameters
		windex  [Input]
			project window index
		opts  [Result]
			construction options

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_finit,fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_opts_init_func) (
	fzrt_int				windex,
	fz_objt_sweep_cnstr_opts_ptr	*opts
	);

/****************************************************************

	fz_objt_cnstr_sweep_opts_get



	Description
		Get the value of a sweep object's construction option. Note, that 
		only those values in fz_objt_sweep_parm_enum are accepted 
		which are tagged for the construction options. See 
		fz_objt_sweep_parm_enum for details.

	Parameters
		windex  [Input]
			project window index
		opts  [Input]
			construction options
		which  [Input]
			parameter id
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_init,fz_objt_cnstr_sweep_opts_finit,fz_objt_cnstr_sweep_opts_set

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_opts_get_func) (
	fzrt_int					windex,
	fz_objt_sweep_cnstr_opts_ptr		opts,
	fz_objt_sweep_parm_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_objt_cnstr_sweep_opts_set



	Description
		Set the value of a sweep object's construction option. Note, that 
		only those values in fz_objt_sweep_parm_enum are accepted 
		which are tagged for the construction options. See 
		fz_objt_sweep_parm_enum for details.

	Parameters
		windex  [Input]
			project window index
		opts  [Input]
			construction options
		which  [Input]
			parameter id
		data  [Input]
			parameter data

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_init,fz_objt_cnstr_sweep_opts_finit,fz_objt_cnstr_sweep_opts_get

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_opts_set_func) (
	fzrt_int					windex,
	fz_objt_sweep_cnstr_opts_ptr		opts,
	fz_objt_sweep_parm_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_objt_cnstr_sweep_opts_finit



	Description
		Frees sweep object construction options. The opts pointer 
		is deallocated and returned back as NULL.

	Parameters
		windex  [Input]
			project window index
		opts  [Input, Result]
			options (will become NULL)

	Returns
		Error codes


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_cnstr_sweep_opts_init,fz_objt_cnstr_sweep_opts_get,fz_objt_cnstr_sweep_opts_set

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_cnstr_sweep_opts_finit_func) (
	fzrt_int				windex,
	fz_objt_sweep_cnstr_opts_ptr	*opts
	);

/****************************************************************

	fz_objt_edit_sweep_parm_get



	Description
		Gets a parameter of a sweep object.

	Parameters
		windex  [Input]
			project window index
		obj  [Input]
			object
		which  [Input]
			parameter id
		data  [Result]
			parameter data

	Returns
		Error codes 
		MODEL_ERR_WRONG_OBJT_TYPE	- object passed in is not a sweep object


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_set,fz_objt_edit_parm_regen

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_edit_sweep_parm_get_func) (
	fzrt_int					windex,
	fz_objt_ptr			obj,
	fz_objt_sweep_parm_enum	which,
	fz_type_td				*data
	);

/****************************************************************

	fz_objt_edit_sweep_parm_set



	Description
		Sets a parameter of a sweep object. Note, that the shape of the sweep object 
		is not changed until the function fz_objt_edit_parm_regen has 
		been called.

	Parameters
		windex  [Input]
			project window index
		obj  [Input]
			object
		which  [Input]
			parameter id
		data  [Input]
			parameter data

	Returns
		Error codes 
		MODEL_ERR_WRONG_OBJT_TYPE	- object passed in is not a sweep object


	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_cnstr_sweep.fsl

	See Also
		fz_objt_edit_sweep_parm_get,fz_objt_edit_parm_regen

 ****************************************************************/
typedef fzrt_error_td  (*fz_objt_edit_sweep_parm_set_func) (
	fzrt_int					windex,
	fz_objt_ptr			obj,
	fz_objt_sweep_parm_enum	which,
	fz_type_td				*data
	);

#ifdef __cplusplus
}
#endif

#endif
