/*************************************************************************

	formZ by autodessys, Inc.
	Copyright c 1994 autodessys, Inc.  All rights reserved.
	
	
	File :	fz_palt_api.h
	
	By :	
	
	Created on :  		17 June 2002
	Last Updated on :  	17 June 2002
	
	Description :	
					
***************************************************************************/

#ifndef _FORMZ_PALT_API_H
#define _FORMZ_PALT_API_H 1

#include "fzpl_plugin.h"
#include "fz_iost_api.h"	
#include "fz_model_core_api.h"	
#include "fz_fuim_api.h"
#include "fz_api.h"
#include "fz_proj_api.h"

/* @doc EXTERNAL palt_cbak */

#ifdef __cplusplus
extern "C" {
#endif




/********************************************************************************************************

	@module

	@desc	A palette is a floating window that contains a variety of interface elements. 
			Palettes are flexible extensions as a lot of functionality can be included in a palette. The interface 
			of the palette is controlled by the extension through a formZ user interface (fuim) template. The palette is 
			automatically added as a menu item to a group at the bottom of the Palettes menu. To make 
			portions of the functionality of the palette available to key shortcuts, a separate system command 
			plugin function set must be provided.

			System palettes are used for extensions that are not tied to a project or window. 
			System palettes are defined using the FZ_PALT_CBAK_SYST_FSET_TYPE 
			and the fz_palt_cbak_syst_fset function set as described in the following 
			sections. The user opens or closes a system palette from a menu item in the Palette menu or 
			through a key shortcut. 


	@toc	cmds_cbak_palt_syst

*********************************************************************************************************/

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette initialization (Required). This function is called once by formZ as soon as the palette 
				is registered.
				
				Returning an error other than FZRT_NOERR will cause the palette NOT to be registered

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_syst_finit

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_init_func)(
	void
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette finalization (Optional). This function is called once by formZ when the palette is unregistered at termination.
				This function is useful for finalizing any data that was initialized in fz_palt_cbak_syst_init.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_syst_init

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_finit_func)(
	void
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette name (Recommended). This function is called by formZ to get the name of the palette. It is recommended that 
				the name is stored in .fzr file so that it is localizable. If this function is not provided, 
				the name of the extension file will be used.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_syst_uuid,fz_palt_cbak_syst_help

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_name_func)(	
	char 		*name, 					/* @parm	<out> | string to be filled with the palette name 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of name parameter	*/			
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette id (Recommended). This function is called by formZ to get the id (uuid) of the palette. 
				This unique id is used by formZ to maintain all persistent information about the palette. If this 
				function is not provided, formZ will assign a default id. Note that there are a limited number of 
				default ids and if the number of palettes changes from session to session, the id will not be maintained. 
				In this situation, any persistent information (preferences for example) will be lost. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_syst_name,fz_palt_cbak_syst_help

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_uuid_func)(
	fzrt_UUID_td 	uuid 				/* @parm	<out> | uuid to be filled with the palette id 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette help (Recommended). This function is called by formZ to get the help text of the palette. 
				It is recommended that the help text is stored in .fzr file so that it is localizable. 
				If this function is not provided, no help string is shown. Help text is used in the 
				Help dialogs and the Key shortcuts manager.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_syst_name,fz_palt_cbak_syst_uuid

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_help_func)(	
	char 		*help, 					/* @parm	<out> | string to be filled with the help text 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of help text parameter	*/			
	);


	

/********************************************************************************************************

	@fsfunc_t

	@desc		Palette Options interface template function (Required)
				This function is called by formZ to generate the interface for the options for a system palette.<nl>
				
				The interface is constructed by calls to FUIM template functions

	@return		-1 for failure, ! = -1 for success

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_iface_tmpl_func)(
	fz_fuim_tmpl_ptr 				tmpl_ptr,			/* @parm	<in> | pointer to formZ fuim template 	*/	
	fzrt_ptr						tmpl_data			/* @parm	<in> | pointer to formZ fuim template data 	*/	
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Preference file I/O (Optional). This function is called by formZ to read and write the palettes preference information 
				when a preference file or the defaults file is read or written. The preference files use an IFF type of file format. 
				
				Version control is handled through the version parameter. When writing a block you should set the version variable to
				the variant of data that you are writing. The default is zero. It is recommended that you increment this number each time
				when you change the format of the data that you write to the stream. When reading the block, the version parameter
				contains the version which was used to write the block. You can then take the appropriate action to read the variant 
				of data in the block. 

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_pref_io_func)(
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 			/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);

/************************************************************************

	@fsfunc_t

	@desc		Instructs a formZ extension to register its help data.
							
	@avail		6.0.0.0

	@toc		cmds_cbak_palt_syst

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_palt_cbak_syst_pdf_help_func)(void);

/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to get the default, min and max sizes				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_size_func)(
	fzrt_int	*def_width,			/* @parm	<out> | palette window default width 	*/	
	fzrt_int	*def_height,		/* @parm	<out> | palette window default height 	*/		
	fzrt_int	*min_width,			/* @parm	<out> | palette window min width 	*/	
	fzrt_int	*min_height,		/* @parm	<out> | palette window min height 	*/		
	fzrt_int	*max_width,			/* @parm	<out> | palette window max width 	*/	
	fzrt_int	*max_height			/* @parm	<out> | palette window max height 	*/		
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to when the palette size is changed				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_resize_func)(
	fzrt_int	new_width,			/* @parm	<in> | new palette window width 	*/	
	fzrt_int	new_height			/* @parm	<in> | new palette window height 	*/		
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to when the palette's visibility is chnaged				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_syst

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_syst_visible_func)(
	fzrt_boolean				is_visible			/* @parm	<in> | is nthe palette now visible or not 	*/		
	);


/************************************************************************

	@plfset	fz_palt_cbak_syst_fset

	@desc	Function set for System Palettes. The functions in this set are supplied by the plugin or script to provide
			the functionality for the palette.

	@avail	5.0.0.0

	@toc	cmds_cbak_palt_syst

*************************************************************************/

/********************************************************************************************************

	@fsdef	formZ System Palette callback function set id.

*********************************************************************************************************/
/* UUID : {30dcda43-cc6d-44f5-afe1-c5b190a51aaf} */
#define FZ_PALT_CBAK_SYST_FSET_TYPE		"\x30\xdc\xda\x43\xcc\x6d\x44\xf5\xaf\xe1\xc5\xb1\x90\xa5\x1a\xaf"	/* Reference 2554 */
/********************************************************************************************************

	@fsdef	formZ System Palette callback function set name.

*********************************************************************************************************/
#define FZ_PALT_CBAK_SYST_FSET_NAME		"formZ System Palette Callback Functions"
/********************************************************************************************************

	@fsdef	formZ System Palette callback function set version.

*********************************************************************************************************/
#define FZ_PALT_CBAK_SYST_FSET_VERSION	FZPL_VERS_MAKE(6,0,0,0)


typedef struct fz_palt_cbak_syst_fset
{	
	fzpl_fset_prvt_data_td					prvt_data;

	fz_palt_cbak_syst_init_func			fz_palt_cbak_syst_init;				/* @fsfunc */
	fz_palt_cbak_syst_finit_func		fz_palt_cbak_syst_finit;				/* @fsfunc */
	fz_palt_cbak_syst_name_func			fz_palt_cbak_syst_name;				/* @fsfunc */
	fz_palt_cbak_syst_uuid_func			fz_palt_cbak_syst_uuid;				/* @fsfunc */
	fz_palt_cbak_syst_help_func			fz_palt_cbak_syst_help;				/* @fsfunc */
	
	fz_palt_cbak_syst_iface_tmpl_func	fz_palt_cbak_syst_iface_tmpl;			/* @fsfunc */

	fz_palt_cbak_syst_pref_io_func		fz_palt_cbak_syst_pref_io;			/* @fsfunc */	
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_palt_cbak_syst_pdf_help_func			fz_palt_cbak_syst_pdf_help;			/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 7.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_palt_cbak_syst_size_func				fz_palt_cbak_syst_size;			/* @fsfunc */
	fz_palt_cbak_syst_resize_func			fz_palt_cbak_syst_resize;			/* @fsfunc */
	fz_palt_cbak_syst_visible_func			fz_palt_cbak_syst_visible;			/* @fsfunc */

} fz_palt_cbak_syst_fset;

/********************************************************************************************************

	@module

	@desc	A palette is a floating window that contains a variety of interface elements. 
			Palettes are flexible extensions as a lot of functionality can be included in a palette. The interface 
			of the palette is controlled by the extension through a formZ user interface (fuim) template. The palette is 
			automatically added as a menu item to a group at the bottom of the Palettes menu. To make 
			portions of the functionality of the palette available to key shortcuts, a separate project 
			command plugin function set must be provided.

			Project palettes are defined using the FZ_PALT_CBAK_PROJ_FSET_TYPE 
			and the fz_palt_cbak_proj_fset function sets. The user opens or closes a project palette 
			from a menu item in the Palette menu or through a key shortcut. Project palettes 
			are dependent on a project window and are expected to function on the provided project window. 
			Project palettes are unavailable when there is no open project window. 
			Project palettes may store data in the formZ 
			project file at the project or window level.

	@toc	cmds_cbak_palt_proj

*********************************************************************************************************/


/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette information (Required). This function is called once by formZ when the palette 
				is successfully loaded and registered immediately after the initialization function (if
				provided). It establishes the level and kind of the palette.<nl>
				
				The level parameter indicates the context of the palette (project, modeling, drafting, window, etc.).
				formZ uses the value in this parameter to determine when the palette should be shown and when it should
				be updated.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_info_func)(
	fz_proj_level_enum 		*level 			/* @parm	<out> | to be filled with the palette level 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette initialization (Optional). This function is called once by formZ as soon as the palette 
				is registered.
				
				Returning and error other than FZRT_NOERR will cause the palette NOT to be registered

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_proj_finit

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_init_func)(
	void
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette finalization (Optional). This function is called once by formZ when the palette is unregistered at termination.
				This function is useful for finalizing any data that was initialized in fz_palt_cbak_proj_init.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_proj_init

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_finit_func)(
	void
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette name (Recommended). This function is called by formZ to get the name of the palette. It is recommended that 
				the palette name is stored in .fzr file so that it is localizable. If this function is not provided, 
				the name of the plugin, name of the plugin file, name of the script, or name of the script file 
				will be used.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_proj_uuid,fz_palt_cbak_proj_help

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_name_func)(	
	char 		*name, 					/* @parm	<out> | string to be filled with the palette name 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of name parameter	*/			
	);


/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette id (Recommended). This function is called by formZ to get the id (uuid) of the palette. 
				This unique id is used by formZ to maintain all persistent information about the palette. If this 
				function is not provided, formZ will assign a default id. Note that there are a limited number of 
				default ids and if the number of palettes changes from session to session, the id will not be maintained. 
				In this situation, any persistent information (preferences for example) will be lost. 

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_proj_name,fz_palt_cbak_proj_help

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_uuid_func)(
	fzrt_UUID_td 	uuid 				/* @parm	<out> | uuid to be filled with the palette id 	*/		
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Palette help (Recommended). This function is called by formZ to get the help text of the palette. 
				It is recommended that the help text is stored in .fzr file so that it is localizable. 
				If this function is not provided, no help string is shown. Help text is used in the 
				Help dialogs and the Key shortcuts manager.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_palt_cbak_proj_name,fz_palt_cbak_proj_uuid

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_help_func)(	
	char 		*help, 					/* @parm	<out> | string to be filled with the help text 	*/		
	fzrt_int		max_len				/* @parm	<in> | length of help text parameter	*/			
	);


	

/********************************************************************************************************

	@fsfunc_t

	@desc		Palette Options interface template function (Recommended)
				This function is called by formZ to generate the interface for the options for a project palette. 
				
				The interface is constructed by calls to FUIM template functions

	@return		-1 for failure, ! = -1 for success

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_iface_tmpl_func)(
	fzrt_int						windex,				/* @parm	<in> | project window index */
	fz_fuim_tmpl_ptr 			tmpl_ptr,			/* @parm	<in> | pointer to formZ fuim template manager 	*/		
	fzrt_ptr					tmpl_data			/* @parm	<in> | pointer to formZ fuim template data 	*/	
	);



/********************************************************************************************************

	@fsfunc_t	

	@desc		Preference file I/O (Optional). This function is called by formZ to read and write the palettes preference information 
				when a preference file or the defaults file is read or written. The preference files use an IFF type of file format. 
				
				Version control is handled through the version parameter. When writing a block you should set the version variable to
				the variant of data that you are writing. The default is zero. It is recommended that you increment this number each time
				when you change the format of the data that you write to the stream. When reading the block, the version parameter
				contains the version which was used to write the block. You can then take the appropriate action to read the variant 
				of data in the block. 

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_pref_io_func)(
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 			/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		formZ calls this function to read and write any palette specific project data to a formZ project
				file. This function is called once when reading and writing formZ project files. The file IO is
				performed using the IO streams (iost) interface. This interface provides functions for reading and
				writing data from a file (stream) and handles all cross platform endian issues. The iost
				parameter is the pointer to the formZ project file and should be used in all IO Stream function
				calls. The IO Stream functions are fully documented in the formZ API reference.
				The dir parameter indicates if the file is being written with a value of FZ_IOST_WRITE or read
				with a value of FZ_IOST_READ. The version parameter should return the version of the data
				that was written when outputting a file. When reading a file, the version parameter contains the
				version of the data that was written to in the file (and hence being read). The size parameter is
				only valid when dir == FZ_IOST_READ (read). This is the size of the data that was written in
				the file.
				It is the responsibility of the plugin to maintain version changes of the plugin data.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_data_io_func)(
	fzrt_int					windex,				/* @parm	<in> | project window index */
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 				/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);



/********************************************************************************************************

	@fsfunc_t	

	@desc		formZ calls this function to read and write any palette specific project window data to a formZ
				project file. This function is called once for each window in the project when reading and writing
				formZ project files. The file IO is performed using the IO streams (iost) interface. This interface
				provides functions for reading and writing data from a file (stream) and handles all cross platform
				endian issues. The iost parameter is the pointer to the formZ Project file and should be used in
				all IO Stream function calls. The IO Stream functions are fully documented in the formZ API
				reference.
				The dir parameter indicates if the file is being written with a value of FZ_IOST_WRITE or read
				with a value of FZ_IOST_READ. The version parameter should return the version of the data
				that was is written when writing a file. When reading a file, the version parameter contains the
				version of the data that was written to in the file (and hence being read). The size parameter is
				only valid when dir == FZ_IOST_READ (read). This is the size of the data that was written in
				the file.
				It is the responsibility of the plugin to maintain version changes of the plugin data.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_wind_data_io_func)(
	fzrt_int					windex,				/* @parm	<in> | project window index */
	fz_iost_ptr 			iost, 				/* @parm	<in> | io stream point for current stream. */		
	fz_iost_dir_td_enum 			dir,				/* @parm	<in> | io direction for current stream. */		
	fzpl_vers_td * const version, 				/* @parm	<in_out> | 	if dir == FZ_IOST_READ then this is the version of the block being read. 
																		if dir == FZ_IOST_WRITE then this is the version of the block that was written (default == 0). */
	fzrt_unsigned_int 			size 				/* @parm	<in> | if dir == FZ_IOST_READ then this is the size of the block being read */
	);

/************************************************************************

	@fsfunc_t

	@desc		Instructs a formZ extension to register its help data.
							
	@avail		6.0.0.0

	@toc		cmds_cbak_palt_proj

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_palt_cbak_proj_pdf_help_func)(void);

/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to get the default, min and max sizes				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_size_func)(
	fzrt_int	*def_width,			/* @parm	<out> | palette window default width 	*/	
	fzrt_int	*def_height,		/* @parm	<out> | palette window default height 	*/		
	fzrt_int	*min_width,			/* @parm	<out> | palette window min width 	*/	
	fzrt_int	*min_height,		/* @parm	<out> | palette window min height 	*/		
	fzrt_int	*max_width,			/* @parm	<out> | palette window max width 	*/	
	fzrt_int	*max_height			/* @parm	<out> | palette window max height 	*/		
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to when the palette size is changed				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_resize_func)(
	fzrt_int	new_width,			/* @parm	<in> | new palette window width 	*/	
	fzrt_int	new_height			/* @parm	<in> | new palette window height 	*/		
	);


/********************************************************************************************************

	@fsfunc_t

	@desc		formZ calls this function to when the palette's visibility is chnaged				
				

	@return		-1 for failure, ! = -1 for success

	@avail		7.0.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_visible_func)(
	fzrt_boolean				is_visible			/* @parm	<in> | is nthe palette now visible or not 	*/		
	);

	
/********************************************************************************************************

	@enum	

	@desc	Defines bits than can be set in the palette attributes function
 
	@avail	8.7.0.0
			
	@see	<auto>

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef enum fz_palt_attrs_enum
{
	FZ_PALT_ATTR_VISIBLE_BIT = 0,		/*	@emem make palette visible at birth */
	FZ_PALT_ATTR_PRIVATE_BIT,   		/*	@emem Private palette, not in Palettes manu	*/
	FZ_PALT_ATTR_MAX,
} fz_palt_attrs_enum;

/********************************************************************************************************

	@fsfunc_t

	@desc		returns optional palette attributes
				

	@return		-1 for failure, ! = -1 for success

	@avail		8.7.0.0

	@see		

	@exfile		

	@toc		cmds_cbak_palt_proj

*********************************************************************************************************/
typedef fzrt_error_td (*fz_palt_cbak_proj_attributes_func)(
	fzrt_int						*attrs			/* @parm	<out> | attribute bits 	*/
	);

/************************************************************************

	@plfset	fz_palt_cbak_proj_fset

	@desc	Function set for Project Palettes. The functions in this set are supplied by the plugin or script to provide
			the functionality for the palette.

	@avail	5.0.0.0

	@toc	cmds_cbak_palt_proj

*************************************************************************/

/********************************************************************************************************

	@fsdef	formZ Project Palette callback function set id.

*********************************************************************************************************/
/* UUID : {4825dde6-bdc0-4293-897c-b1c281fbce43} */
#define FZ_PALT_CBAK_PROJ_FSET_TYPE		"\x48\x25\xdd\xe6\xbd\xc0\x42\x93\x89\x7c\xb1\xc2\x81\xfb\xce\x43"	/* Reference 2555 */
/********************************************************************************************************

	@fsdef	formZ Project Palette callback function set name.

*********************************************************************************************************/
#define FZ_PALT_CBAK_PROJ_FSET_NAME		"formZ Project Palette Callback Functions"
/********************************************************************************************************

	@fsdef	formZ Project Palette callback function set version.

*********************************************************************************************************/
#define FZ_PALT_CBAK_PROJ_FSET_VERSION	FZPL_VERS_MAKE(8,7,0,0)


typedef struct fz_palt_cbak_proj_fset
{	
	fzpl_fset_prvt_data_td						prvt_data;

	fz_palt_cbak_proj_info_func				fz_palt_cbak_proj_info;				/* @fsfunc */
	fz_palt_cbak_proj_init_func				fz_palt_cbak_proj_init;				/* @fsfunc */
	fz_palt_cbak_proj_finit_func			fz_palt_cbak_proj_finit;			/* @fsfunc */
	fz_palt_cbak_proj_name_func				fz_palt_cbak_proj_name;				/* @fsfunc */
	fz_palt_cbak_proj_uuid_func				fz_palt_cbak_proj_uuid;				/* @fsfunc */
	fz_palt_cbak_proj_help_func				fz_palt_cbak_proj_help;				/* @fsfunc */
	
	fz_palt_cbak_proj_iface_tmpl_func		fz_palt_cbak_proj_iface_tmpl;		/* @fsfunc */
	
	fz_palt_cbak_proj_pref_io_func			fz_palt_cbak_proj_pref_io;			/* @fsfunc */	
	fz_palt_cbak_proj_data_io_func			fz_palt_cbak_proj_data_io;			/* @fsfunc */	
	fz_palt_cbak_proj_wind_data_io_func		fz_palt_cbak_proj_wind_data_io;		/* @fsfunc */	
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_palt_cbak_proj_pdf_help_func			fz_palt_cbak_proj_pdf_help;			/* @fsfunc */

	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 7.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_palt_cbak_proj_size_func				fz_palt_cbak_proj_size;				/* @fsfunc */
	fz_palt_cbak_proj_resize_func			fz_palt_cbak_proj_resize;			/* @fsfunc */
	fz_palt_cbak_proj_visible_func			fz_palt_cbak_proj_visible;			/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8.7.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	fz_palt_cbak_proj_attributes_func		fz_palt_cbak_proj_attributes;		/* @fsfunc */
	
} fz_palt_cbak_proj_fset;

#ifdef __cplusplus
}
#endif


#endif /* _FORMZ_PALT_API_H  */
