/*************************************************************************

	formZ by autodessys, Inc.
	Copyright c 1994 autodessys, Inc.  All rights reserved.
	
	
	File :	fz_syst_api.h
	
	By :	
	
	Created on :  		15 November 2001
	Last Updated on :  	15 November 2001
	
	Description :	
					
***************************************************************************/

#ifndef __FZ_SYST_API_H
#define __FZ_SYST_API_H 1


#include "fz_api.h"
#include "fzpl_plugin.h"
#include "fz_type_api.h"
#include "fz_file_paths_api.h"

#ifdef __cplusplus
extern "C" {
#endif

/* @doc EXTERNAL cbak_notf */

/********************************************************************************************************

	@module

	@desc	formZ system management functions. These functions are used to access data and perform system operations
			(independent of any particular formZ project).

	@toc	syst

*********************************************************************************************************/

/********************************************************************************************************

	@enum	

	@desc	Selector enum for the project information. 
			
	@avail	5.0.0.0
			
	@see	<auto>
			
	@toc	syst

*********************************************************************************************************/
typedef enum fz_appl_info_enum
{	FZ_APPL_VERSION =0,		/*	@emem version string in the form of "v [major].[minor].[rev][release letter][fix] (Build [build number])". <nl>
									[major]          : major version number. <nl>
									[major]          : minor version number. <nl>
									[rev]            : revision version number. <nl>
									[release letter] : a = alpha, b = beta, g = gamma, f = final. <nl>
									[fix]            : bug fix version number. <nl>
									[build number]   : sequential build number. <nl>			
									For example "v 5.5.1f0 (Build 5187)"
									*/
	FZ_APPL_COPYRIGHT,			/*	@emem copyright string */
	FZ_APPL_NAME,				/*	@emem application logical name (formZ, formZ RenderZone etc..)*/
	FZ_APPL_USER_NAME,			/*	@emem application user name (entered during installation) */
	FZ_APPL_USER_COMPANY,		/*	@emem application user company name (entered during installation) */
	FZ_APPL_USER_SERIAL,		/*	@emem application user serial number in the form of Z-XXXX-XXXXXXX */
	FZ_APPL_BUILD_NUMBER,		/*	@emem build number string. Can be easily converted from the string 
										  to a fzrt_int via the standard c function atol */
	
	FZ_APPL_FORMZ_FOLDER,		/*	@emem localized name for the "formz Folder" in documents folder */
	FZ_APPL_MAX
} fz_appl_info_enum;

/********************************************************************************************************

	@fsfunc_t

	@desc		Get the application information strings.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_syst_appl_get_info_func)(
	fz_appl_info_enum	which,			/* @parm	<in> | selector for string */
	char 			*info, 			/* @parm	<out> | pointer to fill with selected string  */
	fzrt_int 			info_length 	/* @parm	<in> | length of the info parameter  */
	);


/********************************************************************************************************

	@fsfunc_t

	@desc		Get the application file location.

	@return		Error codes

	@avail		5.0.0.0

	@see		

	@exfile		

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_appl_get_floc_func)( 
	fzrt_floc_ptr floc 		/* @parm	<out> | pointer to fill with application floc string  */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Executes a plugin system level utility.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_proj_script_exec_util, fz_syst_script_exec_util, fz_proj_plugin_exec_util

	@exfile		

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_plugin_exec_util_func)(
			fzrt_floc_ptr		floc,			/* @parm <in> | The file location of the plugin file containing the utility */
			fzrt_UUID_td		syst_id			/* @parm <in> | The id of the utility to run. */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Executes a script system level utility.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_proj_script_exec_util, fz_syst_plugin_exec_util, fz_proj_plugin_exec_util

	@exfile		

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_script_exec_util_func)(
			fzrt_floc_ptr		floc			/* @parm <in> | The file location of the script file containing the utility */
			);

/********************************************************************************************************

	@enum	

	@desc	Extended cursor option parameters.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_extd_crsr_opts_parm_enum
{	FZ_SYST_EXTD_CRSR_SHOW_SNAP = 0,		/*	@emem 		Indicates the extended cursor is shown at the snapped position. 
															If set, the extended cursor is shown at the snapped position if grid snap is on, 
															otherwise the extended cursor is shown at the mouse location.<nl>
															If not set, the cursor will be shown at the mouse position even if grid snap is on.
												@ptype		fzrt_boolean
												@default	TRUE
											*/

	FZ_SYST_EXTD_CRSR_SIZE_CUSTOM,			/*	@emem 		Indicates how the extended cursor size is shown. 
															If this is set, the cursor is extended by the value in the custom pixels, 
															otherwise the cursor is extended to the limits of the window.
												@ptype		fzrt_boolean
												@default	FALSE
											*/

	FZ_SYST_EXTD_CRSR_SIZE_CUSTOM_SIZE,		/*	@emem 		Indicates the number of pixels the extended cursor size is drawn when 
															FZ_SYST_EXTD_CRSR_SIZE_CUSTOM is set.
												@ptype		fzrt_int
												@default	40
												@rimin		1
											*/

	FZ_SYST_EXTD_CRSR_SIZE_MARKER,			/*	@emem 		Indicates that a small square, which is aligned parallel to the active plane, is 
															drawn around the location of the extended cursor. 
															If not set, the square isn't drawn.
												@ptype		fzrt_boolean
												@default	TRUE
											*/

	FZ_SYST_EXTD_CRSR_LOCK,					/*	@emem 		Indicates that the extended cursor availabilty is locked. This prevents other code
															from turning the extended cursor on or off, and disables the extended cursor menu item.
												@ptype		fzrt_boolean
												@default	TRUE
											*/
	FZ_SYST_EXTD_CRSR_MAX
} fz_syst_extd_crsr_opts_parm_enum;

/********************************************************************************************************

	@fsfunc_t

	@desc		Gets an extended cursor option.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_extd_crsr_opts,fz_syst_get_extd_crsr,fz_syst_set_extd_crsr

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_get_extd_crsr_opts_func)(
			fz_syst_extd_crsr_opts_parm_enum	which,			/* @parm <in> | parameter identifier */
			fz_type_td							*value			/* @parm <out> | parameter data */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Sets an extended cursor option.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_extd_crsr_opts,fz_syst_get_extd_crsr,fz_syst_set_extd_crsr

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_set_extd_crsr_opts_func)(
			fz_syst_extd_crsr_opts_parm_enum	which,			/* @parm <in> | parameter identifier */
			fz_type_td							*value			/* @parm <in> | parameter data */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Gets whether the extended cursor is on or off.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_extd_crsr,fz_syst_get_extd_crsr_opts,fz_syst_set_extd_crsr_opts

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_get_extd_crsr_func)(
			fzrt_boolean	*value			/* @parm <out> | is the extended cursor on<nl>
															TRUE if on, FALSE if off */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Sets the extended cursor to be on or off.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_extd_crsr,fz_syst_get_extd_crsr_opts,fz_syst_set_extd_crsr_opts

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_set_extd_crsr_func)(
			fzrt_boolean	value			/* @parm <in> | set the extended cursor<nl>
															TRUE if on, FALSE if off */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Gets whether autoscrolling is on or off.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_autoscroll

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_get_autoscroll_func)(
			fzrt_boolean	*value			/* @parm <out> | is autoscrolling on<nl>
															TRUE if on, FALSE if off */
			);

/********************************************************************************************************

	@fsfunc_t

	@desc		Sets autoscrolling to be on or off.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_autoscroll

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_set_autoscroll_func)(
			fzrt_boolean	value			/* @parm <in> | set autoscrolling<nl>
															TRUE if on, FALSE if off */
			);


/********************************************************************************************************

	@enum	

	@desc	Undo type option parameters.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_undo_type_enum
{	FZ_SYST_UNDO_TYPE_UNLIMITED = 0,		/*	@emem 		Indicates undos are unlimited.	*/

	FZ_SYST_UNDO_TYPE_OPERATIONS,			/*	@emem 		Indicates undos are limited by a number of operations.	*/

	FZ_SYST_UNDO_TYPE_DISK,					/*	@emem 		Indicates undos are limited by a set amount of disk space (in kilobytes KB).	*/

	FZ_SYST_UNDO_TYPE_TIME,					/*	@emem 		Indicates undos are limited by an amount of time (in minutes).	*/

	FZ_SYST_UNDO_TYPE_MAX
} fz_syst_undo_type_enum;

/********************************************************************************************************

	@enum	

	@desc	Undo option parameters.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_undo_opts_parm_enum
{	FZ_SYST_UNDO_ON = 0,		/*	@emem 		Indicates undos are on.
									@ptype		fzrt_boolean
									@default	TRUE
								*/

	FZ_SYST_UNDO_TYPE,			/*	@emem 		Indicates the type of undos to use.
									@ptype		fz_syst_undo_type_enum
									@default	FZ_SYST_UNDO_TYPE_UNLIMITED
								*/

	FZ_SYST_UNDO_OPERATIONS,	/*	@emem 		Indicates the number of operations available to undo.
									@ptype		fzrt_int
									@default	20
								*/

	FZ_SYST_UNDO_DISK,			/*	@emem 		Indicates the amount of disk space, in kilobytes (KB), available to undo.
									@ptype		fzrt_int
									@default	1024
								*/

	FZ_SYST_UNDO_TIME,			/*	@emem 		Indicates the amount of time, in minutes, available to undo.
									@ptype		fzrt_int
									@default	20
								*/

	FZ_SYST_UNDO_SHOW,			/*	@emem 		Indicates the number of operations to show in the undo list.
									@ptype		fzrt_int
									@default	10
								*/

	FZ_SYST_UNDO_SAVE,			/*	@emem 		Indicates undos are saved with the project.
									@ptype		fzrt_boolean
									@default	FALSE
								*/

	FZ_SYST_UNDO_RESET,			/*	@emem 		Indicates undos are reset when the project is saved.
									@ptype		fzrt_boolean
									@default	TRUE
								*/

	FZ_SYST_UNDO_MAX
} fz_syst_undo_opts_parm_enum;

/************************************************************************

	@fsfunc_t	
	
	@desc		Gets an undo option parameter.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_undo_parm_data

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*************************************************************************/
typedef fzrt_error_td	 (FZRT_SPEC *fz_syst_get_undo_parm_data_func) (	
	fz_syst_undo_opts_parm_enum		which,			/* @parm <in> | parameter identifier	*/
	fz_type_td						*value			/* @parm <out> | parameter data			*/
	);

/************************************************************************

	@fsfunc_t	
	
	@desc		Sets an undo option parameter.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_undo_parm_data

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*************************************************************************/
typedef fzrt_error_td	 (FZRT_SPEC *fz_syst_set_undo_parm_data_func) (	
	fz_syst_undo_opts_parm_enum		which,			/* @parm <in> | parameter identifier	*/
	fz_type_td						*value			/* @parm <in> | parameter data			*/
	);


/********************************************************************************************************

	@enum	

	@desc	Animation Export parameters.  This enum identifies which
			option is wanted to retrieve or change.  This is used with the fz_syst_anim_expt_get_parm,
			fz_syst_anim_expt_set_parm functions.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_anim_expt_parm_enum
{
	FZ_SYST_ANIM_EXPT_PARM_FRAMES_MENU,		/*	@emem		What choice of frames to use for animation.
												@ptype		fz_anim_frames_enum
												@default	FZ_ANIM_FRAMES_ALL
											*/
	FZ_SYST_ANIM_EXPT_PARM_FRAME_START,		/*	@emem		Starting frame for range of frames to use for animation.
												@ptype		fzrt_int
												@default	0
												@rimin_imax	0 | FZ_SYST_ANIM_EXPT_PARM_FRAME_END
											*/
	FZ_SYST_ANIM_EXPT_PARM_FRAME_END,		/*	@emem		Ending frame for range of frames to use for animation.
															If a number larger than the last frame index of the
															source animation file is used during exportation,
															it will be capped to the last frame index.
												@ptype		fzrt_int
												@default	last frame index
												@rimin_imax	FZ_SYST_ANIM_EXPT_PARM_FRAME_START | last frame index
											*/
	FZ_SYST_ANIM_EXPT_PARM_FRAME_INCR,		/*	@emem		Whether or not to skip/increment/jump a certain number of frames between frames.
												@ptype		fzrt_boolean
												@default	FALSE
											*/
	FZ_SYST_ANIM_EXPT_PARM_FRAME_INCR_INP,	/*	@emem		Number of frames to increment between frames
												@ptype		fzrt_int
												@default	10
												@rimin_imax	1 | number of frames
											*/
	FZ_SYST_ANIM_EXPT_PARM_FRAME_LIST,		/*	@emem		Comma separated list of ranges of frames to use for animation.
												@ptype		char*
												@default	"0 - 299"
											*/
	FZ_SYST_ANIM_EXPT_PARM_MAX
}fz_syst_anim_expt_parm_enum;

/********************************************************************************************************

	@fsfunc_t	

	@desc		Gets the parameter indicated by the parameter id of the "Export Animation" options.
				

	@return		Error codes.

	@avail		5.0.0.0

	@see		fz_syst_anim_expt_set_parm, fz_syst_anim_export

	@exfile

	@toc		syst	

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_syst_anim_expt_get_parm_func)(
	fz_syst_anim_expt_parm_enum	parm_id,		/* @parm    <in> | the id of the parameter to retrieve */
	fz_type_td*					data			/* @parm	<out> | parameter data */
	);

/********************************************************************************************************

	@fsfunc_t	

	@desc		Sets the parameter indicated by the parameter id of the "Export Animation" options.
				

	@return		Error codes.

	@avail		5.0.0.0

	@see		fz_syst_anim_expt_get_parm, fz_syst_anim_export

	@exfile		<scdir>/Utilities | util_anim_export.fsl

	@toc		syst	

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_syst_anim_expt_set_parm_func)(
	fz_syst_anim_expt_parm_enum	parm_id,		/* @parm    <in> | the id of the parameter to change */
	fz_type_td*					data			/* @parm	<in> | parameter data */
	);



/********************************************************************************************************

	@fsfunc_t	

	@desc		Exports an animation of a formZ animation file (.fan) to another animation file format.
				Instead of passing in the dst_floc with an extension, the correct extension will
				be appended to the destination file name based on the ref_id parameter.  Among the valid
				file formats (if supported by the operating system) are QuickTime, AVI, and a number of
				image formats like .bmp and .tiff.

	@return		Error codes.

	@avail		5.0.0.0

	@see		fz_syst_anim_expt_get_parm, fz_syst_anim_expt_set_parm

	@exfile		<scdir>/Utilities | util_system_opts.fsl
	@exfile		<scdir>/Utilities | util_anim_export.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_syst_anim_export_func)(
	fzrt_floc_ptr				src_floc,		/* @parm	<in> | source formZ animation file (.fan). */
	fzrt_floc_ptr				dst_floc,		/* @parm	<in> | destination file for exported animation (without an extenstion). */
	fz_ffmt_ref_td				ref_id			/* @parm	<in> | destination file format type. */
	);

/********************************************************************************************************

	@enum	

	@desc	Entities save options. 
			Used by fz_syst_get_proj_opts_parm_data and fz_syst_set_proj_opts_parm_data 
			when the parameter id is FZ_PROJ_SAVE_OPTS_MODEL_ENTITIES or FZ_PROJ_SAVE_OPTS_DRAFT_ENTITIES.<nl>
			Note : If either FZ_SYST_SAVE_ENTITIES_VISIBLE or FZ_SYST_SAVE_ENTITIES_PICKED is attempted to be 
			set, but there are no visible or picked entities, respectively, in the active project, 
			the option will default back to FZ_SYST_SAVE_ENTITIES_ALL.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_save_entities_enum
{	FZ_SYST_SAVE_ENTITIES_ALL = 0,		/*	@emem	Save all entities.	*/

	FZ_SYST_SAVE_ENTITIES_VISIBLE,		/*	@emem	Save only visible entities.	*/

	FZ_SYST_SAVE_ENTITIES_PICKED,		/*	@emem	Save only picked entities.	*/

	FZ_SYST_SAVE_ENTITIES_NONE,			/*	@emem	Save no entities.	*/

	FZ_SYST_SAVE_ENTITIES_MAX
} fz_syst_save_entities_enum;

/********************************************************************************************************

	@enum	

	@desc	Project save option parameters. 
			Used for the "which" argument of fz_syst_get_print_setup_parm_data and fz_syst_set_print_setup_parm_data.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_save_opts_parm_enum
{	FZ_SYST_SAVE_OPTS_BACKUP = 0,					/*	@emem		Indicates if a backup version is kept.
														@ptype		fzrt_boolean
														@default	TRUE
													*/

	FZ_SYST_SAVE_OPTS_COMPRESS,						/*	@emem		Indicates if the saved file is to be compressed.
														@ptype		fzrt_boolean
														@default	FALSE
													*/

	FZ_SYST_SAVE_OPTS_VERSION,						/*	@emem		Indicates what file version to save as.<nl>
																	FZPL_VERS_MAKE must be called before setting a version.<nl>
																	FZPL_VERS_GET_MAJ, FZPL_VERS_GET_MIN, FZPL_VERS_GET_BUG_MAJ, and 
																	FZPL_VERS_GET_BUG_MIN can be called to get individual values for the 
																	version after getting it.
														@ptype		fzpl_vers_td
													*/

	FZ_SYST_SAVE_OPTS_MODEL_ENTITIES,				/*	@emem		Indicates what part of the modeling project to save.
														@ptype		fz_syst_save_entities_enum
														@default	FZ_SYST_SAVE_ENTITIES_ALL
													*/

	FZ_SYST_SAVE_OPTS_DRAFT_ENTITIES,				/*	@emem		Indicates what part of the drafting project to save.
														@ptype		fz_syst_save_entities_enum
														@default	FZ_SYST_SAVE_ENTITIES_ALL
													*/

	FZ_SYST_SAVE_OPTS_SYMBOLS,						/*	@emem		Indicates if symbol libraries are to be saved with the project. 
																	If there are no symbols in the active project, setting this options will have no effect.
														@ptype		fzrt_boolean
														@default	FALSE
													*/

	FZ_SYST_SAVE_OPTS_SYMBOL_LIBRARY_ALL,			/*	@emem		Indicates if all attached symbol libraries are to be saved with the project.
																	If FALSE, only libraries with referenced definitions will be saved.
														@ptype		fzrt_boolean
														@default	TRUE
													*/

	FZ_SYST_SAVE_OPTS_SYMBOL_SYMBOL_ALL,			/*	@emem		Indicates if all symbol definitions are to be saved with the project.
																	If FALSE, only referenced symbol definitions will be saved.
														@ptype		fzrt_boolean
														@default	TRUE
													*/

	FZ_SYST_SAVE_OPTS_IMAGE,						/*	@emem		Indicates if images are to be saved with the project. If TRUE, copies of the image files will be saved in the same folder as the project.
														@ptype		fzrt_boolean
														@default	FALSE
													*/

	FZ_SYST_SAVE_OPTS_IMAGE_REF_ONLY,				/*	@emem		Indicates if only referenced images are to be saved with the project. If TRUE, copies of the image files that are referenced by the project will be saved in the same folder as the project.
														@ptype		fzrt_boolean
														@default	TRUE
													*/

	FZ_SYST_SAVE_OPTS_FONT,							/*	@emem		Indicates if fonts are to be saved with the project. If TRUE, copies of the font files will be saved in the same folder as the project.
														@ptype		fzrt_boolean
														@default	FALSE
													*/

	FZ_SYST_SAVE_OPTS_FONT_REF_ONLY,				/*	@emem		Indicates if only referenced fonts are to be saved with the project. If TRUE, copies of the font files that are referenced by the project will be saved in the same folder as the project.
														@ptype		fzrt_boolean
														@default	TRUE
													*/

	FZ_SYST_SAVE_OPTS_MAX
} fz_syst_save_opts_parm_enum;

/********************************************************************************************************

	@fsfunc_t

	@desc		Gets formZ project options parameters.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_proj_opts_parm_data

	@exfile		<scdir>/Utilities | util_proj_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_get_proj_opts_parm_data_func)(
	fz_syst_save_opts_parm_enum		which,		/* @parm <in> | parameter identifier */
	fz_type_td						*value		/* @parm <out> | parameter data */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Sets formZ project options parameters.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_proj_opts_parm_data

	@exfile		<scdir>/Utilities | util_proj_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_set_proj_opts_parm_data_func)(
	fz_syst_save_opts_parm_enum		which,		/* @parm <in> | parameter identifier */
	fz_type_td						*value		/* @parm <in> | parameter data */
	);

/********************************************************************************************************

	@enum	

	@desc	How the colors of entities are treated when the entities are pasted from one project to another.
			
	@avail	5.0.0.0
			
	@see	<auto>

	@toc	syst

*********************************************************************************************************/
typedef enum fz_syst_color_paste_enum
{	FZ_SYST_COLOR_PASTE_ACTIVE,			/*	@emem		Use active color. 
														The pasted entity takes the color or surface style 
														that is active in the receiving project. */

	FZ_SYST_COLOR_PASTE_BEST_MATCH,		/*	@emem		Use best matching color. 
														The pasted entity takes the color or surface style 
														in the receiving project that is closest to the 
														original color. */

	FZ_SYST_COLOR_PASTE_TRANSFER,		/*	@emem		Transfer original color. 
														If the color or surface style of the entity already 
														exists in the receiving project, it is assigned that 
														color or surface style. Otherwise, the color or surface 
														style is added to the receiving project and assigned to 
														the entity. */

	FZ_SYST_COLOR_PASTE_MAX
} fz_syst_color_paste_enum;

/********************************************************************************************************

	@fsfunc_t

	@desc		Gets how the color is handled when pasting between projects.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_set_color_paste

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_get_color_paste_func)(
	fz_syst_color_paste_enum	*value		/* @parm <out> | color handling when pasting between projects. */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Sets how the color is handled when pasting between projects.

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_color_paste

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td 	 (FZRT_SPEC *fz_syst_set_color_paste_func)(
	fz_syst_color_paste_enum	value		/* @parm <in> | color handling when pasting between projects. */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Determines whether a tool uses the status of objects option in its tool palette.
				This function is designed to be called from the fz_tool_cbak_init callback function
				of a plugin or script tool. if it is not called, the tool identified by its uuid
				is assumed not to use the status of objects functionality. If this function is
				called, the opnd_opt and volm_opt parameters determine, whether the Operand option
				is used (opnd_opt == TRUE) or not (opnd_opt == FALSE) and whether the New Objects
				option is used (volm_opt == TRUE) or not (volm_opt == FALSE). These options are
				equivalent to the options presented in the Status Of Objects tabs of the respective
				modeling tools (Revolve for example).
				It is up to the tool to make sure, that the status of objects options are also executed by
				calling the respective API function (fz_objt_edit_handle_status_of_opnd, fz_objt_edit_handle_new_objt_volms).

	@return		Error codes

	@avail		5.0.0.0

	@see		fz_syst_get_color_paste

	@exfile		<scdir>/Utilities | util_system_opts.fsl

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_syst_cmnd_set_status_of_objt_func) (
	fzrt_UUID_td 	uuid, 		/* @parm <in> | The uuid of the tool for which to set the status of objects options. */
	fzrt_boolean 	opnd_opt,	/* @parm <in> | Set to TRUE, if the tool will be using the Operands option and FALSE otherwise. */ 
	fzrt_boolean 	volm_opt 	/* @parm <in> | Set to TRUE, if the tool will be using the New Objects option and FALSE otherwise. */ 
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		DEPRICATED:   Adds an entry into the formZ pdf help system.

	@return		Error codes

	@avail		6.0.0.0

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_syst_help_add_entry_func) (
	fzrt_UUID_td 	uuid, 				/* @parm <in> | The uuid of the tool, dialog, palette for which to set the help entry. */
	char 		 	*description,		/* @parm <in> | A description of tool, dialog, palette. */ 
	char 		 	*pdf_file_name,		/* @parm <in> | The name of the pdf file to link to. */ 
	char			*pdf_destination, 	/* @parm <in> | The destination string from the pdf file describing which page to link to. */ 
	char			*title_string		/* @parm <in> | A string to display to the user identifying the section title in the pdf document. */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Adds an entry into the formZ pdf help system.

	@return		Error codes

	@avail		6.0.0.0

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_syst_help_add_floc_entry_func) (
	fzrt_UUID_td 	uuid, 				/* @parm <in> | The uuid of the tool, dialog, palette for which to set the help entry. */
	char 		 	*description,		/* @parm <in> | A description of tool, dialog, palette. */ 
	fzrt_floc_ptr	pdf_path_floc,		/* @parm <in> | An floc containing the full path to the pdf file.  This is usually the same floc as the plugin. */
	char 		 	*pdf_file_name,		/* @parm <in> | The name of the pdf help file without the .pdf file extension.  FormZ will add the language specifier and the .pdf file extension. */ 
	char			*pdf_destination, 	/* @parm <in> | The destination string from the pdf file describing which page to link to. */ 
	char			*title_string		/* @parm <in> | A string to display to the user identifying the section title in the pdf document. */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Make the tool active

	@return		Error codes

	@avail		8.5.0.1

	@see		fz_syst_cmnd_get_active

	@exfile

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_syst_cmnd_set_active_func) (
	fzrt_UUID_td 	uuid 		/* @parm <in> | The uuid of the tool to be made the active. */
	);

/********************************************************************************************************

	@fsfunc_t

	@desc		Get the currently active tool

	@return		Error codes

	@avail		8.5.0.1

	@see		fz_syst_cmnd_set_active

	@exfile

	@toc		syst

*********************************************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_syst_cmnd_get_active_func) (
	fzrt_UUID_td 	uuid 		/* @parm <in> | The uuid of the tool that is active. */
	);
	
/************************************************************************

	@fset	fz_syst_fset

	@desc	formZ system function set

	@avail	5.0.0.0

	@toc	syst

*************************************************************************/

/* System Functions */
/* uuid : {6767927d-091a-4138-81cb-23cc1aefd23c} */
/********************************************************************************************************

	@fsdef	formZ system function set UUID.
	
*********************************************************************************************************/
#define FZ_SYST_FSET_TYPE		"\x67\x67\x92\x7d\x09\x1a\x41\x38\x81\xcb\x23\xcc\x1a\xef\xd2\x3c"	/* Reference 60 */

/********************************************************************************************************

	@fsdef	formZ system function set name.
	
*********************************************************************************************************/
#define FZ_SYST_FSET_NAME		"formZ system functions"

/********************************************************************************************************

	@fsdef	formZ system function set version.
	
*********************************************************************************************************/
#define FZ_SYST_FSET_VERSION	FZPL_VERS_MAKE(8,6,0,0)

typedef struct fz_syst_fset
{
	fzpl_fset_prvt_data_td					prvt_data;
		
	fz_syst_appl_get_info_func				fz_syst_appl_get_info;			/* @fsfunc */
	fz_syst_appl_get_floc_func				fz_syst_appl_get_floc;			/* @fsfunc */

	fz_syst_plugin_exec_util_func			fz_syst_plugin_exec_util;		/* @fsfunc */
	fz_syst_script_exec_util_func			fz_syst_script_exec_util;		/* @fsfunc */

	fz_syst_get_extd_crsr_opts_func			fz_syst_get_extd_crsr_opts;		/* @fsfunc */
	fz_syst_set_extd_crsr_opts_func			fz_syst_set_extd_crsr_opts;		/* @fsfunc */
	fz_syst_get_extd_crsr_func				fz_syst_get_extd_crsr;			/* @fsfunc */
	fz_syst_set_extd_crsr_func				fz_syst_set_extd_crsr;			/* @fsfunc */
	fz_syst_get_autoscroll_func				fz_syst_get_autoscroll;			/* @fsfunc */
	fz_syst_set_autoscroll_func				fz_syst_set_autoscroll;			/* @fsfunc */
	fz_syst_get_undo_parm_data_func			fz_syst_get_undo_parm_data;		/* @fsfunc */
	fz_syst_set_undo_parm_data_func			fz_syst_set_undo_parm_data;		/* @fsfunc */
	fz_syst_anim_export_func				fz_syst_anim_export;			/* @fsfunc */
	fz_syst_anim_expt_get_parm_func			fz_syst_anim_expt_get_parm;		/* @fsfunc */
	fz_syst_anim_expt_set_parm_func			fz_syst_anim_expt_set_parm;		/* @fsfunc */
	fz_syst_get_proj_opts_parm_data_func	fz_syst_get_proj_opts_parm_data;	/* @fsfunc */
	fz_syst_set_proj_opts_parm_data_func	fz_syst_set_proj_opts_parm_data;	/* @fsfunc */
	fz_syst_get_color_paste_func			fz_syst_get_color_paste;		/* @fsfunc */
	fz_syst_set_color_paste_func			fz_syst_set_color_paste;		/* @fsfunc */

	fz_syst_cmnd_set_status_of_objt_func	fz_syst_cmnd_set_status_of_objt;	/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_syst_help_add_entry_func			fz_syst_help_add_entry;			/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 6.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_syst_help_add_floc_entry_func			fz_syst_help_add_floc_entry;			/* @fsfunc */

	/*** SEARCH PATHS ***/
	fz_file_paths_init_func						fz_file_paths_init;						/* @fsfunc */
	fz_file_paths_finit_func					fz_file_paths_finit;					/* @fsfunc */
	fz_file_paths_get_count_func				fz_file_paths_get_count;				/* @fsfunc */
	fz_file_paths_add_func						fz_file_paths_add;						/* @fsfunc */
	fz_file_paths_remove_func					fz_file_paths_remove;					/* @fsfunc */
	fz_file_paths_search_func					fz_file_paths_search;					/* @fsfunc */
	fz_file_paths_dialog_func					fz_file_paths_dialog;					/* @fsfunc */

	fz_file_path_item_get_active_func			fz_file_path_item_get_active;			/* @fsfunc */
	fz_file_path_item_get_locked_func			fz_file_path_item_get_locked;			/* @fsfunc */
	fz_file_path_item_get_nested_func			fz_file_path_item_get_nested;			/* @fsfunc */
	fz_file_path_item_get_type_func				fz_file_path_item_get_type;				/* @fsfunc */
	fz_file_path_item_get_floc_func				fz_file_path_item_get_floc;				/* @fsfunc */
	fz_file_path_item_set_active_func			fz_file_path_item_set_active;			/* @fsfunc */
	fz_file_path_item_set_locked_func			fz_file_path_item_set_locked;			/* @fsfunc */
	fz_file_path_item_set_nested_func			fz_file_path_item_set_nested;			/* @fsfunc */
	fz_file_path_item_set_type_func				fz_file_path_item_set_type;				/* @fsfunc */

	fz_syst_cmnd_set_active_func				fz_syst_cmnd_set_active;				/* @fsfunc */
	fz_syst_cmnd_get_active_func				fz_syst_cmnd_get_active;				/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8,5,0,1 functions.									*
	 *																					*
	 ************************************************************************************/

	fz_file_paths_iost_func						fz_file_paths_iost;						/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8.6.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

}  fz_syst_fset;

#ifdef __cplusplus
}
#endif

#endif /*__FZ_SYST_API_H */

