/*********************************************************************************************
 *                                                                                           *
 *                                                                                           *
 *   Copyright 1989-2018 by AutoDesSys, Inc.                                                 *
 *                                                                                           *
 *   This software and its associated documentation contains proprietary,                    *
 *   confidential, and trade secret information of AutoDesSys Inc. and may not be            *
 *   disclosed, copied in any form, or used for any purpose except as specifically           *
 *   permitted in writing by AutoDesSys, Inc.                                                *
 *                                                                                           *
 *   AutoDesSys, Inc. grants permission to use the software to those who purchase            *
 *   a license.  Such a usage will be made according to the terms and conditions             *
 *   specified in the license agreement to which a licensee agrees before installing         *
 *   and commencing usage of the software.  The license agreement can be found               *
 *   on the envelop that contains the software CD.                                           *
 *                                                                                           *
 *   AutoDesSys, Inc. also grants permission to licensees to develop extensions to           *
 *   the software, which can be in the form of scripts or plugins.  The permission           *
 *   includes calling the API and call back functions provided by AutoDesSys, Inc.           *
 *   and accessing information relevant to these functions and the general workings          *
 *   of form.Z, but only up to the point to which AutoDesSys, Inc. has chosen to             *
 *   expose, through its form.Z SDK Manual.  All extensions developed by third               *
 *   parties will include a copy of this copyright note in an easily accessible place or     *
 *   a clear instruction to read this original note by AutoDesSys, Inc.                      *
 *                                                                                           *
 *   All the software distributed under a license, either for the purpose of using it or     *
 *   for the purpose of developing extensions, is distributed AS IS, WITHOUT                 *
 *   WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND                                    *
 *   AutoDesSys, Inc. HEREBY DISCLAIMS ALL SUCH WARRANTIES,                                  *
 *   INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF                                         *
 *   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.                                    *
 *                                                                                           *
 *   formZ, RenderZone, and RadioZity are registered trademarks of AutoDesSys, Inc.          *
 *                                                                                           *
 *   form.Z SDK, Version 8.6.3, Dec 03, 2018                                                 *
 *                                                                                           *
 *                                                                                           *
 *********************************************************************************************/



#ifndef __FORMZ_TEXT_API_H
#define __FORMZ_TEXT_API_H 1


#include "fzpl_plugin.h"

#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************

	Text Fonts and Styles


	Description
		formZ font and text styles

		fonts in formZ are specified by two strings.  The first is the font 
		family name (i.e. courier, helvetica).  The second is the font face 
		name (i.e. bold, normal).

	Availability	5.5.0.0

 ****************************************************************/




#define	FZ_TEXT_CONTEXT_ID 		12
typedef enum fz_text_errors_enum
{
	FZ_TEXT_FONT_NOT_FOUND		= 1,
	
	FZ_TEXT_ERRORS_MAX
}  fz_text_errors_enum;

/****************************************************************

	FZ_TEXT_RANGE_START


	Description
		Specifies the start of a string as the start of a range.

	Availability	5.5.0.0

 ****************************************************************/
#define FZ_TEXT_RANGE_START		0

/****************************************************************

	FZ_TEXT_RANGE_END


	Description
		Specifies the end of a string as the end of a range.

	Availability	5.5.0.0

 ****************************************************************/
#define FZ_TEXT_RANGE_END		0x7fffffff



/****************************************************************

	fz_font_id_td


	Description
		A runtime font identifier.

	Availability	5.5.0.0

 ****************************************************************/
typedef fzrt_int	fz_font_id_td;

/****************************************************************

	FZ_FONT_NONE


	Description
		The is of an unknown or nonexistant font.

	Availability	5.5.0.0

 ****************************************************************/
#define FZ_FONT_NONE		(-1)


/****************************************************************

	fz_font_type_enum


	Description
		Font types.

	Members
		FZ_FONT_TYPE_TRUE_TYPE
			TrueType font
		FZ_FONT_TYPE_POSTSCRIPT
			Postscript font
		FZ_FONT_TYPE_BITMAP
			Bitmap font

	Availability	5.0.0.0


	See Also
		fz_rmtl_delete, 

 ****************************************************************/
typedef enum fz_font_type_enum
{
	FZ_FONT_TYPE_TRUE_TYPE = 0,
	FZ_FONT_TYPE_POSTSCRIPT,
	FZ_FONT_TYPE_BITMAP
}  fz_font_type_enum;

/****************************************************************

	fz_font_parms_enum


	Description
		Parameter identifiers for a font. 
		These parameters are read only. 
		Used for the "which" argument of fz_font_parm_get_parm.

	Members
		FZ_FONT_PARM_FAMILY_NAME
			font family name
			Type: string
		FZ_FONT_PARM_FACE_NAME
			font face name
			Type: string
		FZ_FONT_PARM_TYPE
			font type
			Type: fz_font_type_enum

	Availability	5.0.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl

	See Also
		fz_font_parm_get_parm, fz_font_parm_get_parm

 ****************************************************************/
typedef enum fz_font_parms_enum
{
	FZ_FONT_PARM_FAMILY_NAME = 0,
	FZ_FONT_PARM_FACE_NAME,
	FZ_FONT_PARM_TYPE,
	
	FZ_FONT_PARM_MAX
}  fz_font_parms_enum;



/****************************************************************

	fz_font_find_by_name



	Description
		Gets the font id from a font family name and font face name. 
		If the specified font is not loaded in formZ (or the system), 
		the font id will be set to FZ_FONT_NONE and a FZ_FONT_NOT_FOUND 
		error will be returned.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		font_family_name  [Input]
			font family name
		font_face_name  [Input]
			font face name
		font_id  [Result]
			font id of matching font

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_find_by_name_func)(
			fzrt_int 				windex,
			char				*font_family_name,
			char				*font_face_name,
			fz_font_id_td		*font_id
			);
			
/****************************************************************

	fz_font_get_count



	Description
		Gets the number of fonts loaded in formZ.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		num_fonts  [Result]
			the number of fonts

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_get_count_func)(
			fzrt_int 				windex,
			fzrt_int				*num_fonts
			);

/****************************************************************

	fz_font_get_next



	Description
		Given a font id, get the id of the next font loaded in formZ. 
		This can be used to iterate over all fonts loadedin formZ. 
		Passing in FZ_FONT_NONE for font_id will set next_font_id to the id 
		of the first font.  next_font_id will be set to FZ_FONT_NONE to 
		indicate the end of the fonts.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		font_id  [Input]
			font id
		next_font_id  [Result]
			the next font id

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_get_next_func)(
			fzrt_int 				windex,
			fz_font_id_td		font_id,
			fz_font_id_td		*next_font_id
			);
			
/****************************************************************

	fz_font_parm_get_parm



	Description
		Gets a parameter of a font.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		font_id  [Input]
			font id
		which  [Input]
			parameter identifier
		data  [Result]
			parameter data

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_parm_get_parm_func)(
			fzrt_int 				windex,
			fz_font_id_td		font_id,
			fz_font_parms_enum	which,
			fz_type_td			*data
			);

/****************************************************************

	fz_font_face_get_count



	Description
		Gets the number of faces defined for a font.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		font_family_name  [Input]
			font family name
		num_faces  [Result]
			number of faces

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_face_get_count_func)(
			fzrt_int 				windex,
			char				*font_family_name,
			fzrt_int				*num_faces
			);

/****************************************************************

	fz_font_face_get_next



	Description
		Given a font family name and a font id, get the font id of the next font 
		within font family.  fz_font_parm_get_parm can be used on next_font_id to 
		get the face name.  Passing in FZ_FONT_NONE for font_id will set next_font_id 
		to the id of the first font.  next_font_id will be set to FZ_FONT_NONE to 
		indicate the end of the fonts.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		font_family_name  [Input]
			font family name
		font_id  [Input]
			font id
		next_font_id  [Input]
			id of the next font in the font family

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_font_face_get_next_func)(
			fzrt_int 				windex,
			char				*font_family_name,
			fz_font_id_td		font_id,
			fz_font_id_td		*next_font_id
			);


			

/****************************************************************

	fz_text_style_ptr


	Description
		A text style.  A text style consists of a font id and a point size.

	Availability	5.5.0.0

 ****************************************************************/
typedef struct fz_text_style_td		*fz_text_style_ptr;



/****************************************************************

	fz_text_style_init



	Description
		Initializes a text style.

	Parameters
		style  [Input, Result]
			The text style

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_text_style_init_func)(
			fz_text_style_ptr		*style
			);

/****************************************************************

	fz_text_style_finit



	Description
		Uninitializes a text style.

	Parameters
		style  [Input, Result]
			The text style

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_text_style_finit_func)(
			fz_text_style_ptr		*style
			);

/****************************************************************

	fz_text_style_get_info



	Description
		Gets the font is and point size from a style.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		style  [Input]
			The text style
		font_id  [Result]
			font id
		point_size  [Result]
			point size

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_text_style_get_info_func)(
			fzrt_int 					windex,
			fz_text_style_ptr		style,
			fz_font_id_td			*font_id,
			fzrt_int					*point_size
			);

/****************************************************************

	fz_text_style_set_info



	Description
		Sets the font is and point size to a style.

	Parameters
		windex  [Input]
			project window index, use -1 to access the system font table
		style  [Input]
			The text style
		font_id  [Input]
			font id
		point_size  [Input]
			point size

	Returns
		Error codes


	Availability	5.5.0.0

	Example Files
		../../../../Scripts/Samples/Utilities/util_objt_unistr_text.fsl
 ****************************************************************/
typedef fzrt_error_td (*fz_text_style_set_info_func)(
			fzrt_int 					windex,
			fz_text_style_ptr		style,
			fz_font_id_td			font_id,
			fzrt_int					point_size
			);

/****************************************************************

	fz_font_text_get_used_font_list



	Description
		Gets a list of fonts used by a project.

	Parameters
		windex  [Input]
			project window index
		max_str_len  [Input]
			masimum length of each font name string
		f_list_ptr  [Input, Result, Optional]
			an array of font names
		l_font_ptr  [Input, Result, Optional]
			the number of font names in the array
		s_font_ptr  [Input, Result, Optional]
			the allocated size of the array

	Returns
		Error codes


	Availability	5.0.0.0

 ****************************************************************/
typedef fzrt_int  (*fz_font_text_get_used_font_list_func)(
	fzrt_int		windex,
	short	max_str_len,
	char	**f_list_ptr,
	short	*l_font_ptr,
	short	*s_font_ptr
	);


/****************************************************************

	fz_text_fset


	FZ_TEXT_FSET_TYPE
		formZ text fonts and styles function set UUID.

	FZ_TEXT_FSET_NAME
		formZ text fonts and styles function set name.

	FZ_TEXT_FSET_VERSION
		formZ text fonts and styles function set version.

	Description
		formZ text fonts and styles function set

	Availability	5.0.0.0

 ****************************************************************/


#define FZ_TEXT_FSET_TYPE		"\x4f\xc5\xe7\x78\xb3\xc0\x46\xc1\x95\xc4\x50\x20\x30\x23\xe5\xa8"

#define FZ_TEXT_FSET_NAME		"formZ text functions"

#define FZ_TEXT_FSET_VERSION	FZPL_VERS_MAKE(5,0,0,0)

typedef struct fz_text_fset
{
	fzpl_fset_prvt_data_td						prvt_data;
	
	fz_font_find_by_name_func					fz_font_find_by_name;
	fz_font_get_count_func						fz_font_get_count;
	fz_font_get_next_func						fz_font_get_next;
	fz_font_parm_get_parm_func					fz_font_parm_get_parm;
	fz_font_face_get_count_func					fz_font_face_get_count;
	fz_font_face_get_next_func					fz_font_face_get_next;
			
	fz_text_style_init_func						fz_text_style_init;
	fz_text_style_finit_func					fz_text_style_finit;
	fz_text_style_get_info_func					fz_text_style_get_info;
	fz_text_style_set_info_func					fz_text_style_set_info;


	fz_font_text_get_used_font_list_func		fz_font_text_get_used_font_list;
																		
}  fz_text_fset;



/****************************************************************

	Text Search and Replace


	Description
		Text Search and Replace

		Text in text objects and draft elements can be search.  Objects/elements which match 
		a specified search criteria can either be picked or the text, font or style (bold, 
		italic, etc.) can be replaced.  The capabilities of these functions closely resemble 
		the Text search and replace tools and the Select by for Text for modeling and drafting.

	Availability	6.0.0.0

 ****************************************************************/

/****************************************************************

	fz_text_search_parm_ptr


	Description
		Text Search and Replace Parameters.  This data type stores text search criteria. 
		This data type is used for searching objects and draft elements, however, different 
		functions initialize it for modeling and drafting.

	Availability	6.0.0.0


	See Also
		fz_objt_text_search_init, fz_objt_text_search_finit, 
		fz_objt_text_search_parm_set, fz_objt_text_search_parm_get, 
		fz_objt_text_search_text_parm_set, fz_objt_text_search_text_parm_get, 
		fz_objt_text_search_objt, fz_objt_text_search_all_objts

 ****************************************************************/
typedef struct fz_text_search_parm_td	*fz_text_search_parm_ptr;

/****************************************************************

	fz_text_interactive_search_data_ptr


	Description
		Interactive Text Search and Replace Data.  This stores the search parameters 
		and the current search result. 
		This data type is used for searching objects and draft elements, however, different 
		functions initialize it for modeling and drafting.

	Availability	6.0.0.0


	See Also
		fz_objt_text_interactive_search_init, fz_objt_text_interactive_search_finit, 
		fz_objt_text_interactive_search_find_next, fz_objt_text_interactive_search_do_replace, 
		fz_objt_text_interactive_search_get_current

 ****************************************************************/
typedef struct fz_text_interactive_search_data_td	*fz_text_interactive_search_data_ptr;

/****************************************************************

	fz_text_search_action_enum


	Description
		Text Search and Replace Action.  This determines what formZ should do with 
		search results (pick the objects or replace text).

	Members
		FZ_TEXT_SRCH_ACTION_PICK
			Pick matching text objects/elements.
		FZ_TEXT_SRCH_ACTION_REPLACE
			Replace text in matching text objects/elements.

	Availability	6.0.0.0

 ****************************************************************/
typedef enum fz_text_search_action_enum
{
	FZ_TEXT_SRCH_ACTION_PICK = 0,
	FZ_TEXT_SRCH_ACTION_REPLACE
}  fz_text_search_action_enum;

/****************************************************************

	fz_text_search_range_enum


	Description
		Numeric Value Search Range.  Numeric parameters of text can be searched by 
		defining a range of values.  This type is used to define whether the minimum 
		or maximum value of the range is to be set or get.

	Members
		FZ_TEXT_SEARCH_RANGE_MIN
			The minimum value of the range.
		FZ_TEXT_SEARCH_RANGE_MAX
			The maximum value of the range.

	Availability	6.0.0.0

 ****************************************************************/
typedef enum fz_text_search_range_enum
{
	FZ_TEXT_SEARCH_RANGE_MIN = 0,
	FZ_TEXT_SEARCH_RANGE_MAX
}  fz_text_search_range_enum;

/****************************************************************

	fz_text_search_criteria_enum


	Description
		Text Search Criteria.  Criteria for matching text of a text object/element.

	Members
		FZ_TEXT_SRCH_CRIT_TEXT_CONTAINS
			Text containing the search string is considered a match.
		FZ_TEXT_SRCH_CRIT_TEXT_STARTS
			Text which begins with the search string is considered a match.
		FZ_TEXT_SRCH_CRIT_TEXT_ENDS
			Text which ends with the search string is considered a match.
		FZ_TEXT_SRCH_CRIT_TEXT_IS
			Text which is exactly the search string is considered a match.
		FZ_TEXT_SRCH_CRIT_TEXT_IS_NOT
			Text which is not exactly the search string is considered a match.
		FZ_TEXT_SRCH_CRIT_TEXT_CONTAINS_NOT
			Text which does not contain the search string is considered a match.

	Availability	6.0.0.0

 ****************************************************************/
typedef enum fz_text_search_criteria_enum
{
	FZ_TEXT_SRCH_CRIT_TEXT_CONTAINS = 0,
	FZ_TEXT_SRCH_CRIT_TEXT_STARTS,
	FZ_TEXT_SRCH_CRIT_TEXT_ENDS,
	FZ_TEXT_SRCH_CRIT_TEXT_IS,
	FZ_TEXT_SRCH_CRIT_TEXT_IS_NOT,
	FZ_TEXT_SRCH_CRIT_TEXT_CONTAINS_NOT
}  fz_text_search_criteria_enum;

/****************************************************************

	fz_text_search_parm_enum


	Description
		Text Search Parameters.  This enumeration defines parameters of a text search 
		or replace.  This is used by fz_objt_text_search_parm_get, fz_objt_text_search_text_parm_set, 
		fz_draft_text_search_parm_get, and fz_draft_text_search_text_parm_set to set values for 
		these parameters.

	Members
		FZ_TEXT_SRCH_PARM_SEARCH_FONT
			The font id of the font to search for.  Use fz_font_parm_get_parm to get the name of the font.
			Type: fzrt_int
			Default Value: The default font defined in the formZ preferences.
		FZ_TEXT_SRCH_PARM_SEARCH_FONT_NAME
			The font name of the font to search for.  Use fz_font_parm_get_parm to get the name of the font.
			Type: fzrt_int
			Default Value: The default font defined in the formZ preferences.
		FZ_TEXT_SRCH_PARM_SEARCH_FACE
			The font face id to search for.
			Type: fzrt_int
			Default Value: 0
		FZ_TEXT_SRCH_PARM_SEARCH_SIZE
			The font size to search for.
			Type: fzrt_int
			Default Value: 24
		FZ_TEXT_SRCH_PARM_REPLACE_FONT
			The id of the font to set to matching text during a replace operation.  Use fz_font_parm_get_parm to get the name of the font.
			Type: fzrt_int
			Default Value: The default font defined in the formZ preferences.
		FZ_TEXT_SRCH_PARM_REPLACE_FONT_NAME
			The name of the font to set to matching text during a replace operation.  Use fz_font_parm_get_parm to get the name of the font.
			Type: fzrt_int
			Default Value: The default font defined in the formZ preferences.
		FZ_TEXT_SRCH_PARM_REPLACE_FACE
			The font face to set to matching text during a replace operation.
			Type: fzrt_int
			Default Value: 0
		FZ_TEXT_SRCH_PARM_REPLACE_SIZE
			The font size to set to matching text during a replace operation.
			Type: fzrt_int
			Default Value: 24
		FZ_TEXT_SRCH_PARM_SEARCH_MATCH_CASE
			If the value is TRUE, match the case of the searched text.
			Type: Boolean
			Default Value: FALSE
		FZ_TEXT_SRCH_PARM_SEARCH_MATCH_WHOLE_WORD
			If the value is TRUE, match the whole word of the searched text.
			Type: Boolean
			Default Value: FALSE
		FZ_TEXT_SRCH_PARM_SEARCH_ALL_OBJECTS
			If the value is TRUE, search all objects; otherwise, only search picked objects.
			Type: Boolean
			Default Value: TRUE
		FZ_TEXT_SRCH_PARM_SEARCH_CRITERIA
			The text matching cirteria.
			Type: fz_text_search_criteria_enum
			Default Value: FZ_TEXT_SRCH_CRIT_TEXT_CONTAINS
		FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_NORM
			The text facetted front rounding normal.  For modeling only.
			Type: boolean
			Default Value: FALSE
		FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_RND
			The text facetted front rounding round.  For modeling only.
			Type: boolean
			Default Value: FALSE
		FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_BEVL
			The text facetted front rounding bevel.  For modeling only.
			Type: boolean
			Default Value: FALSE
		FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_NORM
			The text facetted back rounding normal.  For modeling only.
			Type: boolean
			Default Value: FALSE
		FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_RND
			The text facetted back rounding round.  For modeling only.
			Type: boolean
			Default Value: FALSE
		FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_BEVL
			The text facetted back rounding bevel.  For modeling only.
			Type: boolean
			Default Value: FALSE

	Availability	6.0.0.0


	See Also
		fz_objt_text_search_parm_get, fz_objt_text_search_text_parm_set

 ****************************************************************/
typedef enum fz_text_search_parm_enum
{
	FZ_TEXT_SRCH_PARM_SEARCH_FONT = 0,
	FZ_TEXT_SRCH_PARM_SEARCH_FONT_NAME,
	FZ_TEXT_SRCH_PARM_SEARCH_FACE,
	FZ_TEXT_SRCH_PARM_SEARCH_SIZE,
	FZ_TEXT_SRCH_PARM_REPLACE_FONT,
	FZ_TEXT_SRCH_PARM_REPLACE_FONT_NAME,
	FZ_TEXT_SRCH_PARM_REPLACE_FACE,
	FZ_TEXT_SRCH_PARM_REPLACE_SIZE,
	FZ_TEXT_SRCH_PARM_SEARCH_MATCH_CASE,
	FZ_TEXT_SRCH_PARM_SEARCH_MATCH_WHOLE_WORD,
	FZ_TEXT_SRCH_PARM_SEARCH_ALL_OBJECTS,
	FZ_TEXT_SRCH_PARM_SEARCH_CRITERIA,
	FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_NORM,
	FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_RND,
	FZ_OBJT_TEXT_PARM_FRND_FRNT_TYPE_BEVL,
	FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_NORM,
	FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_RND,
	FZ_OBJT_TEXT_PARM_FRND_BACK_TYPE_BEVL,
}  fz_text_search_parm_enum;


#ifdef __cplusplus
}
#endif

#endif
																		
																		
																		
																		
																		
																		
																		
