/*************************************************************************

	formZ by autodessys, Inc.
	Copyright c 1994 autodessys, Inc.  All rights reserved.
	
	
	File :	fzrt_api.h
	
	By :	Keith Agee
	
	Created on :  		11 Sep 2003
	Last Updated on :  	11 Sep 2003
	
	Description :	
					
***************************************************************************/

#ifndef _FZRT_API_H
#define _FZRT_API_H 1

#ifdef __cplusplus
extern "C" {
#endif
#include "fzrt_platform.h"
#include "fzrt_extern.h"

#include "fzpl_plugin.h"

#include "fzrt_api_type.h"

/* @doc EXTERNAL FZRT */
/********************************************************************************************************

	@module FZRT | formZ Runtime functions

	@desc	formZ Runtime Library functions. This is a group of functions that perform operating system or runtime level operations
			within a platform independent interface. This is functionality that is available in any application.
			
			The library is organized into 6 areas of memory, file, basic, utility, graphics and user interface. 
			The use of the runtime library provides a common interface to use on both Macintosh and Windows platforms.
			It also provides a cross-platform method of supplying strings and menus than can be easily translated for use on different
			localized versions of the application.
			

	@toc	fzrt

***************************************************************************/

/******************************************************************************************************

	@enum	
	
	@desc	formZ runtime library error codes. 
			
	@avail	5.0.0.0

	@see	<auto>
	
	@toc	fzrt_file

*******************************************************************************************************/
typedef enum fzrt_context_error_enum
{
	FZRT_BAD_PARAM_ERROR				=1,				/* @emem Bad parameter passed to a function. */
	FZRT_UNKNOWN_ERROR					=2,				/* @emem Unknown error. */
	FZRT_NOT_IMPLEMENTED_ERROR			=3,				/* @emem Function not implemented. */
	FZRT_DATA_NOT_INITIALIZED_ERROR		=4,				/* @emem Data not initialized. */
	FZRT_MALLOC_ERROR					=5,				/* @emem Memory Allocation Failed. */
	FZRT_NO_ACTION_FUNC_ERROR			=6,				/* @emem No action function defined for control. */
	FZRT_NO_FUNC_SET_ERROR				=7,				/* @emem No action function set defined for control. */
	FZRT_ERR_NOT_FZRF_FILE				=8,				/* @emem Not an formZ resource file. */
	FZRT_ERR_INVALID_FZRF_FORMAT		=9,				/* @emem Invalid formZ resource file. */
	FZRT_ERR_RSRC_NOT_FOUND				=10,			/* @emem formZ resource file not found. */
	FZRT_ERR_DATA_NOT_CACHED			=11,			/* @emem Data not cached. */
	FZRT_ERROR_LAST_LANG				=12,			/* @emem return when lang iterator has finished list */
	FZRT_BAD_VSPEC_ERROR				=13,			/* @emem Invalid or unknown vspec in get_vspec_type */
	FZRT_NO_PBUF_FOR_PORT				=14,			/* @emem No pixel buffer for port. */
	FZRT_PBUF_NOT_DRAWABLE_ERROR		=15,			/* @emem Pixel buffer not drawable. */
	FZRT_PIX_UNIMPLEMENTED_ERROR		=16,			/* @emem Pixel format not supported by formZ. */
	FZRT_DATA_OVERFLOW_ERROR			=17,			/* @emem Attempt to copy pixels beyond pixel buffer size. */
	FZRT_UNEXPECTED_EOF_ERROR			=18,			/* @emem Unexpected end of file. */
	FZRT_INVALID_FOLDER_ERROR			=19,			/* @emem Invalid folder. */
	FZRT_FILE_OPEN_ERROR				=20,			/* @emem File open error. */
	FZRT_NO_MATCH_ERROR					=21,			/* @emem No matching uuid found. */
	FZRT_INTERNAL_LIMIT_EXEC_FUNC		=22,			/* @emem Internal limit exceeded. */		/* DEK 03/10/04 */
	FZRT_BAD_FILE_TYPE_ERROR			=23,			/* @emem Incorrect file type. */
	FZRT_OUT_OF_CONTEXT_ERROR			=24,			/* @emem Function called at an inappropriate time or from an inappropriate context. */
	FZRT_WARN_IRREGULAR_FZRF_FORMAT		=25,			/* @emem formZ Resource file has a non-standard format, data may not be as expected */
	FZRT_OUTDATED_PARAM_ERROR			=26,			/* @emem An api or a parameter to an api function is outdated. A newer version exists and should be used instead. */
	
	FZRT_ERROR_CODE_MAX
}	fzrt_context_error_enum;

/************************************************************************

	@fsfunc_t

	@desc		Creates an instance of a file locator. This must be called before the floc is used.
				After the floc is used, fzrt_file_floc_finit must be called to dispose of the file locator.
	
	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_finit

	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_file_floc_init_func)(
				fzrt_floc_ptr *floc				/* @parm <out> | New file locator */
				);

/************************************************************************

	@fsfunc_t
	
	@desc		Disposes of a file locator created by fzrt_file_floc_init.
				After disposing a file locator, the file locator is no longer
				valid and cannot be used for any functions that take a file/folder
				locator.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_file_floc_finit_func)(
				fzrt_floc_ptr 			*floc			/* @parm <in> |  pointer to a file locator to release*/
				);


/************************************************************************

	@fsfunc_t

	@desc		Gets the name associated with the file or folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_file_floc_get_name_func)(
					fzrt_floc_ptr				floc,				/* @parm <in> |  pointer to a file or folder */
					char *						name,				/* @parm <out> | buffer that receives the name of file/folder, if successful. Must be allocated by caller. */
					fzrt_unsigned_int				maxlen				/* @parm <in> | size of buffer in name parameter */
					);



/************************************************************************

	@fsfunc_t

	@desc	Returns the current working folder. The current working folder
				is the folder that is used when files are opened using fopen.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_set_crnt, fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_file_floc_get_crnt_func)(
				fzrt_floc_ptr			floc				/* @parm <in_out> |  pointer to an initialized file locator that receives the current working folder */
				);


/************************************************************************

	@fsfunc_t

	@desc		Sets the current folder to the specified folder. If the
				file locator points to a file, this will set the current
				folder to the folder that contains the file.


	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_get_crnt, fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_file_floc_set_crnt_func)(
				fzrt_floc_ptr			floc				/* @parm <in> |  pointer to folder to set as current working folder */
				);


/************************************************************************

	@fsfunc_t

	@desc		Makes a copy of the source file locator in the destination file locator. 
				The destination file locator must be initialized before	
				calling this function.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init
	
	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_file_floc_copy_func)			(
					fzrt_floc_ptr			src_floc,				/* @parm <in> |  source file locator */
					fzrt_floc_ptr			dst_floc				/* @parm <in_out> |  pointer to an initialized file locator 
																						that receives the copy of the source file locator */
					);



/************************************************************************

	@fsfunc_t

	@desc		Compares two file locators for inequality. If either file locator
				is invalid or uninitialized, they are considered not equal.
				File locators are considered equal if they have the same name
				and are located in the same folder.


	@return		TRUE if they are different, FALSE if they are the same.

	@avail		5.0.0.0
	
	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_boolean  (FZRT_SPEC * fzrt_file_floc_compare_func)(
				fzrt_floc_ptr	 floc1,			/* @parm <in> |  source file locator */
				fzrt_floc_ptr	 floc2			/* @parm <in> |  second file locator */
				);

/************************************************************************

	@fsfunc_t

	@desc	Tests whether the file locator is valid. A file locator is considered
			valid if it has been initialized and it's path or path and name have 
			been set. The file or folder does not have to exist to be valid.

	@return		TRUE if file locator has been initialized and values have been set. , FALSE otherwise.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init, fzrt_file_floc_set_path_name, fzrt_file_floc_set_fullpath, fzrt_file_floc_set_name

	@toc		fzrt_file

*************************************************************************/
typedef  fzrt_boolean  (FZRT_SPEC * fzrt_file_floc_valid_func)(
				fzrt_floc_ptr				floc			/* @parm <in> |  source file locator */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets file locator for the system specified temporary folder. This will
			create a folder with the specified name, if it does already exist, in the system temporary folder,
			and return the file locator of the created folder.


	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_scratch_floc_func)(
				fzrt_floc_ptr 				floc,			/* @parm <out> |  scratch file locator. Must be initialized by caller */
				char *						scratch_name	/* @parm <in> |  scratch folder name. Must be allocated by the caller */
				);

/************************************************************************

	@fsfunc_t

	@desc		Gets file locator for system folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_system_floc_func)(
				fzrt_floc_ptr 			floc			/* @parm <out> |  system file locator. Must be initialized by caller */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets file locator for user preferences folder. This is the system
			preferred location for preferences of the current user.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_prefs_floc_func)(
				fzrt_floc_ptr 			floc			/* @parm <out> |  Pointer to file locator to receive preferences file locator. Must be initialized by caller */
				);


/************************************************************************

	@fsfunc_t

	@desc		Gets a list of file locators for font folders on the host machine.
				If NULL is passed for the file locator list, the function will return the number
				of font folders on the machine in the count parameter.
				
	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_fonts_floc_func)(
				fzrt_floc_ptr 					*floc_list,		/* @parm <out_opt> |  array of file locators.  Must be allocated and initialized by caller */
				fzrt_int *							count			/* @parm <in_out> |  number of font folders on return if floc_list is NULL, 
																							number of file locators in floc_list if floc_list is non-NULL */
															
								);					/* */

/************************************************************************

	@fsfunc_t

	@desc	Gets file locator for system extensions folder, if it exists.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_extensions_floc_func)(
				fzrt_floc_ptr 					floc			/* @parm <out> |  System Extensions file locator. Must be initialized by caller */
				);

/************************************************************************

	@fsfunc_t

	@desc	Gets file locator for the folder that contains the application executable file.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_application_floc_func)(
				fzrt_floc_ptr 					floc			/* @parm <out> |  Application file locator. Must be initialized by caller */
				);

/************************************************************************

	@fsfunc_t

	@desc	Gets file locator for user documents folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_documents_floc_func)(
				fzrt_floc_ptr 				floc			/* @parm <out> |  Documents file locator. Must be initialized by caller */
								);


/************************************************************************

	@fsfunc_t

	@desc	Creates a new folder. The file locator of the new folder is 
			returned. If the folder already exists
			this function returns the file locator of the exiting folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_new_folder_func)(
				fzrt_floc_ptr 					floc,			/* @parm <in> |  parent file locator. Location of new folder*/
				const char *					folder_name,	/* @parm <in> |  Name of new folder as a C string */
				fzrt_floc_ptr					new_floc		/* @parm <out> |  file locator of new folder. Must be initialized by caller */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets the file locator of the folder located inside the specified folder.
				This does not set the current working folder. 

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_open_folder_func)(
				fzrt_floc_ptr					floc,			/* @parm <in> |  parent folder's file locator. Location of folder*/
				const char *					folder_name,	/* @parm <in> |  Name of folder to open. C string */
				fzrt_floc_ptr					new_floc   		/* @parm <out> |  file locator of opened folder. Must be initialized by caller */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets time stamp of a file. The time stamp of a file consists of either
			the modification date or the creation date. The values are system dependent, 
			and are only useful for comparing to other values

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_time_stamp_func)(
				fzrt_floc_ptr					floc,			/* @parm <in> |  File locator. Location of file*/
				fzrt_int *							mod_date,		/* @parm <out_opt> |  Modification date as fzrt_int. Value is system dependent */
				fzrt_int *							cr_date			/* @parm <out_opt> |  Creation date as fzrt_int.  Value is system dependent. */
				);


/************************************************************************

	@fsfunc_t

	@desc	Sets time stamp of a file. The values are system dependent and 
			cannot be calculated. It is only useful to restore the time stamp
			of a file after copying or modifying the file with the time stamp
			obtained from fzrt_file_get_time_stamp.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see		fzrt_file_get_time_stamp

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_set_time_stamp_func)(
				fzrt_floc_ptr 					floc,			/* @parm <in> |  file locator. Location of file*/
				fzrt_int							mod_date,		/* @parm <in> |  Modification date as a fzrt_int. Value is system dependent */
				fzrt_int							cr_date 		/* @parm <in> |  Creation date as a fzrt_int.  Value is system dependent. */
				);


/************************************************************************

	@fsfunc_t

	@desc	Sets the file type and creator of a file (Macintosh) . This will 
			set the Macintosh HFS file type and creator. It will not change
			the file extension.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_set_signature_func)(
				fzrt_floc_ptr 					floc,			/* @parm <in> |  file locator. Location of file*/
				fzrt_ostype						fdCreator,		/* @parm <in> |  File creator, 4-character code */
				fzrt_ostype						fdType			/* @parm <in> |  File type. 4-character code, e.g. 'TEXT'. */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets the size of a file. The size of the file is in bytes.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_size_func)(
				fzrt_floc_ptr 					floc,		/* @parm <in> |  file locator. Location of file*/
				fzrt_unsigned_int *					size		/* @parm <out> |  Size of the file in bytes. */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets the amount of free space on disk volume. 
			The file locator can be either a file or a folder. The free space
			is the amount of disk space for the volume on which the file resides.

	@return		FZRT_NOERR on success, otherwise an error ID

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_vol_free_space_func)(
				fzrt_floc_ptr 				floc,			/* @parm <in> |  parent file locator.  */
				fzrt_unsigned_int	*			size			/* @parm <out> | size in bytes of the free space on the disk volume */
				);


/************************************************************************

	@fsfunc_t

	@desc	Sets the file locator from a full path string. The full path
			includes the filename, and must be in the format of the native
			operating system. Windows uses backslash character as the 
			folder separator, Macintosh uses the colon character as the
			folder separator.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_set_fullpath_func)(
				fzrt_floc_ptr 				floc,		/* @parm <out> |  file locator. User must initialize.*/
				char *						path		/* @parm <in> |  full pathname as C strings*/
				);

/************************************************************************

	@fsfunc_t

	@desc	Gets the volume name of a file locator. The volume name is
			the name of the disk on Macintosh. It is the pathname on Windows.
			If the buffer that receives the volume name is too small, the 
			string is truncated and FZRT_NOERR is returned. 
	
	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_get_volume_name_func)(
				fzrt_floc_ptr 				floc,		/* @parm <out> |  file locator. User must initialize.*/
				char *						path,		/* @parm <out> |  pointer to buffer to receive volume name as C string */
				fzrt_unsigned_int				maxlen		/* @parm <in> | size of buffer in path parameter */
				);

/************************************************************************

	@fsfunc_t
	
	@desc		Sets the volume name of a file locator. This does not check 
				for validity of the volume name. It will store the string passed 
				in path, in an internal data structure. This same string will
				be retrieved from fzrt_file_floc_get_volume_name. Setting this
				string to an invalid volume can cause unpredictable results.
	
	@return		FZRT_NOERR on success, Otherwise, an error ID.


	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_set_volume_name_func)(
				fzrt_floc_ptr 				floc,		/* @parm <out> |  file locator. User must initialize.*/
				char *						path		/* @parm <in> |  pointer to buffer to contains volume name as C string */
				);



/************************************************************************

	@fsfunc_t
	
	@desc		Gets the parent folder locator for a file or folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_file_floc_get_parent_func)(
				fzrt_floc_ptr				floc,					/* @parm <in> |  file or file locator */
				fzrt_floc_ptr				parent_floc				/* @parm <out> |  parent file locator. User must initialize.*/
				);

/************************************************************************

	@fsfunc_t

	@desc		Checks if file locator is a folder. This will check whether 
				the file locator represents a folder. 
				If the file locator has not been assigned it will
				return FALSE. This will not determine existence of a folder.
				Setting the name of a file locator to an existing sub folder
				name will not be recognized as a folder locator. Instead, use 
				fzrt_file_open_folder.

	@return		TRUE if floc identifies a folder, FALSE if it is not a folder.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init, fzrt_file_open_folder, fzrt_file_new_folder, fzrt_file_floc_get_crnt, fzrt_file_select_folder

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_boolean (FZRT_SPEC *fzrt_file_floc_is_folder_func)(
				fzrt_floc_ptr				floc					/* @parm <in> |  file or file locator */
				);

/************************************************************************

	@fsfunc_t
  
	@desc		Checks if file locator is a file. This will check whether the  
				file locator represents a file. If the file locator has not been assigned it will
				return FALSE. This will not determine existence of a file.
				A file locator is assumed to be a file if the name has been set
				with a call to fzrt_file_floc_set_name.
				
	@return		TRUE if floc identifies a file, FALSE if it is not a file.

	@avail		5.0.0.0

	@see		fzrt_file_floc_init, fzrt_file_floc_set_name

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_boolean (FZRT_SPEC *fzrt_file_floc_is_file_func)(
				fzrt_floc_ptr				floc					/* @parm <in> |  file or file locator */
				);




/************************************************************************

	@fsfunc_t

	@desc	Sets the hidden attribute of a file.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_hide_func)(
				fzrt_floc_ptr 				floc		/* @parm <in> |  file locator */
				);


/************************************************************************

	@fsfunc_t
	
	@desc		Counts the number of files in a folder. 
				This will return the number of files in the specified folder. 
				The count does not include sub-folders.
				

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see		fzrt_file_find_file_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_folder_count_files_func)(
				fzrt_floc_ptr 				floc,			/* @parm <in> |  folder locator */
				fzrt_int *						nfiles			/* @parm <out> |  contains the number of files in the folder */
				);


/************************************************************************

	@fsfunc_t

	@desc		Counts the number of sub-folders in a folder. 
				This will return the number of sub-folders in the specified folder. 

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_find_folder_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_folder_count_folders_func)(
				fzrt_floc_ptr 			floc,				/* @parm <in> |  folder locator */
				fzrt_int *					nfolds				/* @parm <out> |  contains the number of files in the folder */
				);

/************************************************************************

	@fsfunc_t

	@desc	Returns the locked (read-only) state of a file.

	@return		true if locked, false if not locked.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_is_locked_func)(
				fzrt_floc_ptr 				floc,			/* @parm <in> |  file locator */
				fzrt_boolean *				locked			/* @parm <out> | true if file is locked, false if it is not locked */
				);


/************************************************************************

	@fsfunc_t

	@desc	Flushes the contents of the volume to disk. This will cause all
			open files that have been modified to have the contents written
			to disk. This only applies to Macintosh volumes.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_flush_func)(
				fzrt_floc_ptr 				floc			/* @parm <in> |  file locator */
				);



/************************************************************************

	@fsfunc_t

	@desc	Gets the file Macintosh file type and creator of a file.
			On Windows, this will always return a file type of 'TEXT'
			and the creator is left unassigned. 

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_get_signature_func)(
				fzrt_floc_ptr 			floc, 			/* @parm <in> |  file locator. Location of file*/
				fzrt_ostype *			fdCreator, 		/* @parm <out> |  File creator, 4-character code */
				fzrt_ostype	*			fdType			/* @parm <out> |  File type. 4-character code, e.g. 'TEXT'. */
				);


								 
/************************************************************************

	@fsfunc_t
	
	@desc		This will return the file system character or string
				for folder separators and parent folders, useful
				for building full paths and relative paths.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_file_get_char_func)(
				fzrt_unsigned_int			which,		/* @parm <in> |  Which file character to get 
														FZRT_DIR_SEP
														FZRT_DIR_PARENT
													*/
				char *					sep			/* @parm <out> |  C string containing character(s). Must be allocated by user */ 		
				);

/************************************************************************

	@fsfunc_t
	
	@desc	Checks whether character is valid in a filename. The character
			is checked for validity against the current operating systems
			valid file characters. The character is assumed to be in the 
			Mac Roman character set on Macintosh, and Windows ANSI character
			set on Windows.

	@return		true if character can be used in filename, false if not

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_boolean  (FZRT_SPEC *fzrt_file_is_invalid_char_func)(
				char					ch				/* @parm <in> |  character to test. Assumes Mac Roman character set on Macintosh, ANSI character set on Windows */ 	
				);


/************************************************************************

	@fsfunc_t

	@desc	Tests the hidden attribute of a file.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_is_hidden_func)(
				fzrt_floc_ptr 				floc ,			/* @parm <in> |  file locator */
				fzrt_boolean *				hidden			/* @parm <out> | true if file is hidden, false if it is not hidden */
				);
								
/************************************************************************

	@fsfunc_t

	@desc	Initializes the iterator for finding files in a folder. 
			This will create an iterator that can be used to get the 
			file names of all files in a folder. The count does not
			include sub-folders in the folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_find_finit, fzrt_file_find_file_by_index, fzrt_file_find_next_file

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_file_init_func)(
				fzrt_floc_ptr 					floc, 				/* @parm <in> |  file locator */
				fzrt_int *							count, 				/* @parm <out> |  number of files in folder.*/
				fzrt_file_iter_ptr *			iterator			/* @parm <out> | contains an iterator object used to get names of files */
								);
								
/************************************************************************

	@fsfunc_t

	@desc	Gets the specified indexed file in a folder. The index
			of a file is system dependent. It is equivalent to calling
			fzrt_file_find_next_file _n_ times where _n_ is the index
			of the file. The iterator must be created by fzrt_file_find_file_init
			before calling this function.
			 This set the state of the iterator to the specified index.
			 A call to fzrt_file_find_next_file will return the file at index+1, if it exists.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_find_file_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_file_by_index_func)(
				fzrt_file_iter_ptr 				iterator,			/* @parm <in> | iterator object returned by fzrt_file_find_file_init */
				fzrt_int 							index, 				/* @parm <in> |   1-based index of file to get. Must be less than count returned by fzrt_file_find_file_init
																		@rimin		1 */
				char *							file_name			/* @parm <out> |  Name of file as C string. User must allocate.*/
								);
								
/************************************************************************

	@fsfunc_t

	@desc	Gets the next file in a folder. The next file is file that
			follows the file, in folder order, from the previous call to fzrt_file_find_next_file
			with the same iterator. If there has not been a previous call, it returns
			the first file in the folder. If the file returned is the last file
			in the folder, the last parameter is set to true.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see 		fzrt_file_find_file_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_next_file_func)(
				fzrt_file_iter_ptr 				iterator, 		/* @parm <in> | iterator object returned by fzrt_file_find_file_init */
				char *							file_name, 		/* @parm <out> |  Name of file as C string. User must allocate.*/
				fzrt_boolean *					last			/* @parm <out> |  contains true if this is there are no more files after this. The file_name is valid if last is true and the return value is FZRT_NOERR. */
				);

/************************************************************************

	@fsfunc_t

	@desc	Initializes the iterator for finding sub-folders in a folder. 
			This will create an iterator that can be used to get the 
			names of all sub folders in a folder. The count does not
			include files in the folder.
			
	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see 		fzrt_file_find_finit

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_folder_init_func)(
				fzrt_floc_ptr 					floc, 			/* @parm <in> |  pointer to a folder which contains sub-folders to iterate through */
				fzrt_int *							count, 			/* @parm <out> |  number of sub-folders in folder.*/
				fzrt_file_iter_ptr *			iterator   		/* @parm <out> | receives an iterator object used to get names of sub-folders */
								);
								
/************************************************************************

	@fsfunc_t

	@desc	Gets the name of specified, indexed sub-folder in a folder.
			The index of a sub-folder is considered to be the position
			 of the folder in directory order relative to the position of
			 other sub-folders.
			 Directory order is system dependent.
			 This set the state of the iterator to the specified index.
			 A call to fzrt_file_find_next_folder will return the folder at index+1 if it exists.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see 		fzrt_file_find_folder_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_folder_by_index_func)(
				fzrt_file_iter_ptr 			iterator,			/* @parm <in> | iterator object returned by fzrt_file_find_folder_init */
				fzrt_int 						index, 				/* @parm <in> |   1-based index of file. Must be less than count returned by fzrt_folder_find_init
																	@rimin		1  */
				char *						file_name			/* @parm <out> |  pointer to buffer that receives the name of the file as C string. User must allocate the buffer.*/
				);
								
/************************************************************************

	@fsfunc_t
	
	@desc	Gets the next sub-folder in a parent folder. The next sub-folder is the folder that
			follows the one, in directory order, from the previous call to fzrt_file_find_next_folder,
			with the same iterator. If there has not been a previous call, it returns
			the first sub-folder in the folder. If the sub-folder returned is the last sub-folder
			in the parent folder, the last parameter is set to true.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_next_folder_func)(
				fzrt_file_iter_ptr 			iterator, 			/* @parm <in> | iterator object returned by fzrt_file_find_folder_init */
				char *						folder_name,		/* @parm <out> |  pointer to a buffer that receives the name of sub-folder as C string. User must allocate.*/
				fzrt_boolean *				last				/* @parm <out> |  contains true if there are no more sub-folders, false otherwise.
																					The sub-folder name is valid if "last" is true and the return value is FZRT_NOERR */
				);

/************************************************************************

	@fsfunc_t

	@desc	Releases the find file/folder iterator create with fzrt_file_find_folder_init.
				This function must be called after all other functions requiring a find file iterator.
				The find file iterator invalid after this call.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see		fzrt_file_find_folder_init, fzrt_file_find_file_init
	
	@toc		fzrt_file

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_file_find_finit_func)(
				fzrt_file_iter_ptr *iterator					/* @parm <in> | pointer iterator object created by fzrt_file_find_folder_init or fzrt_file_find_file_init */
				);


/************************************************************************

	@fsfunc_t

	@desc	Gets the specified indexed file or folder. The index
			of a file is system dependent. It is equivalent to calling
			fzrt_file_find_next_floc _n_ times where _n_ is the index
			of the file or folder. The iterator must be created by fzrt_file_find_file_init or fzrt_file_find_folder_init
			before calling this function. If the iterator is created with fzrt_file_find_file_init, this function will
			return the indexed file, and if the iterator is created with fzrt_file_find_folder_init, this function will
			return the indexed sub folder.
			 This sets the state of the iterator to the specified index.
			 A call to fzrt_file_find_next_floc will return the file or folder at index+1, if it exists.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_find_file_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_floc_by_index_func)(
				fzrt_file_iter_ptr 				iterator,			/* @parm <in> | iterator object returned by corresponding fzrt_file_find_xxxx_init function */
				fzrt_int 							index, 				/* @parm <in> |   1-based index of floc to get. Must be less than count returned by corresponding fzrt_file_find_xxxx_init function
																		@rimin		1 */
				fzrt_floc_ptr					floc			/* @parm <out> |  file locator of folder or file. Must be created prior to the call*/
								);
								
/************************************************************************

	@fsfunc_t

	@desc	Gets the next file or folder in a folder. The next floc is the file or folder that
			follows the floc from the previous call to this function with the same iterator. 
			If there has not been a previous call, it returns
			the first file in the folder. If the floc returned is the last 
			in the folder, the last parameter is set to true.
			
			The iterator must be created by fzrt_file_find_file_init or fzrt_file_find_folder_init
			before calling this function. If the iterator is created with fzrt_file_find_file_init, this function will
			return the indexed file, and if the iterator is created with fzrt_file_find_folder_init, this function will
			return the indexed sub folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@see 		fzrt_file_find_file_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_find_next_floc_func)(
				fzrt_file_iter_ptr 				iterator, 		/* @parm <in> | iterator object returned by fzrt_file_find_file_init */
				fzrt_floc_ptr					floc, 		/* @parm <out> |  file locator. Must be created prior to the call.*/
				fzrt_boolean *					last			/* @parm <out> |  contains true if this is there are no more files after this. The file_name is valid if last is true and the return value is FZRT_NOERR. */
				);


/************************************************************************

	@fsfunc_t

	@desc	Converts a file name from the default character set to the 
			character set encoding suitable for the std C file system functions.
			On MacOS X, the file system encoding is UTF8

	@return		FZRT_NOERR on success, Otherwise, an error ID.
				If result filename is too fzrt_int to fit in buffer, the result
				filename is truncated and an error is returned.

	@avail		5.0.0.0


	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC * fzrt_file_cnvt_name_func)(
				 const char *					fname,			/* @parm <in> | Null-terminated filename in default character set */
				 char *							fs_name,		/* @parm <out> |  pointer to filename buffer that contains the null-terminated filename in file system encoding.*/
				 fzrt_unsigned_int					buf_len			/* @parm <in> |  size of output filename buffer  */
				 );

/************************************************************************

	@fsfunc_t

	@desc	Copies the source file to the destination folder. The
				src_file floc must specify a valid file. The dst_file
				floc may be either a folder or a file. If the destination
				floc specifies a folder, the destination file has the same
				name as the source file.
				If the destination contains a file of the same name, and
				FZRT_FILE_COPY_FLAGS_REPLACE is not specified in flags, or 
				the destination file is read_only, the copy will fail.
				If FZRT_FILE_COPY_FLAGS_REPLACE is place in the flags parameter,
				and the file exists, it will be replaced with a copy of the source
				file.
				
	@return		FZRT_NOERR on success, Otherwise, an error ID.
				

	@avail		5.5.0.0


	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *  fzrt_file_copy_func)(
				fzrt_floc_ptr					src_file,		/* @parm <in> |  pointer to a file r */
				fzrt_floc_ptr					dst_file,		 /* @parm <in> |  pointer to a file or folder locator */
				fzrt_file_copy_flags_enum		flags			/* @parm <in> | FZRT_FILE_COPY_FLAGS_NONE, FZRT_FILE_COPY_FLAGS_REPLACE */
				);

/************************************************************************

	@fsfunc_t

	@desc	Gets the type of the disk volume specified by the file locator.
			 The type of the disk volume specifies whether the volume is located
			 on a network, a local fixed disk, or a CD-ROM.
	
	@desc		This returns whether the volume is on a local hard disk, remote hard 
				disk, or a CD-ROM

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_get_vspec_type_func)(
				fzrt_floc_ptr 					floc, 				/* @parm <in> |  pointer to a file or folder locator */
				fzrt_volm_type_enum	*			volume_type			/* @parm <out> |  pointer to volm_type enum that receives the disk volume type */
				);




/************************************************************************

	@fsfunc_t
	
	@desc		Sets the file name in a file locator.
				This will set the filename for a file locator. If the file locator
				specified a folder previous, it will be modified to specify the file 
				within the folder after this call. This is no attempt to validate
				the filename or existence of the file.
				If the name specifies a sub-folder within the folder, the file locator
				is not modified to be a folder locator for the sub-folder. If the modified
				file locator is used in a call to fzrt_file_floc_set_crnt, the working 
				folder will be set to the parent folder, not the subfolder. To receive a
				folder locator for a sub-folder, use fzrt_file_open_folder.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_floc_init, fzrt_file_open_folder

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_set_name_func)(
				fzrt_floc_ptr 				floc,		/* @parm <in> |   file spec created with fzrt_file_floc_init */
				 char *						fname 		/* @parm <in> |  pointer to filename as a C string. */
				 );


/************************************************************************

	@fsfunc_t
	
	@desc		Gets the pathname of a file or folder. 
					This will get the path name of the file. The buffer passed in
				path must be allocated by caller. If the file locator specifies folder,
				the path contains the folder name. If the file locator specifies a file,
				the path contains the parent folder, but not the file name.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_get_path_name_func)(
				fzrt_floc_ptr 				floc,		/* @parm <in> |   file locator created with fzrt_file_floc_init */
				char *						path,    	/* @parm <out> |  pointer to buffer that receives the pathname as a C string. Must be allocated by user */
				fzrt_unsigned_int				maxlen		/* @parm <in> | size of buffer in path parameter */
				);


/************************************************************************

	@fsfunc_t
	
	@desc		Sets the pathname of a file locator. 
				This will set the pathname of a file locator. If the 
				locator was previously a folder locator, it is still a folder locator
				for the new path. If the locator was previously a file locator,
				it is now a file locator for the file at the new path.
				The implementation differs on platforms. Macintosh requires that
				the path exist. There is no existence check on Windows.

	@return		FZRT_NOERR on success, Otherwise, an error ID.

	@avail		5.0.0.0
	
	@see 		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_set_path_name_func)(
				fzrt_floc_ptr 				floc,		/* @parm <in> |  file locator created with fzrt_file_floc_init */
				char *						path    	/* @parm <in> |  pointer to pathname as a C string. Must be allocated by user */
				 );



/************************************************************************

	@fsfunc_t
	
	@desc		Gets the full-path string of file locator.
				The full-path is the folder containing the file with the filename.
				If the locator is a folder locator, the full-path contains the folder name.
				There is no difference between fzrt_file_floc_get_fullpath, and fzrt_file_floc_get_pathname
				for folder locators.
				

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0
	
	@see 		fzrt_file_floc_init

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_file_floc_get_fullpath_func)(
				fzrt_floc_ptr 				floc,		/* @parm <in> |   file locator created with fzrt_file_floc_init. */
				char *						path_str,	/* @parm <out> |  pointer to buffer that receives full path. Must be allocated by the caller */
				fzrt_int 						maxlen		/* @parm <in> |   size of the path_str buffer */
				);



/************************************************************************

	@fsfunc_t
	
	@desc		Checks for existence of a file. 
				This will test for the existence of a file. If the locator
				is a folder locator, this function will fail.

	@return		FZRT_NOERR is file exists, error ID otherwise

	@avail		5.0.0.0
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_does_exist_func)(
				fzrt_floc_ptr 				floc			/* @parm <in> |   file locator created with fzrt_file_floc_init */
				);	

/************************************************************************

	@fsfunc_t
	
	@desc		Displays the Get File dialog.
				This will display a system Get File dialog that will
				select a single file. If the user cancels the dialog,
				this function will return FZRT_NOERR. The result parameter
				will indicate the action of the user.
				The selected file is returned in the file located argument.
				The files displayed to the user can be filtered using a file dialog filter.
				The filter is created using fzrt_file_dlg_filter_new and fzrt_file_dlg_filter_set
				functions. Each entry into the dialog filter should have a unique ID associated
				with it. The start_id parameter determines which filter entry is used
				on display of the dialog. If the start_id does not match any of the IDs in 
				the dialog filter, then all files of each type are displayed. If the dialog file
				filter is NULL, then all files are displayed.

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_get_multiple_files_begin
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_get_file_simple_func)(
					const fzrt_floc_ptr								start_folder,			/* @parm <in_opt> | File locator - start browsing from this folder */
					const char *									message,				/* @parm <in_opt> | Post this message (or default message if "") */
					fzrt_file_dlg_filter_ptr						dlg_filter,				/* @parm <in_opt> | If not NULL a menu with formats appears in the open/close dialogs */
					fzrt_int											count,					/* @parm <in> | Count of entries in the dialog file filter. Ignored if dialog file filter is NULL  */
					fzrt_int											start_id,				/* @parm <in> |  file filter ID of entry to use when dialog is first display. Ignored if dialog file filter is NULL. */
					fzrt_unsigned_int									flags,					/* @parm <in> | Unused. Must be 0 */
					fzrt_floc_ptr									file,					/* @parm <out> | file locator of selected file if FZRT_STD_OK returned */
					fzrt_std_enum *									result					/* @parm <out> | FZRT_STD_OK if OK pressed, FZRT_STD_CANCEL if Cancel pressed, FZRT_STD_ERROR otherwise */
					);

/************************************************************************

	@fsfunc_t
	
	@desc		Creates a new dialog file filter.
				This will allocate a new dialog file filter for the Get File /Save File
				dialogs. The dialog file filter will be created with
				the specified number of entries.

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_dlg_filter_delete, fzrt_file_dlg_filter_set, fzrt_file_get_multiple_files_begin, fzrt_file_get_file_simple
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_file_dlg_filter_new_func)(
					  fzrt_file_dlg_filter_ptr *					dlg_filter,			/* @parm <in_out> | Pointer to a dialog file filter. This will be allocated and returned by the function */
					  fzrt_unsigned_int									cnt					/* @parm <in> |  The number of entries in the dialog file filter 
					  																				@rimin				1			*/
					  );


/************************************************************************

	@fsfunc_t
	
	@desc		Disposes of a dialog file filter.
				This will dispose all memory associated with a dialog file filter.
				The file dialog filter pointer is invalid after this call.

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_dlg_filter_new
	
	@toc		fzrt_file

*************************************************************************/

typedef fzrt_error_td (FZRT_SPEC * fzrt_file_dlg_filter_delete_func)(
					fzrt_file_dlg_filter_ptr *					dlg_filter			/* @parm <in_out> | pointer to file dialog filter, Set to NULL on return */
					);

/************************************************************************

	@fsfunc_t
	
	@desc		Sets the entry of dialog file filter.
				This will set the specified entry in a dialog filter for the Get File/Save
				dialog.  The specified entry must be less than the count passed into fzrt_file_dlg_filter_new.
				The file_id must be a unique number for every entry in the dialog file filter.
				 
				The extensions string can contain up to FZRT_STDF_MAX_NUM_EXT extensions.  Each extension must be
				less than FZRT_STDF_MAX_EXT characters.  The extensions are separated by the colon ':' character.
				If this string contains more than FZRT_STDF_MAX_NUM_EXT extensions, only the first FZRT_STDF_MAX_NUM_EXT
				extensions will be used.
				The menu_name is a C string.

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_dlg_filter_new, fzrt_file_get_multiple_files_begin, fzrt_file_get_file_simple
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_file_dlg_filter_set_func)(
					  fzrt_file_dlg_filter_ptr			dlg_filter,				/* @parm <in> | pointer to a file dialog filter. Must be created with fzrt_file_dlg_filter_new */
					  fzrt_int								index,					/*  @parm <in> | index of entry to set in file dialog filter 0-based */
					  fzrt_int								file_id,				/*  @parm <in> | unique ID of entry in dialog filter 
					  																@rimin		2 		*/
					  fzrt_ostype						type,					/*  @parm <in> | Macintosh file type (Macintosh only) */
					  char *							extensions, 			/*  @parm <in> | file extensions. Must be a C string */
					  char *							menu_name				/*  @parm <in> | Description of file type. C string not longer than 256 characters */
					);

/************************************************************************

	@fsfunc_t
	
	@desc		This will create an OS Type constant from a string. The first
				four characters from the string are used to create a constant
				that can be used as a file type for the dialog filter suitable
				for the MacOS. If the string is shorter than 4 characters, the 
				constant contains 0s in the low bytes. i.e. "TXT" -> 'TXT\0'
				The string must be a null terminated.

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_get_file_simple
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_file_dlg_filter_set_ftype_func)(
					char *								type_string,			/* @parm <in> | pointer to a C string */
					fzrt_ostype *						ftype					/*  @parm <out> | pointer to an fzrt_ostype that will receive the file type constant */
					);

/************************************************************************

	@fsfunc_t
	
	  @desc		Displays a multiple select Get File dialog. 
	  			This will display a system Get File dialog that will
				allow the user to select multiple files. If the user cancels the dialog,
				this function will return FZRT_NOERR. The result parameter will
				indicate the action of the user, if the function is successful.
				The list of file locators is returned by the filetab argument.
				Use the fzrt_file_get_one_file function to retrieve the individual
				file locators from the file table. The caller must call fzrt_file_get_multiple_files_end 
				to release the memory associated with the file table.
				For a description of the dialog file filter, see fzrt_file_get_file_simple.
				

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_get_one_file, fzrt_file_get_multiple_files_end, fzrt_file_get_file_simple
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_get_multiple_files_begin_func)(	
						const fzrt_floc_ptr								start_folder,			/* @parm <in> | File locator - start browsing from this folder */
						const char *									message,				/* @parm <in_opt> | Post this message (or default message if "") */
						fzrt_file_dlg_filter_ptr						entrytab,				/* @parm <in_opt> | If not NULL a menu with formats appears in the open/close dialogs */
						fzrt_int											count,					/* @parm <in> | Count of entries in the file filter menu */
						fzrt_int											start_id,				/* @parm <in> | Start-up file ID of filter */
						fzrt_unsigned_int									flags,					/* @parm <in> | Unused. Must be 0 */
						fzrt_stdffile_ptr *								filetab,				/* @parm <out> | Pointer to selected files storage - this is created by get_multiple_files_begin*/
						fzrt_int *											knt,					/* @parm <out> |Returned selected file count (if zero FZRT_CANCEL was selected) */
						fzrt_std_enum *									result					/* @parm <out> | FZRT_STD_OK if OK pressed, FZRT_STD_CANCEL if Cancel pressed, FZRT_STD_ERROR otherwise */
						);				

/************************************************************************

	@fsfunc_t
	
	@desc	Gets the file locator of a file from multiple select get file dialog.
	
	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0
	
	@see		fzrt_file_get_multiple_files_begin

	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_get_one_file_func)(
						fzrt_stdffile_ptr								filetab,				/* @parm <in> | pointer to table of file locators returned by fzrt_file_get_multiple_files_begin*/
						fzrt_int												knt,					/* @parm <in> | The count of files returned by fzrt_file_get_multiple_files_begin */
						fzrt_int												indx,					/* @parm <in> | The 0 based index of the requested file locator in the buffer
																									@rimin_max  0 | knt */
						fzrt_floc_ptr									folder					/* @parm <out> |	the selected file folder if FZRT_STD_OK returned. The file locator must be initialized by the caller. */
						);

/************************************************************************

	@fsfunc_t

	@desc	Disposes of the file table from a fzrt_file_get_multiple_files_begin function.
	
	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_file_get_multiple_files_begin
	
	@toc		fzrt_file

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_file_get_multiple_files_end_func)(
						fzrt_stdffile_ptr*								filetab,				/* @parm <in> | pointer to table of file locators returned by fzrt_file_get_multiple_files_begin*/
						fzrt_int *											knt						/* @parm <in_out> | pointer to the count of file in the file table. Set to 0 on exit  */
						);

/************************************************************************

	@fsfunc_t
	
	@desc		This will display the select folder dialog, and return the file locator of the folder 
				the user selected. The folder locator must be initialized before calling this
				function. The result parameter will indicate the user action of the dialog, whether
				OK or Cancel was pressed. The dialog may be optionally customized by passing in
				strings for the title of the dialog, and the default name for a New Folder.
				
				
	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_select_folder_func)(
						const fzrt_floc_ptr								start_folder,			/* @parm <in_opt> | Start browsing from this folder. NULL to use default. */
						const char *									start_name,				/* @parm <in_opt> | New folder name or "" for default */
						const char *									message,				/* @parm <in_opt> | Post this message (or default message if "") */
						fzrt_unsigned_int									flags,					/* @parm <in> | Unused, must be 0 */
						fzrt_floc_ptr									folder,					/* @parm <out> | Selected folder location if FZRT_STD_OK is returned */
						fzrt_std_enum *									result					/* @parm <out> | FZRT_STD_OK if OK pressed, FZRT_STD_CANCEL if dialog is canceled, FZRT_STD_ERROR otherwise */
						);
/************************************************************************

	@fsfunc_t
	
	@desc		Displays the Save File dialog.
				This will display the default system's save file dialog.
				The result parameter indicates whether the user click OK,
				Cancel, or if an error occurred.
				The destination file locator is returned in the out_file argument,
				if the user presses the OK/Save	button, and the result is FZRT_STD_OK.
				The caller can create a filter of which files to save as
				using the dlg_filter. The file_id argument will contain which entry in 
				the dialog filter was selected by the user, as an indication of the Save as file type.
				If the user passes NULL for dlg_filter,	the result of file_id is undefined. 
				The initial name of the saved file name is passed as a string in the start_name parameter. 

	@return		FZRT_NOERR is successful, error number otherwise

	@avail		5.0.0.0
	
	@see	fzrt_file_dlg_filter_new, fzrt_file_dlg_filter_set
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_file_put_file_func)(
						const fzrt_floc_ptr					start_folder,			/* @parm <in> | File locator - start browsing from this folder */
						const char *						start_name,				/* @parm <in> | default name for saved file) */
						const char *						message,				/* @parm <in_opt> | Post this message (or default message if "") */
						fzrt_file_dlg_filter_ptr			dlg_filter,				/* @parm <in_opt> | If not NULL a menu with formats appears in the open/close dialogs */
						fzrt_int								count,					/* @parm <in> | Count of entries in the file filter menu */
						fzrt_int								start_id,				/* @parm <in> | Start-up file ID of filter */
						fzrt_floc_ptr						out_file,				/* @parm <out> | file locator of selected file if FZRT_STD_OK returned */
						fzrt_int *								file_id	,				/* @parm <out_opt> | the file id of entry in dlg_filter selected when OK is pressed*/
						fzrt_std_enum *						result					/* @parm <out> | FZRT_STD_OK if OK pressed, FZRT_STD_CANCEL if Cancel pressed, FZRT_STD_ERROR otherwise */
					);



/************************************************************************

	@fsfunc_t
	
	@desc		Gets the maximum length of a file name.
				This will return the maximum length of a file name on the current system,
				in characters.

	@return		size in characters of a filename.

	@avail		5.0.0.0
	
	@toc		fzrt_file

*************************************************************************/
typedef fzrt_unsigned_int   (FZRT_SPEC *fzrt_file_get_fnam_max_func)( void );	


/************************************************************************
	FZRT Resource File (FZRF) functions
*************************************************************************/
/******************************************************************************
	
	@fsfunc_t
	
	@desc	Opens a formZ Resource File.
			This will open the resource file with the specified "base" filename. 
			The resource reference that is returned represents collection of resources in two different languages. 
			This function will open up to 2 formZ resource files named from the base name, in the form
			base_filename.XXY.fzr : where XX is a 2 letter code defined by ISO standard 639
			and Y is a letter code for the sub language as defined by Microsoft Windows <nl>
								ENU  - US English <nl>
								FRA  - France <nl>
								DEU	 - German <nl>
								ESP  - Spain <nl>
								ELL  - Greek <nl>
								ITA  - Italy <nl>
								JPN  - Japan (Hirigana, Katakana, Kanji) <nl>
								CHS  - Chinese Simplified (PRC)  <nl>
								CHT  - Chinese Traditional (Taiwan) <nl>
								KOR	 - Korean (Hangul, Hanja) <nl>
								
		The language is determined by the current language as set in the preferences. The default current language is the system language.
		The alternate language of the pair of resource files is always U.S. English.
			
		Resource access functions that take a resource reference as input will search the
		primary file first, if that search fails,  the default language file is searched (US English). 
		If there is already an existing(open) resource reference for the file, the existing reference
		is returned.
		
		If neither the primary language file nor US English language file is found, the function
		will fail and return an error. This function succeeds if either file is opened successfully.

	@return		FZRT_NOERR				on successful open <nl>
				FZRT_ERR_NOT_FZRF_FILE	if either of the files is not a FZRF file.	<nl>							
				error ID otherwise

	@avail		5.0.0.0
	
	@see		fzrt_fzr_close, fzrt_lang_get
	
	@toc		fzrt_base

*******************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_fzr_open_func)(
				const fzrt_floc_ptr			floc,			/* @parm <in> | file locator of the folder to find resource file */
				const char					*base_fname,	/* @parm <in> | base filename of the resource file as a C string */
				fzrt_fzr_ref_td				*fzrf_ref		/* @parm <out> | reference to the resource file */
				);

/************************************************************************

	@fsfunc_t

	@desc		Closes a formZ resource reference.
				This will decrease the reference count on the files, and close
				the files if the reference count reaches zero.		

	@avail		5.0.0.0

	@see		fzrt_fzr_open

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_close_func)(
				fzrt_fzr_ref_td				fzr_ref	/* @parm <in> | reference to resource file */
				);


/************************************************************************

	@fsfunc_t

	@desc		Gets a string from a resource file.
				This will load a string from the specified resource file, that is located in resource ID list_id
				and at the 1-based index. String resources are labeled as STR# in the resource file.
				The result is a C string in the default character set.

	@return		pointer to the string. Same as parameter "str", NULL if not found.

	@avail		5.0.0.0

	@see		fzrt_fzr_open

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_fzr_get_string_func)(
				fzrt_fzr_ref_td				fzr_ref,		/* @parm <in> | reference to the resource file */
				fzrt_unsigned_int				list_id,		/* @parm <in> | ID of the string resource in the file */
				fzrt_unsigned_int				index,			/* @parm <in> | 1-based index to the string in the string list  
																@rimin		1										*/
				fzrt_ptr					str				/* @parm <out> | pointer to buffer, allocated by caller, that holds result C string */
										);

/************************************************************************

	@fsfunc_t

	@desc		Gets a string from the resource file in Pascal string format.
				This will load a string from the specified resource file, that is located in resource ID list_id
				and at the 1-based index. String resources are labeled as STR# in the resource file.
				The result is a Pascal string in default character set

	@return		pointer to the string. Same as parameter "str", NULL if not found.

	@avail		5.0.0.0

	@see		fzrt_fzr_open, fzrt_fzr_get_string

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_fzr_get_stringP_func)(
				fzrt_fzr_ref_td				fzr_ref,		/* @parm <in> | reference to the resource file */
				fzrt_unsigned_int				list_id,		/* @parm <in> | ID of the string resource in the file */
				fzrt_unsigned_int				index,			/* @parm <in> | 1-based index to the string in the string list 
																@rimin	1										*/
				fzrt_ptr					str				/* @parm <out> | pointer to buffer, allocated by caller, that holds result Pascal String */
				);		

/************************************************************************

	@fsfunc_t

	@desc		Gets the number of strings in resource string list.
				This will return the number of string in the specified
				string list in the specified resource file. It will return
				the number of string in the primary resource file, if the
				string list exists in the primary resource file. If the
				string list does not exist in the primary resource file,
				it returns the number of strings in the string list in the
				English version of the resource file.

	@return		FZRT_NOERR if successful, error ID if not found.

	@avail		5.0.0.0

	@see		fzrt_fzr_open

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_string_cnt_func)(
				fzrt_fzr_ref_td			fzr_ref,				/* @parm <in> | reference to resource file returned by fzrt_fzr_open */
				fzrt_unsigned_int			list_id,				/* @parm <in> | ID of STR# resource */
				fzrt_unsigned_int			*string_cnt				/* @parm <out> | pointer to variable to receive number of strings in resource */
				);			


/************************************************************************

	@fsfunc_t

	@desc		Gets a menu from a formZ resource file.
				This will load a menu from the resource file. The menu resource is labeled with  "MENU"
				and is the same format as a string list. The first string is the menu title.
				The first menu item is the second string in the list.
				
	@return		FZRT_NOERR if successful, an error ID otherwise.

	@avail		5.0.0.0

	@see		fzrt_fzr_open

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_get_menu_func)(
				fzrt_fzr_ref_td				fzr_ref,				/* @parm <in> | reference to resource file returned by fzrt_fzr_open */
				fzrt_int						menuID,					/* @parm <in> | ID of MENU resource */
				fzrt_menu_ptr				*menu					/* @parm <out> | pointer to menu_ptr that receives the loaded menu */
				);

/************************************************************************

	@fsfunc_t

	@desc		Gets a string from a menu resource.
				This will load a string a MENU resource from the resource file. 
				The Menu resource is labeled with "MENU", and is the same format as a string list.
				The first string is the menu title.	The first menu item is the second string in the list.
				The index parameter corresponds the index in the string list.

	@return		FZRT_NOERR if successful, an error ID otherwise.

	@avail		5.0.0.0

	@see		fzrt_fzr_get_string

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *  fzrt_fzr_get_menu_string_func)(
				fzrt_fzr_ref_td				fzr_ref,		/* @parm <in> | reference to the resource file */
				fzrt_unsigned_int				list_id,		/* @parm <in> | ID of the string resource in the file */
				fzrt_unsigned_int				index,			/* @parm <in> | 1-based index to the string in the string list  
																@rimin		1                                    */
				fzrt_ptr					str				/* @parm <out> | pointer to buffer, allocated by caller, that holds result C string */
				);

/************************************************************************

	@fsfunc_t

	@desc		Gets a Pascal string from a menu resource.
				This will load a string a MENU resource from the resource file. The Menu resource is labeled with the MENU
				name. The resource is the same format as a string list. The first string is the menu title.
				The first menu item is the second string in the list. The index parameter corresponds the index in the
				string list. The string is a Pascal String.

	@return		FZRT_NOERR if successful, an error ID otherwise.

	@avail		5.0.0.0

	@see		fzrt_fzr_get_stringP, fzrt_fzr_get_menu_string

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *  fzrt_fzr_get_menu_stringP_func)(
				fzrt_fzr_ref_td					fzr_ref,		/* @parm <in> | reference to the resource file */				
				fzrt_unsigned_int					list_id,
				fzrt_unsigned_int					index,
				fzrt_ptr						str
				);
													  
/************************************************************************

	@fsfunc_t

	@desc		Gets the resource ID of the specified type in the resource file.
				This will return the ID the resource as specified by the index. Index order is the 
				order the resource is located in the file from the top of the text file.
				It is recommended that resources be accessed by ID rather than by index. 

	@return		FZRT_NOERR if successful, error ID otherwise.

	@avail		5.0.0.0

	@see		fzrt_fzr_open

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_get_rsrc_ind_ID_func)(
				const fzrt_fzr_ref_td		fzr_ref,		/* @parm <in> | reference to the resource file */			
				fzrt_unsigned_int  			type,  				/* @parm <in> | type of resource to count (FZRT_FZR_STRING_ID or FZRT_FZR_MENU_ID) */
				fzrt_unsigned_int 			index, 				/* @parm <in> | index of resource
																@rimin		1	 */
				fzrt_unsigned_int 			*ID					/* @parm <out> | ID of resource */
				);
										
/************************************************************************

	@fsfunc_t
	
	@desc		Gets the number of resources of specified type in the resource file.
				This will return the number of resource of the requested type in the primary
				resource file only.

	@return		FZRT_NOERR if successful, error ID otherwise.

	@avail		5.0.0.0

	@see		fzrt_fzr_open, fzrt_fzr_get_rsrc_ind_ID

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_get_rsrc_cnt_func)(
				const fzrt_fzr_ref_td		fzr_ref,			/* @parm <in> | reference to the resource file */
				fzrt_unsigned_int  				type,				/* @parm <in> | type of resource to count ( currently only FZRT_FZR_STRING_ID or FZRT_FZR_MENU_ID) */
				fzrt_unsigned_int				*cnt				/* @parm <out> |- pointer fzrt_unsigned_int contains number of resources of type */
				);				
										

/************************************************************************

	@fsfunc_t
	
	@desc		Write the source resource file to the new destination using the
				current platform and language character set. The source floc parameter
				indicates the source folder of the resource file, and the destination
				floc designates where the result resource file is written. The base filename
				is used for both the source and destination resource files. The current
				language preference is used for both the source and destination filenames. 
				The source and destination folders must be different.
				Comments inside the resource file are not preserved.

	@return		FZRT_NOERR if successful, error ID otherwise.

	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_fzr_write_file_func)(
				const fzrt_floc_ptr					src_floc,				/* @parm <in> | file locator of the source folder of resource file */
				const fzrt_floc_ptr					dst_floc,				/* @parm <in> | file locator of the destination folder to write the new resource file */
				const char							*base_fname				/* @parm <in> | base filename of the resource file. Filename without Language or file extension */
				);				


/************************************************************************

	@fsfunc_t
	
	@desc		Get information about a resource file.
				This will get information about a resource file and the
				data in the resource file. All output parameters are optional.
				Pass NULL if the information is not needed.
				The version of the resource format will always be 40.
				The fver is the resource author's version string. If it is not
				included in the file, the null string is passed back.

	@return		FZRT_NOERR if successful, error ID otherwise.

	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC  *fzrt_fzr_get_info_func)(
				const fzrt_fzr_ref_td		fzr_ref,			/* @parm <in> | reference to the resource file */
				short						*version,			/* @parm <out_opt> | file format version of fzrf. This will always be 40 */
				fzrt_lang_td				*lang,				/* @parm <out_opt> | language of the resource file, as determined by filename extension */
				fzrt_charset_td				*char_set,			/* @parm <out_opt> | character set used in resource file. Determined by language and CHAR= value in FZRF block */
				char						*fver,				/* @parm <out_opt> | version of the resource file as indicated by FVER= string in FZRF block */
				fzrt_floc_ptr				floc				/* @parm <out_opt> | pointer to file locator. Must be initialized before call */
				);

/************************************************************************

	@fsfunc_t
	
	@desc		Gets the size of the string
				This will return the number of bytes needed to store the 
				string, including any terminating or length bytes. This is
				not the same as the string length. This function is useful
				if you need to allocated storage before reading the string.

	@return		FZRT_NOERR if successful, error ID otherwise.

	@see		fzrt_fzr_open, fzrt_fzr_get_string
	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_fzr_get_string_size_func)(
				 const fzrt_fzr_ref_td		fzr_ref,			/* @parm <in> | reference to the resource file */
				fzrt_unsigned_int				list_id,			/* @parm <in> | ID of the string resource in the file */
				fzrt_unsigned_int				index,				/* @parm <in> | 1-based index to the string in the string list  */
				fzrt_unsigned_int				*size				/* @parm <out> | pointer to an unsigned short that receives the number of bytes in the string */
				);
	
/******************************************************************************
	Runtime Memory functions
******************************************************************************/


/************************************************************************

	@fsfunc_t

	@desc		Move the contents of memory.
				This will copy the contents of memory from one pointer
				to another. This call will move memory in the most
				efficient manner and handles blocks of overlapping memory.
						
	@avail		5.0.0.0

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_block_move_func)(
				const void *			srcPtr,			/* @parm <in> | pointer to source memory location */
				void *					destPtr,		/* @parm <in> | pointer to destination memory location. Must be allocated by user*/
				fzrt_size 				byteCount		/* @parm <in> | size in bytes of memory to move */
				);

/************************************************************************

	@fsfunc_t

	@desc		version of fzrt_block_move that uses 64 bit byteCount parameter (ie > 2 GB)
						
	@avail		8.5.0.1

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_block_move_64_func)(
				const void *			srcPtr,			/* @parm <in> | pointer to source memory location */
				void *					destPtr,		/* @parm <in> | pointer to destination memory location. Must be allocated by user*/
				fzrt_int64 				byteCount		/* @parm <in> | size in bytes of memory to move */
				);

/************************************************************************

	@fsfunc_t

	@desc		Gets the amount of free memory.
				This will return the amount of free memory in the 
				system available to the application. Under virtual
				memory systems, this is just an estimate.
				This includes all fragmented blocks in the heap, so 
				it is possible that you cannot allocate this much
				memory with one allocation. 

	@return		The amount of free memory in bytes
						
	@avail		5.0.0.0

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_free_mem_func)(
				fzrt_unsigned_int *free_mem				/* @parm <out> | The size, in bytes, of the available memory in the heap */
				);


/************************************************************************

	@fsfunc_t

	@desc		Gets the maximum available memory.
				This will return the size, in bytes, of the largest contiguous free block 
				in the application heap	after purging and compacting. Under virtual
				memory systems, this is just an estimate.
				The grow parameter is the amount by which the heap can grow. 
						
	@return		The size, in bytes, of the largest contiguous free block in the heap
				after purging and compacting.

	@avail		5.0.0.0

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_max_mem_func)(
				fzrt_size *					grow,	/* @parm <out> | The maximum number of bytes by which the current heap zone can grow. */
				fzrt_size *					size	/* @parm <out> | The size, in bytes, of the largest contiguous free block in the heap
																		after purging and compacting */
				);


/************************************************************************

	@fsfunc_t

	@desc		version of fzrt_max_mem that uses 64 bit byteCount parameter (ie > 2 GB).
						
	@return		The size, in bytes, of the largest contiguous free block in the heap
				after purging and compacting.

	@avail		5.0.0.0

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_max_mem_64_func)(
				fzrt_int64 *					grow,	/* @parm <out> | The maximum number of bytes by which the current heap zone can grow. */
				fzrt_int64 *					size	/* @parm <out> | The size, in bytes, of the largest contiguous free block in the heap
																		after purging and compacting */
				);


/************************************************************************

	@fsfunc_t

	@desc		Disposes of memory allocated by fzrt_new_ptr.
				The pointer is invalid after the call to this function.

	@return		FZRT_NOERR if successful, otherwise an error ID

	@avail		5.0.0.0

	@see		fzrt_new_ptr

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_dispose_ptr_func)(
				fzrt_ptr 					p		/* @parm <in> | pointer to memory allocated by fzrt_new_ptr */
				);




/************************************************************************

	@fsfunc_t

	@desc		Allocates a non-relocatable block of memory initialized to zeros.
				This will allocate a block of memory in the applications heap and
				initialize the memory to all zeros.
				This block is non-relocatable on MacOS 9, and may cause fragmentation
				of memory, therefore it should only be used to allocate temporary
				memory. Persistent blocks of dynamic memory should use mem zones.		

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		5.0.0.0

	@see		fzrt_dispose_ptr, fz_mem_zone_alloc

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_new_ptr_clear_func)(
				fzrt_size 					byteCount,		/* @parm <in> | the desired size of the memory block, in bytes */
				fzrt_ptr	*				new_ptr			/* @parm <out> | pointer to a pointer that receives the pointer to a newly allocated memory block */
				);

/************************************************************************

	@fsfunc_t

	@desc		version of fzrt_new_ptr_clear that uses 64 bit byteCount parameter (ie > 2 GB).

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		8.5.0.1

	@see		fzrt_new_ptr, fzrt_dispose_ptr, fz_mem_zone_alloc

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_new_ptr_clear_64_func)(
				fzrt_int64 					byteCount,		/* @parm <in> | the desired size of the memory block, in bytes */
				fzrt_ptr	*				new_ptr			/* @parm <out> | pointer to a pointer that receives the pointer to a newly allocated memory block */
				);

/************************************************************************

	@fsfunc_t

	@desc		Allocates a non-relocatable block of memory.
				This will allocate a block of memory in the applications heap and set the contents to zeros.
				This block is non-relocatable on MacOS 9, and may cause fragmentation
				of memory, therefore it should only be used to allocate temporary
				memory. Persistent blocks of dynamic memory should use mem zones.		

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		5.0.0.0

	@see		fzrt_dispose_ptr, fz_mem_zone_alloc, fzrt_new_ptr_clear_64

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_new_ptr_func)(
				fzrt_size 					byteCount,		/* @parm <in> | the desired size of the memory block */
				fzrt_ptr	*				new_ptr			/* @parm <out> | pointer to a pointer that receives the pointer to a newly allocated memory block */
				);


/************************************************************************

	@fsfunc_t

	@desc		version of fzrt_new_ptr that uses 64 bit byteCount parameter (ie > 2 GB).

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		8.5.0.1

	@see		fzrt_new_ptr, fzrt_dispose_ptr, fz_mem_zone_alloc

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fzrt_new_ptr_64_func)(
				fzrt_int64 					byteCount,		/* @parm <in> | the desired size of the memory block, in bytes */
				fzrt_ptr	*				new_ptr			/* @parm <out> | pointer to a pointer that receives the pointer to a newly allocated memory block */
				);

/************************************************************************

	@fsfunc_t

	@desc		Gets the amount of free physical memory.
				This function will return amount of memory that has not been 
				allocated than can still fit in physical RAM. This does not necessarily
				mean that you can allocate this much memory, or that allocating this
				amount of memory is guaranteed to remain in physical memory.

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		5.0.0.0

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_FreeMemPhys_func)(
				 fzrt_unsigned_int *phys_mem);

/************************************************************************

	@fsfunc_t
	
	@desc		Gets the amount of physical RAM.
				This returns the size, in bytes of the amount of physical RAM installed
				on the system.

	@return		FZRT_NOERR if successful, err code otherwise

	@avail		5.0.0.0

	@see		fzrt_free_mem

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_total_mem_phys_func)(
				fzrt_unsigned_int * mem_free);

/************************************************************************

	@fsfunc_t
	
	@desc		Reallocates memory. This will reallocate memory allocated by fzrt_new_ptr, in
				a more robust way but it may potentially move memory and 
				change the pointer value. Any copies of the pointer will be invalid, in this case.

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_free_mem

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_realloc_mem_func)(
				void 				**ptr, 			/* @parm <in_out> | pointer to memory allocated by fzrt_new_ptr
															on exit contains pointer to reallocated memory */
				fzrt_unsigned_int 		newsize			/* @parm <in> | desired size of new memory block */
								);

/************************************************************************

	@fsfunc_t

	@desc		Gets the amount of free memory. This will return the amount of free memory available to the application. 
				This include memory that is in the heap and unused memory in the mem zones. 
				Since this includes all the free memory available to the program, not necessarily
				contiguous memory, it is unlikely that a allocation request of this size will succeed.
						
	@return		FZRT_NOERR on success, otherwise an error ID
	
	@avail		5.0.0.0
	
	@see		fzrt_free_mem, fz_mem_zone_static_init, fz_mem_zone_dynamic_init

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_mem_free_func)(
				fzrt_unsigned_int	*mem_free			/* @parm <out> | pointer to fzrt_unsigned_int to receive the amount of free memory */
				);




/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of 2 bytes. This is useful for switching between endian	
				byte order of a 2 byte value
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_2_bytes_func)(
				void 			*data							/* @parm <In_out> | pointer to 2 bytes to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a short integer. This is useful for switching between endian	
				byte order of a short integer
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_2_bytes, fzrt_endian_swap_long

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_short_func)(
				short 			*data						/* @parm <In_out> | pointer to short to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of an unsigned short integer. 
				This is useful for switching between endian	byte order of a unsigned short integer
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_2_bytes, fzrt_endian_swap_long

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_ushort_func)(
				unsigned short 			*data				/* @parm <In_out> | pointer to unsigned short to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of 4 bytes.
				This will swap 4 bytes of memory, useful for switch endian	
				byte order of a fzrt_int integer
						
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_2_bytes, fzrt_endian_swap_long

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_4_bytes_func)(
				void 			*		data				/* @parm <In_out> | pointer to 4 bytes to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a fzrt_int integer. This is useful for switching between endian	
				byte order of a fzrt_int integer
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_long_func)(
				fzrt_int 			*		data				/* @parm <In_out> | pointer to fzrt_int to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of an fzrt_unsigned_int integer. 
				This is useful for switching between endian	byte order of a fzrt_int integer
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_endian_swap_ulong_func)(
				fzrt_unsigned_int 			*data				/* @parm <In_out> | pointer to fzrt_unsigned_int to swap, contains swapped value on exit */
				);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a floating point value. 
				This is useful for switching between endian	floating point number
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_float_func)(
						float *							data				/* @parm <In_out> | pointer to float to swap, contains swapped value on exit */
						);
/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of array of bytes 
				This is useful for switching between endian	byte order of any integer size.
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_n_bytes_func)(
						void *						data, 							/* @parm <In_out> | pointer to array of bytes to swap, contains swapped values on exit */
						short 						num_bytes						/* @parm <In> | number of bytes to swap*/
						);
/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a array of fzrt_int integers. 
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_n_longs_func)(
						fzrt_int *						data, 					/* @parm <In_out> | pointer to array of longs to swap, contains swapped values on exit */
						fzrt_int 						num_longs				/* @parm <In> | number of fzrt_int values to swap */
						);
/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of an array floating point numbers.
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_n_floats_func)(
						float *					data, 						/* @parm <In_out> | pointer to array of floats to swap, contains swapped values on exit */
						fzrt_int 					num_floats					/* @parm <In> | number of floats to swap */
						);
/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a 64 bit value. 
				This is useful for switching between endian	 byte order of a 64 bit value
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_8_bytes_func)(
						void *					data					/* @parm <In_out> | pointer to 8 bytes to swap, contains swapped value on exit */
						);
/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a 96 bit (12 byte) value. 
				This is useful for switching between endian	byte order of a 96 bit value
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_12_bytes_func)(
							void *				data					/* @parm <In_out> | pointer to 21 bytes to swap, contains swapped value on exit */
							);

/************************************************************************

	@fsfunc_t

	@desc		Swaps byte order of a floating point double. 
				This is useful for switching between endian	
				byte order of a double value
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td	(FZRT_SPEC *fzrt_endian_swap_double_func)(
						double *					data					/* @parm <In_out> | pointer to double to swap, contains swapped value on exit */
						);

/************************************************************************

	@fsfunc_t

	@desc		Initializes memory.
				This will set the number of bytes of memory pointed to by ptr to zeros.
				
	@return		FZRT_NOERR on success, otherwise an error ID
						
	@avail		5.0.0.0
	
	@see		fzrt_endian_swap_4_bytes, fzrt_endian_swap_short

	@toc		fzrt_mem

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_init_mem_block_func)(
						fzrt_ptr 					ptr, 					/* @parm <In_out> | pointer to memory to clear */
						fzrt_int 						size					/* @parm <In> | The number of bytes to clear */
						);

/************************************************************************

	@fsfunc_t

	@desc		Gets the available memory in mem zones.
				This will return the amount of free memory across all mem zones
				in the application.		

	@return		The size in bytes, not allocated in all mem zones.

	@avail		5.0.0.0

	@see		fz_mem_zone_free_bytes, fz_mem_zone_dynamic_init

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_unsigned_int (FZRT_SPEC *fz_mem_zone_get_free_bytes_func)(
						void);

/************************************************************************

	@fsfunc_t

	@desc		Gets the available memory in specified mem zone.
					This will return the amount of free memory in one mem zone.

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_free_bytes, fz_mem_zone_dynamic_init

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fz_mem_zone_free_bytes_func)(
				fzrt_zone_ptr 			zone_ptr,					/* @parm <in> | mem zone of which to get the free bytes */
				fzrt_unsigned_int *			free						/* @parm <out> | pointer to fzrt_int that receives the amount of free space in bytes */
				);


/************************************************************************

	fsfunc_t

	return		The error of the last mem zone call.

	avail		5.0.0.0

	toc		fzrt_mem

*************************************************************************/
typedef fzrt_oserr   (FZRT_SPEC *mem_zone_error_func)(
				void);

/************************************************************************

	@fsfunc_t

	@desc		This will create a mem zone with a set amount of memory. 
				Pointers can be allocated from this zone until the zone	
				is full.		

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_alloc, fz_mem_zone_free

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_static_init_func)(
				fzrt_zone_ptr 			*zone_static_ptr, 	/* @parm <out> | the newly create static mem zone */
				fzrt_int 					size				/* @parm <in> | the size of the memory zone */
				);
							
/************************************************************************

	@fsfunc_t

	@desc		This will create a mem zone that will grow as needed. 
				Pointers can be allocated from this zone and the zone will
				grow by the specified block size if there is not enough
				memory in the zone to fulfill the request

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_alloc, fz_mem_zone_free

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_dynamic_init_func)(
				fzrt_zone_ptr 			*zone_dynamic_ptr, 		/* @parm <out> | the newly create zone */	
				fzrt_int 					zone_block_size			/* @parm <in> | block size to use for zone */
				);

/************************************************************************

	@fsfunc_t

	@desc		This will dispose of all memory in the zone and any associate
				zone structures. The mem zone is invalid for any mem zone calls
				after this call.

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_static_init

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_mem_zone_finit_func)(
				fzrt_zone_ptr 			*zone__ptr		/* @parm <in> | mem zone to release */
				);

/************************************************************************

	@fsfunc_t

	@desc		This will allocate a memory block from a mem zone. The
				mem zone can be either static or dynamic, or NULL. If 
				the mem zone is NULL, the call is the same a fzrt_new_ptr.
				The memory must be freed with a call to fz_mem_zone_free, using
				the same mem zone. A pointer to the newly allocated memory is returned
				if successful; it is NULL if unsuccessful

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_static_init, fz_mem_zone_free

	@toc		fzrt_mem

*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC *fz_mem_zone_alloc_func)(
				fzrt_zone_ptr 			zone_ptr, 	/* @parm <in_opt> | mem zone to allocate memory from, NULL to allocate from system */
				fzrt_int 					size, 		/* @parm <in> | size of memory to allocate */
				fzrt_boolean			clearit,	/* @parm <in> | TRUE to clear memory to zeros, FALSE otherwise */
				fzrt_ptr *				new_ptr		/* @parm <out> | pointer to pointer that receives the newly allocated pointer */
				);

/************************************************************************

	@fsfunc_t

	@desc		Disposes of a memory block.
				This will release a memory block back to the mem zone. The
				mem zone can be either static or dynamic, or NULL. If 
				the mem zone is NULL, the call is the same a fzrt_dispose_ptr.
				The memory must be freed using the same mem zone that it was allocated with.

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_static_init, fz_mem_zone_alloc

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fz_mem_zone_free_func)(
				fzrt_zone_ptr 			zone_ptr,  	/* @parm <in> | mem zone to allocate memory from, NULL to release from system */
				fzrt_ptr 					*ptr	/* @parm <in> | pointer to memory to release */
				);
				
/************************************************************************

	@fsfunc_t

	@desc		Reallocates a memory block.
				This will reallocate a memory block from a mem zone. The
				mem zone can be either static or dynamic, or NULL. If 
				the mem zone is NULL, the call is the same a fzrt_realloc_mem.
				The memory must be reallocated using the same mem zone that it was allocated with.

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_dynamic_init, fz_mem_zone_static_init, fz_mem_zone_alloc

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_realloc_func)(
				fzrt_zone_ptr 			zone_ptr,   	/* @parm <in> | mem zone to allocate memory from, NULL to release from system */
				fzrt_ptr 					*ptr, 		/* @parm <in_out> | pointer to pointer to memory to reallocate. On exit contains the new pointer to reallocated memory */
				fzrt_int 					size			/* @parm <in> | desired new size of memory block */
				);


/************************************************************************

	@fsfunc_t

	@desc		This will reallocate a memory block in a memory zone by the 
				specified size, where size is calculated by data_size*(block_size+incr).
				The block size is updated to the newly reallocated number
				of data elements		

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_incr_ptr_long, fz_mem_zone_incr_ptr_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_incr_ptr_func)(
				fzrt_zone_ptr 			zone_ptr, 		/* @parm <in> | mem zone that pointer is a member of */ 
				fzrt_ptr 				*ptr, 			/* @parm <in> | pointer to memory of which to increase the size */	
				fzrt_int 					*block_size, 	/* @parm <in_out> | pointer to an fzrt_int. On input this is the current size of elements in the pointer, on exit this is updated by the increment */
				fzrt_int					data_size, 		/* @parm <in> | the size of the data elements */
				fzrt_int 					incr			/* @parm <in> | the number of data elements to increase the size by */
				);

/************************************************************************

	@fsfunc_t

	@desc		This will reallocate a memory block in a memory zone by the 
				specified size, where the size if the data_size*(block_size+incr).
				The block size is updated to the newly reallocated number
				of data elements		

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_incr_ptr, fz_mem_zone_incr_ptr_short

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_incr_ptr_long_func)(
				fzrt_zone_ptr 			zone_ptr, 		/* @parm <in> | mem zone that pointer is a member of */ 
				fzrt_ptr 				*ptr, 			/* @parm <in> | pointer to memory of which to increase the size */	
				fzrt_int 					*block_size,    /* @parm <in_out> | pointer to an fzrt_int. On input this is the current size of elements in the pointer, on exit this is updated by the increment */
				fzrt_int 					data_size, 		/* @parm <in> | the size of the data elements */
				fzrt_int 					incr			/* @parm <in> | the number of data elements to increase the size by */
				);

/************************************************************************

	@fsfunc_t

	@desc		Resize a block of memory save number of elements in a short integer.
				This will reallocate a memory block in a memory zone by the 
				specified size, where the size if the data_size*(block_size+incr).
				The block size is updated to the newly reallocated number
				of data elements		

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_incr_ptr, fz_mem_zone_incr_ptr_long

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_incr_ptr_short_func)(
				fzrt_zone_ptr 			zone_ptr, 		/* @parm <in> | mem zone that pointer is a member of */ 
				fzrt_ptr 				*ptr, 		/* @parm <in> | pointer to memory of which to increase the size */	
				short 					*block_size,    /* @parm <in_out> | pointer to an short. On input this is the current size of elements in the pointer, on exit this is updated by the increment */
				fzrt_int 					data_size, 		/* @parm <in> | the size of the data elements */
				short 					incr			/* @parm <in> | the number of data elements to increase the size by */
				);

/************************************************************************

	@fsfunc_t

	@desc		Resize a block of memory based on data size.
				This will reallocate a memory block , if necessary, to 
				hold the more data elements as specified by the increment.
				The function will reallocate in predefined block sizes
				to avoid memory fragmentation and improve performance.
				This function may increase the memory block by more than the requested
				increment. Each subsequent call to this function should pass the same
				pointer for the size parameter, as this contains the true allocated size.
				The parameter "last" contains the logical number of elements used in the 
				pointer, this parameter should always be less than or equal to the "size" parameter. 

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_incr_ptr

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_incr_ptr_general_func)(
				fzrt_zone_ptr 			zone_ptr, 	/* @parm <in> | mem zone that pointer is a member of */ 
				fzrt_ptr 				*ptr, 		/* @parm <in> | pointer to memory of which to increase the size */	
				fzrt_int 					data_size,  /* @parm <in> | the size of the data elements */
				fzrt_int					last, 		/* @parm <in> | Then number of data elements currently used in the memory block */
				fzrt_int 					*size, 		 /* @parm <in_out> | pointer to an integer. On input this is the current number of data elements the pointer can hold, on exit this is the updated number of elements the pointer could hold. */
				fzrt_int 					incr		/* @parm <in> | the minimum number of data elements to increase the size */
				);

/************************************************************************

	@fsfunc_t

	@desc		Resize a block of memory containing an array of same sized elements.
				This will reallocate a memory block , if necessary to 
				hold the required number of data elements as specified.
				The function will reallocate by multiples of the defined increment.
				The user must specify a pointer to an fzrt_int integer which holds the 
				current size of the pointer, in number of data elements. 

	@return		FZRT_NOERR on success, error ID otherwise.

	@avail		5.0.0.0

	@see		fz_mem_zone_incr_ptr

	@toc		fzrt_mem

*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC *fz_mem_zone_incr_array_func)(
				fzrt_zone_ptr			zone_ptr, 		/* @parm <in> | mem zone that pointer is a member of */ 
				fzrt_ptr				*array, 		/* @parm <in> | pointer to memory array of which to increase the size */	
				fzrt_int 					*curr_size, 	 /* @parm <in_out> | pointer to an fzrt_int. On input this is the current number of data elements the pointer can hold, on exit this is updated number of elements the pointer could hold*/
				fzrt_int 					item_size, 		/* @parm <in> | the size of the data elements */
				fzrt_int 					req_size, 		/* @parm <in> | Then required number of data elements the array needs to hold */
				fzrt_int 					dincr			/*@parm <in> | Then number of elements to increase the array by, if necessary */
				);
				
				
/********************************************************************************************************

Runtime Error functions

*********************************************************************************************************/
/************************************************************************

	@fsfunc_t

	@desc		Writes a message to the error log.
				This will write a time stamped message to the error log file.		

	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_error_log_msg_func)(
						fzrt_int	id,		/* @parm <in> | error id */
						char	*str	/* @parm <in> | Message to write to log file, as a C string */
						);
						
/************************************************************************

	@fsfunc_t

	@desc		Sets the current error with detail.
				This assigns an error ID to an error with an additional detail id.	
				The detail ID can have an additional string associated with it.		

	@return		Runtime error ID associated with this instance of the err.

	@avail		5.0.0.0

	@see		fzrt_error_add_detail, fzrt_error_set

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_error_set_with_detail_func)(
				fzrt_int 					err,				/* @parm <in> | Specific error ID */
				fzrt_error_severity_enum	severity,			/* @parm <in> | Severity of the error */
				fzrt_error_context_enum	context,			/* @parm <in> | The context of the error */
				fzrt_int					context_id,			/* @parm <in> | the specific ID within the context, for plugins this is the plugin ID */
				fzrt_int					detail_id			/* @parm <in> | associated detail ID */
				);

/************************************************************************

	@fsfunc_t

	@desc		Sets the current error.
				This assigns a unique error ID to an error. The error ID can be
				returned to a calling function. The context and Context ID
				are used to make the error unique within the system.	

	@return		The unique runtime error ID.

	@avail		5.0.0.0

	@see		fzrt_error_set, fzrt_error_get_info

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_error_set_func)(
				fzrt_int 					err, 		/* @parm <in> | Specific error ID */
				fzrt_error_severity_enum 	severity,   /* @parm <in> | Severity of the error */
				fzrt_error_context_enum 	context,    /* @parm <in> | The context of the error */
				fzrt_int 					context_id  /* @parm <in> | the specific ID within the context, for plugins this is the plugin ID */
				);

/************************************************************************

	@fsfunc_t

	@desc		Adds a detail string to an error.
				This will associate a detail message with an error.
				The error must first be set using fzrt_error_set_with_detail.

	@avail		5.0.0.0

	@see		fzrt_error_set_with_detail

	@toc		fzrt_base

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_error_add_detail_func)(
				fzrt_error_td 				err_id,		/* @parm <in> | Error ID set by fzrt_error_set_with_detail */
				fzrt_int 						detail_id,	/* @parm <in> | Detail ID passed in to fzrt_error_set_with_detail */
				char 						*detail_str	/* @parm <in> | C string with detail message */
				);

/************************************************************************

	@fsfunc_t
	
	@desc		Adds debugging information to an error.
				This writes the debug ID and message to the error log. 
				

	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_error_add_debug_func)(
				fzrt_error_td err_id,	/* @parm <in> | Error ID */
				fzrt_int debug_id,			/* @parm <in> | Debug ID */
				char *debug_str			/* @parm <in> | C String Debug message */
				);

/************************************************************************

	@fsfunc_t

	@desc		This will clear the last error. This forces fzrt_error_get_last to 
				return FZRT_NOERR until another error is set.		

	@avail		5.0.0.0

	@see		fzrt_error_get_last

	@toc		fzrt_base

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_error_clear_func)(
				void);

/************************************************************************

	@fsfunc_t

	@desc		Gets the last error.
				This will return the last error set by fzrt_error_set.		

	@return		An Error ID.

	@avail		5.0.0.0

	@see		fzrt_error_set, fzrt_error_get_info, fzrt_error_clear

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_error_td  (FZRT_SPEC *fzrt_error_get_last_func)(
				void);

/************************************************************************

	@fsfunc_t

	@desc		Gets a string associated with an error.
				This will return a string suitable for display, for the
				specified error ID.		

	@return		FALSE if successful, TRUE if err id is not valid, or context does
				supply strings for errors..

	@avail		5.0.0.0

	@see		

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_boolean  (FZRT_SPEC *fzrt_error_get_string_func)(
				fzrt_error_td 			err_id, 		/* @parm <in> | Error ID */
				char 					*str, 			/* @parm <in> | pointer to buffer to receive the error */
				fzrt_int 					maxlen);		/* @parm <in> | Size of the string buffer */

/************************************************************************

	@fsfunc_t

	@desc		Gets information on a error.
				This will return information about the error ID, including the
				error ID, and error string, the severity of the error, the Context of the error and the context ID within
				the context, the name of the context, if any, and any specific detail ID.		

	@return		FALSE if successful, TRUE if error is invalid or if a string can't be found for the error.

	@avail		5.0.0.0

	@see		

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_boolean  (FZRT_SPEC *fzrt_error_get_info_func)(
				fzrt_error_td			err_id, 				/* @parm <in> | The error ID to get information about.*/
				char 					*str, 					/* @parm <in_opt> | pointer to buffer to receive string, NULL if not needed */
				fzrt_int 					maxlen, 				/* @parm <in_opt> | size of buffer to receive string, ignored if str is NULL */
				fzrt_int 					*err, 					/* @parm <in_opt> | pointer to fzrt_int to receive actual error ID, NULL if not needed */
				fzrt_error_severity_enum 	*severity,				/* @parm <in_opt> | pointer to fzrt_error_severity_enum to receive severity, NULL if not needed */
				fzrt_error_context_enum 	*context, 				/* @parm <in_opt> | pointer to fzrt_error_context_enum to receive context, NULL if not needed */
				fzrt_int 					*context_id,			/* @parm <in_opt> | pointer to fzrt_int to receive context id, NULL if not needed */
				char					*context_id_name,		/* @parm <in_opt> | pointer to buffer to receive context name, NULL if not needed */
				fzrt_int					context_id_name_len,	/* @parm <in_opt> | size of buffer for context name, ignored if context_id_name is NULL */
				fzrt_int					*detail_id				/* @parm <in_opt> | pointer to fzrt_int to receive detail id, NULL if not needed */
				);			

/************************************************************************

	@fsfunc_t
	
	@desc		This will check for the existence of a specific type of error.

	@return		TRUE if error occurred, FALSE otherwise.

	@avail		5.0.0.0

	@see		fzrt_error_set, fzrt_error_severity_enum, fzrt_error_context_enum

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_boolean  (FZRT_SPEC *fzrt_error_exists_func)(
				fzrt_int 					err, 				/* @parm <in> | error ID to check */
				fzrt_int 					severity_mask, 		/* @parm <in> | Mask of severity levels to check */
				fzrt_int 					context_mask, 		/* @parm <in> | Mask of contexts to check. Bits in mask can be bitwise OR-ed together using fzrt_error_context_enum  */
				fzrt_unsigned_int 			context_id, 		/* @parm <in> | context id to check or FZRT_ERROR_CONTEXT_ANY for all */
				fzrt_boolean			last_only			/* @parm <in> | TRUE to check only the most recent error */
				);

/************************************************************************

	@fsfunc_t

	@desc		This will mark the error as posted. If the error was the last error,
				it will also clear the last error.		

	@avail		5.0.0.0

	@see		fzrt_error_was_posted, fzrt_error_clear

	@toc		fzrt_base

*************************************************************************/
typedef void  (FZRT_SPEC *fzrt_error_posted_func)(
				fzrt_error_td 			err_id			/* @parm <in> | error ID to check */
				);

/************************************************************************

	@fsfunc_t

	@desc		Checks if an error has been posted.
				This will mark the error as posted. if the error was the last error
				it will also clear the last error.		

	@return		TRUE if it has been posted, FALSE otherwise.

	@avail		5.0.0.0

	@see		fzrt_error_posted

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_boolean  (FZRT_SPEC *fzrt_error_was_posted_func)(
				fzrt_error_td 			err_id			/* @parm <in> | error ID to check */
				);


/************************************************************************

	@fsfunc_t

	@desc		Causes an error alert to be posted when an error is set.
				If set is TRUE, an error alert dialog will be displayed for every
				subsequent call to fzrt_error_set or fzrt_error_set_with_detail.
				Setting set to FALSE, will turn off this behavior and an error
				alert dialog will only be displayed when fz_fuim_alrt_std_error
				is called.

	@avail		5.0.0.0

	@see		fzrt_error_set, fzrt_error_set_with_detail, fz_fuim_alrt_std_error

	@toc		fzrt_base

*************************************************************************/
typedef void (FZRT_SPEC *fzrt_error_set_alrt_on_error_func)(
				fzrt_boolean		set			/* @parm <in> | If TRUE, an error alert will be displayed every time a set error function is called. */
				);

/************************************************************************
	KDA 9/19/03: RLIB API
************************************************************************/

/************************************************************************

	@fsfunc_t
  
	@desc		Loads a resource from a runtime file.
				This will load a resource from the runtime module.
				To load native ICON resources use FZRT_RLIB_CICON as the type.

	@return		Memory pointer to resource. The resource pointer will
				be in the native format, so that Macintosh will return
				a macintosh Handle, and Windows will return a HGLOBAL.
				or fzrt_icon_ptr for ICON, FZRT_RLIB_CICON resources
				NULL if resource can't be found.

	@avail		5.0.0.0

	@toc		fzrt_base

*************************************************************************/
typedef fzrt_ptr ( FZRT_SPEC *fzrt_rlib_get_resource_func)(	
				short								rlib_indx,			/* @parm <in> | reference to the loaded module */ 
				fzrt_ostype							type, 				/* @parm <in> | Resource type, must be 4-char constant or number */
				short								resID				/* @parm <in> | ID of the resource to load */ 
				);
				
/************************************************************************

	@fsfunc_t
  
	@desc		Loads an icon resource from a runtime module.
				This will load an icon from the runtime module.
				If the icon resource cannot be loaded, the icon_ptr
				is set to NULL.
				The icons need to be saved in the resource of the 
				runtime module (dll, shared library).
				NOTE: On Macintosh, black and white icons are stored as an ICON resource,
				and color icons are stored as a CICN resource.
				On Windows,  black and white icons are 1 bit deep bitmaps and must be stored as a "FZICON" resource,
				color icons can be either a Windows native BITMAP or ICON resource. If the 
				color icon is stored as a BITMAP resource, it will not have a mask associated
				with it.

	@return		FZRT_NOERR on success, error code otherwise.

	@avail		5.0.0.0

	@toc		fzrt_base
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_rlib_load_icon_func)(	
				short								rlib_indx,			/* @parm <in> | reference to the loaded module */ 
				fzrt_icon_enum						icon_type, 			/* @parm <in> | type of icon FZRT_LOAD_ICON_COLOR, FZRT_LOAD_ICON_BW */
				short								resID,				/* @parm <in> | ID of the resource to load */ 
				fzrt_icon_ptr *						icon_ptr			/* @parm <out> | pointer to an icon_ptr that receives the loaded icon */
				);


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the current tick count.
				This will get the current system time in 1/60th second increments
				since the system was started.

	@return		Number of ticks (1/60th second increments) since system boot

	@avail		5.0.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_int  (FZRT_SPEC *  fzrt_tick_count_func)(	void);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the current time.
				This will get the current time in seconds since a predefined
				time. The predefined time is system dependent, so this value
				should not be stored in a cross-platform setting.<nl>
				Macintosh uses January 1, 1904<nl>
				Windows uses January 1, 1970

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC *fzrt_get_date_time_func)(
				fzrt_unsigned_int *						date_time			/* @parm <out> | pointer to fzrt_unsigned_int that receives the current time in seconds */
				);


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the month. This will get the month from a date time object

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_month_func)(
				fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
				fzrt_int *								month);				/*	 @parm <out> | received the month from the date_time object */

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the day. This will get the day of the month from a date time object
	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time
  
	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_day_func)(
						fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
						fzrt_int *								day);			/*	 @parm <out> | received the day of the month from the date_time object */

/************************************************************************

	 @fsfunc_t
  
	@desc		Gets the year. This will get the year from a date time object

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time
  
	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_year_func)(
					fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
					 fzrt_int *								year);				/*	 @parm <out> | received the year from the date_time object */

/************************************************************************

  @fsfunc_t
  
	@desc		Gets the hour. This will get the hour (0--23) from a date time object

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time
  
	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_hour_func)(
					fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
					 fzrt_int *								hour);				/* @parm <out> | received the hours from the date_time object */

/************************************************************************

	@fsfunc_t
  
	@desc		This will get the minute (0..59) from a date time object

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_minute_func)(
					fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
					 fzrt_int *							minutes);			/* @parm <out> | receives the minutes from the date_time object */

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the seconds This will get the seconds (0..59) from a date time object

	@return		FZRT_NOERR if successful, error ID otherwise

	@avail		5.0.0.0

	@see		fzrt_get_date_time

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_date_time_get_seconds_func)(
						fzrt_unsigned_int						date_time,				/* @parm <in> | seconds from fzrt_get_date_time*/
						 fzrt_int *							seconds);			/* @parm <out> | received the seconds from the date_time object */

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the attributes of the current OS. 
			This will return the specific version level of the OS, such as OS X Native or MacOS Classic
				, Windows NT or Windows_95

	@return		The type of system 

	@avail		5.0.0.0

	@see		fzrt_get_platform, fzrt_get_os_long

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_syst_attr_enum (FZRT_SPEC * fzrt_get_os_attrs_func)(void);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the current platform.
				This will return the platform on which the application is currently running.

	@return		The current OS platform

	@avail		5.0.0.0

	@see		fzrt_get_os_attrs, fzrt_get_os_long

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_platform_enum (FZRT_SPEC * fzrt_get_platform_func)(void);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the OS Version number. 
				This will return the version of the OS as a fzrt_int with the 
				high word containing the major version number. 
				The high byte of the low word containing the minor version number,
				and the low byte of the low word containing the patch/build number.

	@return		The operating system version number

	@avail		5.0.0.0

	@see		fzrt_get_os_attrs, fzrt_get_os_long

	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_int (FZRT_SPEC * fzrt_get_os_long_func)(void);

/************************************************************************

	@fsfunc_t
  
  @desc			Gets a string describing the current OS.

	@return		pointer to string, same as parameter os_str., NULL if there is an error

	@avail		5.0.0.0

	@see		fzrt_get_os_attrs, fzrt_get_os_long

	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_os_string_func)(
				  fzrt_int						extended,		/* @parm <in> | 1 to includes details (e.g. for NT the Build and Service Pack numbers) */
				  char *					os_str,			/* @parm <in> | pointer to character buffer to receive C string, must be allocated by caller*/
				  fzrt_unsigned_int				maxlen			/* @parm <in> | length of character buffer os_str */
				 );

/************************************************************************

	@fsfunc_t
  
  	@desc		Plays the system beep.

	@avail		5.0.0.0


	@toc		fzrt_util
		
*************************************************************************/
typedef  void (FZRT_SPEC * fzrt_sys_beep_func)(
				  fzrt_int						extended			/* @parm <in> | duration of beep in sixieths of a second */
				 );

/************************************************************************

	@fsfunc_t
  
  @desc			Converts a string.
  				This will convert a string from one character set to another. The 
  				resulting string is placed in the same buffer as the source string.
  				The buffer containing the string must be large enough to hold the
  				destination string, it will not be resized.
  				This function will optionally convert line endings to the current platform default.
  				macintosh line ending are carriage return <CR> (0x13)
  				Windows line endings is a pair of carriage return and line feed <CR><LF> (0x13,0x10)

	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.0.0.0


	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_cnvt_str_func)(
					char *					str,				/* @parm <in_out> | buffer containing the NULL terminated string on input and the converted string return */
					fzrt_charset_td			src_cs,				/* @parm <in> | character set of the input string		*/
					fzrt_charset_td			out_cs,				/* @parm <in> | character set of the resulting string */
					fzrt_int					str_size,			/* @parm <in> | size of the string buffer */
					fzrt_int *					out_len,			/* @parm <out> | pointer to fzrt_int that receives size of the resulting string in bytes not including null terminator */
					fzrt_unsigned_int			cnvt_flags			/* @parm <in> | FZRT_CNVT_NO_CRLF to leave carriage return/line feed as is, otherwise they are converted to current platform default */
				 );
/************************************************************************

	@fsfunc_t

	@desc		Gets the current character set in use.
					This will return the character set of the current language
				of the application.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.0.0.0


	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_current_charset_func)(
					fzrt_charset_td			*src_cs				/* @parm <out> | character set of the current language and system		*/
				 );


/************************************************************************

	@fsfunc_t

	@desc		Gets the current language.
				This will return the language of the application.
				The current language is the language used for the menus and dialogs.
  
	@return		The application language.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_lang_td (FZRT_SPEC * fzrt_lang_get_func)(void);		/*  */


/************************************************************************

	@fsfunc_t

	@desc		Gets the current system language.
				This will return the language of the system.
				The current system language is the language of the Operation System.
				On MacOS X, this is the first language in the specified in international System Preferences list.
				On Windows, it is the language specified in the Regional Options Control Panel.
  
	@return		The System language.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_lang_td (FZRT_SPEC * fzrt_lang_system_get_func)(void);

/************************************************************************

	@fsfunc_t

	@desc		Initialize the language information pointer.
				This will allocation a language information pointer, and return it in the lang_info
				parameter. The values of the language information pointer are initialized to English.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *  fzrt_lang_info_init_func)(
				  fzrt_lang_info_ptr 				*lang_info	/* @parm <out> | pointer to the initialized language information pointer	*/
				  );


/************************************************************************

	@fsfunc_t

	@desc		Releases the language information pointer.
				This will de-allocate a language information pointer. The pointer is invalid after this call.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

  	@see		fzrt_lang_info_init

	@toc		fzrt_util
		
*************************************************************************/
typedef void (FZRT_SPEC *  fzrt_lang_info_finit_func)(		fzrt_lang_info_ptr 					*lang_info);

/************************************************************************

	@fsfunc_t

	@desc		Gets language information for the specified language.
				This will fill the information pointer with the values for the specific language.
				To retrieve the values, call the language information accessor functions.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

  	@see		fzrt_lang_info_init

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_lang_get_info_func)(
					fzrt_lang_td						lang,				/* parm <in> | language identifier		*/			
					fzrt_lang_info_ptr					lang_info			/* parm <in_out> | language information pointer that receives language information, must be initialized before calling*/
					);
/************************************************************************

	@fsfunc_t

	@desc		Find out if language is available
				This will return the value indicating whether the language is installed. A language is installed if the formZ resource files for that language
				are available.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC * fzrt_lang_info_is_avail_func)( 
					fzrt_lang_info_ptr					lang_info,			/* parm <in> | language information pointer,  must be initialized before calling*/
					fzrt_boolean						*avail				/* parm <out> | pointer to boolean. TRUE if language is installed, FALSE if it is not */
																   );

/************************************************************************

	@fsfunc_t

	@desc		Get the name of the specified language
				This will return the name value for the language. The name is returned in the current language. If the buffer is not large enough to contain
				the name, the name is truncated.
  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC * fzrt_lang_info_get_name_func)(
					fzrt_lang_info_ptr					lang_info,			/* parm <in> | language information pointer,  must be initialized before calling*/
					char								*name,				/* parm <out> | pointer to buffer that receives the language name.*/
					short								 maxlen				/* parm <in> | size of the name buffer */	
																   );

/************************************************************************

	@fsfunc_t

	@desc		Get the extension of a language
				This will return the 3 character extension value for the language. 
				The tag is the 3 character extension used in the formZ resource filenames.
				The tag buffer must be at least 4 bytes fzrt_int to accommodate the 3 characters and the NULL terminator.
				The currently supported language tags are:
								ENU  - US English <nl>
								FRA  - France <nl>
								DEU	 - German <nl>
								ESP  - Spain <nl>
								ELL  - Greek <nl>
								ITA  - Italy <nl>
								JPN  - Japan (Hirigana, Katakana, Kanji) <nl>
								CHS  - Chinese Simplified (PRC)  <nl>
								CHT  - Chinese Traditional (Taiwan) <nl>
								KOR	 - Korean (Hangul, Hanja) <nl>

  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC * fzrt_lang_info_get_tag_func)( 
					fzrt_lang_info_ptr					lang_info,			/* parm <in> | language information pointer,  must be initialized before calling*/
					char								*tag				/* parm <out> | pointer to buffer of at least 4 bytes that receives the language tag.*/
																  );

/************************************************************************

	@fsfunc_t

	@desc		Get the language identifier value of the language information pointer.
				This will return the language identifier value for the language. 

  
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td   (FZRT_SPEC * fzrt_lang_info_get_lang_func)( 
					fzrt_lang_info_ptr						lang_info,			/* parm <in> | language information pointer,  must be initialized before calling*/ 
					fzrt_lang_td							*lang				/* parm <out> | pointer to fzrt_lang_td that receives the language identifier.*/
																   );


/************************************************************************

	@fsfunc_t

	@desc		Get the type of character byte for a specific byte in a multi-byte character string. 

  
	@return		value of type fzrt_char_byte_td representing a single, first, middle or last byte of a character.

	@avail		5.5.0.0

	@toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_char_byte_td  (FZRT_SPEC * fzrt_char_byte_func)(
								fzrt_ptr 				textBuf,		/* parm <in> | pointer to a buffer containing multi-byte character string*/
								 short 					textOffset,		/* parm <in> | byte offset into the buffer for the character byte we want information on.*/
								 fzrt_charset_td 		char_set		/* parm <in> | character set to use for the analysis of the string */
								 );

/************************************************************************

	fsfunc_t

	desc		Gets the what the event is.
					This will return the what the event is.
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_event_what_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					unsigned short	*				what				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Sets the what the event is.
					This will set what the event is.
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_set_event_what_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					unsigned short					what				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );


/************************************************************************

	fsfunc_t

	desc		Gets the when the event occurred.
					This will return the when the event happened.
					The time of the event is recorded in ticks (1/60th of second).
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0
	
	see		fzrt_tick_count

	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_event_when_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_unsigned_int	*				when				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Sets the when the event occurred.
					This will set the when the event happened.
					The time of the event is recorded in ticks (1/60th of second).
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_set_event_when_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_unsigned_int					when				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Gets the where the event occurred.
					This will return the where the event happened.
					The location is a point in global coordinates
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0
	
	see		fzrt_tick_count

	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_event_where_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_point	*					where				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Sets the where the event occurred.
					This will set the where the event happened.
					The location is a point in global coordinates
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_set_event_where_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_point						where				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );
/************************************************************************

	fsfunc_t

	desc		Gets the event message.
					This will return the extra event message.
					The message is event specific data.
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0
	
	see		fzrt_tick_count

	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_event_message_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_unsigned_int	*				message				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Sets the event message.
					This will get the extra event message.
					The message is event specific data.
					
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_set_event_message_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					fzrt_unsigned_int					message				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );
/************************************************************************

	fsfunc_t

	desc		Gets the event modifiers.
					This will return the set of modifiers keys that are down
					when the event happened.
  
	return		FZRT_NOERR on success, an error id otherwise.

	avail		5.0.0.0
	
	see		fzrt_tick_count

	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_get_event_modifiers_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					unsigned short	*				message				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	fsfunc_t

	desc		Sets the event modifiers.
					This will set the set of modifiers keys that are down
					when the event happened.
					
	return		FZRT_NOERR on success, an error id otherwise.

	avail      5.0.0.0


	toc		fzrt_util
		
*************************************************************************/
typedef  fzrt_error_td (FZRT_SPEC * fzrt_set_event_modifiers_func)(
					fzrt_event_record_ptr			event,				/* parm <in> | pointer to event		*/
					unsigned short					message				/* parm <out> | pointer to unsigned short that receives what the event is */					
				 );

/************************************************************************

	@fsfunc_t

	@desc		Get the pressed state of a key.
					This will return the state of the specified virtual key.
					Currently this is only implemented for the modifier keys.
					Note that this converts the actual key pressed into virtual key presses.
					For example, On Windows the actual keys Control+Shift combine to make the virtual key FZRT_OPTIONKEY.
					So the return value of FZRT_VIRT_OPTIONKEY will be FZRT_KEY_PRESSED only if the actual keys control AND shift are both pressed.
					To the state of the actual keys, use fzrt_evnt_get_actual_key_state.

					The actual key combinations are as follows:
					WINDOWS: FZRT_VIRT_OPTIONKEY -> control+shift
							FZRT_VIRT_CMDKEY	-> control
							FZRT_VIRT_CONTROLKEY -> control+alt
							FZRT_VIRT_SHIFT	-> shift
							FZRT_VIRT_ALT -> alt

					MACINTOSH: FZRT_VIRT_OPTIONKEY -> option
							FZRT_VIRT_CMDKEY	-> command
							FZRT_VIRT_CONTROLKEY -> control
							FZRT_VIRT_SHIFT	-> shift
							FZRT_VIRT_ALT -> N/A
					
					
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail      5.0.0.0

	@see		fzrt_evnt_get_actual_key_state_func


	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_evnt_get_key_state_func)(
					fzrt_evnt_virtual_key_enum virt_key,				/* @parm <in> |  virtual key code		*/
					fzrt_key_state_enum *state							/* @parm <out> | pointer to fzrt_unsigned_int that receives the state of key.  */					
					);


/************************************************************************

	@fsfunc_t

	@desc		Get the pressed state of a key.
					This will return the state of the specified virtual key.
					Currently this is only implemented for the modifier keys
					Note that this return the key pressed state of the actual keys and 
					not the virtual modifier keys that are used to make cross-platform
					key equivalents consistent.
					
					
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail      5.0.0.0

	@see		fzrt_evnt_get_key_state


	@toc		fzrt_util
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_evnt_get_actual_key_state_func)(
					fzrt_evnt_virtual_key_enum virt_key,				/* @parm <in> |  virtual key code		*/
					fzrt_key_state_enum *state							/* @parm <out> | pointer to fzrt_unsigned_int that receives the state of key.  */					
					);



typedef struct fzrt_timer_td *fzrt_timer_ptr;

typedef void (FZRT_SPEC *fzrt_evnt_timer_callback_func)(void *data, double time_since_start, double time_since_last_fire);


typedef fzrt_timer_ptr (FZRT_SPEC *fzrt_evnt_one_shot_timer_func)(
					double							delay,									
					fzrt_evnt_timer_callback_func	func,									
					void							*data									
					);

typedef void (FZRT_SPEC *fzrt_evnt_one_shot_timer_cancel_func)(
					fzrt_timer_ptr *timer									
					);


typedef fzrt_timer_ptr (FZRT_SPEC *fzrt_evnt_periodic_timer_func)(
					double							delay,									
					fzrt_evnt_timer_callback_func	func,									
					void							*data									
					);

typedef void (FZRT_SPEC *fzrt_evnt_periodic_timer_cancel_func)(
					fzrt_timer_ptr *timer									
					);

	
typedef struct fzrt_lnch_parm_td
{
	char					param[512];
}	fzrt_lnch_parm_td;

typedef struct fzrt_lnch_parms_td
{
	fzrt_int					count;
	fzrt_lnch_parm_td		*param;
}	fzrt_lnch_parms_td;
	
typedef fzrt_oserr  (FZRT_SPEC *fzrt_launch_application_func)(	fzrt_floc_ptr		floc,
																 fzrt_lnch_parms_td	*params,
																 fzrt_boolean		elevate,
																 fzrt_boolean		hide);
	
	/************************************************************************

	@fsfunc_t

	@desc		Show or hide the mouse cursor.
					This will show or hide the mouse cursor on the screen.
					It does not count the number of times it has been called, so 
					passing a value of TRUE will immediately show the cursor regardless
					of how many calls have been made with the value of FALSE to hide the cursor.
					It does not change the cursor bitmap.
										
	@return		FZRT_NOERR on success, an error id otherwise.

	@avail		5.0.0.0


	@toc		fzrt_util
		
*************************************************************************/
typedef  void  (FZRT_SPEC *fzrt_show_cursor_func)(
					 fzrt_boolean show						/* @parm <in> | boolean value, TRUE to show current cursor, FALSE to hide cursor */
					 );

/************************************************************************

************************************************************************/
typedef  void  (FZRT_SPEC *fzrt_cursor_set_location_func)(
					fzrt_point *pixel
				);

/************************************************************************

	@fsfunc_t
  
	@desc	Sets the text of a menu item.
			This will set the menu item to the specified string.
			The string must be in C format, the index is 1-based

	@return	FZRT_NOERR on success, error id otherwise

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_set_item_text_func)(
				fzrt_menu_ptr 							theMenu,				/* @parm <in> | the menu pointer */
				short 									item,					/*  @parm <in> | the 1-based index of the menu item */
				const  char * 					itemString);			/*  @parm <in> | pointer to a C string */


/************************************************************************

	@fsfunc_t
  
	@desc	Gets the text of a menu item.
			This will get the text of a specified menu item.
			The string will be in Pascal format, the index is 1-based

  @return	FZRT_NOERR on success, error id otherwise
  
	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_get_item_text_func)(
				fzrt_menu_ptr 						theMenu,					/*  @parm <in> | the menu pointer */
				short 								item,						/*  @parm <in> | the 1-based index of the menu item
																					@rimin		1 */
				char * 							itemString);					/*  @parm <out> | pointer to a buffer that received a C string  - user must allocate the buffer large enough */


/************************************************************************

	@fsfunc_t
  
	@desc	Sets the checkmark on a menu item.
			This will set or remove the checkmark state of a menu item

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_menu_check_item_func)(
				fzrt_menu_ptr 						theMenu,				/*  @parm <in> | the menu pointer */
				short 								item,					/*  @parm <in> | the 1-based index of the menu item 
																				@rimin		1*/
				fzrt_boolean 						checked);				/*  @parm <in> | TRUE to set check mark, FALSE to remove it */


/************************************************************************

	@fsfunc_t
  
	@desc	Enables a menu item.
			This will enable a menu item at the 1-based index

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_menu_enable_item_func)(
				fzrt_menu_ptr 					theMenu,				/*  @parm <in> | the menu pointer */
				short 							item);					/*  @parm <in> | the 1-based index of the menu item 
																			@rimin		1   */


/************************************************************************

	@fsfunc_t
  
	@desc	Disables a menu item. 
			This will disable a menu item at the 1-based index

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_menu_disable_item_func)(
				fzrt_menu_ptr 					theMenu,				/*  @parm <in> | the menu pointer */
				short 							item);					/*  @parm <in> | the 1-based index of the menu item */


/************************************************************************

	@fsfunc_t
  
	@desc	This will delete the menu item at the 1-based index

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_menu_delete_item_func)(
				fzrt_menu_ptr 					theMenu,				/*  @parm <in> | the menu pointer */
				short 							item);					/*  @parm <in> | the 1-based index of the menu item */


/************************************************************************

	@fsfunc_t	
  
	@desc	Changes	the attributes of a menu item.
		This will change 1 or more attributes of the menu item at the 1-based index
			If a menu attribute is in neither the add nor remove, the attribute is unchanged.
			Specifying the same attribute in both add and remove yields unpredictable results

  @return	FZRT_NOERR on success, error id otherwise
  
	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_change_item_attributes_func)(
				fzrt_menu_ptr 							menu, 					/*  @parm <in> | menu pointer */
				short 									menuindex, 				/*  @parm <in> | the 1-based index of the menu item */
				fzrt_menu_attributes_enum 				addAttribs, 			/*  @parm <in> |  menu attributes to add to menu item */
				fzrt_menu_attributes_enum 				removeAttribs);			/*  @parm <in> |  menu attributes to remove from menu item */

/************************************************************************

	@fsfunc_t	
  
	@desc	Gets the attributes of a menu item.
			This will get the attributes of the menu item at the 1-based index

  @return	FZRT_NOERR on success, error id otherwise
  
	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_get_item_attributes_func)(
				fzrt_menu_ptr							menu,					/*  @parm <in> | menu pointer */
				short									menuindex,				/*  @parm <in> | the 1-based index of the menu item */
				fzrt_menu_attributes_enum *				attribs);				/*  @parm <out> | pointer to menu attributes receives the attributes of menu item */
	

/************************************************************************

	@fsfunc_t
  
	@desc	This will add a menu separator to the bottom of a menu

  @return	FZRT_NOERR on success, error id otherwise
  
	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_append_separator_func)(
															fzrt_menu_ptr 				menu);	/* @parm <in> | menu pointer */

/************************************************************************

	@fsfunc_t
  
	@desc	This will add a menu separator after the specified menu item of a menu

  @return	FZRT_NOERR on success, error id otherwise
  
	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_insert_separator_func)(
															fzrt_menu_ptr 				menu,		/* @parm <in> | menu pointer */
															short					afterID );		/* @parm <in> | 1-based index of menu item */

/************************************************************************

	@fsfunc_t
  
	@desc	This will create a new, empty menu with the specified title.
			The title should be a C string.

	@return	pointer to a new menu, NULL if unsuccessful

	@avail	5.0.0.0

	@see	fzrt_fzr_get_menu

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_menu_ptr ( FZRT_SPEC *fzrt_menu_create_func)(
							char					*menuTitle	/* @parm <in_opt> | C style string to use as menu title. */

							);	
/************************************************************************

	@fsfunc_t
  
	@desc	This will create a new menu that is filled with the names of fonts installed on the system.
			If a valid font name is passed in the default_font parameter, the menu_sel result parameter
			will contain the menu index of that font.

	@return	FZRT_NOERR if successful, error ID otherwise.
	
	@avail	5.0.0.0

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_create_font_menu_func)(
					   fzrt_menu_ptr 					*menu,					/* @parm <out> | pointer to menu pointer, receives newly create menu */
					   char 							*default_font,			/* @parm <in_opt> | pointer to c string that has default font name */
					   short 							*menu_sel				/* @parm <out_opt> | pointer to short in that has menu index of default font name */
					   );

/************************************************************************

	@fsfunc_t
  
	@desc	This will get the font id of a font that is at the specified menu index..

	@return	FZRT_NOERR if successful, error ID otherwise.

	@avail	5.0.0.0

	 @see	fzrt_menu_create_font_menu
  
	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_font_id_get_func)(
						fzrt_menu_ptr 					menu,					/* @parm <in> |  menu pointer create with fzrt_menu_create_font_menu*/
						short 							menu_pos,				 /* @parm <in> | 1-based index of menu item */
						short *							fontid					/* @parm <out> | pointer to short contains the font id of fontname at menu position */
					);

/************************************************************************

	@fsfunc_t
  
	@desc	This will append a menu item to the bottom of the menu with the 
			specified text. Text is a C string. NOTE: this will not
			process meta characters ala the Macintosh function AppendMenu

	@return	FZRT_NOERR if successful, error ID otherwise.
	
	@avail	5.0.0.0

	@see	fzrt_menu_append_separator, fzrt_menu_insert_item_text

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_append_item_text_func)(
						fzrt_menu_ptr 					menu,				/* @parm <in> |  menu pointer */
						const  char * 			text);				/* @parm <in> |  pointer to string to use a menu item text as C string */


/************************************************************************

	@fsfunc_t
  
	@desc	This will insert a menu item, after the specified menu index of the menu, with the 
			specified text. Text is a C string. NOTE: this will not
			process meta characters ala the Macintosh function AppendMenu

	@return	FZRT_NOERR if successful, error ID otherwise.
	
	@avail	5.0.0.0

	@see	fzrt_menu_insert_separator, fzrt_menu_append_item_text

	@toc	fzrt_ui
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_menu_insert_item_text_func)( 
						fzrt_menu_ptr					theMenu,			/* @parm <in> |  menu pointer */
						short							nenuindex,			/* @parm <in> | 1-based menu index to insert new menu item after */			
						const  char *			text);				/* @parm <in> |  pointer to string to use a menu item text as C string */

/************************************************************************

	@fsfunc_t
  
	@desc		This will get the number of menu items in the menu, including
				separators.
	
	@return		The number of menu items in the menu, -1 if there was an error
	  
	@avail		5.0.0.0

	@toc			fzrt_ui
		
*************************************************************************/
typedef short ( FZRT_SPEC *fzrt_menu_count_items_func)(
				fzrt_menu_ptr 						menu);						/* @parm <in> |  menu pointer */

/************************************************************************

	@fsfunc_t
  
	@desc		This will dispose of a menu. The menu should not be inserted
				into the menu bar, or in an active popup menu control
	
	@avail		5.0.0.0

	@see			fzrt_menu_create, fzrt_fzr_get_menu
  
	@toc			fzrt_ui
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_menu_dispose_func)(
				fzrt_menu_ptr					menu);					/* @parm <in> |  menu pointer */


/************************************************************************

	@fsfunc_t
  
	@desc		returns 1 for dark mode , 0 for classic light mode
	
	@avail		9.1.0.0

	@see
  
	@toc			fzrt_ui
		
*************************************************************************/
typedef fzrt_int ( FZRT_SPEC *fzrt_appr_is_darkmode_func)(void);					/* @parm <in> |  menu pointer */


/************************************************************************

	@fsfunc_t
  
  	@desc		This disposes all associated memory for a color table. After
  				the call the color table is invalid.
  				
  	
	@avail		5.0.0.0

	@see		fzrt_ctab_create
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_ctab_dispose_func)(
				fzrt_color_table_ptr				CTab		/* @parm <in> | color table pointer to dispose */
				);


/************************************************************************

	@fsfunc_t
  
	@desc		This will create a new color table with the number of colors specified.
			The color entries are undefined in the new color table. The function fzrt_ctab_set_rgb
			can be used to initialize the individual entries in the color table.
			
	
	@return		error number, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_ctab_dispose
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_create_func)( 
					short							nColors,		/* @parm <in> | Number of colors in table */		
					short							flags, 			/* @parm <in> | flags for color table - current unused, set to 0 */
					fzrt_color_table_ptr *			cTab			/* @parm <out> | pointer to color table pointer received newly created color table */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Changes the number of color entries in the color table.
				This will resize the color table to hold the number of color specified.
				If the number of colors is larger than the current table size, that new colors
				are added to the end and the colors are not initialized.
				If the number of colors is less than the current, the colors
				at the end of the table are removed.
	
	@return		error ID, FZRT_NOERR if successful

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_resize_func)( 
						fzrt_color_table_ptr 				ctab, 			/* @parm <in> | color table pointer */
						short 								num_colors		/* @parm <in> | new number of colors in table */
						);

/************************************************************************

	@fsfunc_t
  
	@desc		Copies the entries of a color table.
				This will allocate a buffer with fzrt_new_ptr, and copy
				the entries in the color table into an array of fzrt_rgb_color_td
				structures.
				The caller is responsible for disposing of the buffer with fzrt_dispose_ptr.
	
	@return		error ID, FZRT_NOERR if successful

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_2_cult_func)(
					   fzrt_color_table_ptr ctab,			/* @parm <in> | color table pointer */
					   fzrt_rgb_color_td **cult,			/* @parm <out> | pointer to pointer to array of rgb color records filled in from color table. Allocated by function. Must be disposed by caller with fzrt_dispose_ptr. */
					   fzrt_int *cult_size						/* @parm <out> | pointer to fzrt_int that receives number of entries copied into cult array */
					   );

/************************************************************************

	@fsfunc_t
		
	@desc		Gets the number of colors in a color table.
  
	@return		error number, FZRT_NOERR if successful
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_get_color_count_func)(
					fzrt_color_table_ptr						cTab, 		/* @parm <in> | color table pointer */
					short *										nColors		/* @parm <out> | pointer to short that received the number of colors in the color table */
					);

/************************************************************************

	@fsfunc_t
	
	@desc		Gets the RGB color values of an entry in a color table
  
	@avail		5.0.0.0

	@return		error number, FZRT_NOERR if successful
  
	@see		fzrt_ctab_set_rgb
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_get_rgb_func)(
					fzrt_color_table_ptr					cTab, 		/* @parm <in> | color table pointer */
					short 									index, 		/* @parm <in> | index into the color table 
																			@rimin		1 	*/
					fzrt_rgb_color_td * 					clut_rgb		/* @parm <out> | pointer to RGB Color that received the rgb triple describing the color at the index. */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Sets the RBG color values of an entry in a color table.
				This will set the color at the specified index to the color described by the RGB triple	
	
	@return		error number, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_ctab_get_rgb
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_ctab_set_rgb_func)(
					fzrt_color_table_ptr					cTab, 		/* @parm <in> | color table pointer  */
					short									index, 		/* @parm <in> | index into the color table 
																			@rimin		1		*/
					const fzrt_rgb_color_td *				clut_rgb		/* @parm <in> | RGB Color to set */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		This will create a new pixel buffer with the specified dimensions, pixel depth, and format.
				If the FZRT_PBUF_FLAG_PORT flag is specified, the pixel buffer can be set as the current port
				and will be the target of drawing calls. A Drawable pixel buffer is limited in size on the Macintosh
				to 4096x4096, and on Windows may perform slower than a non-drawable pixel buffer.
				
	
	@return		error number, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_pbuf_dispose, fzrt_pbuf_is_port, fzrt_pbuf_set_port, fzrt_pbuf_get_port
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_new_func)(
				 fzrt_pbuf_ptr				*pixbuf_ptr,		/* @parm <out> | pointer for new pixbuf */
				const fzrt_pbuf_rect_td			*bounds,			/* @parm <in> | bounds of pixbuf */
				short							pixdepth,			/* @parm <in> | depth of pixels in bits, 1,8,24,32 */
				fzrt_pix_format_enum			pix_format,			/* @parm <in> | format of pixel data */
				fzrt_unsigned_int					flags);					/* @parm <in> | flags - FZRT_PBUF_FLAG_PORT */

/************************************************************************

	@fsfunc_t
  
	@desc		This will set the specified pixel buffer as the current port. The pixel buffer
				must be created with the FZRT_PBUF_FLAG_PORT flag in order to be able to be
				set as a current port. Since there are limitations on drawable pixel buffers,
				it is best to check the pixel buffer is drawable first.
				This function can fail if the pixel buffer is not drawable.
				
	
	@return		error id, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_pbuf_new, fzrt_pbuf_is_port, fzrt_pbuf_get_port
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_pbuf_set_port_func)(
						fzrt_pbuf_ptr				pbuf_ptr		/* @parm <in> | pointer to a drawable pixel buffer. Pixel buffer must be created with the FZRT_PBUF_FLAG_PORT flag */
						);

/************************************************************************

	@fsfunc_t
  
	@desc		This will get the pixel buffer pointer of the current port.
				The current port must be set by a call to fzrt_pbuf_set_port.
				Not all ports are pixel buffers, so this function can fail
				if the current port does not have a pixel buffer or if the 
				current port was set through fzrt_set_port.
				
	
	@return		error id, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_pbuf_set_port
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC *fzrt_pbuf_get_port_func)(
						fzrt_pbuf_ptr				*pbuf_ptr		/* @parm <in> | pointer to a fzrt_pbuf_ptr that receives the current port's pixel buffer.  */
						);

/************************************************************************

	@fsfunc_t
  
	@desc		This function will test whether pixel buffer is drawable.
				A pixel buffer is drawable if it is created with the FZRT_PBUF_FLAG_PORT flag.
				A drawable pixel buffer can be set as the current port, but may have
				limitations in size or performance.
	
	@return		TRUE if pixel buffer was created with FZRT_PBUF_FLAG_PORT flag, FALSE otherwise.

	@avail		5.0.0.0

	@see		fzrt_pbuf_new, fzrt_pbuf_set_port, fzrt_pbuf_get_port
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean (FZRT_SPEC *fzrt_pbuf_is_port_func)(
						fzrt_pbuf_ptr				pbuf_ptr		/* @parm <in> | pointer to a pixel buffer to test. */
						);

/************************************************************************

	@fsfunc_t
	
	@desc	Disposes of a pixel buffer.
  
	@avail		5.0.0.0

	@see		fzrt_pbuf_new
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_pbuf_dispose_func)(
				fzrt_pbuf_ptr	*					pixbuf		/* @parm <in> | pointer to a pixel buffer */
				);

/************************************************************************

	@fsfunc_t
  
	@desc	Gets the bounds of a pixel buffer.
			This will return the bounding rectangle of a pixel buffer.
			This is the rectangle that was specified in the call to fzrt_pbuf_new.
			The bounding rectangle does not necessarily have its origin at 0,0.
			Pixels outside the bounding rectangle are invalid and cannot be accessed.
  
	@return		error number, FZRT_NOERR if successful
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_bounds_func)(		
					fzrt_pbuf_ptr				pixbuf,		/* @parm <in> | pointer to a pixel buffer */
					fzrt_pbuf_rect_td			*rect	/* @parm <out> | pointer pbuf rect receives dimensions of pixel buffer */
					);			

/************************************************************************

	@fsfunc_t
  
	@desc	Gets the color table for a pixel buffer.
			A pixel buffer only contains a color table it is created with a pixel
			format of FZRT_PBUF_FORMAT_INDEXED.
  
	@return		error number, FZRT_NOERR if successful
	
	 @avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_clut_func)(
						fzrt_pbuf_ptr					pixbuf, 		/* @parm <in> | pointer to a pixel buffer */
						fzrt_color_table_ptr *				clut		/* @parm <out> | pointer to a color table pointer received the pixel buffer color table */
						);

/************************************************************************

	@fsfunc_t

 	@desc	Gets the size of a pixel
 			This will return the number of bytes used to store one pixel.
  
 
	@return		error number, FZRT_NOERR if successful
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_pixbytes_func)(	
						fzrt_pbuf_ptr				pixbuf,		/* @parm <in> | pointer to a pixel buffer */
						short						*pixbytes);		/* @parm <out> | pointer to a short that receives the size of a pixel in bytes */

/************************************************************************

	@fsfunc_t
  
	@desc		This will convert a pbuf rectangle to the system rectangle.
				Note that this may cause overflow if the system rectangle is
				based on 2-byte fields.
	
	@return		error number, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_pbuf_point_to_point
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_rect_to_rect_func)(		
						const fzrt_pbuf_rect_td			*src_rect,	/* @parm <in> | pointer to a pixel buffer rectangle*/
						fzrt_rect						*qd_rect);	/* @parm <out> | pointer to a fzrt_rect that receives the rectangle coordinates */


/************************************************************************

	@fsfunc_t
  
	@desc		Convert a pixel buffer point to a runtime point.
  				Note that this may cause overflow if the system point is
				based on 2-byte fields.
				
	@return		error number, FZRT_NOERR if successful

  @avail		5.0.0.0

	@see		fzrt_pbuf_rect_to_rect
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_point_to_point_func)(	
						const fzrt_pbuf_point_td		*src_point,	/* @parm <in> | pointer to a pixel buffer point*/
						fzrt_point						*qd_Point);	/* @parm <out> | pointer to a fzrt_point that receives the point coordinates */


/************************************************************************

  @fsfunc_t
  
	@desc		Convert a runtime rect to a pixel buffer rect.
  
	@return		error number, FZRT_NOERR if successful

	@avail		5.0.0.0

	@see		fzrt_pbuf_rect_to_rect
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_rect_from_rect_func)(	
							const fzrt_rect				*qd_rect,	/* @parm <in> | pointer to a runtime rectangle*/
							fzrt_pbuf_rect_td			*src_rect	/* @parm <out> | pointer to a fzrt pixel buffer that receives the rectangle coordinates */
							);


/************************************************************************

  @fsfunc_t
  
	@desc	Convert a runtime point to a pixel buffer point

@return		error number, FZRT_NOERR if successful

  @avail		5.0.0.0

	@see		fzrt_pbuf_point_to_point
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_point_from_point_func)(	
					const fzrt_point				*qd_Point,	/* @parm <in> | pointer to a runtime point*/
					fzrt_pbuf_point_td				*src_point);	/* @parm <out> | pointer to a fzrt pixel buffer that receives the point coordinates */


/************************************************************************

	@fsfunc_t
  
	@desc		Moves memory based pixel data into a pixel buffer.
				This will load a memory buffer filled with pixel data into
				an allocated pixel buffer, converting the pixel data if necessary.
	
	@return		FZRT_NOERR if successful, error number otherwise

	@avail		5.0.0.0

	@see		fzrt_pbuf_get_pixels
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_put_pixels_func)(
						fzrt_ptr						src_ptr,			/* @parm <in> | pointer to pixel data */
						fzrt_int							src_rowbytes,		/* @parm <in> |number of bytes per scanline */
						short							src_pixdepth,		/* @parm <in> |number of bits per pixel (1,8,16,24,32) */
						short							src_pixbytes,		/* @parm <in> |number of bytes per pixel */
						fzrt_pix_format_enum			src_pix_format,		/* @parm <in> |format of pixel data (RGB, BGR, RGBA, PIX555, indexed...)*/
						const fzrt_pbuf_rect_td			*src_bounds,		/* @parm <in> |bounding rectangle to read pixel data from */
						fzrt_color_table_ptr			src_ctab,			/* @parm <in> |source Color Table pointer*/
						fzrt_pbuf_ptr					dst_pixbuf,			/* @parm <in> |Destination GWorld fzrt_ptr */
						const fzrt_pbuf_point_td		*dst_off,			/* @parm <in> |offset to (top,left) dst. pixel (size is same src rect) */
						fzrt_pbuf_apply_alpha_enum		apply_alpha,		/* @parm <in> |apply alpha channel to destination */
						fzrt_boolean					flip_buff,			/* @parm <in> |flip pixel data along x & y axis when pushing to destination */
						unsigned char			*		 gamma_lut);		/* @parm <in_opt> |gamma look up table to apply to src during copy- NULL if none */


/************************************************************************

  @fsfunc_t
  
	@desc		Copies pixel data from a pixel buffer into memory.
				This will load a memory buffer with pixel data to
				an allocated pixel buffer, converting the pixel data if necessary.
				
	@return		FZRT_NOERR if successful, error number otherwise

	@avail		5.0.0.0

	
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_pixels_func)(	
					fzrt_pbuf_ptr					src_pbuf,			/*  @parm <in> |Destination GWorld fzrt_ptr */
					const fzrt_pbuf_rect_td			*src_bounds,		/*  @parm <in> |offset to (top,left) src. pixel  */
					fzrt_ptr						dst_ptr,			/*  @parm <out> |pointer to pixel data must be allocated by the caller */
					fzrt_int							dst_rowbytes,		/*  @parm <in> |number of bytes per scanline */
					fzrt_int							dst_scanlines,		/*  @parm <in> |number of scanlines in destination buffer */
					short							dst_pixdepth,		/*  @parm <in> |number of bits per pixel (1,8,16,24,32) */
					short							dst_pixbytes,		/*  @parm <in> |number of bytes per pixel */
					fzrt_pix_format_enum			dst_pix_format,		/*  @parm <in> |format of pixel data (RGB, BGR, RGBA, PIX555, indexed...)*/
					const fzrt_pbuf_point_td		*dst_off,			/*  @parm <in> |bounding rectangle to read pixel data from */
					fzrt_color_table_ptr			*dst_ctab,			/*  @parm <out> |pointer to destination color table */
					fzrt_pbuf_apply_alpha_enum		apply_alpha	);		/*  @parm <in> |handle the alpha data */


/************************************************************************

	@fsfunc_t
  
	@desc		Copies one pixel buffer to another.
				This will copy the contents of one pixel buffer to another.
				The destination pixel buffer must be allocated before copy.
				If the src and destination rectangles are different sizes,
				the pixels will be stretched (or compressed).
	
	@return		FZRT_NOERR if successful, error number otherwise
	  
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_copy_func)(				
										fzrt_pbuf_ptr				src_pixbuf,		/* @parm <in> | allocated pixbuf */
										fzrt_pbuf_rect_td			*src_rect,		/* @parm <in> | bounds of src pixels to copy */
										fzrt_pbuf_ptr				dst_pixbuf,		/* @parm <in> | destination pixbuf - already allocated*/
										fzrt_pbuf_rect_td			*dst_rect,		/* @parm <in> | destination rectangle */
										fzrt_unsigned_int				flags);			/* @parm <in> |flags - 0 for now */


/************************************************************************

	@fsfunc_t
  
	@desc		Copies pixel data from a port into a pixel buffer.
				This will copy the contents of graf ptr to a pixel buffer.
				The destination pixel buffer must be allocated before copy.
				If the src and destination rectangles are different sizes,
				the pixels will be stretched (or compressed).
				
	@return		FZRT_NOERR if successful, error code otherwise

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_copy_from_graf_port_func)(
										fzrt_graf_ptr							src_port,	/* @parm <in> | graf pointer*/
										fzrt_rect *							src_rect,		/* @parm <in> | source rectangle*/
										fzrt_pbuf_ptr						dst_pixbuf,		/* @parm <in> | destination pixbuf - already allocated*/
										fzrt_pbuf_rect_td *					dst_rect,	/* @parm <in> | destination rectangle*/
										fzrt_unsigned_int							flags		/* @parm <in> | flags - 0 for now*/
										);


/************************************************************************
	@fsfunc_t

	@desc		Copies pixel data from a pixel buffer to a port.
				This will copy the contents of pixel buffer to a graf ptr.
				The destination pixel buffer must be allocated before copy.
				If the src and destination rectangles are different sizes,
				the pixels will be stretched (or compressed).

	@return		FZRT_NOERR if successful, error code otherwise

	@avail		5.0.0.0

	@toc		fzrt_graf

		  
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_copy_to_graf_port_func)(
										fzrt_pbuf_ptr						src_pixbuf,		/* @parm <in> | source pixel buffer*/
										fzrt_pbuf_rect_td *					src_rect,		/* @parm <in> | source rectangle*/
										fzrt_graf_ptr						dst_port,		/* @parm <in> | destination graf pointer*/
										fzrt_rect *							dst_rect,		/* @parm <in> | destination rectangle*/
										fzrt_unsigned_int						flags			/* @parm <in> | flags - 0 for now*/
										);


/************************************************************************

	@fsfunc_t
  
	@desc	Updates a pixel buffer with the new color table.
			This will replace the existing color table, if any, with the table supplied.
			This will not change the pixel format of the pixel. The format must be FZRT_PBUF_FORMAT_INDEXED.
	
	@return		FZRT_NOERR if successful, error code otherwise

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_update_clut_func)(		
					fzrt_pbuf_ptr						pixbuf,			/* @parm <in> | allocated pixbuf */
					fzrt_color_table_ptr				clut);			/* @parm <in> | pointer to color table */

/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the size of a pixel row in a pixel buffer.
  	
	@return		FZRT_NOERR if successful, error number otherwise
	
	  @avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_rowbytes_func)(		
					fzrt_pbuf_ptr						pixbuf,			/* @parm <in> | allocated pixbuf */
					fzrt_unsigned_int						*rowbytes);		/* @parm <out> | number of bytes for one scanline  */


/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the size of a pixel in a pixel buffer.
  				This will return the depth of a pixel buffer. The depth
  				is considered the number of bits used for color data in a pixel.
  				Generally this is either 8, 16, 24, or 32.
  	
	@return		FZRT_NOERR if successful, error number otherwise
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_pixelsize_func)(
						fzrt_pbuf_ptr				pixbuf,			/* @parm <in> | allocated pixbuf */
						short*						pixel_size);	/* @parm <out> | number of bits needed for one pixel */



/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the format of a pixel in a pixel buffer.
  	
	@return		FZRT_NOERR if successful, error number otherwise
	
	  @avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_pbuf_get_pixelformat_func)(	
						fzrt_pbuf_ptr					pixbuf,			/* @parm <in> | allocated pixbuf */
						fzrt_pix_format_enum			*pix_format);	/* @parm <out> | format of pixel Data - FZRT_PBUF_FORMAT_RGB, FZRT_PBUF_FORMAT_RGBA etc */


/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the current drawing port.
  				A drawing port cannot be created, but is associated with
  				a window or a pixel buffer. This function should only be
  				called with a port that is obtained from a call to fzrt_get_port
  				or fzrt_get_window_port.
  	
	@avail		5.0.0.0

	@see		fzrt_get_port
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_graf_ptr ( FZRT_SPEC *fzrt_set_port_func)(					
					fzrt_graf_ptr					port		/* @parm <in> | graf ptr to make current */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current drawing port.
  	
	@return		FZRT_NOERR if successful, error number otherwise
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_port_func)(					
						fzrt_graf_ptr *				port		/* @parm <out> | pointer graf ptr that receives current port */
						);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the drawing port associated with a window.
  	
	@return		graf port of the window
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_graf_ptr  (FZRT_SPEC *fzrt_get_window_port_func)(
							fzrt_window_ptr 				theWindow			/* @parm <in> | window pointer */
							);

/************************************************************************

	@fsfunc_t

  	@desc		Gets the window associated with a drawing port.
  				Not all drawing ports have a window associated with it, such as a pixel buffer port.

	@return		window pointer associated with the graf ptr, NULL if graf ptr has no windows (It is an offscreen graf port )
  
	@avail		5.0.0.0

	@see		fzrt_get_window_port

	@toc		fzrt_graf

*************************************************************************/
typedef  fzrt_window_ptr  (FZRT_SPEC *fzrt_get_window_from_port_func)(
							fzrt_graf_ptr				thePort				/* @parm <in> | graf ptr */
							);


/************************************************************************

	@fsfunc_t
  
	  @desc		Converts point from local to global coordinates

	@avail		5.0.0.0

	@see		fzrt_global_to_local
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_local_to_global_func)(			
					fzrt_point *					pt		/* @parm <in> | point in local coordinate system */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Converts point from global to local coordinates

	@avail		5.0.0.0

	@see		fzrt_local_to_global
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_global_to_local_func)(				
					fzrt_point *					pt		/* @parm <in> | point in global coordinate system */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the current clipping region
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_clip_func)(					
				fzrt_rgn_ptr				rgn			/* @parm <out> | allocated region receives current clip region */
				);	


/************************************************************************

	@fsfunc_t

	@desc		Sets the current clipping region.
	  
	@avail		5.0.0.0

  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_set_clip_func)(					
				fzrt_rgn_ptr				rgn			/* @parm <in> | Region to set clipping. The system makes copy of region, the caller is responsible for disposing the region. */
				);


/************************************************************************

	@fsfunc_t
  
	@desc		Sets the height and width of the pen.
				This will set the size of the current pen.  If the underlying
				OS does not support non-square pens, then the width is used for
				the size of the pen, and height parameter is unused.

	@avail		5.0.0.0

  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_pen_size_func)(					
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					short							width,	/* @parm <in> | the width of the pen  - 0 will hide the pen. */
					short							height	/* @parm <in> | the height of the pen  - 0 will hide the pen. */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		Sets the current line weight.

	@avail		5.0.0.0

  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_line_weight_func)(					
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					float					lwidth		/* @parm <in> | the width of the line. */
					);



/************************************************************************

	@fsfunc_t

	@desc		Sets the transfer mode of the pen.
				This function will set how the pen combines the current
				foreground color and pattern with the destination.
  
	@avail		5.0.0.0

	@see		fzrt_pattern_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_pen_mode_func)(
				fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
				short							mode	/* @parm <in> | the transfer mode of the pen  */
				);




/************************************************************************

	@fsfunc_t
	
	@desc		Sets the pen to default values.
				This will reset the pen to a 1x1, black pen with FZRT_PATCOPY transfer
				mode.
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_pen_normal_func)(
					fzrt_graf_ptr			port		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					);											/* */

/************************************************************************

	@fsfunc_t
	
	@desc		Sets the pen pattern.
				This will set the pen's 8x8 bitmap pattern.

	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_pen_pat_func)(
					fzrt_graf_ptr port,							/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					const fzrt_pen_pattern_ptr	 pat			/* @parm <out> | pointer to a pen pattern */
						);

/************************************************************************

	@fsfunc_t
	
	@desc		Disposes of a pen pattern.
	
	@return		FZRT_NOERR on success, and error ID otherwise
	
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_pen_pat_dispose_func)(
					fzrt_pen_pattern_ptr *pat		/* @parm <in_out> | pointer to a pen pattern Must be a pointer allocated by fzrt_pen_pat_new */
					);

/************************************************************************

	@fsfunc_t
	
	@desc		Creates a new, blank pen pattern.
				This will allocate a new pen pattern. The pattern is
				initialized to all black (zero bits).
	
	@return		FZRT_NOERR on success, and error ID otherwise
	

	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC * fzrt_pen_pat_new_func)(
					fzrt_pen_pattern_ptr *			pat		/* @parm <in> | pointer to a pen pattern Must be a pointer allocated to at least 8 bytes  */
					);


/************************************************************************

	@fsfunc_t
	
	@desc		Gets a standard pen pattern.
				This will return one of the runtime standard pen patterns.
	
	@return		FZRT_NOERR on success, and error ID otherwise
	
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_error_td  (FZRT_SPEC *fzrt_get_std_penpat_func)(	
				fzrt_std_pen_pat_enum				pattype,			/*  @parm <in> | A standard pen pattern */
				fzrt_pen_pattern_ptr 				pat					/* @parm <in> | pointer to a pen pattern Must be a pointer allocated by fzrt_pen_pat_new  */
				);

/************************************************************************

	@fsfunc_t
	
	@desc		Gets the pen pattern.
				This will get the pen's 8x8 bitmap pattern of the specified port.
				If the port parameter is NULL, it gets the pen pattern of the current port
				set by the last call to fzrt_set_port.
				The caller must allocate a pen pattern before calling this function.
				and dispose of it when it is no longer needed.
	
	@avail		5.0.0.0
  
  	@see		fzrt_pen_pat_new, fzrt_pen_pat_dispose
  	
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_get_pen_pat_func)(
					fzrt_graf_ptr port,				/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					fzrt_pen_pattern_ptr pat		/* @parm <in_out> | pointer to a pen pattern that receives current pen pattern. Must be a pointer allocated by fzrt_pen_pat_new  */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Sets the current font.
				This will set the font family of the specified graphics port
				The font face, size, are left unchanged. The font IDs may
				change from one run to the next
	
	@avail		5.0.0.0

	@see		fzrt_get_fnum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_text_font_func)(
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					short					font	/* @parm <in> | the id of the font to use in the port . */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the current font.
				This will get the font family of the specified drawing port.
				
	@return		The font ID
	
	@avail		5.0.0.0

	@see		fzrt_get_fnum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  short  ( FZRT_SPEC * fzrt_get_text_font_func)(
						fzrt_graf_ptr port_param	/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
						);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the font ID.
				This will retrieve the font ID of a font with the given name.
				The name is the name of a font family (e.g. Arial)

	@return		FZRT_NOERR if successful, error id otherwise.
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_error_td  ( FZRT_SPEC *fzrt_get_fnum_func)(
							const char *		name,				/* @parm <in> | name of the font to retrieve. */
							short *						familyID	/* @parm <out> | the ID of the font. */
							);

/************************************************************************

	@fsfunc_t
  
	@desc		Get a font family name.
					This will retrieve the font family name of a font with the given ID.

	@return		FZRT_NOERR if successful, error id otherwise.
	
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_error_td  ( FZRT_SPEC *fzrt_get_font_name_func)(		
							short					familyID,				/* @parm <in> | the ID of the font. */ 
							char *					name					/* @parm <out> | pointer to buffer that receives name of font- C string */
							);

/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the current font face.
  				This will set the style of the font. The style can be
  				FZRT_FONT_NORMAL or a combination of fzrt_font_style_enum 
  				values OR-ed together using bitwise OR operator.
  	
	@avail		5.0.0.0

	@see		fzrt_font_style_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_text_face_func)(					
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					fzrt_style				face		/* @parm <in> | the face of the font to set. This can be FZRT_FONT_NORMAL or combination of fzrt_font_style_enum values */
					);

/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current font face.
  	
	@avail		5.0.0.0

	@return		the font face

	@see		fzrt_font_style_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  fzrt_style  ( FZRT_SPEC * fzrt_get_text_face_func)(
						fzrt_graf_ptr port_param	/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
						);


/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the current text drawing mode.

	@avail		5.0.0.0

	@see		fzrt_source_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_text_mode_func)(
				fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
				short					mode			/* @parm <in> | the transfer mode to apply to text */
				);

/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current text drawing mode.

	@return		The current text mode

	@avail		5.0.0.0

	@see		fzrt_source_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  short  (FZRT_SPEC *fzrt_get_text_mode_func)(
					fzrt_graf_ptr				port			/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					);


/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the current text font size.

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_text_size_func)(					
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					short					size		/* @parm <in> | Size of the font in pixels. */
					);


/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current text font size.
  				The font size is the size in pixels of the fonts ascent plus
  				descent.

	@return		The font size, in pixels

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef  short  ( FZRT_SPEC * fzrt_get_text_size_func)(
					fzrt_graf_ptr port_param		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the width of a character.
				This will return the width of the specified char using
				the current font family, size, and style selected into the current
				drawing port.

  @return		Width of character, in pixels, if drawn to current port.
  
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef short ( FZRT_SPEC *fzrt_char_width_func)(				
					short						ch		/* @parm <in> | character to get the width */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Gets the width of a string.
				This will return the width of the specified string using
				the current font family, size, style selected into the current
				port. The string is a C string.
	
	@return		Width of string, in pixels, if drawn to current port.
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef short ( FZRT_SPEC *fzrt_string_width_func)(
					char *							s			/* @parm <in> |String to measure C string */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Gets the width of a string of text.
				This will return the width of the specified string or substring using
				the current font family, size, style selected into the current
				port. The text string does not need to be null terminated.
				

	@return		Width of string if drawn to current port.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef short ( FZRT_SPEC *fzrt_text_width_func)(					
						const void *			textBuf,		/* @parm <in> | buffer containing text to measure */
						short					firstByte,		/* @parm <in> | index into buffer pointing to the first character of string */
						short					byteCnt			/* @parm <in> | length of string in bytes to read from buffer */ 
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Draws a character.
				This will draw a character to the current port using the
				the current font, using the current language script.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_draw_char_func)(	
						short					ch		/* @parm <in> | Character to draw */
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Draws a string of text to the current port using the font
				currently selected into the port. The character set of the
				string is determined by the current language.
	
	@return		FZRT_NOERR if successful, error code otherwise

	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_draw_string_func)(				
					char *					s		/* @parm <in> | string to draw. C string */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Draws text to the current port using the font
				currently selected into the port. The text does 
				not need to be NULL terminated. The character set of the
				text is determined by the current language.
	
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_draw_text_func)(					
					 const void *			textBuf,	/* @parm <in> | buffer containing text to draw */
					 short					firstByte,	/* @parm <in> | index into buffer pointing to the first character of string */
					 short					byteCnt		/* @parm <in> | length of string in bytes to read from buffer */ 
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Displays the system Choose color dialog.
				This will display the system default Choose Color dialog.
				The location of the upper, left corner of the dialog is defined by
				the where parameter. The initial color displayed in the dialog
				is defined by the RGB triple, inColor. The selected color is returned
				in the outColor RGB structure. The outColor value is undefined if 
				the user cancels the dialog, or if an error occurred.

	@return - TRUE if user clicks OK, FALSE if user clicks Cancel or error occurs
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_get_color_func)(					
							fzrt_point						*where,			/* @parm <in_opt> | pointer to point structure that specifies where the top, left corner of the dialog should be located. NULL or (0,0) Centers on main screen. */						
							const char *			prompt,			/* @parm <in> | Text to display as prompt in dialog. C string */
							const fzrt_rgb_color_td *		inColor,		/* @parm <in> | color to use as starting color in dialog */
							fzrt_rgb_color_td *				outColor		/* @parm <out> | color selected by user. Undefined if user cancels dialog */
							);


/************************************************************************

	@fsfunc_t
  
  	@desc		Convert HSL to RGB color.
  				This will convert a color from the Hue, Saturation, and Lightness model
  				to the nearest color in the Red, Blue, Green model.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_2_hsl
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_hsl_2_rgb_func)(					
					fzrt_hsl_color_td *				hColor,				/* @parm <in> | color is hsl color space */	
					fzrt_rgb_color_td *				rColor				/* @parm <out> | closest color in rgb color space */
					);				



/************************************************************************

	@fsfunc_t
  
  	@desc		Convert RGB to HSL color
  				This will convert a color from the Red, Blue, Green model
  				to the nearest color in the Hue, Saturation, and Lightness model.
	
	@avail		5.0.0.0

	@see		fzrt_hsl_2_rgb
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_rgb_2_hsl_func)(					
				fzrt_rgb_color_td *				rColor,				/* @parm <in> | color in rgb color space to be converted */	
				fzrt_hsl_color_td *				hColor			/* @parm <out> | closest color to rgb color in hsl color space */		
				);



/************************************************************************

	@fsfunc_t
  
  	@desc		Convert CMY to RGB color
  				This will convert a color from the Cyan, Magenta, and Yellow model
  				to the nearest color in the Red, Blue, Green model.
  	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_cmy_2_rgb_func)(					
					fzrt_cmy_color_td *				cColor,		/* @parm <in> | color in cmy color space to be converted */			
					fzrt_rgb_color_td *				rColor		/* @parm <out> | closest color to cmy color in rgb color space */				
					);


/************************************************************************

	@fsfunc_t
  
  	@desc		Convert HSV to RGB color
  				This will convert a color from the Hue, Saturation, and Value model
  				to the nearest color in the Red, Blue, Green model.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_2_hsv
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_hsv_2_rgb_func)(					
					fzrt_hsv_color_td *				hColor,					/* @parm <in> | color in hsv color space to be converted */			
					fzrt_rgb_color_td *				rColor					/* @parm <out> | closest color to hsv color in rgb color space */				
					);



/************************************************************************

	@fsfunc_t
  
	
  	@desc		Converts an RGB color to HSV color
   				This will convert a color from the Red, Blue, Green model
  				to the nearest color in the Hue, Saturation, and Value model.
 	
	@avail		5.0.0.0

	@see		fzrt_hsv_2_rgb
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_rgb_2_hsv_func)(					
					fzrt_rgb_color_td *				rColor,				/* @parm <in> | color in rgb color space to be converted */				
					fzrt_hsv_color_td *				hColor);			/* @parm <out> | closest color to rgb color in hsv color space */					


/************************************************************************

	@fsfunc_t
  
  	@desc		Paints a rectangle.
  				This will fill a rectangle with the current foreground color.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_fore_color
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_paint_rect_func)(					
					const fzrt_rect *			r		/* @parm <in> | pointer to rectangle to paint */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc 	Erases a rectangle with the current color.
  			This will erase a rectangle with the current background color.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_back_color
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_erase_rect_func)(					
					const fzrt_rect *			r		/* @parm <in> | pointer to rectangle to paint */
					);




/************************************************************************

	@fsfunc_t
  
	@desc		Paints a rounded rectangle.
				This paints a rounded rectangle with the current fore ground color.
				The corners of the rectangle are rounded by the oval defined in oval width and oval height.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_paint_round_rect_func)(			
				const fzrt_rect *			r,							/* @parm <in> | bounding rectangle to paint */
				short						ovalWidth,					/* @parm <in> | Width of oval defining rounded corners */
				short						ovalHeight);				/* @parm <in> | Height of oval defining rounded corners */



/************************************************************************

	@fsfunc_t
  
	@desc		This will paint an oval inside the specified rectangle, with
				the current fore ground color 
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_paint_oval_func)(					
					const fzrt_rect *			r	/* @parm <in> | bounding box of oval to paint*/
					);



/************************************************************************

	@fsfunc_t
  
	@desc		This will frame an oval inside the specified rectangle, with
				the current pen. 
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_oval_func)(					
				const fzrt_rect *			r				/* @parm <in> | bounding box of oval to Frame*/
				);



/************************************************************************

	@fsfunc_t
  
	@desc		Erases an oval.
				This will erase an oval inside the specified rectangle, with
				the current background color. 
				
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_erase_oval_func)(				
					const fzrt_rect *			r				/* @parm <in> | bounding box of oval to erase*/
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Draw an arc.
				This will draw an arc with the current pen. The arc is a section of an oval
				starting at the start angle, with 0 being at the noon position, 90 is at the
				3 o'clock position. 45 is at a line drawn from the center of the rectangle
				through the top, right corner. The length of the arc is defined by the srcAngle
				defining how many degrees the arc covers. A negative number goes counter-clockwise.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_arc_func)(					
				const fzrt_rect *			r,				/* @parm <in> | bounding box of oval defining the curve of the arc */		
				short						startAngle,		/* @parm <in> | the starting position of the arc in degrees */		
				short						arcAngle		/* @parm <in> | the number of degrees defining the arc's extent */
				);				



/************************************************************************

	@fsfunc_t
  
	@desc		Begins a polygon creation.
				This starts saving lines for processing a polygon description.
				Calls to fzrt_line and fzrt_line_to will save the endpoints of 
				the line in the polygon description. The pen is hidden, so the 
				lines are not drawn to the current port.
	
	@return		Pointer to a polygon description 

	@avail		5.0.0.0

	@see		fzrt_line, fzrt_line_to, fzrt_close_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_polygon_ptr ( FZRT_SPEC *fzrt_open_poly_func)(	void);											/* */



/************************************************************************

	@fsfunc_t
	@desc		Closes a polygon.
				This will end the creation of a polygon. The polygon is stored
				in the polygon pointer returned by fzrt_open_poly.
				The pen is shown again. All drawing after this call is 
				drawn to the current port.
  
	@avail		5.0.0.0

	@see		fzrt_line, fzrt_line_to, fzrt_open_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_close_poly_func)(					void);											/* */



/************************************************************************

	@fsfunc_t
  
  	@desc		Disposes of a polygon.
  	
	@avail		5.0.0.0

	@see		fzrt_open_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_kill_poly_func)(					
					fzrt_polygon_ptr				poly	/* @parm <in> | pointer to a polygon description that is to be disposed */
					);


/************************************************************************

	@fsfunc_t
  
	 @desc		Paints a polygon.
	 			This will fill a polygon with the current foreground color.
  	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_paint_poly_func)(		
					fzrt_polygon_ptr				poly	/* @parm <in> | pointer to a polygon description that is to be painted */
					);




/************************************************************************

	@fsfunc_t
  
  	@desc		Erases a polygon.
  				This will erase a polygon with the current background color.
  	
	@avail		5.0.0.0

	@see		fzrt_open_poly, fzrt_paint_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_erase_poly_func)(				
					fzrt_polygon_ptr				poly	/* @parm <in> | pointer to a polygon description that is to be erased */
					);





/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the value of a pixel. True for non-white, false for white.
  	
	@return		TRUE if pixel is black, FALSE otherwise		
	
	@avail		5.0.0.0

	@see		fzrt_get_cpixel
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_get_pixel_func)(		
							short					h,			/* @parm <in> | horizontal position of pixel in port */
							short					v			/* @parm <in> | vertical position of pixel in port */
							);

/************************************************************************

	@fsfunc_t
  
	@desc		This will return the color of the pixel specified in the current port.
	
	@avail		5.0.0.0

	@see		fzrt_set_cpixel
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC *fzrt_get_cpixel_func)(
					short						h,			/* @parm <in> | horizontal position of pixel  */
					short						v,			/* @parm <in> | vertical position of pixel */
					fzrt_rgb_color_td *			cPix		/* @parm <out> | pointer to rgb structure that receives the color of the pixel */
					);

/************************************************************************

	@fsfunc_t
  
  	@desc		Sets a pixel to a color.
  	
	@avail		5.0.0.0

	@see		fzrt_get_cpixel
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_set_cpixel_func)(
					 short							h,			/* @parm <in> | horizontal position of pixel in port */
					 short							v,			/* @parm <in> | vertical position of pixel in port */
					 fzrt_rgb_color_td *			cPix		/* @parm <out> | pointer to rgb structure that has the new color of the pixel */
					 );


/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the foreground color.
  				The foreground color is used for drawing with the current pen.
  	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_rgb_fore_color_func)(			
					fzrt_rgb_color_td *				color		/* @parm <in> | pointer to a rgb structure that has new foreground color */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the background color.
  				The background color is used for erasing graphic objects and 
  				in certain transfer mode in fzrt_copy_bits and drawing text.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_back_color
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_rgb_back_color_func)(	
					fzrt_rgb_color_td *				color		/* @parm <in> | pointer to a rgb structure that has new background color */
					);

/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current foreground color.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_fore_color
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_fore_color_func)(			
					fzrt_rgb_color_td *				color	/* @parm <out> | pointer to a rgb structure that receives foreground color */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc		Gets the current background color.
  	
	@avail		5.0.0.0

	@see		fzrt_rgb_back_color
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_back_color_func)(
					fzrt_rgb_color_td *				color	/* @parm <out> | pointer to a rgb structure that receives background color */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Draws an icon.
				Draws an icon in the current rectangle, stretching or shrinking it
				if necessary. Icons can be loaded with fzrt_rlib_load_icon.
	
	@avail		5.0.0.0

  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_plot_icon_func)(				
					const fzrt_rect *			theRect,				/* @parm <in> | pointer to rectangle that is location of the icon */		
					fzrt_icon_ptr				theIcon					/* @parm <in> | pointer to an icon object */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc		Creates a new, empty region
  	
	@return		The newly created region, NULL if there is an error.

	@avail		5.0.0.0

	@see	fzrt_dispose_rgn	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_rgn_ptr ( FZRT_SPEC *fzrt_new_rgn_func)(void);	



/************************************************************************

	fsfunc_t
  
	desc		Begin definition of region.
				This will start saving drawing calls to define a region. All calls to fzrt_line, fzrt_line_to
				and any framing operation, except arcs will add to the description of the
				region. The pen is hidden until the corresponding call to fzrt_close_rgn.
	
	avail		6.5.0.0

	see		fzrt_line, fzrt_line_to, fzrt_close_rgn
  
	toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_open_rgn_func)(void);											/* */



/************************************************************************

	fsfunc_t
  
	desc		Finishes defining a region.
				This will finish the description of a region, and passes back the finished
				region. The region must first be started with fzrt_open_rgn.
	
	avail		6.5.0.0

	see		fzrt_open_rgn, fzrt_new_rgn.
  
	toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_close_rgn_func)(					
				   fzrt_rgn_ptr					dstRgn	/* @parm <out> | pointer to receive finished region. Must be allocated with fzrt_new_rgn before call */
				   );



/************************************************************************

	@fsfunc_t
  
  	@desc		Checks if the region is empty.
  	
	@return		TRUE if region is empty, FALSE otherwise		
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_empty_rgn_func)(				
							fzrt_rgn_ptr				rgn	/* @parm <in> | pointer to region to test for emptiness */
							);



/************************************************************************

	@fsfunc_t
  
	@desc		Erases a region.
				This will fill a region with the background color.
	
	@avail		5.0.0.0
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_erase_rgn_func)(					
				   fzrt_rgn_ptr				rgn		/* @parm <in> | pointer to region to erase */
				   );



/************************************************************************

	@fsfunc_t
  
	@desc		Insets a region.
				This will inset a region by the amounts specified. If the 
				values are negative, the region is outset.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_inset_rgn_func)(		
					fzrt_rgn_ptr				rgn,				/* @parm <in> | pointer to region to inset */	
					short						dh,					/* @parm <in> | amount to inset the region horizontally */	
					short						dv				/* @parm <in> | amount to inset the region vertically */	
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Assigns a region to the specified rectangle.
					This will change the existing region to a rectangular region.
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_rect_rgn_func)(	
					fzrt_rgn_ptr				rgn,			/* @parm <in> | pointer to region to change */
					const fzrt_rect *			r				/* @parm <in> | pointer to rectangle to set */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Gets a regions bounding rectangle.
				This will return the rectangle that completely encloses the specified region.
				
	@return		FZRT_NOERR on success, and error ID otherwise.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td ( FZRT_SPEC *fzrt_get_rgn_bbox_func)(		
					   fzrt_rgn_ptr				rgn,		/* @parm <in> | pointer to a region */
					   fzrt_rect *				rect		/* @parm <out> | pointer to a rectangle that receives the bounding rectangle */
					   );




/************************************************************************

	@fsfunc_t
  
	@desc		Sets the cursor to a standard cursor.
				This will reset the mouse cursor to one of the standard
				cursor types. Currently either FZRT_CURS_ARROW or FZRT_CURS_WATCH.
				FZRT_CURS_WATCH will set to the static wait cursor. To use an
				animated wait cursor use fzrt_set_std_wait_cursor.
	
	@avail		5.0.0.0

	@see		fzrt_std_cursor_enum, fzrt_set_std_wait_cursor
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_set_std_cursor_func)(
					fzrt_std_cursor_enum					curs_type		/* @parm <in> | standard cursor  */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		Sets the wait cursor step.
				This will set the system animated wait cursor at the desired step.
				To animate the cursor, this function needs to be called repeatedly with
				the stage incremented by one on each call. 
	
	@avail		5.0.0.0

	@see		fzrt_set_std_cursor
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_set_std_wait_cursor_func)(
					 fzrt_int					stage			/* @parm <in> | the step for an animated cursor */
					 );


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the pen location.
				This will return the coordinates of the pen in the current port
	
	@avail		5.0.0.0

	@see		fzrt_move, fzrt_move_to
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_pen_func)(
				fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
				fzrt_point *			pt			/* @parm <out> | pointer to point that receives the pen position in local coordinates */
				);



/************************************************************************

	@fsfunc_t
  
	@desc		Moves the pen in the current port to the desired location.
	
	@avail		5.0.0.0

	@see		fzrt_move
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_move_to_func)(					
				short					h,	/* @parm <in> | Horizontal position in local coordinates */
				short					v	/* @parm <in> | Vertical position in local coordinates */
				);



/************************************************************************

	@fsfunc_t
  
	@desc		Moves the pen.
				This will move the pen the desired distance relative to the current location.
	
	@avail		5.0.0.0

	@see		fzrt_get_pen, fzrt_move_to
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_move_func)(		
					short					dh,			/* @parm <in> | Horizontal distance from current pen location */
					short					dv			/* @parm <in> | Vertical distance from current pen location */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Draws a line.
				This draws a line with the current pen from the current pen location
				to the specified location.
	
	@avail		5.0.0.0

	@see		fzrt_line
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_line_to_func)(
					short					h,		/* @parm <in> | Horizontal position of line endpoint */
					short					v		/* @parm <in> | Vertical position of line endpoint */
					);	



/************************************************************************

	@fsfunc_t
  
	@desc		Draws a line.
				This will draw a line from the current location of the 
				specified length. Positive values for the distance will
				draw up and to the right, negative values will draw
				down and to the left.
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_line_func)(		
					short					dh,		/* @parm <in> | Horizontal length of the line */
					short					dv		/* @parm <in> | Vertical height of the line */
					);



/************************************************************************

	@fsfunc_t
  
  	@desc		Sets the coordinates of a rectangle.
  	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_set_rect_func)(			
				fzrt_rect *					r,				/* @parm <out> | pointer to rectangle to set */
				short					left,				/* @parm <in> | new left (x) coordinate of rectangle */
				short					top,				/* @parm <in> | new top (y) coordinate of rectangle */
				short					right,				/* @parm <in> | new right (x) coordinate of rectangle */
				short					bottom				/* @parm <in> | new bottom (x) coordinate of rectangle */
				);			

/************************************************************************

	@fsfunc_t
  
  	@desc		Intersects two rectangles.
  				This will assign the intersection of two rectangles to the destination 
  				rectangle. If the two source rectangles do not intersect, the functions
  				returns FALSE and the destination rectangle is empty
  	
	@return		TRUE if rectangles intersect, FALSE if they do not intersect
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_sect_rect_func)(	
						fzrt_rect *					src1,					/* @parm <in> | pointer to first source rectangle */
						fzrt_rect *					src2,					/* @parm <in> | pointer to first source rectangle */
						fzrt_rect *					dstRect);				/* @parm <out> | pointer to rectangle that receives the result of the intersection */



/************************************************************************

	@fsfunc_t
  
	@desc		Checks if a rectangle is empty.
			This function determines whether the specified rectangle is empty. 
			A empty rectangle is one that has no area; that is, the coordinate of the right side is 
			less than or equal to the coordinate of the left side, or the coordinate of the bottom 
			side is less than or equal to the coordinate of the top side. 

	@return		TRUE if rectangle is empty, FALSE otherwise.

	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_empty_rect_func)(
						fzrt_rect *					r	/* @parm <in> |Pointer to a rectangle structure that contains the logical coordinates of the rectangle. */
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Frames a polygon.
				This will frame a polygon with the current pen. Polygon must be closed before
				the polygon can be framed.
	
	@avail		5.0.0.0

	@see		fzrt_open_poly, fzrt_close_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_poly_func)(	
					fzrt_polygon_ptr				p	/* @parm <in> | pointer to polygon that is to be framed */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Frames a rounded rectangle.
				This will frame a rounded rectangle with the current pen. The corner of 
				the rounded rect is defined by the oval width and height.  
	
	@avail		5.0.0.0

	
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_round_rect_func)(			
				const fzrt_rect *			r,			/* @parm <in> | pointer to rectangle that contains coordinates of enclosing rectangle */
				short						w,			/* @parm <in> |width of oval that defines rounded corner */
				short						h			/* @parm <in> |height of oval that defines rounded corner */
				);	



/************************************************************************

	@fsfunc_t
  
	@desc		Inverts a rectangle.
				This will fill a rectangle with the inverse colors already 
				filling the rectangle. The inverse color is a bitwise NOT of the
				current pixel values.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_invert_rect_func)(	
					fzrt_rect *					r	/* @parm <in> | pointer to rectangle that contains coordinates of rectangle to invert */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		Fills a rectangle.
				This will fill a rectangle with the specified pattern. 
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_fill_rect_func)(
					const fzrt_rect *			r_param,			/* @parm <in> | pointer to rectangle that contains coordinates of rectangle to fill */
					const fzrt_pen_pattern_ptr 	pat					/* @parm <in> | pointer to pen pattern to fill the rect */
					);


/************************************************************************

	@fsfunc_t
	
	  @desc		Disposes of a region.
	  			This will dispose the memory associated with a region.
				After this call, the region pointer is invalid to use
				with other region functions.
	  
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_dispose_rgn_func)(			
				fzrt_rgn_ptr				rgn		/* @parm <in> | pointer to region to dispose */
				);



/************************************************************************

	@fsfunc_t
  
  	@desc		Tests if a point in inside a region.
  	
	@return		TRUE if point is located inside region, FALSE if point is outside of rectangle.		
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_pt_in_rgn_func)(		
					fzrt_point					pt,		/* @parm <in> | pointer to point to test */
					fzrt_rgn_ptr				rgn		/* @parm <in> | pointer to region to test */
					);				



/************************************************************************

	@fsfunc_t
  
	@desc		Tests if a rectangle is in a region.
				This function determines whether any part of the specified rectangle is within the boundaries of a region. 	

	@return		TRUE if any part of the rectangle is inside the region, FALSE otherwise
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_rect_in_rgn_func)(	
						fzrt_rect *					r,					/* @parm <in> | pointer to rectangle to test */
						fzrt_rgn_ptr				rgn					/* @parm <in> | pointer to region */
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Calculates the difference between two regions.
				This function will calculate the parts of first region that are not part of the second region.
				The regions do not need to be distinct, so the destination region may be either of the source regions.
				The destination region needs to be created by the caller before calling this function
	
	@avail		5.0.0.0

  @see		fzrt_new_rgn, fzrt_sect_rgn, fzrt_union_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_diff_rgn_func)(					
				fzrt_rgn_ptr				srcRgnA,				/* @parm <in> | pointer to the first source region */
				fzrt_rgn_ptr				srcRgnB,				/* @parm <in> | pointer to the second source region*/
				fzrt_rgn_ptr				dstRgn					/* @parm <out> | pointer to region the receives the difference of the two source regions */
				);	



/************************************************************************

  @fsfunc_t
  
	@desc		Calculates the intersection between two regions.
				This function will calculate the parts of first region that are also part of the second region.
				The regions do not need to be distinct, so the destination region may be either of the source regions.
				The destination region needs to be created by the caller before calling this function
				
				  
	@avail		5.0.0.0

	@see		fzrt_new_rgn, fzrt_diff_rgn, fzrt_union_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_sect_rgn_func)(					
					fzrt_rgn_ptr				srcRgnA,			/* @parm <in> | pointer to the first source region */	
					fzrt_rgn_ptr				srcRgnB,			/* @parm <in> | pointer to the second source region*/	
					fzrt_rgn_ptr				dstRgn				/* @parm <out> | pointer to region the receives the intersection of the two source regions */
					);				



/************************************************************************

	@fsfunc_t
  
	@desc		Sets a region to the empty region.
				This will reset a region to the empty region. The region must be
				created before calling this function.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_set_empty_rgn_func)(				
				   fzrt_rgn_ptr				rgn			/* @parm <in> | pointer to the region to set to empty. Must be created before call */
				   );



/************************************************************************

	@fsfunc_t
  
	@desc		Frames a region.
				This function will draw border around a region with the current pen.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_rgn_func)(		
					fzrt_rgn_ptr				rgn		/* @parm <in> | pointer to region that is to be framed */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		This function will fill a region with the specified pen pattern
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC * fzrt_fill_rgn_func)(
					fzrt_rgn_ptr rgn_param,				/* @parm <in> | pointer to region that is to be filled */
					const fzrt_pen_pattern_ptr  pat		/* @parm <in> | pointer to pen pattern used to fill the region */
					);

/************************************************************************

	@fsfunc_t
  
	  @desc		Calculates the union of two rectangles.
	  			This function creates the union of two rectangles. The union is the 
	  			smallest rectangle that contains both source rectangles. 
		
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_union_rect_func)(
				const fzrt_rect *			src1,				/* @parm <in> | pointer to first source rectangle */
				const fzrt_rect *			src2,				/* @parm <in> | pointer to second source rectangle */
				fzrt_rect *					dstRect				/* @parm <out> | pointer to rectangle that receives the union of the two source rectangles */
				);



/************************************************************************

	@fsfunc_t
  
	@desc		Tests if a point in inside a rectangle.
				This function test whether the specified point is located
				within the specified rectangle. A point that is on the edges
				of a rectangle is considered to be in the rectangle

	@return		TRUE if point is inside the rectangle, FALSE if it is not
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_pt_in_rect_func)(					
							fzrt_point					pt,			/* @parm <in> | pointer to a point structure that contains the specified point */
							fzrt_rect *					r			/* @parm <in> | pointer to a rectangle structure that contains the specified rectangle */
							);			


/************************************************************************

	@fsfunc_t
  
	@desc		Tests if two rectangles are equal.
				This function will compare two rectangles for equality. 
				Rectangle are equal if their top, left and bottom right
				corners are the same. Rectangles of the same width and height
				are not equal if they do not have the same corner points.

	@return		TRUE if rectangles are equal, FALSE otherwise
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_equal_rect_func)(				
						fzrt_rect *					rect1,				/* @parm <in> | pointer to a rectangle structure that contains the first rectangle */
						fzrt_rect *					rect2				/* @parm <in> | pointer to a rectangle structure that contains the second rectangle */ 
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Frames a rectangle.
				This function will draw a border around the specified rectangle
				using the current pen.
	
	@avail		5.0.0.0

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_frame_rect_func)(					
							const fzrt_rect *			r		/* @parm <in> | pointer to a rectangle structure that contains the specified rectangle */
							);						/* */



/************************************************************************

	@fsfunc_t
  
	@desc		Inverts a region.
				This function will invert the colors of the specified region.
				The region must be a valid region, created with fzrt_new_rgn.
				The inverse color is a bitwise NOT of the current colors in the region.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_invert_rgn_func)(
					fzrt_rgn_ptr				rgn				/* @parm <in> | pointer to region to invert */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Paints a region.
				This function will fill a region with the current foreground
				color
	
	@avail		6.5.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_paint_rgn_func)(	
				   fzrt_rgn_ptr				rgn				/* @parm <in> | pointer to region to paint */
				   );



/************************************************************************

	@fsfunc_t
  
	@desc		Copies a region.
				This function will make a copy of the specified region. The
				destination region must be created before calling this function.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_copy_rgn_func)(
				fzrt_rgn_ptr				srcRgn,						/* @parm <in> | pointer to region to copy */
				fzrt_rgn_ptr				dstRgn						/* @parm <out> | pointer to region to receive the copy */
				);



/************************************************************************

	@fsfunc_t
  
	@desc		Calculate the union of two regions.
				This function will calculate the union of the first region and second region.
				The regions do not need to be distinct, so the destination region may be either of the source regions.
				The destination region needs to be created by the caller before calling this function
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn, fzrt_sect_rgn, fzrt_diff_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_union_rgn_func)(					
				fzrt_rgn_ptr				srcRgnA,				/* @parm <in> | pointer to the first source region */
				fzrt_rgn_ptr				srcRgnB,				/* @parm <in> | pointer to the second source region*/
				fzrt_rgn_ptr				dstRgn					/* @parm <out> | pointer to region the receives the union of the two source regions */
				);	


/************************************************************************

	@fsfunc_t
  
	@desc		Insets a rectangle.
				This function increases or decreases the width and height of the specified rectangle. 
				It adds dx units to the left and subtracts dx units from the right side of the rectangle
				and adds dy units to the top and subtracts dy units from the bottom side. 
				The dx and dy parameters are signed values; positive values decrease the width and height, 
				and negative values increase them. 
	
	@avail		5.0.0.0

	@see		fzrt_offset_rect

	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_inset_rect_func)(
					fzrt_rect *					r,				/* @parm <in_out> | pointer to a rectangle structure to inset. On return it contain the new inset rectangle */
					short					dx,					/* @parm <in> | the amount to inset the left and right edges */
					short					dy				/* @parm <in> | the amount to inset the top and bottom edges */
					);



/************************************************************************

	@fsfunc_t
  
	@desc		Offsets a rectangle.
				This function moves the specified rectangle by the specified offsets.
	
	@avail		5.0.0.0

	@see		fzrt_inset_rect
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_offset_rect_func)(	
					fzrt_rect *						r,			/* @parm <in_out> | pointer to a rectangle structure to offset. On return it contain the new offset rectangle */
					short							dx,			/* @parm <in> | the amount to offset the rectangle along the x coordinate */
					short							dy			/* @parm <in> | the amount to offset the rectangle along the y coordinate */
					);


/************************************************************************

	@fsfunc_t
  
	@desc		Test if two regions are equal.
				This function checks the two specified regions to determine whether they are identical. 
				The function considers two regions identical if they are equal in size, shape, and postion. 
	
	@return		TRUE if the two regions are identical, FALSE otherwise
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_boolean ( FZRT_SPEC *fzrt_equal_rgn_func)(	
						fzrt_rgn_ptr				rgn1,					/* @parm <in> | pointer to first source region to test */
						fzrt_rgn_ptr				rgn2					/* @parm <in> | pointer to first source region to test */
						);



/************************************************************************

	@fsfunc_t
  
	@desc		Offsets a region.
				This function will move a region by the desired offsets.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_offset_rgn_func)(		
					fzrt_rgn_ptr			rgn,				/* @parm <in> | pointer to region to offset */
					short					dx,						/* @parm <in> | the amount to offset the region along the x coordinate */
					short					dy);					/* @parm <in> | the amount to offset the region along the y coordinate */


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the bits of a pen pattern.
				This function will return the 8x8 bitmap pattern used for a pen pattern.
				The pattern is a monochrome bitmap, stored as 8 bytes, with each bit in
				a byte representing a pixel in the 8x8 bitmap.
				The first bytes is the top row, with the high bit being the leftmost pixel.
	
	@avail		5.0.0.0

	@see		fzrt_new_rgn
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef fzrt_error_td (FZRT_SPEC * fzrt_graf_get_pattern_bits_func)(
							fzrt_pen_pattern_ptr pattern,	/* @parm <in> | pointer to pattern */
							unsigned char *bits				/* @parm <out> | pointer to memory to received the bits. This must be allocated by the caller and have at least 8 bytes */
							);

/************************************************************************

	@fsfunc_t
  
	@desc		Inverts a polygon.
				This function will invert the colors of a polygon. The 
				polygon must be closed with fzrt_close_poly before calling 
				this function.
	
	@avail		5.0.0.0

	@see		fzrt_open_poly, fzrt_close_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_invert_poly_func)(
					fzrt_polygon_ptr 			poly			/* @parm <in> | pointer to polygon to invert */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Fills a polygon.
				This function will fill a polygon with the specified bitmap pattern.
	
	@avail		5.0.0.0

	@see		fzrt_open_poly, fzrt_close_poly
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef  void  (FZRT_SPEC  * fzrt_fill_poly_func)(
					fzrt_polygon_ptr 			poly,	/* @parm <in> | pointer to polygon created with fzrt_open_poly */
					const fzrt_pen_pattern_ptr  pat		/* @parm <in> | pointer to pen pattern to fill the polygon with */
					);

/************************************************************************

	@fsfunc_t
  
	@desc		Gets the pen size.
				This will get the size of the current pen. The size of the
				pen is stored as a point for width and height. If the underlying
				OS does not support non-square pens, then the h coordinate contains
				the size of the pen, and v coordinate is unused.

	@avail		5.0.0.0

	@see		fzrt_pen_size
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_pen_size_func)(			
				fzrt_graf_ptr			port,				/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
				fzrt_point					*pnSize			/* @parm <in_out> | pointer to a point structure to receive pen size. x is size of pen in horizontal direction, y is size of pen in vertical direction. */
				);


/************************************************************************

	@fsfunc_t
  
	@desc		Gets the pen mode.
				This will get the transfer mode for the current pen.
	
	@avail		5.0.0.0

	@see		fzrt_pen_mode, fzrt_pattern_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_get_pen_mode_func)(	
					fzrt_graf_ptr			port,		/* @parm <in_opt> | pointer to graphics port. NULL to use current port */
					short					*mode		/* @parm <out> | pointer to short that receives the current pen mode */
					);




/************************************************************************

	@fsfunc_t
  
	@desc		This function will copy the pixels from a source port to a 
				destination port. If the srcRect and destination rectangle
				are different sizes, the image is stretched, or shrunk to
				fit the destination. The bits are combined based on the transfer
				mode specified. The current foreground and background colors are 
				used during the transfer.
				The user can define a mask region to limit what part of the
				rectangle is copied.
				The source and destination port can be the same. 
	
	@avail		5.0.0.0

	@see		fzrt_source_mode_enum, fzrt_arith_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_copy_bits_func)( 
						 const	fzrt_graf_ptr		srcBits,				/* @parm <in> | pointer to graf port to read source pixel from*/
						 const fzrt_graf_ptr		dstBits,				/* @parm <in> | pointer to graf port to write source pixel to*/
						 const fzrt_rect *			srcRect,				/* @parm <in> | pointer to rectangle that defines where in port to read pixels */
						 const fzrt_rect *			dstRect,				/* @parm <in> | pointer to rectangle that defines where in destination port to put resulting pixels*/
						 short						mode,					/* @parm <in> | the transfer mode to use when combining pixel from source port to destination port */
						 fzrt_rgn_ptr				maskRgn					/* @parm <in_opt> | pointer to a region that defines mask to use */
						 );



/************************************************************************

	@fsfunc_t
  
	@desc		Copies the pixels from the source port to the destination colorizing with the specified colors.
				This function will copy the pixels from a source port to a 
				destination port. If the srcRect and destination rectangle
				are different sizes, the image is stretched, or shrunk to
				fit the destination. The bits are combined based on the transfer
				mode specified. The foreground and background colors are set
				to those specified before the transfer is made.
				The user can define a mask region to limit or clip what part of the
				rectangle is copied.
	
	@avail		5.0.0.0

	@see		fzrt_source_mode_enum, fzrt_arith_mode_enum
  
	@toc		fzrt_graf
		
*************************************************************************/
typedef void ( FZRT_SPEC *fzrt_copy_bits_colorize_func)(			
					const	fzrt_graf_ptr		srcBits,				/* @parm <in> | pointer to graf port to read source pixel from*/
					const fzrt_graf_ptr			dstBits,				/* @parm <in> | pointer to graf port to write source pixel to*/
					const fzrt_rect *			srcRect,				/* @parm <in> | pointer to rectangle that defines where in port to read pixels */
					const fzrt_rect *			dstRect,				/* @parm <in> | pointer to rectangle that defines where in destination port to put resulting pixels*/
					short						mode,					/* @parm <in> | the transfer mode to use when combining pixel from source port to destination port */
					fzrt_rgn_ptr				maskRgn,				/* @parm <in_opt> | pointer to a region that defines mask to use */
					fzrt_rgb_color_td			*fcolor,				/* @parm <in_opt> | pointer to rgb structure that contains the color to use for foreground color , if no color is specified, black is use for foreground */
					fzrt_rgb_color_td			*bcolor					/* @parm <in_opt> | pointer to rgb structure that contains the color to use for background color , if no color is specified, white is use for background */
					);			


/********************************************************************************************************

	@module FZRT_BASE | Basic formZ Runtime functions

	@desc	Basic Runtime functions. This is the basic set of runtime functionality that will most likely
			be used by any plugin or script. This includes error handling functions (fzrt_error*), formz Resource Files (fzrt_fzr*) functions,
			and UUID functions (fzrt_UUID*)

	@toc	fzrt_base

***************************************************************************/


/************************************************************************
																	 
	@fset	fz_fzrt_base_fset
	
	@desc	formZ Runtime Base Function Set

	@avail	5.0.0.0

	@toc	fzrt_base

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime Base function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_BASE_FSET_TYPE			"\x80\x30\x46\x63\xc4\xe8\x43\x64\x86\x91\xeb\xb6\x28\x06\xd8\xd6"	/* {80304663-c4e8-4364-8691-ebb62806d8d6} Reference : 1943 */

/********************************************************************************************************

	@fsdef	formZ Runtime Base function set name.

*********************************************************************************************************/
#define FZ_FZRT_BASE_FSET_NAME			"formZ Runtime Base functions"
/********************************************************************************************************

	@fsdef	formZ Runtime Base function set version.

*********************************************************************************************************/
#define FZ_FZRT_BASE_FSET_VERSION		FZPL_VERS_MAKE(5,0,0,0)

typedef struct fz_fzrt_base_fset
{
	fzpl_fset_prvt_data_td						prvt_data;

	fzrt_error_log_msg_func						fzrt_error_log_msg;					/* @fsfunc */
	fzrt_error_set_with_detail_func				fzrt_error_set_with_detail;			/* @fsfunc */
	fzrt_error_set_func							fzrt_error_set;						/* @fsfunc */
	fzrt_error_add_detail_func					fzrt_error_add_detail;				/* @fsfunc */
	fzrt_error_add_debug_func					fzrt_error_add_debug;				/* @fsfunc */
	fzrt_error_clear_func						fzrt_error_clear;					/* @fsfunc */
	fzrt_error_get_last_func					fzrt_error_get_last;				/* @fsfunc */
	fzrt_error_get_string_func					fzrt_error_get_string;				/* @fsfunc */
	fzrt_error_get_info_func					fzrt_error_get_info;				/* @fsfunc */
	fzrt_error_exists_func						fzrt_error_exists;					/* @fsfunc */
	fzrt_error_posted_func						fzrt_error_posted;					/* @fsfunc */
	fzrt_error_was_posted_func					fzrt_error_was_posted;				/* @fsfunc */
	fzrt_error_set_alrt_on_error_func			fzrt_error_set_alrt_on_error;		/* @fsfunc */

	/*** FZR functions ***/
	fzrt_fzr_open_func							fzrt_fzr_open;						/* @fsfunc */
	fzrt_fzr_close_func							fzrt_fzr_close;						/* @fsfunc */
	fzrt_fzr_get_string_func					fzrt_fzr_get_string;				/* @fsfunc */
	fzrt_fzr_get_stringP_func					fzrt_fzr_get_stringP;				/* @fsfunc */
	fzrt_fzr_string_cnt_func					fzrt_fzr_string_cnt;				/* @fsfunc */
	fzrt_fzr_get_menu_func						fzrt_fzr_get_menu;					/* @fsfunc */
	fzrt_fzr_get_menu_stringP_func				fzrt_fzr_get_menu_stringP;			/* @fsfunc */
	fzrt_fzr_get_menu_string_func				fzrt_fzr_get_menu_string;			/* @fsfunc */
	fzrt_fzr_get_rsrc_ind_ID_func				fzrt_fzr_get_rsrc_ind_ID;			/* @fsfunc */
	fzrt_fzr_get_rsrc_cnt_func					fzrt_fzr_get_rsrc_cnt;				/* @fsfunc */
	fzrt_fzr_write_file_func					fzrt_fzr_write_file;				/* @fsfunc */
	fzrt_fzr_get_info_func						fzrt_fzr_get_info;					/* @fsfunc */
	fzrt_fzr_get_string_size_func				fzrt_fzr_get_string_size;			/* @fsfunc */
	
	fzrt_rlib_get_resource_func					fzrt_rlib_get_resource;				/* @fsfunc */
	fzrt_rlib_load_icon_func					fzrt_rlib_load_icon;				/* @fsfunc */

	/*** UUID functions ***/
	fzrt_UUID_copy_func							fzrt_UUID_copy;						/* @fsfunc */
	fzrt_UUID_is_NULL_func						fzrt_UUID_is_NULL;					/* @fsfunc */
	fzrt_UUID_is_notNULL_func					fzrt_UUID_is_notNULL;				/* @fsfunc */
	fzrt_UUID_set_NULL_func						fzrt_UUID_set_NULL;					/* @fsfunc */
	fzrt_UUID_is_equal_func						fzrt_UUID_is_equal;					/* @fsfunc */
	fzrt_UUID_is_notequal_func					fzrt_UUID_is_notequal;				/* @fsfunc */
	fzrt_UUID_compare_func						fzrt_UUID_compare;					/* @fsfunc */
	fzrt_UUID_is_less_func						fzrt_UUID_is_less;					/* @fsfunc */
	fzrt_UUID_is_greater_func					fzrt_UUID_is_greater;				/* @fsfunc */
	fzrt_UUID_is_less_equal_func				fzrt_UUID_is_less_equal;			/* @fsfunc */
	fzrt_UUID_is_greater_equal_func				fzrt_UUID_is_greater_equal;			/* @fsfunc */
	fzrt_UUID_to_string_func					fzrt_UUID_to_string;				/* @fsfunc */
	fzrt_string_to_UUID_func					fzrt_string_to_UUID;				/* @fsfunc */
	fzrt_UUID_create_func						fzrt_UUID_create;					/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
}  fz_fzrt_base_fset;


/********************************************************************************************************

	@module FZRT_FILE | formZ Runtime file functions

	@desc	Runtime File functions. This is a set of file representation functions. 
			The file locator, fzrt_floc_ptr, is used to represent any file or folder in the file system.
			Functions in this set can get or set the current folder, retrieve information on files or 
			folders, and search for files and folders. There are no functions to provide direct access
			to files for reading or writing. The ANSI C functions should be used instead for opening and 
			reading/writing of files.  

	@toc	fzrt_file

***************************************************************************/

/************************************************************************
																	 
	@fset	fz_fzrt_file_fset
	
	@desc	formZ Runtime File Function Set

	@avail	5.0.0.0

	@toc	fzrt_file

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_FILE_FSET_TYPE			"\x8b\xb1\x52\x4e\x67\xfd\x4a\x8a\xac\xfd\xa0\x31\x2e\xa2\xe6\x6c"	/* {8bb1524e-67fd-4a8a-acfd-a0312ea2e66c} Reference : 1944 */
/********************************************************************************************************

	@fsdef	formZ Runtime File function set name.

*********************************************************************************************************/
#define FZ_FZRT_FILE_FSET_NAME			"formZ Runtime File functions"
/********************************************************************************************************

	@fsdef	formZ Runtime File function set version.

*********************************************************************************************************/
#define FZ_FZRT_FILE_FSET_VERSION		FZPL_VERS_MAKE(5,0,0,0)
typedef struct fz_fzrt_file_fset
{
	fzpl_fset_prvt_data_td						prvt_data;
	
	/*** volume spec functions ***/
	fzrt_file_floc_init_func					fzrt_file_floc_init;		/* @fsfunc */
	fzrt_file_floc_finit_func					fzrt_file_floc_finit;		/* @fsfunc */
	fzrt_file_floc_get_crnt_func				fzrt_file_floc_get_crnt;	/* @fsfunc */
	fzrt_file_floc_set_crnt_func				fzrt_file_floc_set_crnt;	/* @fsfunc */
	fzrt_file_floc_copy_func					fzrt_file_floc_copy;		/* @fsfunc */
	fzrt_file_floc_compare_func					fzrt_file_floc_compare;		/* @fsfunc */
	fzrt_file_floc_valid_func					fzrt_file_floc_valid;		/* @fsfunc */
	
	/*** volume folder functions ***/
	fzrt_file_get_scratch_floc_func				fzrt_file_get_scratch_floc;		/* @fsfunc */
	fzrt_file_get_system_floc_func				fzrt_file_get_system_floc;			/* @fsfunc */
	fzrt_file_get_prefs_floc_func				fzrt_file_get_prefs_floc;			/* @fsfunc */
	fzrt_file_get_fonts_floc_func				fzrt_file_get_fonts_floc;			/* @fsfunc */
	fzrt_file_get_extensions_floc_func			fzrt_file_get_extensions_floc;		/* @fsfunc */
	fzrt_file_get_application_floc_func			fzrt_file_get_application_floc;	/* @fsfunc */
	fzrt_file_get_documents_floc_func			fzrt_file_get_documents_floc;		/* @fsfunc */
	fzrt_file_new_folder_func					fzrt_file_new_folder;				/* @fsfunc */
	fzrt_file_open_folder_func					fzrt_file_open_folder;				/* @fsfunc */
	
	/*** volume disk functions ***/
	fzrt_file_vol_free_space_func				fzrt_file_vol_free_space;	/* @fsfunc */
	fzrt_file_floc_flush_func					fzrt_file_floc_flush;		/* @fsfunc */
	fzrt_file_get_vspec_type_func				fzrt_file_get_vspec_type;	/* @fsfunc */
	
	/*** volume file functions ***/
	fzrt_file_get_time_stamp_func				fzrt_file_get_time_stamp;			/* @fsfunc */
	fzrt_file_set_time_stamp_func				fzrt_file_set_time_stamp;			/* @fsfunc */
	fzrt_file_get_signature_func				fzrt_file_get_signature;			/* @fsfunc */
	fzrt_file_set_signature_func				fzrt_file_set_signature;			/* @fsfunc */
	fzrt_file_get_size_func						fzrt_file_get_size;					/* @fsfunc */
	fzrt_file_hide_func							fzrt_file_hide;						/* @fsfunc */
	fzrt_file_is_hidden_func					fzrt_file_is_hidden;				/* @fsfunc */
	fzrt_file_is_locked_func					fzrt_file_is_locked;				/* @fsfunc */
	fzrt_file_get_char_func						fzrt_file_get_char;					/* @fsfunc */
	fzrt_file_is_invalid_char_func				fzrt_file_is_invalid_char;			/* @fsfunc */
	fzrt_file_folder_count_files_func			fzrt_file_folder_count_files;		/* @fsfunc */
	fzrt_file_folder_count_folders_func			fzrt_file_folder_count_folders;		/* @fsfunc */
	
	/*** volume find functions ***/
	fzrt_file_find_file_by_index_func			fzrt_file_find_file_by_index;		/* @fsfunc */
	fzrt_file_find_next_file_func				fzrt_file_find_next_file;			/* @fsfunc */
	fzrt_file_find_folder_init_func				fzrt_file_find_folder_init;			/* @fsfunc */
	fzrt_file_find_file_init_func				fzrt_file_find_file_init;			/* @fsfunc */
	fzrt_file_find_folder_by_index_func			fzrt_file_find_folder_by_index;		/* @fsfunc */
	fzrt_file_find_next_folder_func				fzrt_file_find_next_folder;			/* @fsfunc */
	fzrt_file_find_finit_func					fzrt_file_find_finit;				/* @fsfunc */
	
	/*** file spec functions ***/
	fzrt_file_floc_get_name_func				fzrt_file_floc_get_name;			/* @fsfunc */
	fzrt_file_floc_set_name_func				fzrt_file_floc_set_name;			/* @fsfunc */
	fzrt_file_floc_get_path_name_func			fzrt_file_floc_get_path_name;		/* @fsfunc */
	fzrt_file_floc_set_path_name_func			fzrt_file_floc_set_path_name;		/* @fsfunc */
	fzrt_file_floc_get_fullpath_func			fzrt_file_floc_get_fullpath;		/* @fsfunc */
	fzrt_file_floc_set_fullpath_func			fzrt_file_floc_set_fullpath;		/* @fsfunc */
	fzrt_file_floc_get_volume_name_func			fzrt_file_floc_get_volume_name;		/* @fsfunc */
	fzrt_file_floc_set_volume_name_func			fzrt_file_floc_set_volume_name;		/* @fsfunc */
	fzrt_file_does_exist_func					fzrt_file_does_exist;				/* @fsfunc */
	fzrt_file_get_fnam_max_func					fzrt_file_get_fnam_max;				/* @fsfunc */
	
	fzrt_file_floc_get_parent_func				fzrt_file_floc_get_parent;			/* @fsfunc */
	fzrt_file_floc_is_folder_func				fzrt_file_floc_is_folder;			/* @fsfunc */
	fzrt_file_floc_is_file_func					fzrt_file_floc_is_file	;			/* @fsfunc */
	
	fzrt_file_dlg_filter_new_func				fzrt_file_dlg_filter_new;			/* @fsfunc */
	fzrt_file_dlg_filter_delete_func			fzrt_file_dlg_filter_delete;		/* @fsfunc */
	fzrt_file_dlg_filter_set_func				fzrt_file_dlg_filter_set;			/* @fsfunc */
	fzrt_file_dlg_filter_set_ftype_func			fzrt_file_dlg_filter_set_ftype;		/* @fsfunc */

	fzrt_file_get_file_simple_func				fzrt_file_get_file_simple;			/* @fsfunc */
	fzrt_file_get_multiple_files_begin_func		fzrt_file_get_multiple_files_begin;	/* @fsfunc */
	fzrt_file_get_multiple_files_end_func		fzrt_file_get_multiple_files_end;	/* @fsfunc */
	fzrt_file_get_one_file_func					fzrt_file_get_one_file;				/* @fsfunc */
	fzrt_file_select_folder_func				fzrt_file_select_folder	;			/* @fsfunc */
	fzrt_file_put_file_func						fzrt_file_put_file;					/* @fsfunc */
	
	/*** 5.0.0.1 ****/
	fzrt_file_find_floc_by_index_func			fzrt_file_find_floc_by_index;		/* @fsfunc */
	fzrt_file_find_next_floc_func				fzrt_file_find_next_floc;			/* @fsfunc */

	fzrt_file_cnvt_name_func					fzrt_file_cnvt_name;				/* @fsfunc */
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	/* 5.5.0.0 */
	fzrt_file_copy_func							fzrt_file_copy;						/* @fsfunc */

	

}  fz_fzrt_file_fset;

/********************************************************************************************************

	@module FZRT_MEM | formZ Runtime memory functions

	@desc	Runtime memory functions. This is a set of memory management functions. The basic memory
			functions are provided to allocate, resize, move/copy and dispose of memory.  These
			functions should only be used for allocating and using memory in a temporary fashion.
			
			The fz_mem_zone functions provide a more advanced method for managing memory.
			Mem Zones are allocated in large chunks of memory then sub-allocated for pointer
			allocation. There are two types of memory zones, static and dynamic. Static
			mem zones are allocated once and provide only a fixed zone size from which to sub-allocate.
			This is useful for allocating global variables and tables/arrays of fixed or maximum size.
			Dynamic mem zones will grow by a specified increment size as required for mem allocation
			or pointer reallocation. 

	@toc	fzrt_mem

***************************************************************************/

/************************************************************************
																	 
	@fset	fz_fzrt_mem_fset
	
	@desc	formZ Runtime Memory Function Set

	@avail	5.0.0.0

	@toc	fzrt_mem

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime Memory function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_MEM_FSET_TYPE				"\x1e\xa9\xd3\x0a\xf1\x76\x43\x3c\x99\x19\x6d\x76\x6f\x48\x41\xbc"	/* {1ea9d30a-f176-433c-9919-6d766f4841bc} Reference : 1945 */
/********************************************************************************************************

	@fsdef	formZ Runtime Memory function set name.

*********************************************************************************************************/
#define FZ_FZRT_MEM_FSET_NAME				"formZ Runtime Memory functions"
/********************************************************************************************************

	@fsdef	formZ Runtime Memory function set version.

*********************************************************************************************************/
#define FZ_FZRT_MEM_FSET_VERSION		FZPL_VERS_MAKE(8,5,0,1)
typedef struct fz_fzrt_mem_fset
{
	fzpl_fset_prvt_data_td						prvt_data;
	
	/* memory functions */
	fzrt_block_move_func						fzrt_block_move;					/* @fsfunc */
	fzrt_free_mem_func							fzrt_free_mem;						/* @fsfunc */
	fzrt_max_mem_func							fzrt_max_mem;						/* @fsfunc */
	//fzrt_mem_error_func						fzrt_mem_error;	
//	fzrt_get_ptr_size_func						fzrt_get_ptr_size;
	fzrt_dispose_ptr_func						fzrt_dispose_ptr;					/* @fsfunc */
//	fzrt_set_ptr_size_func						fzrt_set_ptr_size;	
	fzrt_new_ptr_clear_func						fzrt_new_ptr_clear;					/* @fsfunc */
	fzrt_new_ptr_func							fzrt_new_ptr;						/* @fsfunc */
	fzrt_FreeMemPhys_func						fzrt_FreeMemPhys;					/* @fsfunc */
	fzrt_total_mem_phys_func					fzrt_total_mem_phys;				/* @fsfunc */
	fzrt_realloc_mem_func						fzrt_realloc_mem;					/* @fsfunc */
	fzrt_mem_free_func							fzrt_mem_free;						/* @fsfunc */
	fzrt_endian_swap_2_bytes_func				fzrt_endian_swap_2_bytes;			/* @fsfunc */
	fzrt_endian_swap_4_bytes_func				fzrt_endian_swap_4_bytes;			/* @fsfunc */
	fzrt_endian_swap_8_bytes_func				fzrt_endian_swap_8_bytes;			/* @fsfunc */
	fzrt_endian_swap_12_bytes_func				fzrt_endian_swap_12_bytes;			/* @fsfunc */
	fzrt_endian_swap_n_bytes_func				fzrt_endian_swap_n_bytes;			/* @fsfunc */
	fzrt_endian_swap_short_func					fzrt_endian_swap_short;				/* @fsfunc */
	fzrt_endian_swap_long_func					fzrt_endian_swap_long;				/* @fsfunc */
	fzrt_endian_swap_ushort_func				fzrt_endian_swap_ushort;			/* @fsfunc */
	fzrt_endian_swap_ulong_func					fzrt_endian_swap_ulong;				/* @fsfunc */
	fzrt_endian_swap_float_func					fzrt_endian_swap_float;				/* @fsfunc */
	fzrt_endian_swap_n_longs_func				fzrt_endian_swap_n_longs;			/* @fsfunc */
	fzrt_endian_swap_n_floats_func				fzrt_endian_swap_n_floats;			/* @fsfunc */
	fzrt_endian_swap_double_func				fzrt_endian_swap_double;			/* @fsfunc */
	fzrt_init_mem_block_func					fzrt_init_mem_block;				/* @fsfunc */
	
	/* mem zone functions */
	fz_mem_zone_get_free_bytes_func				fz_mem_zone_get_free_bytes;			/* @fsfunc */
	fz_mem_zone_free_bytes_func					fz_mem_zone_free_bytes;				/* @fsfunc */
//	mem_zone_last_error_func					mem_zone_last_error;
//	mem_zone_error_func							mem_zone_error;		
	fz_mem_zone_static_init_func					fz_mem_zone_static_init;				/* @fsfunc */
	fz_mem_zone_dynamic_init_func					fz_mem_zone_dynamic_init;				/* @fsfunc */
	fz_mem_zone_finit_func							fz_mem_zone_finit;						/* @fsfunc */
	fz_mem_zone_alloc_func							fz_mem_zone_alloc;						/* @fsfunc */
	fz_mem_zone_free_func							fz_mem_zone_free;						/* @fsfunc */
	fz_mem_zone_realloc_func						fz_mem_zone_realloc;					/* @fsfunc */
//	fz_mem_zone_get_ptr_size_func					fz_mem_zone_get_ptr_size;
	fz_mem_zone_incr_ptr_func						fz_mem_zone_incr_ptr;					/* @fsfunc */
	fz_mem_zone_incr_ptr_long_func					fz_mem_zone_incr_ptr_long;				/* @fsfunc */
	fz_mem_zone_incr_ptr_short_func				fz_mem_zone_incr_ptr_short;			/* @fsfunc */
	fz_mem_zone_incr_ptr_general_func				fz_mem_zone_incr_ptr_general;			/* @fsfunc */
	fz_mem_zone_incr_array_func					fz_mem_zone_incr_array;				/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	fzrt_block_move_64_func							fzrt_block_move_64;						/* @fsfunc */
	fzrt_new_ptr_clear_64_func						fzrt_new_ptr_clear_64;					/* @fsfunc */
	fzrt_new_ptr_64_func							fzrt_new_ptr_64;						/* @fsfunc */
	fzrt_max_mem_64_func							fzrt_max_mem_64;						/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8.5.0.1 functions.									*
	 *																					*
	 ************************************************************************************/

}  fz_fzrt_mem_fset;



/********************************************************************************************************

	@module FZRT_UTIL | formZ Runtime utility functions

	@desc	Runtime utility functions. This is a set of functions that provide utiilty operations.
	
			This includes functions for determining the underlying operating system type and version, 
			getting the current date and time, basic system timer information, and converting character
			sets for strings.
			

	@toc	fzrt_util

***************************************************************************/
/************************************************************************
																	 
	@fset	fz_fzrt_util_fset
	
	@desc	formZ Runtime Utility Function Set

	@avail	5.0.0.0

	@toc	fzrt_util

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime Utility function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_UTIL_FSET_TYPE			"\x27\xef\x68\x61\x79\xbe\x4c\xb1\x98\xfc\x04\x2c\x56\xd1\x9f\x48"	/* {27ef6861-79be-4cb1-98fc-042c56d19f48} Reference : 1946 */
/********************************************************************************************************

	@fsdef	formZ Runtime Utility function set name.

*********************************************************************************************************/
#define FZ_FZRT_UTIL_FSET_NAME			"formZ Runtime Utility functions"
/********************************************************************************************************

	@fsdef	formZ Runtime Utility function set version.

*********************************************************************************************************/
#define FZ_FZRT_UTIL_FSET_VERSION		FZPL_VERS_MAKE(8,5,8,0)
typedef struct fz_fzrt_util_fset
{
	fzpl_fset_prvt_data_td						prvt_data;
	fzrt_tick_count_func						fzrt_tick_count;			/* @fsfunc */
	fzrt_get_date_time_func						fzrt_get_date_time;			/* @fsfunc */
	fzrt_date_time_get_month_func				fzrt_date_time_get_month;		/* @fsfunc */
	fzrt_date_time_get_day_func					fzrt_date_time_get_day;			/* @fsfunc */
	fzrt_date_time_get_year_func				fzrt_date_time_get_year;		/* @fsfunc */
	fzrt_date_time_get_hour_func				fzrt_date_time_get_hour;		/* @fsfunc */
	fzrt_date_time_get_minute_func				fzrt_date_time_get_minute;		/* @fsfunc */
	fzrt_date_time_get_seconds_func				fzrt_date_time_get_seconds;		/* @fsfunc */
	fzrt_get_os_attrs_func						fzrt_get_os_attrs;				/* @fsfunc */
	fzrt_get_platform_func						fzrt_get_platform;				/* @fsfunc */
	fzrt_get_os_long_func						fzrt_get_os_long;				/* @fsfunc */
	fzrt_get_os_string_func						fzrt_get_os_string;				/* @fsfunc */
	fzrt_sys_beep_func							fzrt_sys_beep;					/* @fsfunc */
	fzrt_cnvt_str_func							fzrt_cnvt_str;					/* @fsfunc */	
	fzrt_get_current_charset_func				fzrt_get_current_charset;		/* @fsfunc */	
	fzrt_get_event_what_func					fzrt_get_event_what;			
	fzrt_set_event_what_func					fzrt_set_event_what;			
	fzrt_get_event_when_func					fzrt_get_event_when;			
	fzrt_set_event_when_func					fzrt_set_event_when;			
	fzrt_get_event_where_func					fzrt_get_event_where;			
	fzrt_set_event_where_func					fzrt_set_event_where;			
	fzrt_get_event_message_func					fzrt_get_event_message;			
	fzrt_set_event_message_func					fzrt_set_event_message;			
	fzrt_get_event_modifiers_func				fzrt_get_event_modifiers;		
	fzrt_set_event_modifiers_func				fzrt_set_event_modifiers;		

	fzrt_evnt_get_key_state_func				fzrt_evnt_get_key_state;		/* @fsfunc */
	fzrt_evnt_get_actual_key_state_func			fzrt_evnt_get_actual_key_state;		/* @fsfunc */
	fzrt_show_cursor_func						fzrt_show_cursor;				/* @fsfunc */
	fzrt_cursor_set_location_func				fzrt_cursor_set_location;

	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	
	fzrt_lang_get_func							fzrt_lang_get;					/* @fsfunc */
	fzrt_lang_system_get_func					fzrt_lang_system_get;			/* @fsfunc */
	fzrt_lang_get_info_func						fzrt_lang_get_info;				/* @fsfunc */
	fzrt_lang_info_init_func					fzrt_lang_info_init;			/* @fsfunc */
	fzrt_lang_info_finit_func					fzrt_lang_info_finit;			/* @fsfunc */
	fzrt_lang_info_is_avail_func				fzrt_lang_info_is_avail;		/* @fsfunc */
	fzrt_lang_info_get_name_func				fzrt_lang_info_get_name;		/* @fsfunc */
	fzrt_lang_info_get_tag_func					fzrt_lang_info_get_tag;			/* @fsfunc */
	fzrt_lang_info_get_lang_func				fzrt_lang_info_get_lang;		/* @fsfunc */
	
	fzrt_char_byte_func							fzrt_char_byte;		/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 6.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	
	fzrt_evnt_one_shot_timer_func					fzrt_evnt_one_shot_timer;			/* @fsfunc */
	fzrt_evnt_one_shot_timer_cancel_func			fzrt_evnt_one_shot_timer_cancel;	/* @fsfunc */
	fzrt_evnt_periodic_timer_func					fzrt_evnt_periodic_timer;			/* @fsfunc */
	fzrt_evnt_periodic_timer_cancel_func			fzrt_evnt_periodic_timer_cancel;	/* @fsfunc */

	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 7.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

	fzrt_launch_application_func					fzrt_launch_application;			/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 8.5.8.0 functions.									*
	 *																					*
	 ************************************************************************************/

}  fz_fzrt_util_fset;

/********************************************************************************************************

	@module FZRT_GRAF | formZ Runtime graphics functions

	@desc	Runtime graphics functions. This is a set of functions that provide graphics operations.
	
			The basic graphic element is the pixel buffer (fzrt_pbuf*). There are two types of pixel buffers,
			a basic pixel store, and a pixel buffer graphics port. Both types have an underlying pixel store,
			but the graphics port pixel buffer can be the destination for drawing operations.
			The drawing port is tied to the underlying operating system graphics context.
			
			The basic pixel store has advantages over the drawing port pixel buffer. On the Macintosh platform,
			the drawing port is limited to 4096x4096 bounding rectangle. On the Windows platform, the drawing 
			port is slower in operations that involve pixel pushing (fzrt_copy_bits, fzrt_pbuf_put_pixels.
			
			All drawing is made to the current graphics port. Not all graphics ports are associated with a pixel
			buffer, most notably the graphics port of a window is not associated with a pixel buffer.
	
			

	@toc	fzrt_graf

***************************************************************************/
/************************************************************************
																	 
	@fset	fz_fzrt_graf_fset
	
	@desc	formZ Runtime Graphics Function Set

	@avail	5.0.0.0

	@toc	fzrt_graf

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime Graphics function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_GRAF_FSET_TYPE			"\xb8\x0e\x70\xa8\xf8\x25\x43\x6d\xbb\xe8\x72\x4d\x32\x46\xff\x91"	/* {b80e70a8-f825-436d-bbe8-724d3246ff91} Reference : 1947	 */
/********************************************************************************************************

	@fsdef	formZ Runtime Graphics function set name.

*********************************************************************************************************/
#define FZ_FZRT_GRAF_FSET_NAME			"formZ Runtime Graphics functions"
/********************************************************************************************************

	@fsdef	formZ Runtime Graphics function set version.

*********************************************************************************************************/
#define FZ_FZRT_GRAF_FSET_VERSION		FZPL_VERS_MAKE(5,0,0,0)
typedef struct fz_fzrt_graf_fset
{
	fzpl_fset_prvt_data_td						prvt_data;

	fzrt_ctab_dispose_func						fzrt_ctab_dispose;		/* @fsfunc */
	fzrt_ctab_create_func						fzrt_ctab_create;			/* @fsfunc */
	fzrt_ctab_get_color_count_func				fzrt_ctab_get_color_count;		/* @fsfunc */
	fzrt_ctab_get_rgb_func						fzrt_ctab_get_rgb;			/* @fsfunc */
	fzrt_ctab_set_rgb_func						fzrt_ctab_set_rgb;			/* @fsfunc */
	fzrt_ctab_resize_func						fzrt_ctab_resize;			/* @fsfunc */
	fzrt_ctab_2_cult_func						fzrt_ctab_2_cult;			/* @fsfunc */

	fzrt_pbuf_new_func							fzrt_pbuf_new;			/* @fsfunc */
	fzrt_pbuf_dispose_func						fzrt_pbuf_dispose;		/* @fsfunc */
	fzrt_pbuf_get_bounds_func					fzrt_pbuf_get_bounds;		/* @fsfunc */
	fzrt_pbuf_get_clut_func						fzrt_pbuf_get_clut;			/* @fsfunc */
	fzrt_pbuf_get_pixbytes_func					fzrt_pbuf_get_pixbytes;		/* @fsfunc */
	fzrt_pbuf_get_pixelformat_func				fzrt_pbuf_get_pixelformat;		/* @fsfunc */
	fzrt_pbuf_get_pixelsize_func				fzrt_pbuf_get_pixelsize;		/* @fsfunc */
	fzrt_pbuf_get_rowbytes_func					fzrt_pbuf_get_rowbytes;		/* @fsfunc */
	fzrt_pbuf_update_clut_func					fzrt_pbuf_update_clut;		/* @fsfunc */
	fzrt_pbuf_copy_func							fzrt_pbuf_copy;				/* @fsfunc */
	fzrt_pbuf_copy_to_graf_port_func			fzrt_pbuf_copy_to_graf_port;		/* @fsfunc */
	fzrt_pbuf_copy_from_graf_port_func			fzrt_pbuf_copy_from_graf_port;		/* @fsfunc */
	fzrt_pbuf_put_pixels_func					fzrt_pbuf_put_pixels;		/* @fsfunc */
	fzrt_pbuf_get_pixels_func					fzrt_pbuf_get_pixels;		/* @fsfunc */
	fzrt_pbuf_rect_to_rect_func					fzrt_pbuf_rect_to_rect;		/* @fsfunc */
	fzrt_pbuf_rect_from_rect_func				fzrt_pbuf_rect_from_rect;		/* @fsfunc */
	fzrt_pbuf_point_to_point_func				fzrt_pbuf_point_to_point;		/* @fsfunc */
	fzrt_pbuf_point_from_point_func				fzrt_pbuf_point_from_point;		/* @fsfunc */
	fzrt_pbuf_is_port_func						fzrt_pbuf_is_port;	/* @fsfunc */
	fzrt_pbuf_set_port_func						fzrt_pbuf_set_port;	/* @fsfunc */
	fzrt_pbuf_get_port_func						fzrt_pbuf_get_port;	/* @fsfunc */
		
	fzrt_get_window_port_func					fzrt_get_window_port;			/*@fsfunc */
	fzrt_get_window_from_port_func				fzrt_get_window_from_port;			/*@fsfunc */
	fzrt_get_port_func							fzrt_get_port;		/* @fsfunc */
	fzrt_set_port_func							fzrt_set_port;		/* @fsfunc */
	fzrt_get_clip_func							fzrt_get_clip;		/* @fsfunc */
	fzrt_set_clip_func							fzrt_set_clip;		/* @fsfunc */
	fzrt_pen_size_func							fzrt_pen_size;		/* @fsfunc */
	fzrt_get_pen_size_func						fzrt_get_pen_size;		/* @fsfunc */
	fzrt_pen_mode_func							fzrt_pen_mode;		/* @fsfunc */
	fzrt_get_pen_mode_func						fzrt_get_pen_mode;		/* @fsfunc */
	fzrt_pen_pat_func							fzrt_pen_pat;		/* @fsfunc */
	fzrt_pen_normal_func						fzrt_pen_normal;		/* @fsfunc */
	fzrt_text_font_func							fzrt_text_font;		/* @fsfunc */
	fzrt_text_face_func							fzrt_text_face;		/* @fsfunc */
	fzrt_text_size_func							fzrt_text_size;		/* @fsfunc */
	fzrt_text_mode_func							fzrt_text_mode;		/* @fsfunc */
	fzrt_get_pen_pat_func						fzrt_get_pen_pat;		/* @fsfunc */
	fzrt_pen_pat_new_func						fzrt_pen_pat_new;		/* @fsfunc */
	fzrt_pen_pat_dispose_func					fzrt_pen_pat_dispose;		/* @fsfunc */
	fzrt_get_std_penpat_func					fzrt_get_std_penpat;	/* @fsfunc */
	fzrt_get_text_font_func						fzrt_get_text_font;		/* @fsfunc */
	fzrt_get_text_face_func						fzrt_get_text_face;		/* @fsfunc */
	fzrt_get_text_size_func						fzrt_get_text_size;		/* @fsfunc */
	fzrt_get_text_mode_func						fzrt_get_text_mode;		/* @fsfunc */
	fzrt_get_fnum_func							fzrt_get_fnum;		/* @fsfunc */
	fzrt_get_font_name_func						fzrt_get_font_name;		/* @fsfunc */
	fzrt_string_width_func						fzrt_string_width;		/* @fsfunc */
	fzrt_text_width_func						fzrt_text_width;		/* @fsfunc */
	fzrt_draw_char_func							fzrt_draw_char;		/* @fsfunc */
	fzrt_draw_string_func						fzrt_draw_string;		/* @fsfunc */
	fzrt_draw_text_func							fzrt_draw_text;		/* @fsfunc */
	fzrt_get_color_func							fzrt_get_color;		/* @fsfunc */
	fzrt_hsl_2_rgb_func							fzrt_hsl_2_rgb;		/* @fsfunc */
	fzrt_cmy_2_rgb_func							fzrt_cmy_2_rgb;		/* @fsfunc */
	fzrt_hsv_2_rgb_func							fzrt_hsv_2_rgb;		/* @fsfunc */
	fzrt_rgb_2_hsl_func							fzrt_rgb_2_hsl;		/* @fsfunc */
	fzrt_rgb_2_hsv_func							fzrt_rgb_2_hsv;		/* @fsfunc */
			
	fzrt_paint_rect_func						fzrt_paint_rect;		/* @fsfunc */
	fzrt_frame_rect_func						fzrt_frame_rect;		/* @fsfunc */
	fzrt_erase_rect_func						fzrt_erase_rect;		/* @fsfunc */
	fzrt_invert_rect_func						fzrt_invert_rect;		/* @fsfunc */
	fzrt_fill_rect_func							fzrt_fill_rect;			/* @fsfunc */
	fzrt_paint_round_rect_func					fzrt_paint_round_rect;		/* @fsfunc */
	fzrt_frame_round_rect_func					fzrt_frame_round_rect;		/* @fsfunc */
	fzrt_paint_oval_func						fzrt_paint_oval;		/* @fsfunc */
	fzrt_frame_oval_func						fzrt_frame_oval;		/* @fsfunc */
	fzrt_erase_oval_func						fzrt_erase_oval;		/* @fsfunc */
	fzrt_frame_arc_func							fzrt_frame_arc;		/* @fsfunc */
	fzrt_copy_bits_func							fzrt_copy_bits;		/* @fsfunc */
	fzrt_copy_bits_colorize_func				fzrt_copy_bits_colorize;		/* @fsfunc */
	fzrt_open_poly_func							fzrt_open_poly;		/* @fsfunc */
	fzrt_close_poly_func						fzrt_close_poly;		/* @fsfunc */
	fzrt_kill_poly_func							fzrt_kill_poly;		/* @fsfunc */
	fzrt_paint_poly_func						fzrt_paint_poly;		/* @fsfunc */
	fzrt_erase_poly_func						fzrt_erase_poly;		/* @fsfunc */
	fzrt_invert_poly_func						fzrt_invert_poly;		/* @fsfunc */
	fzrt_fill_poly_func							fzrt_fill_poly;		/* @fsfunc */
	fzrt_get_pixel_func							fzrt_get_pixel;		/* @fsfunc */
	fzrt_get_cpixel_func						fzrt_get_cpixel;		/* @fsfunc */
	fzrt_set_cpixel_func						fzrt_set_cpixel;		/* @fsfunc */
	fzrt_rgb_fore_color_func					fzrt_rgb_fore_color;		/* @fsfunc */
	fzrt_rgb_back_color_func					fzrt_rgb_back_color;		/* @fsfunc */
	fzrt_get_fore_color_func					fzrt_get_fore_color;		/* @fsfunc */
	fzrt_get_back_color_func					fzrt_get_back_color;		/* @fsfunc */
	fzrt_plot_icon_func							fzrt_plot_icon;		/* @fsfunc */
	fzrt_new_rgn_func							fzrt_new_rgn;		/* @fsfunc */
	fzrt_dispose_rgn_func						fzrt_dispose_rgn;		/* @fsfunc */
	fzrt_rect_rgn_func							fzrt_rect_rgn;		/* @fsfunc */
	fzrt_empty_rgn_func							fzrt_empty_rgn;		/* @fsfunc */
	fzrt_pt_in_rgn_func							fzrt_pt_in_rgn;		/* @fsfunc */
	fzrt_set_empty_rgn_func						fzrt_set_empty_rgn;		/* @fsfunc */
	fzrt_copy_rgn_func							fzrt_copy_rgn;		/* @fsfunc */
	fzrt_union_rgn_func							fzrt_union_rgn;		/* @fsfunc */
	fzrt_sect_rgn_func							fzrt_sect_rgn;		/* @fsfunc */
	fzrt_diff_rgn_func							fzrt_diff_rgn;		/* @fsfunc */
	fzrt_rect_in_rgn_func						fzrt_rect_in_rgn;		/* @fsfunc */
	fzrt_offset_rect_func						fzrt_offset_rect;					/* @fsfunc*/
	fzrt_equal_rgn_func							fzrt_equal_rgn;					/* @fsfunc*/
	fzrt_offset_rgn_func						fzrt_offset_rgn;					/* @fsfunc */
	fzrt_frame_rgn_func							fzrt_frame_rgn;		/* @fsfunc */
	fzrt_fill_rgn_func							fzrt_fill_rgn;		/* @fsfunc */
	fzrt_get_rgn_bbox_func						fzrt_get_rgn_bbox;		/* @fsfunc */
	fzrt_set_std_cursor_func					fzrt_set_std_cursor;		/* @fsfunc */
	fzrt_set_std_wait_cursor_func				fzrt_set_std_wait_cursor;		/* @fsfunc */
	fzrt_get_pen_func							fzrt_get_pen;			/* @fsfunc */
	fzrt_move_to_func							fzrt_move_to;			/* @fsfunc */
	fzrt_move_func								fzrt_move;				/* @fsfunc */
	fzrt_line_to_func							fzrt_line_to;			/* @fsfunc */
	fzrt_line_func								fzrt_line;				/* @fsfunc */
	fzrt_set_rect_func							fzrt_set_rect;			/* @fsfunc */
	fzrt_empty_rect_func						fzrt_empty_rect;		/* @fsfunc */
	fzrt_inset_rect_func						fzrt_inset_rect;		/* @fsfunc */
	fzrt_local_to_global_func					fzrt_local_to_global;		/* @fsfunc */
	fzrt_global_to_local_func					fzrt_global_to_local;		/* @fsfunc */
	fzrt_graf_get_pattern_bits_func				fzrt_graf_get_pattern_bits;	/* @fsfunc */
	
	/* 1,0,0,1 */
	fzrt_invert_rgn_func						fzrt_invert_rgn;					/* @fsfunc */
	fzrt_inset_rgn_func							fzrt_inset_rgn;					/* @fsfunc */
	fzrt_frame_poly_func						fzrt_frame_poly;					/* @fsfunc */
	fzrt_equal_rect_func						fzrt_equal_rect	;				/* @fsfunc */
	fzrt_char_width_func						fzrt_char_width	;				/* @fsfunc */
	fzrt_erase_rgn_func							fzrt_erase_rgn	;				/* @fsfunc */
	fzrt_pt_in_rect_func						fzrt_pt_in_rect	;				/* @fsfunc */	
	fzrt_sect_rect_func							fzrt_sect_rect	;				/* @fsfunc */	
	fzrt_union_rect_func						fzrt_union_rect	;				/* @fsfunc */	
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	 
	 fzrt_paint_rgn_func						fzrt_paint_rgn;				/* @fsfunc */
	 fzrt_open_rgn_func							fzrt_open_rgn;				/* @fsfunc */
	 fzrt_close_rgn_func						fzrt_close_rgn;				/* @fsfunc */
	 
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	
	fzrt_line_weight_func						fzrt_line_weight;		/* @fsfunc */
} fz_fzrt_graf_fset;


/********************************************************************************************************

	@module FZRT_UI | formZ Runtime User Interface functions

	@desc	Runtime user interface functions. This is a set of functions that provide user interface operations.
	
			Currently, the only user interface operations available are operations on menus that are loaded from
			a formZ Resource File (fzr file) or created with fzrt_menu_create.
			
			Most other user interface functionality is provided through the fuim API.
				
	@toc	fzrt_ui

***************************************************************************/
/************************************************************************
																	 
	@fset	fz_fzrt_ui_fset
	
	@desc	formZ Runtime User Interface Function Set

	@avail	5.0.0.0

	@toc	fzrt_ui

*************************************************************************/
/********************************************************************************************************

	@fsdef	formZ Runtime User Interface function set UUID.
  
*********************************************************************************************************/
#define FZ_FZRT_UI_FSET_TYPE			"\xe7\x68\x4e\x95\xa6\xed\x4b\x56\x8f\xbf\xc7\x40\x1e\x1f\x51\x85"	/* {e7684e95-a6ed-4b56-8fbf-c7401e1f5185} Reference : 1948 */
/********************************************************************************************************

	@fsdef	formZ Runtime User Interface function set name.

*********************************************************************************************************/
#define FZ_FZRT_UI_FSET_NAME			"formZ Runtime Graphics functions"
/********************************************************************************************************

	@fsdef	formZ Runtime User Interface function set version.

*********************************************************************************************************/
#define FZ_FZRT_UI_FSET_VERSION		FZPL_VERS_MAKE(9,1,0,0)

typedef struct fz_fzrt_ui_fset
{
	fzpl_fset_prvt_data_td						prvt_data;

	fzrt_menu_enable_item_func					fzrt_menu_enable_item;					/* @fsfunc */
	fzrt_menu_disable_item_func					fzrt_menu_disable_item;					/* @fsfunc */
	fzrt_menu_check_item_func					fzrt_menu_check_item;					/* @fsfunc */
	fzrt_menu_get_item_text_func				fzrt_menu_get_item_text;			/* @fsfunc */
	fzrt_menu_set_item_text_func				fzrt_menu_set_item_text;			/* @fsfunc */
	fzrt_menu_delete_item_func					fzrt_menu_delete_item;				/* @fsfunc */
	fzrt_menu_change_item_attributes_func		fzrt_menu_change_item_attributes_obsolete;	/* @fsfunc */
	fzrt_menu_get_item_attributes_func			fzrt_menu_get_item_attributes_obsolete;		/* @fsfunc */
	fzrt_menu_append_separator_func				fzrt_menu_append_separator;			/* @fsfunc */
	fzrt_menu_insert_separator_func				fzrt_menu_insert_separator;			/* @fsfunc */
	fzrt_menu_create_func						fzrt_menu_create;					/* @fsfunc */
	fzrt_menu_create_font_menu_func				fzrt_menu_create_font_menu;				/* @fsfunc */
	fzrt_menu_font_id_get_func					fzrt_menu_font_id_get;				/* @fsfunc */
	fzrt_menu_append_item_text_func				fzrt_menu_append_item_text;			/* @fsfunc */
	fzrt_menu_insert_item_text_func				fzrt_menu_insert_item_text;			/* @fsfunc */
	fzrt_menu_count_items_func					fzrt_menu_count_items;				/* @fsfunc */
	fzrt_menu_dispose_func						fzrt_menu_dispose;					/* @fsfunc */
	
	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 5.0.0.0 functions.									*
	 *																					*
	 ************************************************************************************/
	
	fzrt_appr_is_darkmode_func					fzrt_appr_is_darkmode;					/* @fsfunc */

	/************************************************************************************
	 *																					*
	 *		DO NOT MAKE ANY CHANGES TO THE ABOVE FUNCTIONS!!!							*
	 *		DO NOT ADD ANY FUNCTION ABOVE THIS LINE!!!									*
	 *		@comm  End of version 9.1.0.0 functions.									*
	 *																					*
	 ************************************************************************************/

}  fz_fzrt_ui_fset;	



#ifdef __cplusplus
}
#endif


#endif	/* _FZRT_API_H */
